package com.kingtom.kirin.api.auth.dto;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import javax.xml.stream.events.Comment;

import com.kingtom.kirin.api.auth.mapper.AuthMapper;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.StringUtils;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023 /8/30
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class AuthMenuTreeDTO{

    private String id;

    private String name;

    private String otherName;

    private String code;

    private String parentId;

    private Integer hide;

    private Integer preset;

    private Integer orderNum;

    private String type;

    private String actionType;

    private String icon;

    private String routeUri;

    private String ext1;

    private String ext2;

    private String gmtCreate;

    private String gmtModified;

    private String sysId;

    private String testExt3;

    private String parentName;

    private List<AuthMenuTreeDTO> children;


    /**
     * 根据集合构建出多颗树.
     *
     * @param list the list
     * @return the list
     */
    public static List<AuthMenuTreeDTO> build(List<AuthMenuDTO> list){
        if(CollectionUtils.isEmpty(list)){
            return new ArrayList<>();
        }
        list = new ArrayList<>(list);
        list.sort(Comparator.comparing(e -> StringUtils.join(e.getPath(), CommonConst.EMPTY_STRING)));
        Map<String, List<AuthMenuDTO>> map = list.stream().collect(Collectors.groupingBy(AuthMenuDTO::getParentId));
        return buildChildren(map, list.get(0).getSysId(), CommonConst.EMPTY_STRING);
    }

    private static List<AuthMenuTreeDTO> buildChildren(Map<String, List<AuthMenuDTO>> map, String key,
            String parentName){
        List<AuthMenuDTO> menus = map.remove(key);
        if(CollectionUtils.isEmpty(menus)){
            return null;
        }
        return menus.stream().map(menu -> {
            AuthMenuTreeDTO dto = AuthMapper.INSTANCT.convertMenuToTree(menu);
            dto.setChildren(buildChildren(map, menu.getId(), menu.getName()));
            dto.setParentName(parentName);
            return dto;
        }).sorted(Comparator.comparing(AuthMenuTreeDTO::getOrderNum)).collect(Collectors.toList());
    }

}
