package com.kingtom.kirin.api.auth.mapper;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import com.kingtom.kirin.api.auth.dto.AuthMenuDTO;
import com.kingtom.kirin.api.auth.dto.AuthMenuTreeDTO;
import com.kingtom.kirin.api.auth.dto.AuthPermissionDTO;
import com.kingtom.kirin.api.auth.dto.AuthRoleDTO;
import com.kingtom.kirin.app.auth.model.*;
import com.kingtom.kirin.core.common.json.JObject;
import com.kingtom.kirin.core.resource.ResourceObject;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Named;
import org.mapstruct.factory.Mappers;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/8/29
 */
@Mapper
public interface AuthMapper{

    public static final AuthMapper INSTANCT = Mappers.getMapper(AuthMapper.class);

    @Mapping(source = "source", target = "resources", qualifiedByName = "authRoleUserIds2DTO")
    @Mapping(source = "ext3", target = "testExt3", qualifiedByName = "authRoleExt32DTO")
    @Mapping(target = "gmtCreate", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "gmtModified", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "menuIds", ignore = true)
    AuthRoleDTO convert(AuthRole data);

    @Named("authRoleUserIds2DTO")
    default List<ResourceObject<String>> authRoleUserIds2DTO(JObject<AuthRoleSource> jObject){
        if(jObject == null || jObject.getE() == null || jObject.getE().getResources() == null){
            return null;
        }
        return jObject.getE().getResources().stream().map(e -> {
            ResourceObject<String> r = new ResourceObject<>();
            r.setName(e.getName());
            r.setObject(e.getObject());
            r.setResId(e.getResId());
            r.setResType(e.getResType());
            return r;
        }).collect(Collectors.toList());

    }

    @Named("authRoleExt32DTO")
    default String authRoleExt32DTO(JObject<AuthRoleExt3> jObject){
        return jObject == null || jObject.getE() == null ? null : jObject.getE().getTestExt3();
    }

    default AuthRole convert(AuthRoleDTO dto){
        if(dto == null){
            return null;
        }
        AuthRole r = convertToBean(dto);
        AuthRoleExt3 e = convertToExt3Bean(dto);
        AuthRoleSource ra = convertToResBean(dto);
        r.setExt3(new JObject<>(e));
        r.setSource(new JObject<>(ra));
        return r;
    }

    @Mapping(target = "gmtCreate", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "gmtModified", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "source", ignore = true)
    @Mapping(target = "ext3", ignore = true)
    AuthRole convertToBean(AuthRoleDTO dto);

    AuthRoleExt3 convertToExt3Bean(AuthRoleDTO dto);

    AuthRoleSource convertToResBean(AuthRoleDTO dto);


    @Mapping(source = "route", target = "path", qualifiedByName = "authMenuRoute2DTO")
    @Mapping(source = "ext3", target = "testExt3", qualifiedByName = "authMenuExt32DTO")
    @Mapping(target = "gmtCreate", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "gmtModified", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "parentName", ignore = true)
    AuthMenuDTO convert(AuthMenu data);

    @Named("authMenuRoute2DTO")
    default List<String> authMenuRoute2DTO(JObject<AuthMenuRoute> jObject){
        if(jObject == null || jObject.getE() == null || jObject.getE().getPath() == null){
            return null;
        }
        return new ArrayList<>(jObject.getE().getPath());

    }

    @Named("authMenuExt32DTO")
    default String authMenuExt32DTO(JObject<AuthMenuExt3> jObject){
        return jObject == null || jObject.getE() == null ? null : jObject.getE().getTestExt3();
    }

    default AuthMenu convert(AuthMenuDTO dto){
        if(dto == null){
            return null;
        }
        AuthMenu r = convertToBean(dto);
        AuthMenuExt3 e = convertToExt3Bean(dto);
        AuthMenuRoute ra = convertToRouteBean(dto);
        r.setExt3(new JObject<>(e));
        r.setRoute(new JObject<>(ra));
        return r;
    }

    @Mapping(target = "gmtCreate", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "gmtModified", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "route", ignore = true)
    @Mapping(target = "ext3", ignore = true)
    AuthMenu convertToBean(AuthMenuDTO dto);

    AuthMenuExt3 convertToExt3Bean(AuthMenuDTO dto);

    AuthMenuRoute convertToRouteBean(AuthMenuDTO dto);

    @Mapping(source = "ext3", target = "testExt3", qualifiedByName = "authPermissionExt32DTO")
    @Mapping(target = "gmtCreate", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "gmtModified", dateFormat = "yyyy-MM-dd HH:mm:ss")
    AuthPermissionDTO convert(AuthPermission data);

    @Named("authPermissionExt32DTO")
    default String authPermissionExt32DTO(JObject<AuthPermissionExt3> jObject){
        return jObject == null || jObject.getE() == null ? null : jObject.getE().getTestExt3();
    }

    default AuthPermission convert(AuthPermissionDTO dto){
        if(dto == null){
            return null;
        }
        AuthPermission r = convertToBean(dto);
        AuthPermissionExt3 e = convertToExt3Bean(dto);
        r.setExt3(new JObject<>(e));
        return r;
    }

    @Mapping(target = "gmtCreate", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "gmtModified", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "ext3", ignore = true)
    AuthPermission convertToBean(AuthPermissionDTO dto);

    AuthPermissionExt3 convertToExt3Bean(AuthPermissionDTO dto);

    @Mapping(target = "children", ignore = true)
    AuthMenuTreeDTO convertMenuToTree(AuthMenuDTO dto);

}
