package com.kingtom.kirin.api.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.api.affair.mapper.AffairMapper;
import com.kingtom.kirin.api.affair.model.AffairInfoDTO;
import com.kingtom.kirin.api.security.SaSecurityUtils;
import com.kingtom.kirin.app.affair.model.AffairInfoCurrentDO;
import com.kingtom.kirin.app.affair.model.AffairInfoDO;
import com.kingtom.kirin.app.affair.service.IAffairService;
import com.kingtom.kirin.app.auth.base.AuthInit;
import com.kingtom.kirin.app.auth.base.Menu;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.base.Result;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/10/21 10:55
 */
@Tag(name = "AffairController", description = "事务中心接口")
@RestController
@AuthInit(menus = {
        @Menu(name = "事务中心", otherName = "系统配置", code = "affairCenter", icon = "setting", routeUri = "sysSetting",
                orderNum = 5),
        @Menu(name = "事务中心:待办事务", otherName = "待办事务", code = "affairCenter_waitdo", icon = "",
                routeUri = "waitdo", orderNum = 51),
        @Menu(name = "事务中心:已办事务", otherName = "已办事务", code = "affairCenter_done", icon = "",
                routeUri = "done", orderNum = 52)}, roles = {})
public class AffairController {

    @Autowired
    private IAffairService affairService;

    @Operation(summary = "获取待办事务列表",description = "没有query")
    @GetMapping("/affairs/waitdo_list")
    @SaCheckPermission(value = {"AuthMenu:affairCenter_waitdo:Execute"})
    public Result<PageInfo<AffairInfoDTO>> getAffairInfosOfWaitdo(@RequestParam(required = false) String groupName,
                                                                @RequestParam(required = false) String search,
                                                                  @RequestParam(required = false) String token,
                                                                  @RequestParam(required = false) String excludeToken,
                                                                Integer pageNum, Integer pageSize) {
        String userId = SaSecurityUtils.findSessionData().getUserId();
        PageInfo<AffairInfoCurrentDO> infos = affairService.findAffairs(userId, CommonConst.StateEnum.Enable,
                groupName, search, token, excludeToken, pageNum, pageSize);
        PageInfo<AffairInfoDTO> r = CollectionUtils.convertPage(infos, AffairMapper.INSTANCT::convertToDTO);
        return Result.ok("获取待办事务列表", r);
    }

    @Operation(summary = "获取待办事务总数", description = "没有query")
    @GetMapping("/affairs/waitdo_count")
    @SaCheckPermission(value = {"AuthMenu:affairCenter_waitdo:Execute"})
    public Result<Long> getAffairInfosOfWaitdoCount(){
        String userId = SaSecurityUtils.findSessionData().getUserId();
        Long res = affairService.findAffairsCount(userId, CommonConst.StateEnum.Enable);
        return Result.ok("获取待办事务总数", res);
    }

    @Operation(summary = "获取已办事务列表",description = "没有query")
    @GetMapping("/affairs/done_list")
    @SaCheckPermission(value = {"AuthMenu:affairCenter_done:Execute"})
    public Result<PageInfo<AffairInfoDTO>> getAffairInfosOfDone(@RequestParam(required = false) String groupName,
                                                                @RequestParam(required = false) String search,
                                                                @RequestParam(required = false) String token,
                                                                @RequestParam(required = false) String excludeToken,
                                                                Integer pageNum, Integer pageSize) {
        String userId = SaSecurityUtils.findSessionData().getUserId();
        PageInfo<AffairInfoCurrentDO> infos = affairService.findAffairs(userId, CommonConst.StateEnum.Disable,
                groupName, search,token, excludeToken, pageNum, pageSize);
        PageInfo<AffairInfoDTO> r = CollectionUtils.convertPage(infos, AffairMapper.INSTANCT::convertToDTO);
        return Result.ok("获取已办事务列表", r);
    }

    @Operation(summary = "获取某事务信息")
    @GetMapping("/affairs/{affairId}/info")
    public Result<AffairInfoDTO> getAffairInfo(@PathVariable String affairId) {
        AffairInfoDO info = affairService.findAffairById(affairId);
        return Result.ok("获取某事务信息", AffairMapper.INSTANCT.convertToDTO(info.getAffairInfo()));
    }

}
