package com.kingtom.kirin.api.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.api.conf.mapper.ConfMapper;
import com.kingtom.kirin.api.conf.model.ConfInfoDTO;
import com.kingtom.kirin.app.conf.model.ConfInfo;
import com.kingtom.kirin.app.conf.service.IConfService;
import com.kingtom.kirin.core.common.base.Result;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import io.swagger.v3.oas.annotations.Operation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/9/5 19:43
 */
@RestController
@Slf4j
public class ConfController {

    @Autowired
    private IConfService confService;

    @Operation(summary = "获取配置列表",description = "query={\"search\":\"搜索\"}")
    @GetMapping("/confs/list")
    @SaCheckPermission(value = {"AuthMenu:sysSetting_env:Execute"})
    public Result<PageInfo<ConfInfoDTO>> getConfInfos(String query, Integer pageNum, Integer pageSize) {
        Map<String,String> qmap = JsonUtils.toObject(query,Map.class);
        PageInfo<ConfInfo> info = confService.findConfInfos(qmap.get("search"),pageNum,pageSize);
        PageInfo<ConfInfoDTO> r = CollectionUtils.convertPage(info, ConfMapper.INSTANCT::convert);
        return Result.ok("获取配置列表", r);
    }

    @Operation(summary = "新增配置信息", description = "内容格式：{\"name\":\"配置名\",\"key\":\"配置KEY\",\"valueType\":\"值类型:String|Boolean|Number|Date|JsonObject\",\"value\":\"值：一律用字符表达\",\"des\":\"描述\",\"ext1\":\"\",\"ext2\":\"\"}")
    @PostMapping("/confs/new")
    @SaCheckPermission(value = {"AuthMenu:sysSetting_env:Execute"})
    public Result<ConfInfoDTO> postCreateConfInfo(@RequestBody String json) {
        ConfInfoDTO m = JsonUtils.toObject(json, ConfInfoDTO.class);
        ConfInfo info = confService.createConfInfo(ConfMapper.INSTANCT.convert(m));
        ConfInfoDTO r = ConfMapper.INSTANCT.convert(info);
        return Result.ok("新增配置信息成功", r);
    }

    @Operation(summary = "获取配置信息")
    @GetMapping("/confs/{id}/info")
    //先加上鉴权注解，后面运行态用这个接口会报错，到时候再处理
    @SaCheckPermission(value = {"AuthMenu:sysSetting_env:Execute"})
    public Result<ConfInfoDTO> getConfInfo(@PathVariable String id) {
        ConfInfo info = confService.findConfInfoById(id);
        ConfInfoDTO r = ConfMapper.INSTANCT.convert(info);
        return Result.ok("获取配置信息成功", r);
    }

    @Operation(summary = "更新配置信息", description = "内容格式：{\"name\":\"配置名\",\"key\":\"配置KEY\",\"valueType\":\"值类型:String|Boolean|Number|Date|JsonObject\",\"value\":\"值：一律用字符表达\",\"des\":\"描述\",\"ext1\":\"\",\"ext2\":\"\"}")
    @PostMapping("/confs/{id}/info")
    @SaCheckPermission(value = {"AuthMenu:sysSetting_env:Execute"})
    public Result<ConfInfoDTO> postUpdateConfInfo(@PathVariable String id,@RequestBody String json) {
        ConfInfoDTO m = JsonUtils.toObject(json, ConfInfoDTO.class);
        m.setId(id);
        ConfInfo info = confService.updateConfInfo(ConfMapper.INSTANCT.convert(m));
        ConfInfoDTO r = ConfMapper.INSTANCT.convert(info);
        return Result.ok("更新配置信息成功", r);
    }

    @Operation(summary = "删除配置信息")
    @PostMapping("/confs/{id}/")
    @SaCheckPermission(value = {"AuthMenu:sysSetting_env:Execute"})
    public Result<String> postDeleteConfInfo(@PathVariable String id) {
        confService.deleteConfInfo(id);
        return Result.ok("删除配置信息成功", null);
    }

    @Operation(summary = "批量删除配置信息", description = "内容格式：{\"ids\":[\"111\",\"222\"],}")
    @PostMapping("/confs/delete")
    @SaCheckPermission(value = {"AuthMenu:sysSetting_env:Execute"})
    public Result<String> postDeleteConfInfos(@RequestBody String json) {
        Map<String,Object> tm = JsonUtils.toObject(json, Map.class);
        confService.deleteConfInfos((List)tm.get("ids"));
        return Result.ok("删除配置信息成功", null);
    }

}
