package com.kingtom.kirin.api.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.api.ddic.mapper.DdicMapper;
import com.kingtom.kirin.api.ddic.model.DdicInfoDTO;
import com.kingtom.kirin.api.ddic.model.DdicItemDTO;
import com.kingtom.kirin.app.ddic.model.DdicInfo;
import com.kingtom.kirin.app.ddic.model.DdicItem;
import com.kingtom.kirin.app.ddic.service.IDdicService;
import com.kingtom.kirin.core.common.base.Result;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/9/1 16:29
 */
@RestController
@Slf4j
@Tag(name="DdicController", description = "字典管理接口")
public class DdicController {

    @Autowired
    private IDdicService ddicService;

    @Operation(summary = "获取字典列表",description = "query={\"search\":\"搜索\"}")
    @GetMapping("/ddics/list")
    @SaCheckPermission(value = {"AuthMenu:sysSetting_dict:Execute"})
    public Result<PageInfo<DdicInfoDTO>> getDdicInfos(String query, Integer pageNum, Integer pageSize) {
        Map<String,String> qmap = JsonUtils.toObject(query,Map.class);
        PageInfo<DdicInfo> info = ddicService.findDdicInfos(qmap.get("search"),pageNum,pageSize);
        PageInfo<DdicInfoDTO> r = CollectionUtils.convertPage(info, DdicMapper.INSTANCT::convert);
        return Result.ok("获取字典列表", r);
    }

    @Operation(summary = "新增字典信息", description = "内容格式：{\"name\":\"字典名称\",\"simpleName\":\"字典简称\",\"showModel\":\"展示类型:simpleName|name\",\"code\":\"字典编码\",\"state\":1,\"des\":\"描述\"}")
    @PostMapping("/ddics/new")
    @SaCheckPermission(value = {"AuthMenu:sysSetting_dict:Execute"})
    public Result<DdicInfoDTO> postCreateDdicInfo(@RequestBody String json) {
        DdicInfoDTO m = JsonUtils.toObject(json, DdicInfoDTO.class);
        DdicInfo info = ddicService.createDdicInfo(DdicMapper.INSTANCT.convert(m));
        DdicInfoDTO r = DdicMapper.INSTANCT.convert(info);
        return Result.ok("新增字典信息成功", r);
    }

    @Operation(summary = "获取字典信息")
    @GetMapping("/ddics/{ddicId}/info")
    @SaCheckPermission(value = {"AuthMenu:sysSetting_dict:Execute"})
    public Result<DdicInfoDTO> getDdicInfo(@PathVariable String ddicId) {
        DdicInfo info = ddicService.findDdicInfoById(ddicId);
        DdicInfoDTO r = DdicMapper.INSTANCT.convert(info);
        return Result.ok("获取字典信息成功", r);
    }

    @Operation(summary = "更新字典信息", description = "内容格式：{\"name\":\"字典名称\",\"simpleName\":\"字典简称\",\"showModel\":\"展示类型:simpleName|name\",\"code\":\"字典编码\",\"state\":1,\"des\":\"描述\"}")
    @PostMapping("/ddics/{ddicId}/info")
    @SaCheckPermission(value = {"AuthMenu:sysSetting_dict:Execute"})
    public Result<DdicInfoDTO> postUpdateDdicInfo(@PathVariable String ddicId,@RequestBody String json) {
        DdicInfoDTO m = JsonUtils.toObject(json, DdicInfoDTO.class);
        m.setId(ddicId);
        DdicInfo info = ddicService.updateDdicInfo(DdicMapper.INSTANCT.convert(m));
        DdicInfoDTO r = DdicMapper.INSTANCT.convert(info);
        return Result.ok("更新字典信息成功", r);
    }

    @Operation(summary = "删除字典信息")
    @PostMapping("/ddics/{ddicId}/")
    @SaCheckPermission(value = {"AuthMenu:sysSetting_dict:Execute"})
    public Result<String> postDeleteDdicInfo(@PathVariable String ddicId) {
        ddicService.deleteDdicInfo(ddicId);
        return Result.ok("删除字典信息成功", null);
    }

    @Operation(summary = "批量删除字典信息", description = "内容格式：{\"ids\":[\"111\",\"222\"],}")
    @PostMapping("/ddics/delete")
    @SaCheckPermission(value = {"AuthMenu:sysSetting_dict:Execute"})
    public Result<String> postDeleteDdicInfos(@RequestBody String json) {
        Map<String,Object> tm = JsonUtils.toObject(json, Map.class);
        ddicService.deleteDdicInfos((List)tm.get("ids"));
        return Result.ok("删除字典信息成功", null);
    }

    @Operation(summary = "获取字典项列表",description = "query={\"search\":\"搜索\"}")
    @GetMapping("/ddics/{ddicId}/item_list")
    @SaCheckPermission(value = {"AuthMenu:sysSetting_dictData:Execute"})
    public Result<PageInfo<DdicItemDTO>> getDdicItems(String query, @PathVariable String ddicId, Integer pageNum, Integer pageSize) {
        Map<String,String> qmap = JsonUtils.toObject(query,Map.class);
        PageInfo<DdicItem> info = ddicService.findDdicItems(qmap.get("search"),ddicId,pageNum,pageSize);
        PageInfo<DdicItemDTO> r = CollectionUtils.convertPage(info, DdicMapper.INSTANCT::convert);
        return Result.ok("获取字典项列表", r);
    }

    @Operation(summary = "新增字典项信息", description = "内容格式：{\"name\":\"项显示值\",\"sort\":\"排序\",\"ddicId\":\"字典ID\",\"parentId\":\"父项ID\",\"des\":\"描述\",\"key\":\"项存储值\",\"ext1\":\"扩展1\",\"ext2\":\"扩展2\"}")
    @PostMapping("/ddics/item_new")
    @SaCheckPermission(value = {"AuthMenu:sysSetting_dictData:Execute"})
    public Result<DdicItemDTO> postCreateDdicItem(@RequestBody String json) {
        DdicItemDTO m = JsonUtils.toObject(json, DdicItemDTO.class);
        DdicItem info = ddicService.createDdicItem(DdicMapper.INSTANCT.convert(m));
        DdicItemDTO r = DdicMapper.INSTANCT.convert(info);
        return Result.ok("新增字典项信息成功", r);
    }

    @Operation(summary = "获取字典项信息")
    @GetMapping("/ddics/item_{itemId}/info")
    @SaCheckPermission(value = {"AuthMenu:sysSetting_dictData:Execute"})
    public Result<DdicItemDTO> getDdicItem(@PathVariable String itemId) {
        DdicItem info = ddicService.findDdicItemById(itemId);
        DdicItemDTO r = DdicMapper.INSTANCT.convert(info);
        return Result.ok("获取字典项信息成功", r);
    }

    @Operation(summary = "更新字典项信息", description = "内容格式：{\"name\":\"项显示值\",\"sort\":\"排序\",\"ddicId\":\"字典ID\",\"parentId\":\"父项ID\",\"des\":\"描述\",\"key\":\"项存储值\",\"ext1\":\"扩展1\",\"ext2\":\"扩展2\"}")
    @PostMapping("/ddics/item_{itemId}/info")
    @SaCheckPermission(value = {"AuthMenu:sysSetting_dictData:Execute"})
    public Result<DdicItemDTO> postUpdateDdicItem(@PathVariable String itemId,@RequestBody String json) {
        DdicItemDTO m = JsonUtils.toObject(json, DdicItemDTO.class);
        m.setId(itemId);
        DdicItem info = ddicService.updateDdicItem(DdicMapper.INSTANCT.convert(m));
        DdicItemDTO r = DdicMapper.INSTANCT.convert(info);
        return Result.ok("更新字典项信息成功", r);
    }

    @Operation(summary = "删除字典项信息")
    @PostMapping("/ddics/item_{itemId}/")
    @SaCheckPermission(value = {"AuthMenu:sysSetting_dictData:Execute"})
    public Result<String> postDeleteDdicItem(@PathVariable String itemId) {
        ddicService.deleteDdicItem(itemId);
        return Result.ok("删除字典项信息成功", null);
    }

    @Operation(summary = "批量删除字典项信息", description = "内容格式：{\"ids\":[\"111\",\"222\"],}")
    @PostMapping("/ddics/items_delete")
    @SaCheckPermission(value = {"AuthMenu:sysSetting_dictData:Execute"})
    public Result<String> postDeleteDdicItems(@RequestBody String json) {
        Map<String,Object> tm = JsonUtils.toObject(json, Map.class);
        ddicService.deleteDdicItems((List)tm.get("ids"));
        return Result.ok("删除字典项信息成功", null);
    }

}
