package com.kingtom.kirin.api.controller;

import java.util.List;
import java.util.Map;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.api.dto.IdsDTO;
import com.kingtom.kirin.api.org.mapper.OrgMapper;
import com.kingtom.kirin.api.org.model.*;
import com.kingtom.kirin.app.org.OrgConst;
import com.kingtom.kirin.app.org.bo.OrgPersonBO;
import com.kingtom.kirin.app.org.bo.OrgUnitBO;
import com.kingtom.kirin.app.org.model.OrgPerson;
import com.kingtom.kirin.app.org.model.OrgStruct;
import com.kingtom.kirin.app.org.model.OrgUnit;
import com.kingtom.kirin.app.org.service.IOrgService;
import com.kingtom.kirin.app.system.utils.SystemUtils;
import com.kingtom.kirin.core.common.base.Result;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/10/14
 */
@Tag(name = "OrgController", description = "组织、部门、岗位、人员相关接口")
@RequestMapping("/org")
@RestController
public class OrgController{

    @Autowired
    private IOrgService orgService;

    @Operation(summary = "新建人员")
    @PostMapping("/org_persons/")
    @SaCheckPermission(value = {"AuthMenu:orgManage_personnel:Execute"})
    public Result<OrgPersonFullDTO> addPerson(@RequestBody OrgPersonFullDTO dto){
        //创建人员
        OrgPerson person = OrgMapper.INSTANCT.convert(dto);
        person.setSysId(SystemUtils.getCurrentSystemId());
        String orgId = orgService.findDefaultUnit().getId();
        OrgPersonBO bo = orgService.createPerson(orgId, dto.getDepartmentIds(), dto.getPostIds(), person);
        //转换处理返回结果
        return Result.ok(OrgPersonFullDTO.build(bo, orgId));
    }

    @Operation(summary = "更新人员")
    @PostMapping("/org_persons/{id}/info")
    @SaCheckPermission(value = {"AuthMenu:orgManage_personnel:Execute"})
    public Result<OrgPersonFullDTO> updatePerson(@PathVariable("id") String id, @RequestBody OrgPersonFullDTO dto){
        OrgPerson person = OrgMapper.INSTANCT.convert(dto);
        person.setId(id);
        String orgId = orgService.findDefaultUnit().getId();
        OrgPersonBO bo = orgService.updatePerson(orgId, dto.getDepartmentIds(), dto.getPostIds(), person);
        //转换处理返回结果
        return Result.ok(OrgPersonFullDTO.build(bo, orgId));
    }

    @Operation(summary = "删除人员")
    @PostMapping("/org_persons/{id}/")
    @SaCheckPermission(value = {"AuthMenu:orgManage_personnel:Execute"})
    public Result<String> deletePerson(@PathVariable("id") String id){
        //todo
        //orgService.deletePersonById(id);
        return Result.ok("操作成功！");
    }

    @Operation(summary = "批量删除人员")
    @PostMapping("/org_persons/delete/")
    @SaCheckPermission(value = {"AuthMenu:orgManage_personnel:Execute"})
    public Result<String> deletePersons(@RequestBody IdsDTO ids){
        //todo
        //orgService.deletePersonByIds(ids.getIds());
        return Result.ok("操作成功！");
    }

    @Parameter(name = "id", description = "人员Id", in = ParameterIn.PATH, required = true)
    @Parameter(name = "orgId", description = "组织Id", in = ParameterIn.QUERY, required = false)
    @Operation(summary = "根据id查找人员")
    @GetMapping("/org_person/{id}/info")
    public Result<OrgPersonFullDTO> getOrgPersonById(@PathVariable("id") String id,
            @RequestParam(value = "orgId", required = false) String orgId){
        orgId = orgService.findDefaultUnit().getId();
        OrgPersonBO bo = orgService.findPerson(id, orgId);
        return Result.ok(OrgPersonFullDTO.build(bo, orgId));
    }

    @Operation(summary = "人员分页查询")
    @Parameters({@Parameter(name = "query", description = "搜索内容", in = ParameterIn.QUERY, required = false),
            @Parameter(name = "structId", description = "机构Id", in = ParameterIn.QUERY, required = false),
            @Parameter(name = "pageNum", description = "当前页", in = ParameterIn.QUERY, required = true),
            @Parameter(name = "pageSize", description = "分页", in = ParameterIn.QUERY, required = true)})
    @GetMapping("/org_person/list")
    @SaCheckPermission(value = {"AuthMenu:orgManage_personnel:Execute"})
    public Result<PageInfo<OrgPersonDTO>> getPersonList(
            @RequestParam(value = "structId", required = false) String structId, @RequestParam("query") String query,
            @RequestParam("pageNum") Integer pageNum, @RequestParam("pageSize") Integer pageSize){
        String search = null;
        String name = null;
        String phone = null;
        String licenseCode = null;
        if(StringUtils.isNotBlank(query)){
            Map<String, String> map = JsonUtils.toObject(query, Map.class);
            search = map.get("search");
            name = map.get("name");
            phone = map.get("phone");
            licenseCode = map.get("licenseCode");
        }
        if(StringUtils.isEmpty(structId)){
            OrgUnit org = orgService.findDefaultUnit();
            OrgStruct rootInnerOrg =
                    orgService.findStructByCodeAndOrgId(OrgConst.OrgStructSubTypeEnum.RootInnerOrg.getCodePrefix(),
                            org.getId());
            structId = rootInnerOrg.getId();
        }
        PageInfo<OrgPerson> page =
                orgService.findPersonPage(structId, search, name, phone, licenseCode, pageNum, pageSize);
        PageInfo<OrgPersonDTO> r = CollectionUtils.convertPage(page, OrgMapper.INSTANCT::convert);
        return Result.ok(r);
    }

    @Parameter(name = "id", description = "组织Id", in = ParameterIn.PATH, required = true)
    @Operation(summary = "根据id查找组织")
    @GetMapping("/org_unit/{id}/info")
    public Result<OrgUnitFullDTO> getOrgUnitById(@PathVariable("id") String id){
        OrgUnitBO org = orgService.findOrgUnit(id);
        return Result.ok(OrgUnitFullDTO.build(org));
    }

    @Operation(summary = "获取组织信息")
    @GetMapping("/org_unit/default/info")
    public Result<OrgUnitFullDTO> getOrgUnitById(){
        OrgUnit orgUnit = orgService.findDefaultUnit();
        OrgUnitBO org = orgService.findOrgUnit(orgUnit.getId());
        return Result.ok(OrgUnitFullDTO.build(org));
    }

    @Operation(summary = "组织分页查询")
    @Parameters({@Parameter(name = "query", description = "搜索内容", in = ParameterIn.QUERY, required = false),
            @Parameter(name = "pageNum", description = "当前页", in = ParameterIn.QUERY, required = true),
            @Parameter(name = "pageSize", description = "分页", in = ParameterIn.QUERY, required = true)})
    @GetMapping("/org_unit/list")
    public Result<PageInfo<OrgUnitDTO>> getUnitList(@RequestParam("query") String query,
            @RequestParam("pageNum") Integer pageNum, @RequestParam("pageSize") Integer pageSize){
        String search = null;
        String name = null;
        String phone = null;
        String licenseType = null;
        String licenseCode = null;
        if(StringUtils.isNotBlank(query)){
            Map<String, String> map = JsonUtils.toObject(query, Map.class);
            search = map.get("search");
            name = map.get("name");
            phone = map.get("phone");
            licenseType = map.get("licenseType");
            licenseCode = map.get("licenseCode");
        }
        PageInfo<OrgUnit> page = orgService.findOrgUnitPage(query, name, licenseCode, pageNum, pageSize);
        PageInfo<OrgUnitDTO> r = CollectionUtils.convertPage(page, OrgMapper.INSTANCT::convert);
        return Result.ok(r);
    }

    @Operation(summary = "新建组织")
    @PostMapping("/org_units/")
    //@SaCheckPermission(value = {"AuthMenu:orgManage_org:Execute"})
    public Result<OrgUnitFullDTO> addUnit(@RequestBody OrgUnitFullDTO dto){
        OrgUnit unit = OrgMapper.INSTANCT.convert(dto);
        unit.setSysId(SystemUtils.getCurrentSystemId());
        OrgUnitBO res = orgService.createUnit(unit, dto.getAdminIds());
        return Result.ok(OrgUnitFullDTO.build(res));
    }

    @Parameter(name = "id", description = "组织Id", in = ParameterIn.PATH, required = true)
    @Operation(summary = "删除组织")
    @PostMapping("/org_units/{id}/")
    //@SaCheckPermission(value = {"AuthMenu:orgManage_org:Execute"})
    public Result<String> deleteUnit(@PathVariable("id") String id){
        orgService.deleteUnitById(id);
        return Result.ok("操作成功！");
    }

    @Operation(summary = "批量删除组织")
    @PostMapping("/org_units/delete/")
    //@SaCheckPermission(value = {"AuthMenu:orgManage_org:Execute"})
    public Result<String> deleteUnits(@RequestBody IdsDTO ids){
        orgService.deleteUnitByIds(ids.getIds());
        return Result.ok("操作成功！");
    }

    @Operation(summary = "更新组织")
    @PostMapping("/org_units/{id}/info")
    //@SaCheckPermission(value = {"AuthMenu:orgManage_org:Execute"})
    public Result<OrgUnitDTO> updateUnit(@PathVariable("id") String id, @RequestBody OrgUnitFullDTO dto){
        OrgUnit unit = OrgMapper.INSTANCT.convert(dto);
        unit.setId(id);
        OrgUnitBO res = orgService.updateUnit(unit, dto.getAdminIds());
        return Result.ok(OrgUnitFullDTO.build(res));
    }

    @Parameter(name = "id", description = "机构Id", in = ParameterIn.PATH, required = true)
    @Operation(summary = "根据id查找机构")
    @GetMapping("/org_struct/{id}/info")
    public Result<OrgStructDTO> getOrgStructById(@PathVariable("id") String id){
        OrgStruct orgStruct = orgService.findStructById(id);
        return Result.ok(OrgMapper.INSTANCT.convert(orgStruct));
    }

    @Parameter(name = "id", description = "机构Id", in = ParameterIn.PATH, required = true)
    @Operation(summary = "删除机构")
    @PostMapping("/org_structs/{id}/")
    //@SaCheckPermission(value = {"AuthMenu:orgManage_org:Execute"})
    public Result<String> deleteStruct(@PathVariable("id") String id){
        boolean f = orgService.deleteStructAndChild(id);
        if(f){
            return Result.ok("操作成功！");
        }
        return Result.fail("操作失败！");
    }

    @Operation(summary = "新建部门")
    @PostMapping("/org_structs/departments/")
    //@SaCheckPermission(value = {"AuthMenu:orgManage_org:Execute"})
    public Result<OrgStructDTO> addDepartment(@RequestBody OrgStructDTO dto){
        OrgStruct struct = OrgMapper.INSTANCT.convert(dto);
        struct.setSysId(SystemUtils.getCurrentSystemId());
        struct.setSubType(OrgConst.OrgStructSubTypeEnum.SolidDepartment.getKey());
        OrgStruct struct1 = orgService.createStruct(struct);
        return Result.ok(OrgMapper.INSTANCT.convert(struct1));
    }

    @Operation(summary = "新建岗位")
    @PostMapping("/org_structs/posts/")
    //@SaCheckPermission(value = {"AuthMenu:orgManage_org:Execute"})
    public Result<OrgStructDTO> addPost(@RequestBody OrgStructDTO dto){
        OrgStruct struct = OrgMapper.INSTANCT.convert(dto);
        struct.setSysId(SystemUtils.getCurrentSystemId());
        struct.setSubType(OrgConst.OrgStructSubTypeEnum.SolidPost.getKey());
        OrgStruct struct1 = orgService.createStruct(struct);
        return Result.ok(OrgMapper.INSTANCT.convert(struct1));
    }

    @Operation(summary = "更新部门")
    @PostMapping("/org_structs/departments/{id}/info")
    //@SaCheckPermission(value = {"AuthMenu:orgManage_org:Execute"})
    public Result<OrgStructDTO> updateDepartment(@PathVariable("id") String id, @RequestBody OrgStructDTO dto){
        OrgStruct struct = OrgMapper.INSTANCT.convert(dto);
        struct.setId(id);
        struct.setSubType(OrgConst.OrgStructSubTypeEnum.SolidDepartment.getKey());
        OrgStruct struct1 = orgService.updateStructById(struct);
        return Result.ok(OrgMapper.INSTANCT.convert(struct1));
    }

    @Operation(summary = "更新岗位")
    @PostMapping("/org_structs/posts/{id}/info")
    //@SaCheckPermission(value = {"AuthMenu:orgManage_org:Execute"})
    public Result<OrgStructDTO> updatePost(@PathVariable("id") String id, @RequestBody OrgStructDTO dto){
        OrgStruct struct = OrgMapper.INSTANCT.convert(dto);
        struct.setId(id);
        struct.setSubType(OrgConst.OrgStructSubTypeEnum.SolidPost.getKey());
        OrgStruct struct1 = orgService.updateStructById(struct);
        return Result.ok(OrgMapper.INSTANCT.convert(struct1));
    }

    @Operation(summary = "部门树")
    @GetMapping("/org_structs/departments/tree")
    //@SaCheckPermission(value = {"AuthMenu:orgManage_org:Execute"})
    public Result<OrgStructNode> getDepartmentTree(@RequestParam("orgId") String orgId){
        List<OrgStruct> list = orgService.findDepartmentTree(orgId);
        OrgStructNode tree = OrgStructNode.build(list, OrgConst.OrgStructSubTypeEnum.RootDepartment.getKey());
        return Result.ok(tree);
    }

    @Operation(summary = "岗位树")
    @GetMapping("/org_structs/posts/tree")
    //@SaCheckPermission(value = {"AuthMenu:orgManage_org:Execute"})
    public Result<OrgStructNode> getPostTree(@RequestParam("orgId") String orgId){
        List<OrgStruct> list = orgService.findPostTree(orgId);
        OrgStructNode tree = OrgStructNode.build(list, OrgConst.OrgStructSubTypeEnum.RootPost.getKey());
        return Result.ok(tree);
    }

}
