package com.kingtom.kirin.api.controller;

import cn.dev33.satoken.annotation.SaIgnore;
import com.kingtom.kirin.api.security.ISaSecurityService;
import com.kingtom.kirin.api.security.dto.LoginParamDTO;
import com.kingtom.kirin.app.auth.AuthConst;
import com.kingtom.kirin.app.auth.base.AuthInit;
import com.kingtom.kirin.app.auth.base.Menu;
import com.kingtom.kirin.app.auth.base.Role;
import com.kingtom.kirin.app.security.base.SessionData;
import com.kingtom.kirin.app.security.utils.SecurityUtils;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.base.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/8/25
 */
@Tag(name = "SecurityController", description = "登录认证相关接口")
@RequestMapping("/web")
@RestController
@AuthInit(menus = {
        @Menu(name = "首页", otherName = "首页", code = "home", icon = "home", routeUri = "home", orderNum = 0),
        @Menu(name = "个人设置", otherName = "个人设置", code = "account", hide = CommonConst.BooleanEnum.True,
                icon = "", routeUri = "account", orderNum = 1),
        @Menu(name = "组织管理", otherName = "组织管理", code = "orgManage", icon = "reconciliation",
                routeUri = "orgManage", orderNum = 2),
        @Menu(name = "组织管理:人员管理", otherName = "人员管理", code = "orgManage_personnel", icon = "",
                routeUri = "personnel", orderNum = 21),
        @Menu(name = "组织管理:组织架构", otherName = "组织架构", code = "orgManage_org", icon = "", routeUri = "org",
                orderNum = 22),
        @Menu(name = "组织管理:岗位管理", otherName = "岗位管理", code = "orgManage_post", icon = "", routeUri = "post",
                orderNum = 23),
        @Menu(name = "权限设置", otherName = "权限设置", code = "perSetting", icon = "usergroup-add",
                routeUri = "perSetting", orderNum = 3),
        @Menu(name = "权限设置:用户管理", otherName = "用户管理", code = "perSetting_user", icon = "",
                routeUri = "user", orderNum = 31),
        @Menu(name = "权限设置:角色管理", otherName = "角色管理", code = "perSetting_role", icon = "",
                routeUri = "role", orderNum = 32),
        @Menu(name = "权限设置:菜单管理", otherName = "菜单管理", code = "perSetting_menu", icon = "",
                routeUri = "menu", orderNum = 33),
        @Menu(name = "系统配置", otherName = "系统配置", code = "sysSetting", icon = "setting", routeUri = "sysSetting",
                orderNum = 4),
        @Menu(name = "系统配置:系统信息", otherName = "系统信息", code = "sysSetting_info", icon = "",
                routeUri = "info", orderNum = 41),
        @Menu(name = "系统配置:数据字典", otherName = "数据字典", code = "sysSetting_dict", icon = "",
                routeUri = "dict", orderNum = 42),
        @Menu(name = "系统配置:字典数据", otherName = "字典数据", code = "sysSetting_dictData",
                hide = CommonConst.BooleanEnum.True, icon = "", routeUri = "dict/detail", orderNum = 43),
        @Menu(name = "系统配置:环境配置", otherName = "环境配置", code = "sysSetting_env", icon = "", routeUri = "env",
                orderNum = 44)}, roles = {
        @Role(name = AuthConst.SUPER_ADMIN_ROLE_CN_NAME, code = AuthConst.SUPER_ADMIN_ROLE_US_NAME, menuCode = {"ALL"},
                display = CommonConst.BooleanEnum.False),
        @Role(name = AuthConst.USER_ROLE_CN_NAME, code = AuthConst.USER_ROLE_US_NAME, menuCode = {"home",
                "affairCenter_waitdo","affairCenter_done"})})
public class SecurityController{

    @Autowired
    private ISaSecurityService authService;

    @Operation(summary = "登录", description = "{\"principal\":\"\",\"credentials\":\"\",\"type\":\"Password\"," +
            "\"encode\":0}")
    @PostMapping("/login")
    @Parameters({@Parameter(name = "principal", description = "账号", required = true, in = ParameterIn.QUERY),
            @Parameter(name = "credentials", description = "密码", required = true, in = ParameterIn.QUERY),
            @Parameter(name = "type", description = "认证类型", example = "Password", required = true,
                    in = ParameterIn.QUERY),
            @Parameter(name = "encode", description = "类型", example = "Password", required = false,
                    in = ParameterIn.QUERY)})
    @SaIgnore
    public Result<SessionData> login(@RequestBody LoginParamDTO loginParamDTO){
        String principal = loginParamDTO.getPrincipal();
        String credentials = loginParamDTO.getCredentials();
        String type = loginParamDTO.getType();
        if(CommonConst.BooleanEnum.True.getKey().equals(loginParamDTO.getEncode())){
            //rsa解密
            principal = SecurityUtils.rsaPrivateKeyDecode(principal);
            credentials = SecurityUtils.rsaPrivateKeyDecode(credentials);
        }
        SessionData KirinSessionData = authService.authenticate(principal, credentials, type, null);
        return Result.ok(KirinSessionData);
    }

    @PostMapping("/logout")
    @Operation(summary = "登出")
    public Result<String> logout(){
        boolean f = authService.logout();
        return Result.ok("登出成功", null);
    }

    @GetMapping("/login_info")
    @Operation(summary = "获取登录信息")
    public Result<SessionData> loginInfo(){
        SessionData r = authService.findSessionInfo();
        return Result.ok("登录信息获取成功", r);
    }

    @GetMapping("/rsa/public_key")
    @Operation(summary = "获取rsa加密的公钥")
    public Result<String> getRsaPublicKey(){
        return Result.ok(SecurityUtils.getRsaPublicKey());
    }

}
