package com.kingtom.kirin.api.controller;

import cn.dev33.satoken.annotation.SaIgnore;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.StrUtil;
import com.kingtom.kirin.api.space.mapper.SpaceMapper;
import com.kingtom.kirin.api.space.model.SpaceFileDTO;
import com.kingtom.kirin.app.space.base.Base64FileSource;
import com.kingtom.kirin.app.space.base.IFileSource;
import com.kingtom.kirin.app.space.base.MultipartFileSource;
import com.kingtom.kirin.app.space.base.SpaceFileReadFileSource;
import com.kingtom.kirin.app.space.model.SpaceFile;
import com.kingtom.kirin.app.space.service.ISpaceService;
import com.kingtom.kirin.app.space.utils.SpaceUtils;
import com.kingtom.kirin.core.common.base.Result;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import com.kingtom.kirin.core.common.utils.ObjectUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.List;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/8/29 10:30
 */
@RestController
@Tag(name="SpaceController", description = "文件接口")
public class SpaceController {

    @Autowired
    private ISpaceService spaceService;

    @PostMapping("/files/uploadbase64")
    @Operation( summary = "临时上传图片-多个", description = "scope：非必填；内容格式:[{\"fileName\":\"test.img\",\"contentType\":\"可以不传\",\"content\":\"data:image/png;base64,/9j/4AAQSkZJ....等编码信息\"}]")
    public Result<List<SpaceFileDTO>> uploadFileByBase64(@RequestParam String scope, @RequestBody String json) {
        List<Base64FileSource> sources = JsonUtils.toObjectList(json, Base64FileSource.class);
        if(CollectionUtils.isNotEmpty(sources)) {
            List<SpaceFile> f = spaceService.createSpaceFileOfTmp(sources);
            if(ObjectUtils.isNotNull(f)) {

                return Result.ok("上传临时图片成功", CollectionUtils.convert(f, SpaceMapper.INSTANCT::convert));
            } else {
                return Result.fail("上传临时图片失败");
            }
        } else {
            return Result.fail("上传临时图片失败");
        }
    }

    @PostMapping("/files/upload")
    @Operation( summary = "临时上传文件-多个", description = "scope：非必填；内容格式：标准文件流")
    public Result<List<SpaceFileDTO>> uploadFile(@RequestParam(required = false) String scope, @RequestPart List<MultipartFile> files) {
        if(CollectionUtils.isNotEmpty(files)) {
            List<IFileSource> fs = CollectionUtils.convert(files, MultipartFileSource::new);
            List<SpaceFile> savedFiles = spaceService.createSpaceFileOfTmp(fs);
            if(ObjectUtils.isNotNull(savedFiles)) {
                return Result.ok("上传临时文件成功", CollectionUtils.convert(savedFiles,SpaceMapper.INSTANCT::convert));
            } else {
                return Result.fail("上传临时文件失败");
            }
        } else {
            return Result.fail("上传临时文件失败");
        }
    }

    @GetMapping("/files/{fileId}/show_image")
    @Operation(summary = "展示图片")
    @SaIgnore
    public void showImage(@PathVariable String fileId, @RequestParam(required = false) String ori, HttpServletRequest request, HttpServletResponse response) {
        SpaceFile f = spaceService.findSpaceFile(fileId);
        if( ObjectUtils.isAnyNull(f) ) {
            throw new AppException("找不到文件");
        } else if(SpaceUtils.isImage(f)) {
            IFileSource s = spaceService.findSpaceFileAsStream(fileId);
            Boolean oriFlag = StrUtil.isBlank(ori) ?Boolean.FALSE:Boolean.valueOf(ori);
            if(s.isCanHttpUrl()) {
                // 跳转到nginx处理
                try{
                    response.sendRedirect(((SpaceFileReadFileSource)s).setOri(oriFlag).getHttpUrl());
                } catch (Exception e) {
                    throw new AppException("图片重定向失败,id="+fileId,e);
                }
            } else {
                InputStream is = null;
                try{
                    response.setContentType(s.getContentType());
                    is = s.getInputStream();
                    OutputStream os = response.getOutputStream();
                    IoUtil.copy(is,os);
                } catch (Exception e) {
                    response.setContentType("application/json");
                    throw new AppException("展示图片失败",e);
                } finally {
                    IoUtil.close(is);
                }
            }
        } else {
            throw new AppException("文件不是图片类型，无法展现");
        }
    }

    @GetMapping("/files/{fileId}/download")
    @Operation(summary = "下载文件")
    @SaIgnore
    public void downloadFile(@PathVariable String fileId, HttpServletRequest request, HttpServletResponse response) {
        IFileSource s = spaceService.findSpaceFileAsStream(fileId);
        if( ObjectUtils.isAnyNull(s) ) {
            throw new AppException("找不到文件");
        } else {
            InputStream is = null;
            try{
                response.setContentType(s.getContentType());
                is = s.getInputStream();
                OutputStream os = response.getOutputStream();
                IoUtil.copy(is,os);
            } catch (Exception e) {
                response.setContentType("application/json");
                throw new AppException("下载文件失败",e);
            } finally {
                IoUtil.close(is);
            }
        }
    }

}
