package com.kingtom.kirin.api.controller;

import cn.dev33.satoken.annotation.SaIgnore;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.kingtom.kirin.api.support.mapper.SupportMapper;
import com.kingtom.kirin.api.support.model.EnumObject;
import com.kingtom.kirin.app.conf.ConfConst;
import com.kingtom.kirin.app.conf.model.ConfInfo;
import com.kingtom.kirin.app.conf.service.IConfService;
import com.kingtom.kirin.app.ddic.DdicConst;
import com.kingtom.kirin.app.ddic.model.DdicInfoDO;
import com.kingtom.kirin.app.ddic.model.DdicItem;
import com.kingtom.kirin.app.ddic.service.IDdicService;
import com.kingtom.kirin.app.ddic.utils.DdicUtils;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.base.IBaseEnum;
import com.kingtom.kirin.core.common.base.Result;
import com.kingtom.kirin.core.common.utils.BaseEnumUtils;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/9/1 16:29
 */
@RestController
@Slf4j
@Tag(name="SupportController", description = "基础使用支撑接口")
public class SupportController {

    @Autowired
    private IDdicService ddicService;
    @Autowired
    private IConfService confService;

    @Operation(summary = "获取字典项信息")
    @GetMapping("/ddics/code_{code}/items")
    @SaIgnore
    public Result<List<EnumObject>> getDdicItem(@PathVariable String code) {
        DdicInfoDO info = ddicService.findDdicInfoDOByCode(code);
        if(ObjectUtil.isNotNull(info)) {
            List<EnumObject> r = CollectionUtils.convert(info.getDdicItems(),SupportMapper.INSTANCT::convert);
            return Result.ok("获取字典项信息成功", r);
        } else {
            return Result.fail("获取字典项信息失败");
        }
    }

    @Operation(summary = "获取字典项信息（多个）")
    @GetMapping("/ddics/code_list/items")
    @SaIgnore
    public Result<Map<String,List<EnumObject>>> getDdicItems(@RequestParam(required = true) String codes) {
        Map<String,List<EnumObject>> r = new LinkedHashMap<>();
        if(StrUtil.isNotBlank(codes)) {
            List<String> codeList = ListUtil.toList(codes.split(CommonConst.COMMA_STRING));
            for(String code:codeList) {
                DdicInfoDO info = ddicService.findDdicInfoDOByCode(code);
                if(ObjectUtil.isNotNull(info)) {
                    List<EnumObject> enumObjectList = CollectionUtils.convert(info.getDdicItems(),SupportMapper.INSTANCT::convert);
                    r.put(code,enumObjectList);
                }
            }
            return Result.ok("获取字典项信息（多个）成功", r);
        } else {
            return Result.fail("获取字典项信息（多个）失败");
        }
    }

    @Operation(summary = "获取枚举项信息")
    @GetMapping("/enums/class_{enumClassName}/items")
    @SaIgnore
    public Result<List<EnumObject>> getEnumItems(@PathVariable String enumClassName) {
        List<IBaseEnum> enumItems = BaseEnumUtils.findEnumItemList(enumClassName);
        if(ObjectUtil.isNotNull(enumItems)) {
            List<EnumObject> r = CollectionUtils.convert(enumItems, SupportMapper.INSTANCT::convert);
            return Result.ok("获取枚举项信息成功", r);
        } else {
            return Result.fail("获取枚举项信息失败");
        }
    }

    @Operation(summary = "获取配置项信息")
    @GetMapping("/confs/code_{confKey}/items")
    @SaIgnore
    public Result<EnumObject> getConf(@PathVariable String confKey) {
        ConfInfo info = confService.findConfInfoByKey(confKey);
        if(ObjectUtil.isNotNull(info)) {
            EnumObject r = SupportMapper.INSTANCT.convert(info);
            return Result.ok("获取配置项信息成功", r);
        } else {
            return Result.fail("获取配置项信息失败");
        }
    }

}
