package com.kingtom.kirin.api.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.dev33.satoken.annotation.SaIgnore;
import cn.hutool.core.collection.CollectionUtil;
import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.api.system.mapper.SystemMapper;
import com.kingtom.kirin.api.system.model.SystemInfoDTO;
import com.kingtom.kirin.api.system.model.SystemUserDTO;
import com.kingtom.kirin.app.system.model.SystemInfo;
import com.kingtom.kirin.app.system.model.SystemUser;
import com.kingtom.kirin.app.system.service.ISystemService;
import com.kingtom.kirin.app.system.service.SystemDefaultService;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.base.Result;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/8/24 19:44
 */
@RestController
@Tag(name="SystemController", description = "系统管理接口")
public class SystemController {

    @Autowired
    private ISystemService systemService;
    @Autowired
    private SystemDefaultService systemDefaultService;

    @Operation(summary = "获取系统信息")
    @GetMapping("/system/info")
    @SaIgnore
    public Result<SystemInfoDTO> getSystemInfo() {
        SystemInfo info = systemDefaultService.findDefaultSystemInfo();
        SystemInfoDTO r = SystemMapper.INSTANCT.convert(info);
        return Result.ok("获取系统信息成功", r);
    }

    @Operation(summary = "更新系统信息", description = "内容格式：{\"id\":\"id\",\"name\":\"成都诚唐科技\",\"otherName\":\"诚唐科技\",\"imageLogo\":\"1fa34fga231sdf5132\",\"imageBackground\":\"123sdf31b5werwzvz\",\"description\":\"牛逼的公司\"}")
    @PostMapping("/system/info")
    @SaCheckPermission(value = {"AuthMenu:sysSetting_info:Execute"})
    public Result<SystemInfoDTO> postSystemInfo(@RequestBody String json) {
        SystemInfoDTO m = JsonUtils.toObject(json, SystemInfoDTO.class);
        SystemInfo info = systemService.updateSystemInfo(SystemMapper.INSTANCT.convert(m));
        SystemInfoDTO r = SystemMapper.INSTANCT.convert(info);
        return Result.ok("更新系统信息成功", r);
    }

    @Operation(summary = "获取用户列表",description = "query={\"search\":\"搜索\",\"name\":\"筛选-名字\",\"phone\":\"筛选-电话\"}")
    @GetMapping("/systemusers/list")
    @SaCheckPermission(value = {"AuthMenu:perSetting_user:Execute"})
    public Result<PageInfo<SystemUserDTO>> getSystemUsers(String query, Integer pageNum, Integer pageSize) {
        Map<String,String> qmap = JsonUtils.toObject(query,Map.class);
        PageInfo<SystemUser> info = systemService.findSystemUsers(qmap.get("search"),qmap.get("name"),qmap.get("phone"),pageNum,pageSize);
        PageInfo<SystemUserDTO> r = CollectionUtils.convertPage(info, SystemMapper.INSTANCT::convert);
        return Result.ok("获取用户列表成功", r);
    }

    @Operation(summary = "新增用户信息", description = "内容格式：{\"name\":\"张珊\",\"otherName\":\"小山子\",\"imageLogo\":\"12ddfg324dfg3412sddf\",\"phone\":\"13211111111\",\"birthday\":\"2023-6-1\",\"sex\":\"Women\",\"username\":\"zhangshan\",\"password\":\"111111\"}")
    @PostMapping("/systemusers/new")
    @SaCheckPermission(value = {"AuthMenu:perSetting_user:Execute"})
    public Result<SystemUserDTO> postCreateSystemUser(@RequestBody String json) {
        SystemUserDTO m = JsonUtils.toObject(json, SystemUserDTO.class);
        SystemUser info = systemService.createSystemUser(SystemMapper.INSTANCT.convert(m));
        SystemUserDTO r = SystemMapper.INSTANCT.convert(info);
        return Result.ok("新增用户信息成功", r);
    }

    @Operation(summary = "获取用户信息")
    @GetMapping("/systemusers/{userId}/info")
    @SaCheckPermission(value = {"AuthMenu:perSetting_user:Execute"})
    public Result<SystemUserDTO> getSystemUser(@PathVariable String userId) {
        SystemUser info = systemService.findSystemUserById(userId);
        SystemUserDTO r = SystemMapper.INSTANCT.convert(info);
        return Result.ok("获取用户信息成功", r);
    }

    @Operation(summary = "更新用户信息", description = "内容格式：{\"name\":\"张珊\",\"otherName\":\"小山子\",\"imageLogo\":\"12ddfg324dfg3412sddf\",\"phone\":\"13211111111\",\"birthday\":\"2023-6-1\",\"sex\":\"Women\",\"username\":\"zhangshan\",\"password\":\"111111\"}")
    @PostMapping("/systemusers/{userId}/info")
    @SaCheckPermission(value = {"AuthMenu:perSetting_user:Execute"})
    public Result<SystemUserDTO> postUpdateSystemUser(@PathVariable String userId,@RequestBody String json) {
        SystemUserDTO m = JsonUtils.toObject(json, SystemUserDTO.class);
        m.setId(userId);
        SystemUser info = systemService.updateSystemUser(SystemMapper.INSTANCT.convert(m));
        SystemUserDTO r = SystemMapper.INSTANCT.convert(info);
        return Result.ok("更新用户信息成功", r);
    }

    @Operation(summary = "删除用户信息")
    @PostMapping("/systemusers/{userId}/")
    @SaCheckPermission(value = {"AuthMenu:perSetting_user:Execute"})
    public Result<String> postDeleteSystemUser(@PathVariable String userId) {
        systemService.deleteSystemUser(userId);
        return Result.ok("删除用户信息成功", null);
    }

    @Operation(summary = "批量删除用户信息", description = "内容格式：{\"ids\":[\"111\",\"222\"],}")
    @PostMapping("/systemusers/users_delete")
    @SaCheckPermission(value = {"AuthMenu:perSetting_user:Execute"})
    public Result<String> postDeleteSystemUsers(@RequestBody String json) {
        Map<String,Object> tm = JsonUtils.toObject(json, Map.class);
        systemService.deleteSystemUsers((List)tm.get("ids"));
        return Result.ok("删除用户信息成功", null);
    }

    @Operation(summary = "更新用户状态",description = "内容格式：{\"state\":1};1-启用|0-停用")
    @PostMapping("/systemusers/{userId}/state")
    @SaCheckPermission(value = {"AuthMenu:perSetting_user:Execute"})
    public Result<String> postChangeState(@PathVariable String userId, @RequestBody String json) {
        Map<String,Object> tm = JsonUtils.toObject(json, Map.class);
        systemService.doChangeState(CollectionUtil.newArrayList(userId),CommonConst.StateEnum.findEnumByKey((Integer) tm.get("state")));
        return Result.ok("更新用户状态成功", null);
    }

    @Operation(summary = "批量更新用户状态",description = "内容格式：{\"ids\":[\"111\",\"222\"],\"state\":1};1-启用|0-停用")
    @PostMapping("/systemusers/users_state")
    @SaCheckPermission(value = {"AuthMenu:perSetting_user:Execute"})
    public Result<String> postChangeState(@RequestBody String json) {
        Map<String,Object> tm = JsonUtils.toObject(json, Map.class);
        systemService.doChangeState((List)tm.get("ids"),CommonConst.StateEnum.findEnumByKey((Integer) tm.get("state")));
        return Result.ok("批量更新用户状态成功", null);
    }

    @Operation(summary = "前端选人组件用的接口")
    @GetMapping("/systemusers/select_module")
    public Result<PageInfo<SystemUserDTO>> selectSystemUser(Integer pageNum, Integer pageSize){
        PageInfo<SystemUser> page = systemService.findSystemUsers(null, null, null, pageNum, pageSize);
        PageInfo<SystemUserDTO> r = CollectionUtils.convertPage(page, SystemMapper.INSTANCT::convert);
        return Result.ok(r);
    }

}
