package com.kingtom.kirin.api.controller;

import java.util.Map;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.kingtom.kirin.api.security.ISaSecurityService;
import com.kingtom.kirin.api.security.SaSecurityUtils;
import com.kingtom.kirin.api.system.mapper.SystemMapper;
import com.kingtom.kirin.api.system.model.SystemUserDTO;
import com.kingtom.kirin.app.security.service.ISecurityService;
import com.kingtom.kirin.app.security.utils.SecurityUtils;
import com.kingtom.kirin.app.system.model.SystemUser;
import com.kingtom.kirin.app.system.service.ISystemService;
import com.kingtom.kirin.core.common.base.Result;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/9/1 11:51
 */
@RestController
@Tag(name = "UserController", description = "用户信息/设置接口")
public class UserConfigController{

    @Autowired
    private ISystemService systemService;

    @Autowired
    private ISaSecurityService saSecurityService;

    @Autowired
    private ISecurityService securityService;


    @Operation(summary = "获取当前用户信息")
    @GetMapping("/currentuser/info")
    public Result<SystemUserDTO> getCurrentUser(){
        String userId = SaSecurityUtils.findSessionData().getUserId();
        SystemUser info = systemService.findSystemUserById(userId);
        SystemUserDTO r = SystemMapper.INSTANCT.convert(info);
        return Result.ok("获取用户信息成功", r);
    }

    @Operation(summary = "更新当前用户信息", description = "内容格式：{\"name\":\"张珊\",\"otherName\":\"小山子\"," + "\"imageLogo" +
                                                           "\":\"12ddfg324dfg3412sddf\"," + "\"phone\":\"13211111111" +
                                                           "\",\"birthday\":\"2023-6-1\"," + "\"sex\":\"Women\"," +
                                                           "\"username\":\"zhangshan\"," + "\"password\":\"111111\"}")
    @PostMapping("/currentuser/info")
    public Result<SystemUserDTO> postUpdateCurrentUser(@RequestBody String json){
        String userId = SaSecurityUtils.findSessionData().getUserId();
        SystemUserDTO m = JsonUtils.toObject(json, SystemUserDTO.class);
        m.setId(userId);
        m.setUsername(null);
        m.setPassword(null);
        SystemUser info = systemService.updateSystemUser(SystemMapper.INSTANCT.convert(m));
        SystemUserDTO r = SystemMapper.INSTANCT.convert(info);
        return Result.ok("更新当前用户信息成功", r);
    }

    @Operation(summary = "重置用户密码信息", description = "内容格式：{\"oldPassword\":\"1111\",\"password\":\"1111\"," +
                                                           "\"rePassword\":\"2222\"}")
    @PostMapping("/currentuser/reset_security")
    public Result<String> postResetSecurity(@RequestBody String json){
        Map<String, String> map = null;
        try{
            ObjectMapper mapper = new ObjectMapper();
            map = mapper.readValue(json, Map.class);
        } catch(JsonProcessingException e){
            throw new AppException("参数错误", e);
        }
        String oldPassword = map.get("oldPassword").toString();
        String password = map.get("password").toString();
        String rePassword = map.get("rePassword").toString();
        if(!password.equals(rePassword)){
            return Result.fail("两次输入的新密码不一样!");
        }

        String userId = SaSecurityUtils.findSessionData().getUserId();
        securityService.updateCredentials(userId,oldPassword, password);
        saSecurityService.logout();
        return Result.ok("重置用户密码信息");
    }

}
