package com.kingtom.kirin.api.controller;

import java.util.Map;

import cn.dev33.satoken.annotation.SaIgnore;
import cn.dev33.satoken.stp.StpUtil;
import com.kingtom.kirin.api.security.ISaSecurityService;
import com.kingtom.kirin.api.security.dto.LoginParamDTO;
import com.kingtom.kirin.app.security.base.SessionData;
import com.kingtom.kirin.app.security.service.ISecurityService;
import com.kingtom.kirin.app.security.utils.SecurityUtils;
import com.kingtom.kirin.app.system.model.SystemUser;
import com.kingtom.kirin.app.wechat.base.mini.service.IWeChatMiniChannel;
import com.kingtom.kirin.app.wechat.service.IWeChatSystemUserService;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.base.Result;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/10/13
 */
@Tag(name = "WechatMiniController", description = "登录认证相关接口")
@RequestMapping("/wechat/mini")
@RestController
public class WechatMiniController{

    @Autowired
    private ISaSecurityService saSecurityService;

    @Autowired
    private ISecurityService securityService;

    @Autowired
    private IWeChatMiniChannel weChatMiniChannel;

    @Autowired
    private IWeChatSystemUserService weChatSystemUserService;

    @Operation(summary = "微信小程序登录",
            description = "内容格式：{\"principal\":\"wx.login获取到的临时凭证\",\"credentials\":\"wx" +
                    ".login获取到的临时凭证\",\"type\":\"WechatMini\",\"encode\":0}")
    @PostMapping("/login")
    @SaIgnore
    public Result<SessionData> loginWechat(@RequestBody String json, HttpServletRequest request){
        LoginParamDTO m = JsonUtils.toObject(json, LoginParamDTO.class);
        String principal = m.getPrincipal();
        String credentials = m.getCredentials();
        String type = m.getType();
        if(CommonConst.BooleanEnum.True.getKey().equals(m.getEncode())){
            //rsa解密
            principal = SecurityUtils.rsaPrivateKeyDecode(principal);
            credentials = SecurityUtils.rsaPrivateKeyDecode(credentials);
        }
        SessionData KirinSessionData = saSecurityService.authenticate(principal, credentials, type, null);
        return Result.ok(KirinSessionData);
    }

    @Operation(summary = "获取用户手机号，需要先登录", description = "内容格式：{\"code\":\"5分钟有效的授权码\"}")
    @PostMapping("/phone")
    public Result<String> getPhone(@RequestBody String json){
        Map<String, String> m = JsonUtils.toObject(json, Map.class);
        String phone = weChatMiniChannel.getPhoneNumber(m.get("code"));
        if(StringUtils.isBlank(phone)){
            return Result.fail("获取手机号失败");
        }
        return Result.ok("获取手机号成功", phone);
    }

    @Operation(summary = "获取用户手机号，并且更新当前登录用户的手机号",
            description = "内容格式：{\"code\":\"5分钟有效的授权码\"}")
    @PostMapping("/userphone")
    public Result<String> getUserPhone(@RequestBody String json){
        Map<String, String> m = JsonUtils.toObject(json, Map.class);
        String phone = weChatMiniChannel.getPhoneNumber(m.get("code"));
        if(StringUtils.isBlank(phone)){
            return Result.fail("获取手机号失败");
        }
        SessionData data = saSecurityService.findSessionInfo();
        SystemUser trueUser = weChatSystemUserService.updateWechatMiniUser(data.getUserId(), phone);
        securityService.logIn(StpUtil.getTokenValue(),trueUser.getId());
        return Result.ok("获取手机号成功", phone);
    }

}
