package com.kingtom.kirin.api.org.base;

import java.util.List;

import com.kingtom.kirin.api.org.mapper.OrgMapper;
import com.kingtom.kirin.api.org.model.OrgPersonDTO;
import com.kingtom.kirin.api.org.model.OrgStructDTO;
import com.kingtom.kirin.api.system.model.SystemUserDTO;
import com.kingtom.kirin.app.org.OrgConst;
import com.kingtom.kirin.app.org.bo.OrgPersonBO;
import com.kingtom.kirin.app.org.model.OrgUnit;
import com.kingtom.kirin.app.org.service.IOrgService;
import com.kingtom.kirin.app.security.base.ISessionDataLoader;
import com.kingtom.kirin.app.security.base.SessionData;
import com.kingtom.kirin.app.security.service.ISecurityContainerService;
import com.kingtom.kirin.app.system.SystemConst;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.resource.ResourceConst;
import jakarta.annotation.PostConstruct;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/10/23
 */
@Component
public class OrgSessionLoader implements ISessionDataLoader{

    @Autowired
    private ISecurityContainerService container;

    @Autowired
    private IOrgService orgService;

    @PostConstruct
    public void init(){
        container.registerSessionLoader(this);
    }

    @Override
    public int getOrder(){
        return CommonConst.INT_1;
    }

    @Override
    public void onLoad(SessionData data){
        OrgUnit orgUnit = orgService.findDefaultUnit();
        //FIXME 因为目前认定只有一个org,所以直接添加org到会话信息,
        data.putAttr(OrgConst.SESSION_ATTR_ORG_UNIT, orgUnit);
        SystemUserDTO systemUser = data.getAttr(SystemConst.SESSION_ATTR_SYSTEM_USER);
        if(systemUser != null && ResourceConst.ResourceTypeEnum.OrgPerson.getKey().equals(systemUser.getResType())){
            OrgPersonBO orgPersonBO = orgService.findPerson(systemUser.getResId(),orgUnit.getId());
            if(orgPersonBO != null){
                OrgPersonDTO dto = OrgMapper.INSTANCT.convert(orgPersonBO.getPerson());
                List<OrgStructDTO> deps = CollectionUtils.convert(orgPersonBO.getDepartments(),OrgMapper.INSTANCT::convert);
                List<OrgStructDTO> posts = CollectionUtils.convert(orgPersonBO.getPosts(),OrgMapper.INSTANCT::convert);
                data.putAttr(OrgConst.SESSION_ATTR_ORG_PERSON, dto);
                data.putAttr(OrgConst.SESSION_ATTR_ORG_Deps, deps);
                data.putAttr(OrgConst.SESSION_ATTR_ORG_Posts, posts);
            }
        }
    }

}
