package com.kingtom.kirin.api.org.mapper;

import java.util.ArrayList;
import java.util.HashMap;

import com.kingtom.kirin.api.org.model.OrgPersonDTO;
import com.kingtom.kirin.api.org.model.OrgStructDTO;
import com.kingtom.kirin.api.org.model.OrgUnitDTO;
import com.kingtom.kirin.app.org.OrgConst;
import com.kingtom.kirin.app.org.model.OrgPerson;
import com.kingtom.kirin.app.org.model.OrgPersonExt3;
import com.kingtom.kirin.app.org.model.OrgStruct;
import com.kingtom.kirin.app.org.model.OrgUnit;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.json.JObject;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.factory.Mappers;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/10/14
 */
@Mapper
public interface OrgMapper{

    public static final OrgMapper INSTANCT = Mappers.getMapper(OrgMapper.class);

    default OrgPersonDTO convert(OrgPerson data){
        if(data == null){
            return null;
        }
        OrgPersonDTO dto = convertToDTO(data);
        dto.setEmails(data.getExt3() == null ? null :
                (data.getExt3().getE() == null ? null : data.getExt3().getE().getEmails()));
        dto.setPhones(data.getExt3() == null ? null :
                (data.getExt3().getE() == null ? null : data.getExt3().getE().getPhones()));
        dto.setSexEnum(OrgConst.OrgPersonSexEnum.findEnumByKey(data.getSex()));
        dto.setLicenseTypeEnum(OrgConst.OrgPersonLicenseTypeEnum.findEnumByKey(data.getLicenseType()));
        dto.setStateEnum(CommonConst.StateEnum.findEnumByKey(data.getState()));
        dto.setAuthStateEnum(CommonConst.StateEnum.findEnumByKey(data.getAuthState()));
        return dto;
    }

    @Mapping(target = "gmtCreate", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "gmtModified", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "birthday", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "licenseStartTime", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "licenseEndTime", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "emails", ignore = true)
    @Mapping(target = "phones", ignore = true)
    @Mapping(target = "sexEnum", ignore = true)
    @Mapping(target = "licenseTypeEnum", ignore = true)
    @Mapping(target = "stateEnum", ignore = true)
    @Mapping(target = "authStateEnum", ignore = true)
    OrgPersonDTO convertToDTO(OrgPerson data);

    default OrgPerson convert(OrgPersonDTO dto){
        if(dto == null){
            return null;
        }
        OrgPerson r = convertToBean(dto);
        OrgPersonExt3 e = convertToExt3Bean(dto);
        r.setExt3(new JObject<>(e));
        return r;
    }

    @Mapping(target = "gmtCreate", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "gmtModified", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "birthday", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "licenseStartTime", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "licenseEndTime", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "ext3", ignore = true)
    OrgPerson convertToBean(OrgPersonDTO dto);

    OrgPersonExt3 convertToExt3Bean(OrgPersonDTO dto);

    default OrgUnitDTO convert(OrgUnit data){
        if(data == null){
            return null;
        }
        OrgUnitDTO r = convertToDTO(data);
        r.setExt3(data.getExt3() == null ? null : data.getExt3().getE());
        r.setPhones(data.getPhones() == null ? null : data.getPhones().getE());
        r.setEmails(data.getEmails() == null ? null : data.getEmails().getE());
        r.setLogoResource(data.getLogoResource() == null ? null : data.getLogoResource().getE());
        r.setImageResource(data.getImageResource() == null ? null : data.getImageResource().getE());
        r.setLoginResource(data.getLoginResource() == null ? null : data.getLoginResource().getE());
        r.setTrademarkResource(data.getTrademarkResource() == null ? null : data.getTrademarkResource().getE());
        r.setTypeEnum(OrgConst.OrgUnitTypeEnum.findEnumByKey(data.getType()));
        r.setIndustryEnum(OrgConst.OrgUnitIndustryEnum.findEnumByKey(data.getIndustry()));
        r.setLicenseStateEnum(OrgConst.OrgUnitLicenseStateEnum.findEnumByKey(data.getLicenseState()));
        r.setStateEnum(CommonConst.StateEnum.findEnumByKey(data.getState()));
        return r;
    }

    @Mapping(target = "gmtCreate", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "gmtModified", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "licenseRegTime", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "licenseCreateTime", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "licenseStartTime", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "licenseEndTime", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "ext3", ignore = true)
    @Mapping(target = "phones", ignore = true)
    @Mapping(target = "emails", ignore = true)
    @Mapping(target = "logoResource", ignore = true)
    @Mapping(target = "imageResource", ignore = true)
    @Mapping(target = "loginResource", ignore = true)
    @Mapping(target = "trademarkResource", ignore = true)
    @Mapping(target = "typeEnum", ignore = true)
    @Mapping(target = "industryEnum", ignore = true)
    @Mapping(target = "licenseStateEnum", ignore = true)
    @Mapping(target = "stateEnum", ignore = true)
    OrgUnitDTO convertToDTO(OrgUnit data);

    default OrgUnit convert(OrgUnitDTO dto){
        if(dto == null){
            return null;
        }
        OrgUnit r = convertToBean(dto);
        r.setExt3(new JObject<>(dto.getExt3() == null ? new HashMap<>() : dto.getExt3()));
        r.setPhones(new JObject<>(dto.getPhones() == null ? new ArrayList<>() : dto.getPhones()));
        r.setEmails(new JObject<>(dto.getEmails() == null ? new ArrayList<>() : dto.getEmails()));
        r.setLogoResource(new JObject<>(dto.getLogoResource() == null ? new ArrayList<>() : dto.getLogoResource()));
        r.setImageResource(new JObject<>(dto.getImageResource() == null ? new ArrayList<>() : dto.getImageResource()));
        r.setLoginResource(new JObject<>(dto.getLoginResource() == null ? new ArrayList<>() : dto.getLoginResource()));
        r.setTrademarkResource(
                new JObject<>(dto.getTrademarkResource() == null ? new ArrayList<>() : dto.getTrademarkResource()));
        return r;
    }

    @Mapping(target = "gmtCreate", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "gmtModified", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "licenseRegTime", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "licenseCreateTime", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "licenseStartTime", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "licenseEndTime", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "ext3", ignore = true)
    @Mapping(target = "phones", ignore = true)
    @Mapping(target = "emails", ignore = true)
    @Mapping(target = "logoResource", ignore = true)
    @Mapping(target = "imageResource", ignore = true)
    @Mapping(target = "loginResource", ignore = true)
    @Mapping(target = "trademarkResource", ignore = true)
    OrgUnit convertToBean(OrgUnitDTO dto);


    default OrgStructDTO convert(OrgStruct data){
        if(data == null){
            return null;
        }
        OrgStructDTO r = convertToDTO(data);
        r.setExt3(data.getExt3() == null ? null : data.getExt3().getE());
        r.setPath(data.getPath() == null ? null : data.getPath().getE());
        r.setTypeEnum(OrgConst.OrgStructTypeEnum.findEnumByKey(data.getType()));
        r.setSubTypeEnum(OrgConst.OrgStructSubTypeEnum.findEnumByKey(data.getSubType()));
        r.setStateEnum(CommonConst.StateEnum.findEnumByKey(data.getState()));
        return r;
    }

    @Mapping(target = "gmtCreate", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "gmtModified", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "ext3", ignore = true)
    @Mapping(target = "path", ignore = true)
    @Mapping(target = "typeEnum", ignore = true)
    @Mapping(target = "subTypeEnum", ignore = true)
    @Mapping(target = "stateEnum", ignore = true)
    OrgStructDTO convertToDTO(OrgStruct data);

    default OrgStruct convert(OrgStructDTO dto){
        if(dto == null){
            return null;
        }
        OrgStruct r = convertToBean(dto);
        r.setExt3(new JObject<>(dto.getExt3() == null ? new HashMap<>() : dto.getExt3()));
        r.setPath(new JObject<>(dto.getPath() == null ? new ArrayList<>() : dto.getPath()));
        return r;
    }

    @Mapping(target = "gmtCreate", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "gmtModified", dateFormat = "yyyy-MM-dd HH:mm:ss")
    @Mapping(target = "ext3", ignore = true)
    @Mapping(target = "path", ignore = true)
    OrgStruct convertToBean(OrgStructDTO dto);

}
