package com.kingtom.kirin.api.org.model;

import java.util.List;
import java.util.stream.Collectors;

import com.kingtom.kirin.api.org.mapper.OrgMapper;
import com.kingtom.kirin.app.org.bo.OrgPersonBO;
import com.kingtom.kirin.app.org.model.OrgStruct;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;
import org.springframework.beans.BeanUtils;

/**
 * 应用模块名称</p>
 * 人员保存提交的参数</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/10/24
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@Schema(description = "人员在某个组织内的完整信息")
public class OrgPersonFullDTO extends OrgPersonDTO{

    @Schema(description = "组织Id", requiredMode = Schema.RequiredMode.NOT_REQUIRED)
    private String orgId;

    @Schema(description = "部门id集合", requiredMode = Schema.RequiredMode.NOT_REQUIRED)
    private List<String> departmentIds;

    @Schema(description = "岗位id集合", requiredMode = Schema.RequiredMode.NOT_REQUIRED)
    private List<String> postIds;

    public static OrgPersonFullDTO build(OrgPersonBO bo, String orgId){
        OrgPersonFullDTO fullDTO = new OrgPersonFullDTO();
        OrgPersonDTO personDTO = OrgMapper.INSTANCT.convert(bo.getPerson());
        BeanUtils.copyProperties(personDTO, fullDTO);
        fullDTO.setOrgId(orgId);
        fullDTO.setPostIds(bo.getPosts().stream().map(OrgStruct::getId).collect(Collectors.toList()));
        fullDTO.setDepartmentIds(bo.getDepartments().stream().map(OrgStruct::getId).collect(Collectors.toList()));
        return fullDTO;
    }

}
