package com.kingtom.kirin.api.org.model;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.kingtom.kirin.app.org.model.OrgStruct;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/10/20
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class OrgStructNode{

    private String id;

    private String name;

    private String parentId;

    private String parentName;

    private String type;

    private String orgId;

    private List<OrgStructNode> children;

    public static OrgStructNode build(List<OrgStruct> list, String type){
        Map<String, OrgStructNode> map = list.stream().collect(Collectors.toMap(OrgStruct::getId,
                e -> OrgStructNode.builder().id(e.getId()).name(e.getName()).parentId(e.getParentId())
                        .type(e.getSubType()).orgId(e.getOrgId()).children(new ArrayList<>()).build()));
        map.values().forEach(e -> {
            OrgStructNode parent = map.get(e.getParentId());
            if(parent != null){
                e.setParentName(parent.getName());
                parent.getChildren().add(e);
            }
        });
        return map.values().stream()
                .filter(e -> e.getParentId() != null && map.get(e.getParentId()) == null && type.equals(e.getType()))
                .findFirst().orElse(null);
    }

}
