package com.kingtom.kirin.api.security;

import com.kingtom.kirin.app.security.base.SessionData;
import com.kingtom.kirin.app.system.SystemConst;
import com.kingtom.kirin.app.system.model.UserSecurity;

import java.util.List;

/**
 * 会话安全模块</p>
 * 身份登录认证 接口</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023 /8/24
 */
public interface ISaSecurityService {

    /**
     * 登录
     *
     * @param principal   the principal
     * @param credentials the credentials
     * @param realmType   the realm type
     * @param encode      the encoded
     * @return the abstract session data
     */
    SessionData authenticate(String principal, String credentials, String realmType, String encode);

    /**
     * 判断是否登录
     *
     * @return boolean 是否已登录
     */
    boolean isAuthenticated();

    /**
     * 退出
     *
     * @return boolean 是否退出成功
     */
    boolean logout();

    /**
     * 获取登录人的会话信息
     *
     * @return the abstract session data
     */
    SessionData findSessionInfo();

}
