package com.kingtom.kirin.api.security;

import cn.dev33.satoken.stp.StpUtil;
import com.kingtom.kirin.app.security.base.Authentication;
import com.kingtom.kirin.app.security.base.IAuthcRealm;
import com.kingtom.kirin.app.security.base.SessionData;
import com.kingtom.kirin.app.security.service.ISecurityContainerService;
import com.kingtom.kirin.app.security.service.ISecurityService;
import com.kingtom.kirin.app.system.SystemConst;
import com.kingtom.kirin.app.system.service.ISystemService;
import com.kingtom.kirin.app.system.service.IUserSecurityService;
import com.kingtom.kirin.core.common.exception.AppException;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/8/25
 */
@Service
public class SaSecurityServiceImpl implements ISaSecurityService{

    @Autowired
    private ISecurityContainerService container;

    @Autowired
    private ISecurityService manager;

    @Override
    public SessionData authenticate(String principal, String credentials, String realmType, String encode){
        SystemConst.AuthcRealmTypeEnum realmTypeEnum = SystemConst.AuthcRealmTypeEnum.findEnumByKey(realmType);
        Authentication authentication = new Authentication();
        authentication.setAuthcRealmTypeEnum(realmTypeEnum);
        authentication.setPrincipal(principal);
        authentication.setCredentials(credentials);

        IAuthcRealm realm = container.getRealm(realmTypeEnum);
        if(realm == null){
            throw new AppException("不存在的认证方式！");
        }
        authentication = realm.doAuth(authentication, null);
        if(authentication == null || !authentication.isUsed() || StringUtils.isEmpty(authentication.getUserId())){
            throw new AppException("登录失败！");
        }
        StpUtil.login(authentication.getUserId());
        if(!StpUtil.isLogin()){
            throw new AppException("登录出错！");
        }
        return manager.findSessionData(StpUtil.getTokenValue());
    }

    @Override
    public boolean isAuthenticated(){
        return StpUtil.isLogin();
    }

    @Override
    public boolean logout(){
        if(StpUtil.isLogin()){
            StpUtil.logout();
        }
        return StpUtil.isLogin();
    }

    @Override
    public SessionData findSessionInfo(){
        return manager.findSessionData(StpUtil.getTokenValue());
    }
}
