package com.kingtom.kirin.api.security;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import cn.dev33.satoken.stp.StpInterface;
import cn.dev33.satoken.stp.StpUtil;
import com.kingtom.kirin.app.auth.model.AuthPermission;
import com.kingtom.kirin.app.auth.model.AuthRole;
import com.kingtom.kirin.app.auth.service.IAuthService;
import com.kingtom.kirin.app.security.base.SessionData;
import com.kingtom.kirin.app.security.service.ISecurityService;
import com.kingtom.kirin.app.security.utils.SecurityUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/8/31
 */
@Component
public class StpInterfaceImpl implements StpInterface{

    @Autowired
    private IAuthService authService;

    @Autowired
    private ISecurityService securityService;

    @Override
    public List<String> getPermissionList(Object loginId, String loginType){
        String userId = getUserId();
        String sysId = getSysId();
        if(StringUtils.isAllBlank(userId, sysId)){
            return new ArrayList<>();
        }
        List<AuthPermission> permissions = authService.findAllAuthPermissions(userId, sysId);
        return securityService.parsePermissions(permissions);
    }

    @Override
    public List<String> getRoleList(Object loginId, String loginType){
        String userId = getSysId();
        String sysId = getUserId();
        if(StringUtils.isAllBlank(userId, sysId)){
            return new ArrayList<>();
        }
        List<AuthRole> role = authService.findRoleBySystemUser(userId, sysId);
        return role.stream().map(AuthRole::getCode).collect(Collectors.toList());
    }

    private String getSysId(){
        SessionData KirinSessionData = securityService.findSessionData(StpUtil.getTokenValue());
        return KirinSessionData == null ? null : KirinSessionData.getSysId();
    }

    private String getUserId(){
        SessionData KirinSessionData = securityService.findSessionData(StpUtil.getTokenValue());
        return KirinSessionData == null ? null : KirinSessionData.getUserId();
    }

}
