package com.kingtom.kirin.api.system.base;

import com.kingtom.kirin.api.system.mapper.SystemMapper;
import com.kingtom.kirin.api.system.model.SystemUserDTO;
import com.kingtom.kirin.app.security.base.ISessionDataLoader;
import com.kingtom.kirin.app.security.base.SessionData;
import com.kingtom.kirin.app.security.service.ISecurityContainerService;
import com.kingtom.kirin.app.system.SystemConst;
import com.kingtom.kirin.app.system.model.SystemUser;
import com.kingtom.kirin.app.system.service.ISystemService;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.exception.AppException;
import jakarta.annotation.PostConstruct;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/8/25
 */
@Component
public class SystemUserSessionLoader implements ISessionDataLoader{

    @Autowired
    private ISystemService systemService;

    @Autowired
    private ISecurityContainerService container;

    @PostConstruct
    public void init(){
        container.registerSessionLoader(this);
    }

    @Override
    public int getOrder(){
        return CommonConst.INT_0;
    }

    @Override
    public void onLogin(SessionData data){
        SystemUser systemUser = systemService.findSystemUserById(data.getUserId());
        if(!CommonConst.StateEnum.Enable.getKey().equals(systemUser.getState())){
            throw new AppException("用户状态异常，不能登录！请联系管理员");
        }
    }

    @Override
    public void onLoad(SessionData data){
        SystemUser systemUser = systemService.findSystemUserById(data.getUserId());
        if(systemUser != null){
            SystemUserDTO dto = SystemMapper.INSTANCT.convert(systemUser);
            dto.setPassword(null);
            data.putAttr(SystemConst.SESSION_ATTR_SYSTEM_USER, dto);
            data.setPhone(dto.getPhone());
        }
    }

}
