package com.kingtom.kirin.config;

import cn.dev33.satoken.exception.NotLoginException;
import cn.dev33.satoken.exception.SaTokenException;
import com.kingtom.kirin.core.common.base.ResultCodeEnum;
import com.kingtom.kirin.core.common.base.Result;
import com.kingtom.kirin.core.common.exception.AppException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/7 20:15
 */
@ControllerAdvice
@Slf4j
public class AppExceptionHandler{

    @ExceptionHandler(value = AppException.class)
    @ResponseBody
    public Result handlerAppException(AppException e){
        log.error("业务发生错误" + e.getMessage() + "，详细信息：" + e.getM(), e);
        Result r = Result.fail(StringUtils.isEmpty(e.getMessage()) ? "业务发生错误" : e.getMessage());
        return r;
    }

    @ExceptionHandler(value = NotLoginException.class)
    @ResponseBody
    public Result handlerException(NotLoginException e){
        log.error("安全错误，需要先登录：" + e.getMessage(), e);
        return Result.build(ResultCodeEnum.Unauthenticated, "请先登录", null);
    }

    @ExceptionHandler(value = SaTokenException.class)
    @ResponseBody
    public Result handlerException(SaTokenException e){
        log.error("安全错误：" + e.getMessage(), e);
        Result r = Result.fail("发生安全错误");
        return r;
    }

    @ExceptionHandler(value = Exception.class)
    @ResponseBody
    public Result handlerException(Exception e){
        log.error("服务发生错误" + e.getMessage(), e);
        Result r = Result.fail("服务发生错误");
        return r;
    }

}
