/**
* <p>Title: RRRR.java</p>
* <p>Description: </p>
* <p>Copyright: Copyright (c) 2018</p>
* <p>Company: www.rzslkj.com</p>
* @author Si.Fengrui
* @date 2018年10月30日
* @version 1.0
*/
package com.kingtom.kirin.config;

import org.aspectj.lang.annotation.Aspect;
import org.springframework.aop.Advisor;
import org.springframework.aop.aspectj.AspectJExpressionPointcut;
import org.springframework.aop.support.DefaultPointcutAdvisor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.annotation.EnableTransactionManagement;
import org.springframework.transaction.interceptor.NameMatchTransactionAttributeSource;
import org.springframework.transaction.interceptor.RollbackRuleAttribute;
import org.springframework.transaction.interceptor.RuleBasedTransactionAttribute;
import org.springframework.transaction.interceptor.TransactionInterceptor;

import java.util.Collections;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/4 17:48
 */
@Aspect
@Configuration
@EnableTransactionManagement
public class TransactionConfig {
	private static final String AOP_POINTCUT_EXPRESSION = "execution (* com..*.service..*.*(..))";

	@Autowired
	private PlatformTransactionManager transactionManager;

	@Bean
	public TransactionInterceptor txAdvice() {

		RuleBasedTransactionAttribute txREQUIRED = new RuleBasedTransactionAttribute();
		txREQUIRED.setPropagationBehavior(TransactionDefinition.PROPAGATION_REQUIRED);
		txREQUIRED.setRollbackRules(Collections.singletonList(new RollbackRuleAttribute(Throwable.class)));

		RuleBasedTransactionAttribute txReadREQUIRED = new RuleBasedTransactionAttribute();
		txReadREQUIRED.setPropagationBehavior(TransactionDefinition.PROPAGATION_REQUIRED);
		txReadREQUIRED.setReadOnly(true);

		NameMatchTransactionAttributeSource source = new NameMatchTransactionAttributeSource();
		source.addTransactionalMethod("create*", txREQUIRED);// 标准添加
		source.addTransactionalMethod("update*", txREQUIRED);// 标准修改
		source.addTransactionalMethod("save*", txREQUIRED);// 标准保存（无-添加，有-修改）
		source.addTransactionalMethod("delete*", txREQUIRED);// 标准删除
		source.addTransactionalMethod("do*", txREQUIRED);// 标准独立业务逻辑
		source.addTransactionalMethod("find*", txReadREQUIRED);// 标准查询
		return new TransactionInterceptor(transactionManager, source);
	}

	@Bean
	public Advisor txAdviceAdvisor() {
		AspectJExpressionPointcut pointcut = new AspectJExpressionPointcut();
		pointcut.setExpression(AOP_POINTCUT_EXPRESSION);
		return new DefaultPointcutAdvisor(pointcut, txAdvice());
	}
}