package com.kingtom.kirin.config;

import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.models.ExternalDocumentation;
import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.oas.models.Operation;
import io.swagger.v3.oas.models.info.Info;
import io.swagger.v3.oas.models.media.StringSchema;
import io.swagger.v3.oas.models.parameters.Parameter;
import org.springdoc.core.customizers.OperationCustomizer;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.method.HandlerMethod;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 * swagger2->openapi3的对应关系如下：
 * *@Api → @Tag
 * *@ApiIgnore → @Parameter(hidden = true) or @Operation(hidden = true) or @Hidden
 * *@ApiImplicitParam → @Parameter
 * *@ApiImplicitParams → @Parameters
 * *@ApiModel → @Schema
 * *@ApiModelProperty(hidden = true) → @Schema(accessMode = READ_ONLY)
 * *@ApiModelProperty → @Schema
 * *@ApiOperation(value = "foo", notes = "bar") → @Operation(summary = "foo", description = "bar")
 * *@ApiParam → @Parameter
 * *@ApiResponse(code = 404, message = "foo") → @ApiResponse(responseCode = "404", description = "foo")
 * @author King
 * @since 2023/6/1 20:13
 */
@Configuration(proxyBeanMethods = false)
public class WebApiConfig {

    @Bean
    public OpenAPI restfulOpenAPI() {
        // 自定义文档主要属性（配合application.yml的springdoc配置）
        return new OpenAPI()
                .info(new Info().title("系统API")
                        .description("系统API文档-openAPI-3")
                        .version("v1.0"))
                // 官方文档链接
                .externalDocs(new ExternalDocumentation()
                        .description("SpringDoc Wiki Documentation")
                        .url("https://springdoc.org/v2"));
    }

    @Bean
    public OperationCustomizer operationCustomizer() {
        return new GlobalHeaderOperationCustomizer();
    }

    public static class GlobalHeaderOperationCustomizer implements OperationCustomizer {
        @Override
        public Operation customize(Operation operation, HandlerMethod handlerMethod) {
            Parameter tokenHeader = new Parameter().in(ParameterIn.HEADER.toString()).name("satoken")
                    .description("传递JWT").schema(new StringSchema()).required(false);

            operation.addParametersItem(tokenHeader);
            return operation;
        }
    }

}
