package com.kingtom.kirin.config;

import com.kingtom.kirin.core.common.utils.IDUtils;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.slf4j.MDC;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.config.annotation.*;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 * web.xml的扩展java实现
 * @author King
 * @since 2023/6/1 21:51
 */
@Configuration(proxyBeanMethods = false)
public class WebAppConfig implements WebMvcConfigurer {

    /**
     * 解决跨域问题
     * @param registry
     */
    @Override
    public void addCorsMappings(CorsRegistry registry) {
        // 针对请求
        CorsRegistration cors = registry.addMapping("/**");
        // 允许哪种header
        cors.allowedHeaders("*");
        // 允许哪个请求来源进行跨域
        cors.allowedOriginPatterns("*");
        // 是否允许携带cookie进行跨域
        cors.allowCredentials(true);
        // 允许那种请求
        cors.allowedMethods("GET", "POST", "DELETE", "PUT");
        // 时效
        cors.maxAge(3600L *24);
    }

    /**
     * 视图跳转控制
     * @param registry
     */
    @Override
    public void addViewControllers(ViewControllerRegistry registry) {
        WebMvcConfigurer.super.addViewControllers(registry);
    }

    /**
     * 视图跳转控制器
     * @param registry
     */
    @Override
    public void configureViewResolvers(ViewResolverRegistry registry) {
        WebMvcConfigurer.super.configureViewResolvers(registry);
    }

    /**
     * 静态资源处理
     * @param registry
     */
    @Override
    public void addResourceHandlers(ResourceHandlerRegistry registry) {
        WebMvcConfigurer.super.addResourceHandlers(registry);
    }

    /**
     * 默认的静态资源处理器
     * @param configurer
     */
    @Override
    public void configureDefaultServletHandling(DefaultServletHandlerConfigurer configurer) {
        WebMvcConfigurer.super.configureDefaultServletHandling(configurer);
    }

    /**
     * 配置内容裁决的一些选项
     * @param configurer
     */
    @Override
    public void configureContentNegotiation(ContentNegotiationConfigurer configurer) {
        WebMvcConfigurer.super.configureContentNegotiation(configurer);
    }

    /**
     * 拦截器配置
     * @param registry
     */
    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        // 日志跟踪ID
        registry.addInterceptor(new LogTraceInterceptor());
    }

    /**
     * 日志跟踪器
     */
    public static class LogTraceInterceptor implements HandlerInterceptor {
        @Override
        public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
            MDC.put("TRACE_ID", IDUtils.newUUID());
            return true;
        }
    }
}
