package com.kingtom.kirin.config;

import cn.dev33.satoken.interceptor.SaInterceptor;
import cn.dev33.satoken.jwt.StpLogicJwtForSimple;
import cn.dev33.satoken.router.SaRouter;
import cn.dev33.satoken.stp.StpLogic;
import cn.dev33.satoken.stp.StpUtil;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 * spring-security6仅支持单独通过构建bean的方式做安全配置
 * @author King
 * @since 2023/6/1 20:41
 */
@Configuration(proxyBeanMethods = false)
public class WebSecurityConfig implements WebMvcConfigurer {

    /**
     * Simple 模式：Token 风格替换
     * 参考https://sa-token.cc/doc.html#/plugin/jwt-extend
     * @return
     */
    @Bean
    public StpLogic getStpLogicJwt() {
        return new StpLogicJwtForSimple();
    }

    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        String[] frees = new String[]{
                "/login",
                "/files/*/show",
                "/login_wechat",
                "/v3/api-docs/**",
                "/swagger-ui/**"
        };
        // 注册路由拦截器，自定义认证规则
        registry.addInterceptor(new SaInterceptor(handle -> SaRouter.match("/**", StpUtil::checkLogin).notMatch(frees)))
                .addPathPatterns("/**").excludePathPatterns(frees);
    }
}