package com.kingtom.kirin.app.affair.model;

import java.util.List;

import cn.hutool.core.collection.ListUtil;
import com.kingtom.kirin.app.affair.AffairConst;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.JsonUtils;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/10/19 20:55
 */
public class AffairInfoDO{

    private AffairInfo affairInfo;

    private List<AffairToken> affairTokens = ListUtil.toList();

    private List<AffairMember> affairMembers = ListUtil.toList();

    public AffairInfoDO(AffairInfo affairInfo){
        this.affairInfo = affairInfo;
    }

    public AffairInfo getAffairInfo(){
        return affairInfo;
    }

    public void setAffairInfo(AffairInfo affairInfo){
        this.affairInfo = affairInfo;
    }

    public List<AffairToken> getAffairTokens(){
        return affairTokens;
    }

    public void setAffairTokens(List<AffairToken> affairTokens){
        this.affairTokens.clear();
        if(affairTokens != null){
            JsonUtils.toObjectTypeList(affairTokens, this.affairTokens, AffairToken.class);
        }
    }

    public List<AffairMember> getAffairMembers(){
        return affairMembers;
    }

    public void setAffairMembers(List<AffairMember> affairMembers){
        this.affairMembers.clear();
        if(affairMembers != null){
            JsonUtils.toObjectTypeList(affairMembers, this.affairMembers, AffairMember.class);
        }
    }

    /*** 功能区 ***/

    public AffairInfoCurrentDO findCurrentDOByTokenId(String tokenId){
        AffairInfoCurrentDO r = new AffairInfoCurrentDO();
        r.setInfo(affairInfo);
        for(AffairToken token : affairTokens){
            if(token.getId().equals(tokenId)){
                r.setToken(token);
                break;
            }
        }
        return r;
    }

    public AffairInfoCurrentDO findCurrentDOByUserId(String userId){
        AffairMember m = null;
        for(AffairMember member : affairMembers){
            if(member.getUserId().equals(userId)){
                m = member;
                break;
            }
        }
        if(m != null){
            AffairInfoCurrentDO r = findCurrentDOByTokenId(m.getTokenId());
            r.setMember(m);
            return r;
        } else{
            return null;
        }
    }

    public AffairToken findTokenById(String tokenId){
        for(AffairToken token : affairTokens){
            if(token.getId().equals(tokenId)){
                return token;
            }
        }
        return null;
    }

    public AffairToken findTokenOfOneOnly(){
        return CollectionUtils.findFirst(this.affairTokens);
    }

    public AffairMember findMemberByUserId(String userId){
        for(AffairMember member : affairMembers){
            if(member.getUserId().equals(userId)){
                return member;
            }
        }
        return null;
    }

    public List<AffairMember> findMemberByTokenId(String tokenId){
        List<AffairMember> r = ListUtil.toList();
        for(AffairMember member : affairMembers){
            if(member.getTokenId().equals(tokenId)){
                r.add(member);
            }
        }
        return r;
    }

    /**
     * 检查代码所有人是否都已处理
     *
     * @param infoDO
     * @return
     */
    public static boolean checkProcessed(AffairInfoDO infoDO){
        AffairConst.AffairInfoActionModelEnum enumByKey =
                AffairConst.AffairInfoActionModelEnum.findEnumByKey(infoDO.getAffairInfo().getActionModel());
        return switch(enumByKey){
            case All -> infoDO.getAffairTokens().stream()
                    .noneMatch(e -> CommonConst.StateEnum.Enable.getKey().equals(e.getState())) &&
                    infoDO.getAffairMembers().stream()
                            .noneMatch(e -> CommonConst.StateEnum.Enable.getKey().equals(e.getState()));
            case One -> infoDO.getAffairTokens().stream()
                    .anyMatch(e -> CommonConst.StateEnum.Disable.getKey().equals(e.getState())) &&
                    infoDO.getAffairMembers().stream()
                            .anyMatch(e -> CommonConst.StateEnum.Disable.getKey().equals(e.getState()));
        };
    }

}
