package com.kingtom.kirin.app.affair.service;

import java.util.*;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.app.affair.AffairConst;
import com.kingtom.kirin.app.affair.base.DynamicCondition;
import com.kingtom.kirin.app.affair.dao.AffairInfoMapper;
import com.kingtom.kirin.app.affair.dao.AffairMemberMapper;
import com.kingtom.kirin.app.affair.dao.AffairTokenMapper;
import com.kingtom.kirin.app.affair.model.*;
import com.kingtom.kirin.app.affair.utils.AffairUtils;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.utils.CaseUtils;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import com.kingtom.kirin.core.resource.ResourceConst;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/10/19 20:49
 */
@Component
@Slf4j
public class AffairServiceImpl implements IAffairService{

    @Autowired
    private AffairInfoMapper infoMapper;

    @Autowired
    private AffairTokenMapper tokenMapper;

    @Autowired
    private AffairMemberMapper memberMapper;

    @Override
    public AffairInfoDO createAffair(AffairInfo info, List<AffairToken> tokens) {
        log.info("准备创建事项：affair="+JsonUtils.toJson(info)+";tokens="+JsonUtils.toJson(tokens));
        CaseUtils.build(info,
                        CaseUtils.caseBeanAndThrow(ObjectUtil::isNull,"事项不能为空"),
                        CaseUtils.caseAttrBlankAndThrow(AffairInfo::getName, "事项名不能为空"),
                        CaseUtils.caseAttrBlankAndThrow(AffairInfo::getResType, "对应业务类型不能为空"),
                        CaseUtils.caseAttrBlankAndThrow(AffairInfo::getResId, "对应业务ID不能为空"))
                .doCase();
        CaseUtils.build(tokens,
                        CaseUtils.caseBeanAndThrow(CollectionUtils::isEmpty,"事项凭证不能为空"))
                .doCase();
        AffairUtils.fillBeforeCreate(info);
        for(AffairToken token:tokens) {
            CaseUtils.build(token,
                            CaseUtils.caseBeanAndThrow(ObjectUtil::isNull,"事项凭证不能为空"),
                            CaseUtils.caseAttrBlankAndThrow(AffairToken::getResType, "事项凭证对应接收者类型不能为空"),
                            CaseUtils.caseAttrBlankAndThrow(AffairToken::getResId, "事项凭证对应接收者不能为空"))
                    .doCase();
            AffairUtils.fillBeforeCreate(token);
            token.setAffairId(info.getId());
        }
        AffairInfoDO infoDO = new AffairInfoDO(info);
        infoDO.setAffairTokens(tokens);
        generatorMembers(infoDO);
        //  保存
        infoMapper.insert(info);
        for(AffairToken token:tokens) {
            tokenMapper.insert(token);
        }
        for(AffairMember member:infoDO.getAffairMembers()) {
            memberMapper.insert(member);
        }
        return fillAffairInfoDO(info, true);
    }

    @Override
    public AffairInfoDO updateAffair(AffairInfo info){
        if(info == null){
            throw new AppException("参数为空，更新待办信息出错！");
        }
        if(StringUtils.isBlank(info.getId())){
            throw new AppException("id为空，更新待办信息出错！");
        }
        //todo 这里可以做一些检查
        info.setGmtModified(new Date());
        infoMapper.updateByPrimaryKeySelective(info);
        return findAffairById(info.getId());
    }

    @Override
    public Boolean doAffairProcess(String tokenId, String userId) {
        AffairMember member = findAffairMemberByTokenAndUser(tokenId,userId);
        log.info("处理事项："+ JsonUtils.toJson(member));
        if(member != null) {
            member.setState(CommonConst.StateEnum.Disable.getKey());
            AffairUtils.fillBeforeUpdate(member,null);
            memberMapper.updateByPrimaryKey(member);
            AffairInfoDO infoDO = findAffairById(member.getAffairId());
            boolean r = AffairInfoDO.checkProcessed(infoDO);
            AffairConst.AffairInfoActionModelEnum enumByKey =
                    AffairConst.AffairInfoActionModelEnum.findEnumByKey(infoDO.getAffairInfo().getActionModel());
            if(AffairConst.AffairInfoActionModelEnum.One.equals(enumByKey)){
                //如果是多个成员，单个执行时，需要把剩余未执行的全部执行掉
                infoDO.getAffairMembers().forEach(e -> {
                    if(e.getState().equals(CommonConst.StateEnum.Enable.getKey())){
                        e.setState(CommonConst.StateEnum.Deleted.getKey());
                        AffairUtils.fillBeforeUpdate(e, null);
                        memberMapper.updateByPrimaryKey(e);
                    }
                });
            }
            log.info("处理事项是否可以下一步的结果："+ r);
            return r;
        } else {
            throw new AppException("不存在该用户的事项").setM(tokenId+CommonConst.UNDER_LINE+userId);
        }
    }

    @Override
    public AffairInfoCurrentDO doAffairAddMember(String tokenId, String userId) {
        AffairInfoCurrentDO infoCurrentDO = findAffairByTokenId(tokenId);
        if(infoCurrentDO == null) {
            throw new AppException("该事项不存在，无法增加处理人").setM(tokenId);
        } else if(AffairConst.AffairTokenEffectModelEnum.Dynamic.getKey().equals(infoCurrentDO.getToken().getEffectModel())) {
            AffairInfoDO infoDO = findAffairById(infoCurrentDO.getInfo().getId());
            generatorMembers(infoDO, infoCurrentDO.getToken().getId(), userId);
            AffairInfoCurrentDO r = infoDO.findCurrentDOByUserId(userId);
            if(r.getToken().getId().equals(tokenId)) {
                tokenMapper.updateByPrimaryKey(r.getToken());
                memberMapper.insert(r.getMember());
                return r;
            } else {
                log.info("该事项已经存在该处理人,tokenId="+tokenId);
                return null;
            }
        } else {
            throw new AppException("该事项设置不支持增加处理人").setM(tokenId);
        }
    }

    @Override
    public AffairInfoCurrentDO findAffairByTokenId(String tokenId) {
        AffairToken token = tokenMapper.selectByPrimaryKey(tokenId);
        if(token != null) {
            AffairInfoDO infoDO = findAffairById(token.getAffairId());
            return infoDO.findCurrentDOByTokenId(tokenId);
        } else {
            return null;
        }
    }

    @Override
    public AffairInfoCurrentDO findAffairByUserId(String tokenId, String userId) {
        AffairToken token = tokenMapper.selectByPrimaryKey(tokenId);
        if(token != null) {
            AffairInfoDO infoDO = findAffairById(token.getAffairId());
            return infoDO.findCurrentDOByUserId(userId);
        } else {
            return null;
        }
    }

    @Override
    public List<AffairInfoDO> findAffairByIds(List<String> ids) {
        if(CollectionUtil.isNotEmpty(ids)) {
            AffairInfoExample e = new AffairInfoExample();
            e.createCriteria().andIdIn(ids);
            List<AffairInfo> beans = infoMapper.selectByExample(e);
            return fillAffairInfoDOs(beans, true);
        } else {
            return ListUtil.toList();
        }
    }

    @Override
    public List<AffairInfoDO> findAffairs(String groupName, String resType, String resId){
        if(StringUtils.isAnyBlank(groupName,resType,resId)){
            return new ArrayList<>();
        }
        AffairInfoExample e = new AffairInfoExample();
        e.createCriteria().andResTypeEqualTo(resType).andResIdEqualTo(resId).andGroupNameEqualTo(groupName);
        e.setOrderByClause("gmt_create desc");
        List<AffairInfo> beans = infoMapper.selectByExample(e);
        return fillAffairInfoDOs(beans, true);
    }

    @Override
    public PageInfo<AffairInfoCurrentDO> findAffairs(String userId, CommonConst.StateEnum state, String groupName,
                                                     String search, String token, String excludeToken,
                                                     Integer pageNum, Integer pageSize) {
        AffairMemberExample e = new AffairMemberExample();
        AffairMemberExample.Criteria c = e.createCriteria();
        c.andUserIdEqualTo(userId);
        if(state != null) {
            c.andStateEqualTo(state.getKey());
        }
        List<AffairMember> members = memberMapper.selectByExample(e);
        List<String> ids = CollectionUtils.convert(members, AffairMember::getAffairId);
        if(CollectionUtils.isNotEmpty(ids)) {
            AffairInfoExample ae = new AffairInfoExample();
            AffairInfoExample.Criteria ac = ae.createCriteria();
            ac.andIdIn(ids);
            if(StrUtil.isNotBlank(groupName)) {
                ac.andNameEqualTo(groupName);
            }
            if(StrUtil.isNotBlank(search)) {
                ac.andNameLike("%"+search+"%");
            }
            if(StrUtil.isNotBlank(token)) {
                ac.andAffairTokenEqualTo(token);
            }
            if(StrUtil.isNotBlank(excludeToken)) {
                ac.andAffairTokenNotEqualTo(excludeToken);
            }
            ae.setOrderByClause("gmt_create desc");
            PageHelper.startPage(pageNum, pageSize);
            List<AffairInfo> infos = infoMapper.selectByExample(ae);
            PageInfo<AffairInfo> pageInfo = new PageInfo<>(infos);
            List<AffairInfoDO> infoDOs = fillAffairInfoDOs(pageInfo.getList(), true);
            List<AffairInfoCurrentDO> infoCurDOs = CollectionUtils.convert(infoDOs, bean->bean.findCurrentDOByUserId(userId));
            PageInfo<AffairInfoCurrentDO> res = new PageInfo<>();
            res.setList(infoCurDOs);
            res.setTotal(pageInfo.getTotal());
            res.setPageNum(pageInfo.getPageNum());
            res.setSize(pageInfo.getSize());
            res.setStartRow(pageInfo.getStartRow());
            res.setEndRow(pageInfo.getEndRow());
            res.setPages(pageInfo.getPages());
            res.setPrePage(pageInfo.getPrePage());
            res.setNextPage(pageInfo.getNextPage());
            res.setIsFirstPage(pageInfo.isIsFirstPage());
            res.setIsLastPage(pageInfo.isIsLastPage());
            res.setHasNextPage(pageInfo.isHasNextPage());
            res.setHasPreviousPage(pageInfo.isHasPreviousPage());
            res.setNavigatePages(pageInfo.getNavigatePages());
            res.setNavigatepageNums(pageInfo.getNavigatepageNums());
            res.setNavigateFirstPage(pageInfo.getNavigateFirstPage());
            res.setNavigateLastPage(pageInfo.getNavigateLastPage());
            return res;
        } else {
            return new PageInfo<>(ListUtil.toList());
        }
    }

    @Override
    public long findAffairsCount(String userId, CommonConst.StateEnum stateEnum){
        AffairMemberExample e = new AffairMemberExample();
        AffairMemberExample.Criteria c = e.createCriteria();
        c.andUserIdEqualTo(userId);
        c.andStateEqualTo(stateEnum.getKey());
        return memberMapper.countByExample(e);
    }

    protected AffairMember findAffairMemberByTokenAndUser(String tokenId, String userId) {
        AffairMemberExample e = new AffairMemberExample();
        e.createCriteria().andTokenIdEqualTo(tokenId).andUserIdEqualTo(userId);
        List<AffairMember> ms = memberMapper.selectByExample(e);
        return CollectionUtils.findFirst(ms);
    }

    protected void generatorMembers(AffairInfoDO infoDO) {
        // 已经存在的处理人集合
        for(AffairToken token:infoDO.getAffairTokens()) {
            generatorMembers(infoDO, token.getId());
        }
    }

    protected void generatorMembers(AffairInfoDO infoDO, String tokenId, String... arguments) {
        AffairToken token = infoDO.findTokenById(tokenId);
        if(CommonConst.StateEnum.Enable.getKey().equals(token.getState())) {
            Map<String,AffairMember> m = CollectionUtils.convertMap(infoDO.getAffairMembers(), AffairMember::getUserId,bean->bean);
            AffairConst.AffairTokenEffectModelEnum effectModel = AffairConst.AffairTokenEffectModelEnum.findEnumByKey(token.getEffectModel());
            switch (effectModel) {
                case GenerateNow -> {
                    if(ResourceConst.ResourceTypeEnum.SystemUser.getKey().equals(token.getResType())) {
                        AffairMember member = new AffairMember();
                        member.setAffairId(token.getAffairId());
                        member.setTokenId(token.getId());
                        member.setUserId(token.getResId());
                        AffairUtils.fillBeforeCreate(member);
                        if(m.containsKey(member.getUserId())) {
                            log.info("该事项接收群体已经有该用户作为处理人。affairId="+infoDO.getAffairInfo().getId());
                        } else {
                            infoDO.getAffairMembers().add(member);
                            m.put(member.getUserId(),member);
                        }
                    } else if(ResourceConst.ResourceTypeEnum.AuthRole.getKey().equals(token.getResType())) {
                        throw new AppException("还不支持角色");
                    }
                    token.setState(CommonConst.StateEnum.Disable.getKey());
                }
                case GenerateEver -> {
                    log.warn("该事项接收群体类型还不支持，无法生成处理人。tikenId=" + token.getId());
                }
                case Dynamic -> {
                    if(ArrayUtil.isEmpty(arguments)) {
                        log.info("收到动态的事项接收群体类型，所以暂不生成处理人。tokenId="+token.getId());
                        token.setState(CommonConst.StateEnum.Enable.getKey());
                    } else {
                        DynamicCondition c = JsonUtils.toObject(token.getEffectCondition(),DynamicCondition.class);
                        Set<String> userIds = CollectionUtil.newHashSet(arguments);
                        for(String a:userIds) {
                            AffairMember member = new AffairMember();
                            member.setAffairId(token.getAffairId());
                            member.setTokenId(token.getId());
                            member.setUserId(a);
                            AffairUtils.fillBeforeCreate(member);
                            if(m.containsKey(member.getUserId())) {
                                log.info("该事项接收群体已经有该用户作为处理人。affairId="+infoDO.getAffairInfo().getId());
                            } else {
                                infoDO.getAffairMembers().add(member);
                                m.put(member.getUserId(),member);
                            }
                            c.setMemberedSize(c.getMemberedSize()+CommonConst.INT_1);
                            if(c.getMemberedSize() >= c.getSize()) {
                                break;
                            }
                        }
                        if(c.getMemberedSize() >= c.getSize()) {
                            token.setState(CommonConst.StateEnum.Disable.getKey());
                        } else {
                            token.setState(CommonConst.StateEnum.Enable.getKey());
                        }
                    }
                }
            }
        } else {
            log.warn("该事项接收群体不是待处理状态，无法生成处理人。tikenId=" + token.getId());
        }
    }

    protected AffairInfoDO fillAffairInfoDO(AffairInfo bean, Boolean containMember){
        List<AffairInfoDO> infos = fillAffairInfoDOs(bean == null?null:ListUtil.toList(bean), containMember);
        return CollectionUtils.findFirst(infos);
    }

    protected List<AffairInfoDO> fillAffairInfoDOs(List<AffairInfo> beans, Boolean containMember){
        List<AffairInfoDO> r = ListUtil.toList();
        if(CollectionUtil.isNotEmpty(beans)) {
            Map<String,AffairInfoDO> m = CollectionUtils.convertMap(beans, AffairInfo::getId, AffairInfoDO::new);
            List<String> ids = ListUtil.toList(m.keySet());
            // 查询token
            AffairTokenExample te = new AffairTokenExample();
            te.createCriteria().andAffairIdIn(ids);
            List<AffairToken> tbeans = tokenMapper.selectByExample(te);
            Map<String,List<AffairToken>> tm = CollectionUtils.toGroupMap(tbeans, AffairToken::getAffairId);
            for(Map.Entry<String,List<AffairToken>> en:tm.entrySet()) {
                m.get(en.getKey()).setAffairTokens(en.getValue());
            }
            // 查询member
            if(containMember) {
                AffairMemberExample me = new AffairMemberExample();
                me.createCriteria().andAffairIdIn(ids);
                List<AffairMember> mbeans = memberMapper.selectByExample(me);
                Map<String,List<AffairMember>> mm = CollectionUtils.toGroupMap(mbeans, AffairMember::getAffairId);
                for(Map.Entry<String,List<AffairMember>> en:mm.entrySet()) {
                    m.get(en.getKey()).setAffairMembers(en.getValue());
                }
            }
            r.addAll(m.values());
        }
        return r;
    }
}
