package com.kingtom.kirin.app.auth.listener;

import java.util.List;

import com.kingtom.kirin.app.auth.base.AuthInitProcessor;
import com.kingtom.kirin.app.auth.model.AuthMenu;
import com.kingtom.kirin.app.auth.model.AuthRole;
import com.kingtom.kirin.app.auth.service.IAuthService;
import com.kingtom.kirin.app.system.event.SystemInfoCreateEvent;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationListener;
import org.springframework.stereotype.Component;

/**
 * 应用模块名称</p>
 * 系统事件监听</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/9/1
 */
@Slf4j
@Component
public class AuthSystemEventListener implements ApplicationListener<SystemInfoCreateEvent>{

    @Autowired
    private IAuthService authService;

    @Autowired
    private AuthInitProcessor authInitProcessor;

    @Override
    public void onApplicationEvent(SystemInfoCreateEvent event){
        if(event == null || event.getSystemInfo() == null){
            return;
        }
        String sysId = event.getSystemInfo().getId();
        //创建菜单
        log.info("监听到系统创建事件，开始初始化角色、菜单、权限!");
        List<AuthMenu> menus = authInitProcessor.createMenuBean(sysId);
        if(CollectionUtils.isNotEmpty(menus)){
            menus = authService.addMenus(menus);
        }
        log.info("共加载了" + (menus == null ? 0 : menus.size()) + "个菜单，并且初始化成功!");
        List<AuthMenu> menuList = authService.findMenuListBySystem(null, sysId);

        //创建角色
        List<AuthRole> roles = authInitProcessor.createRoleBean(sysId);
        if(CollectionUtils.isNotEmpty(roles)){
            for(AuthRole role : roles){
                List<String> roleMenuIds = authInitProcessor.findRoleMenuIds(role.getCode(), menuList);
                authService.addRole(role, roleMenuIds);
            }
        }
        log.info("共加载了" + (roles == null ? 0 : roles.size()) + "个角色，并且初始化成功!");
        log.info("监听到系统创建事件，初始化角色、菜单、权限成功!");
    }

}
