package com.kingtom.kirin.app.auth.service;

import java.util.List;

import com.kingtom.kirin.app.auth.model.AuthMenu;
import com.kingtom.kirin.core.common.CommonConst;

/**
 * 权限模块</p>
 * 菜单 pojo service</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023 /8/24
 */
public interface IAuthMenuService{

    /**
     * 创建菜单
     *
     * @param menu the menu
     * @return auth menu
     */
    AuthMenu createAuthMenu(AuthMenu menu);


    /**
     * Create batch auth menu list.
     *
     * @param menus the menus
     * @return the list
     */
    List<AuthMenu> createBatchAuthMenu(List<AuthMenu> menus);

    /**
     * 更新菜单
     *
     * @param menu the menu
     * @return auth menu
     */
    AuthMenu updateAuthMenuById(AuthMenu menu);

    /**
     * Delete auth menu by id.
     *
     * @param id the id
     */
    void deleteAuthMenuById(String id);

    /**
     * Delete auth menus.
     *
     * @param ids the ids
     * @return 返回被删除的菜单id集合
     */
    void deleteAuthMenuByIds(List<String> ids);

    /**
     * 查找菜单
     *
     * @param id the id
     * @return auth menu
     */
    AuthMenu findAuthMenuById(String id);

    /**
     * Find auth menu by ids auth menu.
     *
     * @param ids the ids
     * @return the auth menu
     */
    List<AuthMenu> findAuthMenuByIds(List<String> ids);

    /**
     * Find auth menu by code auth menu.
     *
     * @param code  the code
     * @param sysId the sys id
     * @return the auth menu
     */
    AuthMenu findAuthMenuByCode(String code, String sysId);

    /**
     * Find menu by sys id list.
     *
     * @param isPreset the is preset
     * @param systemId the system id
     * @return the list
     */
    List<AuthMenu> findMenuBySysId(CommonConst.BooleanEnum isPreset, String systemId);

    /**
     * Find children list.
     *
     * @param id the id
     * @return the list
     */
    List<AuthMenu> findChildren(String id);

}
