package com.kingtom.kirin.app.auth.service;

import java.util.List;

import com.kingtom.kirin.app.auth.model.AuthPermission;
import com.kingtom.kirin.core.resource.ResourceConst;

/**
 * 权限模块</p>
 * 权限 pojo service</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023 /8/24
 */
public interface IAuthPermissionService{

    /**
     * 创建权限
     *
     * @param permission the permission
     * @return auth permission
     */
    AuthPermission create(AuthPermission permission);

    /**
     * Create batch auth permission.
     *
     * @param permission the permission
     * @return the auth permission
     */
    List<AuthPermission> createBatch(List<AuthPermission> permission);

    /**
     * 更新权限
     *
     * @param permission the permission
     * @return auth permission
     */
    AuthPermission update(AuthPermission permission);

    /**
     * 根据Id查询.
     *
     * @param id the id
     * @return the auth permission
     */
    AuthPermission findById(String id);

    /**
     * 根据资源类型、资源id、操作类型、角色Id删除权限.
     *
     * @param resTypeEnums the resource type enums
     * @param resIds       the res ids
     * @param operaEnums   the opera enums
     * @param roleIds      the role ids
     */
    void deleteByRRO(List<ResourceConst.ResourceTypeEnum> resTypeEnums, List<String> resIds,
            List<ResourceConst.ResourceOperaEnum> operaEnums, List<String> roleIds);

    /**
     * 据资源类型、资源id、操作类型、角色Id查找权限
     *
     * @param resTypeEnums the resource type enum
     * @param resIds       the res ids
     * @param operaEnums   the sys id
     * @param roleIds      the role ids
     * @return auth permission
     */
    List<AuthPermission> findByRRO(List<ResourceConst.ResourceTypeEnum> resTypeEnums, List<String> resIds,
            List<ResourceConst.ResourceOperaEnum> operaEnums, List<String> roleIds);

}
