package com.kingtom.kirin.app.auth.service;

import java.util.List;

import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.app.auth.model.AuthRole;
import com.kingtom.kirin.app.auth.model.AuthRoleExample;
import com.kingtom.kirin.core.resource.ResourceConst;
import com.kingtom.kirin.core.resource.ResourceObject;

/**
 * 权限模块</p>
 * 角色 pojo service</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023 /8/24
 */
public interface IAuthRoleService{

    /**
     * Create auth role.
     *
     * @param role the role
     * @return the auth role
     */
    AuthRole createAuthRole(AuthRole role);

    /**
     * Update auth role by id auth role.
     *
     * @param role the role
     * @return the auth role
     */
    AuthRole updateAuthRoleById(AuthRole role);

    /**
     * 角色追加绑定用户资源.
     *
     * @param roleId    the role id
     * @param resources the resource object
     * @return the auth role
     */
    AuthRole updateAuthRoleAddBindResource(String roleId, ResourceObject<String> resources);

    /**
     * 角色追加绑定多个用户资源.
     *
     * @param roleId    the role id
     * @param resources the resource object
     * @return auth role
     */
    AuthRole updateAuthRoleAddBindResources(String roleId, List<ResourceObject<String>> resources);


    /**
     * 角色移除绑定用户资源.
     *
     * @param roleId    the role id
     * @param resources the resource object
     * @return the auth role
     */
    AuthRole updateAuthRoleRemoveBindResource(String roleId, ResourceObject<String> resources);

    /**
     * 角色移除绑定多个用户资源.
     *
     * @param roleId    the role id
     * @param resources the resource object
     * @return auth role
     */
    AuthRole updateAuthRoleRemoveBindResources(String roleId, List<ResourceObject<String>> resources);

    /**
     * 角色更新绑定多个用户资源
     *
     * @param roleId    the id
     * @param resources the resources
     * @return auth role
     */
    AuthRole updateAuthRoleUpdateBindResources(String roleId, List<ResourceObject<String>> resources);

    /**
     * 根据code给角色增加绑定多个用户资源.
     *
     * @param roleCode  the role code
     * @param resources the resources
     * @return the auth role
     */
    void updateAuthRoleAddBindResourcesByCodes(List<String> roleCode, List<ResourceObject<String>> resources);

    /**
     * 根据code给角色移除绑定多个用户资源.
     *
     * @param roleCode  the role code
     * @param resources the resources
     * @return the auth role
     */
    @Deprecated
    void updateAuthRoleRemoveBindResourcesByCodes(List<String> roleCode, List<ResourceObject<String>> resources);

    /**
     * Delete auth role by id auth role.
     *
     * @param roleId the role id
     */
    void deleteAuthRoleById(String roleId);

    /**
     * Delete roles.
     *
     * @param ids the ids
     */
    void deleteRoles(List<String> ids);

    /**
     * Find auth role by id auth role.
     *
     * @param roleId the role id
     * @return the auth role
     */
    AuthRole findAuthRoleById(String roleId);

    /**
     * Find auth role by code auth role.
     *
     * @param code  the code
     * @param sysId the sys id
     * @return the auth role
     */
    AuthRole findAuthRoleByCode(String code, String sysId);

    /**
     * Find auth role by name auth role.
     *
     * @param name  the name
     * @param sysId the sys id
     * @return the auth role
     */
    AuthRole findAuthRoleByName(String name, String sysId);

    /**
     * Find role page info.
     *
     * @param example  the example
     * @param pageNum  the page num
     * @param pageSize the page size
     * @return the page info
     */
    PageInfo<AuthRole> findRolePage(AuthRoleExample example, int pageNum, int pageSize);

    /**
     * Find all list.
     *
     * @param sysId the sys id
     * @return the list
     */
    List<AuthRole> findAll(String sysId);


    /**
     * 找到资源拥有的角色
     *
     * @param resourceTypeEnum the resource type enum
     * @param resId            the res id
     * @param sysId            the sys id
     * @return the list
     */
    List<AuthRole> findBySource(ResourceConst.ResourceTypeEnum resourceTypeEnum, String resId, String sysId);

}
