package com.kingtom.kirin.app.auth.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.regex.Pattern;

import com.kingtom.kirin.app.auth.AuthConst;
import com.kingtom.kirin.app.auth.dao.AuthMenuMapper;
import com.kingtom.kirin.app.auth.model.AuthMenu;
import com.kingtom.kirin.app.auth.model.AuthMenuExample;
import com.kingtom.kirin.app.auth.model.AuthMenuExt3;
import com.kingtom.kirin.app.auth.model.AuthMenuRoute;
import com.kingtom.kirin.app.auth.service.IAuthMenuService;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.json.JObject;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.IDUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 权限模块</p>
 * 菜单 pojo service impl</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023 /8/24
 */
@Service
public class AuthMenuServiceImpl implements IAuthMenuService{

    /**
     * 英文字母开头，由英文字母数字、下划线组成
     */
    private static final Pattern pattern = Pattern.compile("^[a-zA-Z][0-9a-zA-Z_]*$");

    @Autowired
    private AuthMenuMapper mapper;

    @Override
    public AuthMenu createAuthMenu(AuthMenu menu){
        checkBeforeSave(menu);
        fillBeforeSave(menu);
        boolean f = mapper.insertSelective(menu) == 1;
        if(!f){
            throw new AppException("落库出错，新建菜单失败！");
        }
        return findAuthMenuById(menu.getId());
    }

    @Override
    public List<AuthMenu> createBatchAuthMenu(List<AuthMenu> menus){
        List<AuthMenu> menus1 = new ArrayList<>(menus.size() * 2);
        for(AuthMenu menu : menus){
            menus1.add(createAuthMenu(menu));
        }
        return menus1;
    }

    private void checkBeforeSave(AuthMenu menu){
        if(menu == null){
            throw new AppException("菜单为空，新建菜单失败！");
        }
        if(StringUtils.isBlank(menu.getName())){
            throw new AppException("名称为空，新建菜单失败！");
        }
        if(StringUtils.isNotEmpty(menu.getOtherName()) && StringUtils.isBlank(menu.getOtherName())){
            throw new AppException("菜单别名为空，新建菜单失败！");
        }
        if(StringUtils.isBlank(menu.getCode())){
            throw new AppException("标识符为空，新建菜单失败！");
        }
        if(!pattern.matcher(menu.getCode()).matches()){
            throw new AppException("标识符格式错误，新建菜单失败！");
        }
        if(CommonConst.BooleanEnum.findEnumByKey(menu.getPreset()) == null){
            throw new AppException("是否为预置菜单为空，新建菜单失败！");
        }
        if(CommonConst.BooleanEnum.findEnumByKey(menu.getHide()) == null){
            throw new AppException("是否隐藏为空，新建菜单失败！");
        }
        if(StringUtils.isBlank(menu.getSysId())){
            throw new AppException("系统Id为空，新建菜单失败！");
        }
        if(menu.getCode() != null){
            AuthMenu dbByCode = findAuthMenuByCode(menu.getCode(), menu.getSysId());
            if(dbByCode != null){
                throw new AppException("菜单编码重复，新建菜单失败！").setM(menu.getCode());
            }
        }
        if(StringUtils.isBlank(menu.getType())){
            throw new AppException("菜单类型为空，新建菜单失败！");
        }
        AuthConst.AuthMenuTypeEnum menuTypeEnum = AuthConst.AuthMenuTypeEnum.findEnumByKey(menu.getType());
        if(menuTypeEnum == null){
            throw new AppException("菜单类型错误，新建菜单失败！");
        }

        if(AuthConst.AuthMenuTypeEnum.Directory.equals(menuTypeEnum)){
            if(StringUtils.isBlank(menu.getRouteUri())){
                throw new AppException("目录路由地址为空，新建菜单失败！");
            }
            menu.setActionType(null);
        } else if(AuthConst.AuthMenuTypeEnum.Menu.equals(menuTypeEnum)){
            if(StringUtils.isBlank(menu.getRouteUri())){
                throw new AppException("菜单路由地址为空，新建菜单失败！");
            }
            if(StringUtils.isBlank(menu.getActionType())){
                throw new AppException("菜单动作类型为空，新建菜单失败！");
            }
            if(AuthConst.AuthMenuActionTypeEnum.findEnumByKey(menu.getActionType()) == null){
                throw new AppException("菜单动作类型错误，新建菜单失败！");
            }
        } else if(AuthConst.AuthMenuTypeEnum.Function.equals(menuTypeEnum)){
            menu.setIcon(null);
            //menu.setHide(false);
            //menu.setOtherName(null);
            //menu.setRouteUri(null);
            //menu.setModuleUri(null);
            menu.setActionType(null);
        }
    }

    private void fillBeforeSave(AuthMenu menu){
        if(StringUtils.isBlank(menu.getId())){
            //id
            menu.setId(IDUtils.newUUIDByString(menu.getCode() + menu.getSysId()));
        }
        menu.setGmtModified(new Date());
        menu.setGmtCreate(menu.getGmtModified());
        if(menu.getOrderNum() == null){
            menu.setOrderNum(0);
        }
        if(menu.getExt3() == null){
            menu.setExt3(new JObject<>(new AuthMenuExt3()));
        }
        if(menu.getRoute() == null){
            menu.setRoute(new JObject<>());
        }
        if(menu.getRoute().getE() == null){
            menu.getRoute().setE(new AuthMenuRoute());
        }
        if(menu.getRoute().getE().getPath() == null){
            menu.getRoute().getE().setPath(new ArrayList<>());
        }
        //route填充
        if(StringUtils.isBlank(menu.getParentId())){
            menu.setParentId(menu.getSysId());
            menu.getRoute().getE().getPath().clear();
            menu.getRoute().getE().getPath().add(menu.getSysId());
            menu.getRoute().getE().getPath().add(menu.getId());
        } else{
            AuthMenu parent = findAuthMenuById(menu.getParentId());
            if(parent == null){
                throw new AppException("父菜单不存在，新建菜单失败！");
            }
            menu.getRoute().getE().getPath().clear();
            menu.getRoute().getE().getPath().addAll(parent.getRoute().getE().getPath());
            menu.getRoute().getE().getPath().add(menu.getId());
        }

    }

    @Override
    public AuthMenu updateAuthMenuById(AuthMenu menu){
        AuthMenu dbMenu = checkBeforeUpdate(menu);
        AuthMenu updateMenu = new AuthMenu();
        updateMenu.setId(menu.getId());
        updateMenu.setGmtModified(new Date());
        updateMenu.setName(menu.getName());
        updateMenu.setOtherName(menu.getOtherName());
        updateMenu.setCode(menu.getCode());
        updateMenu.setOrderNum(menu.getOrderNum());
        updateMenu.setHide(menu.getHide());
        if(AuthConst.AuthMenuTypeEnum.Directory.getKey().equals(dbMenu.getType())){
            if(StringUtils.isNotEmpty(menu.getIcon()) && StringUtils.isBlank(menu.getIcon())){
                throw new AppException("目录图标为空，更新菜单失败！");
            }
            if(menu.getRouteUri() != null && StringUtils.isBlank(menu.getRouteUri())){
                throw new AppException("目录路由地址为空，更新菜单失败！");
            }
            updateMenu.setIcon(menu.getIcon());
            updateMenu.setRouteUri(menu.getRouteUri());
        } else if(AuthConst.AuthMenuTypeEnum.Menu.getKey().equals(dbMenu.getType())){
            if(StringUtils.isNotEmpty(menu.getIcon()) && StringUtils.isBlank(menu.getIcon())){
                throw new AppException("菜单图标为空，更新菜单失败！");
            }
            if(menu.getActionType() != null && StringUtils.isBlank(menu.getActionType())){
                throw new AppException("菜单动作类型为空，更新菜单失败！");
            }
            if(menu.getRouteUri() != null && StringUtils.isBlank(menu.getRouteUri())){
                throw new AppException("菜单路由地址为空，更新菜单失败！");
            }
            updateMenu.setIcon(menu.getIcon());
            updateMenu.setRouteUri(menu.getRouteUri());
            updateMenu.setActionType(menu.getActionType());
        } else if(AuthConst.AuthMenuTypeEnum.Function.getKey().equals(dbMenu.getType())){
            updateMenu.setIcon(null);
            updateMenu.setRouteUri(null);
            updateMenu.setActionType(null);
        }

        boolean f = mapper.updateByPrimaryKeySelective(updateMenu) == 1;
        if(!f){
            throw new AppException("落库出错，更新菜单失败！");
        }
        return findAuthMenuById(menu.getId());
    }

    private AuthMenu checkBeforeUpdate(AuthMenu menu){
        if(menu == null){
            throw new AppException("菜单为空，更新菜单失败！");
        }
        if(StringUtils.isBlank(menu.getId())){
            throw new AppException("id为空，更新菜单失败！");
        }
        AuthMenu dbMenu = findAuthMenuById(menu.getId());
        if(dbMenu == null){
            throw new AppException("菜单不存在，更新菜单失败！");
        }
        if(CommonConst.BooleanEnum.True.getKey().equals(dbMenu.getPreset())){
            throw new AppException("预置菜单不能修改！");
        }
        if(menu.getName() != null && StringUtils.isBlank(menu.getName())){
            throw new AppException("名称为空，更新菜单失败！");
        }
        if(menu.getOtherName() != null && StringUtils.isBlank(menu.getOtherName())){
            throw new AppException("菜单别名为空，更新菜单失败！");
        }
        if(menu.getHide() != null && CommonConst.BooleanEnum.findEnumByKey(menu.getHide()) == null){
            throw new AppException("菜单是否隐藏为空，更新菜单失败！");
        }
        if(menu.getCode() != null){
            if(StringUtils.isBlank(menu.getCode())){
                throw new AppException("标识符为空，更新菜单失败！");
            }
            if(!pattern.matcher(menu.getCode()).matches()){
                throw new AppException("标识符格式错误，更新菜单失败！");
            }
            AuthMenu dbByCode = findAuthMenuByCode(menu.getCode(), dbMenu.getSysId());
            if(dbByCode != null && !dbByCode.getId().equals(menu.getId())){
                throw new AppException("菜单编码重复，更新菜单失败！").setM(menu.getCode());
            }
        }
        if(menu.getType() != null && !dbMenu.getType().equals(menu.getType())){
            throw new AppException("菜单类型不能修改！");
        }
        return dbMenu;
    }

    @Override
    public void deleteAuthMenuById(String id){
        if(StringUtils.isBlank(id)){
            return;
        }

        boolean f = mapper.deleteByPrimaryKey(id) == 1;
        if(!f){
            throw new AppException("落库出错，删除菜单失败！");
        }
    }

    @Override
    public void deleteAuthMenuByIds(List<String> ids){
        if(CollectionUtils.isEmpty(ids)){
            return;
        }
        AuthMenuExample example = new AuthMenuExample();
        example.createCriteria().andIdIn(ids);
        boolean f = mapper.deleteByExample(example) == ids.size();
        if(!f){
            throw new AppException("落库出错，删除菜单失败！");
        }
    }

    @Override
    public AuthMenu findAuthMenuById(String id){
        if(StringUtils.isBlank(id)){
            return null;
        }
        return mapper.selectByPrimaryKey(id);
    }

    @Override
    public List<AuthMenu> findAuthMenuByIds(List<String> ids){
        if(CollectionUtils.isEmpty(ids)){
            return new ArrayList<>();
        }
        AuthMenuExample example = new AuthMenuExample();
        example.createCriteria().andIdIn(ids);
        return mapper.selectByExample(example);
    }

    @Override
    public AuthMenu findAuthMenuByCode(String code, String sysId){
        if(StringUtils.isAnyBlank(code, sysId)){
            return null;
        }
        AuthMenuExample example = new AuthMenuExample();
        example.createCriteria().andCodeEqualTo(code).andSysIdEqualTo(sysId);
        List<AuthMenu> menus = mapper.selectByExample(example);
        return CollectionUtils.isEmpty(menus) ? null : menus.get(CommonConst.INT_0);
    }

    @Override
    public List<AuthMenu> findMenuBySysId(CommonConst.BooleanEnum isPreset, String systemId){
        if(StringUtils.isBlank(systemId)){
            return new ArrayList<>();
        }
        AuthMenuExample example = new AuthMenuExample();
        AuthMenuExample.Criteria criteria = example.createCriteria();
        criteria.andSysIdEqualTo(systemId);
        if(isPreset != null){
            criteria.andPresetEqualTo(isPreset.getKey());
        }
        return mapper.selectByExample(example);
    }

    @Override
    public List<AuthMenu> findChildren(String id){
        AuthMenu parent = findAuthMenuById(id);
        if(parent == null){
            return new ArrayList<>();
        }
        AuthMenuExample example = new AuthMenuExample();
        example.createCriteria().addCriterion(
                "route::jsonb -> 'e' -> 'path' ??& array['" + StringUtils.join(parent.getRoute().getE().getPath()) +
                        "']");
        return mapper.selectByExample(example);
    }

}
