package com.kingtom.kirin.app.auth.utils;

import java.util.ArrayList;
import java.util.List;

import com.kingtom.kirin.app.auth.AuthConst;
import com.kingtom.kirin.app.auth.model.AuthPermission;
import com.kingtom.kirin.app.auth.model.AuthRole;
import com.kingtom.kirin.core.resource.ResourceConst;
import org.apache.commons.lang3.StringUtils;

/**
 * 权限模块</p>
 * 工具类</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023 /8/31
 */
public class AuthUtils{

    /**
     * 构建一个标准的权限码.
     *
     * @param permission the permission
     * @param code       the code
     * @return the string
     */
    public static String buildPermissionCode(AuthPermission permission, String code){
        List<String> array = new ArrayList<>();
        array.add(permission.getResType());
        array.add(code);
        array.add(permission.getOperaType());
        return StringUtils.join(array, AuthConst.PERMISSION_CODE_SEPARATOR);
    }

    /**
     * 判断一个资源是否是角色的成员
     *
     * @param role             the role
     * @param resourceTypeEnum the resource type enum
     * @param userId           the user id
     * @return boolean
     */
    public static boolean isRoleMember(AuthRole role, ResourceConst.ResourceTypeEnum resourceTypeEnum, String userId){
        return role != null && role.getSource() != null && role.getSource().getE() != null &&
                role.getSource().getE().getResources() != null && role.getSource().getE().getResources().stream()
                .anyMatch(e -> resourceTypeEnum.getKey().equals(e.getResType()) && userId.equals(e.getResId()));
    }

}
