package com.kingtom.kirin.app.conf.service;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.app.conf.dao.ConfInfoMapper;
import com.kingtom.kirin.app.conf.mapper.ConfMapper;
import com.kingtom.kirin.app.conf.model.ConfInfo;
import com.kingtom.kirin.app.conf.model.ConfInfoExample;
import com.kingtom.kirin.app.conf.utils.ConfUtils;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.utils.CaseUtils;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.ObjectUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/9/4 11:02
 */
@Component
@Slf4j
public class ConfServiceImpl implements IConfService {

    @Autowired
    private ConfInfoMapper confInfoMapper;

    @Override
    public ConfInfo createConfInfo(ConfInfo info) {
        CaseUtils.build(info,
                CaseUtils.caseBeanAndThrow(ObjectUtil::isNull,"配置不能为空"),
                CaseUtils.caseAttrBlankAndThrow(ConfInfo::getName,"配置名不能为空"),
                CaseUtils.caseAttrBlankAndThrow(ConfInfo::getKey,"配置KEY不能为空"),
                CaseUtils.caseRelCheckAndThrow(b->findConfInfoByKey(b.getKey()),ObjectUtil::isNotNull,"配置KEY不能重复"))
                        .doCase();
        ConfUtils.fillBeforeCreate(info);
        confInfoMapper.insert(info);
        log.info("配置["+info.getName()+"]创建完成");
        return info;
    }

    @Override
    public ConfInfo updateConfInfo(ConfInfo info) {
        CaseUtils.Context<ConfInfo> context = CaseUtils.build(info,
                        CaseUtils.caseBeanAndThrow(ObjectUtil::isNull,"配置不能为空"),
                        CaseUtils.caseAttrBlankAndThrow(ConfInfo::getId,"配置不能为空"),
                        CaseUtils.caseRelCheckAndThrow(b->findConfInfoById(b.getId()), ObjectUtil::isNull,"配置不存在"),
                        CaseUtils.caseAttrBlankAndThrow(ConfInfo::getName,"配置名不能为空"),
                        CaseUtils.caseAttrBlankAndThrow(ConfInfo::getKey,"配置KEY不能为空"),
                        CaseUtils.caseRelCompareAndThrow(b->findConfInfoByKey(b.getKey()),ConfUtils::isNotSame,"配置KEY不能重复"))
                .doCase();
        ConfInfo dbInfo = context.getCache(CommonConst.INT_2);
        ConfInfo copyDbBean = ConfMapper.INSTANCT.copyDepth(dbInfo);
        ObjectUtils.copyPropertiesByCover(info,copyDbBean,"name","key","valueType","value","des","ext1","ext2");
        ConfUtils.fillBeforeUpdate(copyDbBean,dbInfo);
        confInfoMapper.updateByPrimaryKey(copyDbBean);
        log.info("配置["+info.getName()+"]更新完成");
        return info;
    }

    @Override
    public ConfInfo deleteConfInfo(String id) {
        ConfInfo dbInfo = findConfInfoById(id);
        if(dbInfo != null) {
            confInfoMapper.deleteByPrimaryKey(id);
        } else {
            log.warn("配置["+id+"]已经不存在，无需删除");
        }
        return dbInfo;
    }

    @Override
    public PageInfo<ConfInfo> findConfInfos(String search, Integer pageNum, Integer pageSize) {
        ConfInfoExample e = new ConfInfoExample();
        if(StrUtil.isNotBlank(search)) {
            e.or().andNameLike("%"+search+"%");
            e.or().andKeyLike("%"+search+"%");
        }
        e.setOrderByClause("gmt_create desc");
        PageHelper.startPage(pageNum, pageSize);
        List<ConfInfo> infos = confInfoMapper.selectByExample(e);
        return new PageInfo<>(infos);
    }

    @Override
    public ConfInfo findConfInfoById(String id) {
        return StrUtil.isBlank(id)?null:confInfoMapper.selectByPrimaryKey(id);
    }

    @Override
    public ConfInfo findConfInfoByKey(String key) {
        if(StrUtil.isAllNotBlank(key)) {
            ConfInfoExample e = new ConfInfoExample();
            e.createCriteria().andKeyEqualTo(key);
            return CollectionUtils.findFirst(confInfoMapper.selectByExample(e));
        } else {
            return null;
        }
    }
}
