package com.kingtom.kirin.app.conf.utils;

import cn.hutool.core.util.ObjectUtil;
import com.kingtom.kirin.app.conf.ConfConst;
import com.kingtom.kirin.app.conf.model.ConfInfo;
import com.kingtom.kirin.app.system.utils.SystemUtils;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.utils.IDUtils;

import java.util.Date;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/9/4 11:39
 */
public class ConfUtils {

    private ConfUtils(){}

    /**
     * 完善新增数据
     * @param paramBean 接口传递的待保存数据
     * @return
     */
    public static void fillBeforeCreate(ConfInfo paramBean) {
        Date now = new Date();
        paramBean.setId(IDUtils.newUUID());
        if(paramBean.getState() == null) {
            paramBean.setState(CommonConst.StateEnum.Enable.getKey());
        }
        ConfConst.ConfInfoValueTypeEnum t = ConfConst.ConfInfoValueTypeEnum.findEnumByKey(paramBean.getValueType());
        if(t== null) {
            paramBean.setValueType(ConfConst.ConfInfoValueTypeEnum.String.getKey());
        }
        if(paramBean.getSysFlag() == null) {
            paramBean.setSysFlag(CommonConst.BooleanEnum.False.getKey());
        }
        paramBean.setGmtCreate(now);
        paramBean.setGmtModified(now);
        paramBean.setSysId(SystemUtils.getCurrentSystemId());
    }

    /**
     * 完善修改数据
     * @param paramBean 接口传递的待修改数据
     * @param dbBean 数据库的原始数据
     * @return
     */
    public static void fillBeforeUpdate(ConfInfo paramBean,ConfInfo dbBean) {
        Date now = new Date();
        ConfConst.ConfInfoValueTypeEnum t = ConfConst.ConfInfoValueTypeEnum.findEnumByKey(paramBean.getValueType());
        if(t== null) {
            paramBean.setValueType(ConfConst.ConfInfoValueTypeEnum.String.getKey());
        }
        paramBean.setGmtModified(now);
    }

    public static boolean isNotSame(ConfInfo info1,ConfInfo info2) {
        if(ObjectUtil.isAllEmpty(info1,info2)) {
            return false;
        } else {
            return ObjectUtil.notEqual(info1.getId(),info2.getId());
        }
    }

    public static boolean isSame(ConfInfo info1,ConfInfo info2) {
        return !isNotSame(info1,info2);
    }
}
