package com.kingtom.kirin.app.ddic.service;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.resource.ResourceUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.kingtom.kirin.app.ddic.model.DdicInfo;
import com.kingtom.kirin.app.ddic.model.DdicItem;
import com.kingtom.kirin.app.system.service.SystemDefaultService;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import jakarta.annotation.PostConstruct;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.Resource;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.core.io.support.ResourcePatternResolver;
import org.springframework.stereotype.Component;
import org.springframework.util.ResourceUtils;

import java.io.File;
import java.nio.charset.Charset;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/9/8 10:16
 */
@Component
@Slf4j
public class DdicDefaultService {

    @Autowired
    private IDdicService ddicService;

    @Autowired
    private SystemDefaultService systemDefaultService;
    @Value("${app.init.ddic}")
    private String initDdic;

    public static final String INIT_DDIC_Path = "classpath:ddics/{0}.json";
    public static final String INIT_DDICITEM_Path = "classpath:ddics/{0}-items.json";

    @PostConstruct
    public void init() {
        if(StrUtil.isNotBlank(initDdic)) {
            String[] ls = initDdic.split(",");
            for(String s:ls) {
                if(StrUtil.isNotBlank(s)) {
                    initDdic(s);
                }
            }
        }
    }

    protected void initDdic(String name) {
        try{
            String infopath = MessageFormat.format(INIT_DDIC_Path,name);
            String itempath = MessageFormat.format(INIT_DDICITEM_Path,name);
            log.info("初始化字典["+name+"]准备中，path="+JsonUtils.toJson(ListUtil.toList(infopath,itempath)));
            String json = ResourceUtil.readStr(infopath,Charset.forName("utf-8"));
            DdicInfo info = JsonUtils.toObject(json, DdicInfo.class);
            if(info == null || !info.getCode().equals(name)) {
                throw new AppException("字典不能为空或者编码必须和后缀匹配").setM(name);
            }
            DdicInfo dbinfo = ddicService.findDdicInfoByCode(info.getCode());
            if(dbinfo == null) {
                json = ResourceUtil.readStr(itempath,Charset.forName("utf-8"));
                List<DdicItem> items = JsonUtils.toObjectList(json, DdicItem.class);
                log.info("初始化字典["+name+"]["+info.getName()+"]开始执行");
                doInitDdic(info,items);
            } else {
                log.info("初始化字典["+name+"]["+info.getName()+"]已经存在");
            }
        } catch (Exception e) {
            log.error("初始化字典["+name+"]错误",e);
        }
    }

    protected void doInitDdic(DdicInfo info,List<DdicItem> items) {
        info = ddicService.createDdicInfo(info);
        for(DdicItem item:items) {
            item.setDdicId(info.getId());
        }
        for(DdicItem item:items) {
            item = ddicService.createDdicItem(item);
            log.info("字典项："+item.getFullName());
        }
        log.info("初始化字典["+info.getCode()+"]["+info.getName()+"]完成");
    }

}
