package com.kingtom.kirin.app.ddic.utils;

import java.math.BigDecimal;
import java.util.Date;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.kingtom.kirin.app.ddic.DdicConst;
import com.kingtom.kirin.app.ddic.model.DdicInfo;
import com.kingtom.kirin.app.ddic.model.DdicInfoExt3;
import com.kingtom.kirin.app.ddic.model.DdicItem;
import com.kingtom.kirin.app.ddic.service.IDdicService;
import com.kingtom.kirin.app.system.utils.SystemUtils;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.json.JObject;
import com.kingtom.kirin.core.common.utils.IDUtils;
import com.kingtom.kirin.core.common.utils.SpringContextUtils;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/9/1 17:17
 */
public class DdicUtils {
    private static IDdicService ddicService;

    protected static final BigDecimal D_100 = new BigDecimal("100");
    protected static final BigDecimal D_10000 = new BigDecimal("10000");

    private DdicUtils(){}

    private static IDdicService getDdicService(){
        if(ddicService == null) {
            ddicService = SpringContextUtils.getBean(IDdicService.class);
        }
        return ddicService;
    }

    /**
     * 完善新增数据
     * @param paramBean 接口传递的待保存数据
     * @return
     */
    public static void fillBeforeCreate(DdicInfo paramBean) {
        Date now = new Date();
        paramBean.setId(IDUtils.newUUID());
        if(paramBean.getState() == null) {
            paramBean.setState(CommonConst.StateEnum.Enable.getKey());
        }
        if(paramBean.getSimpleName() == null) {
            paramBean.setSimpleName(paramBean.getName());
        }
        if(paramBean.getExt3() == null) {
            paramBean.setExt3(new JObject<>(new DdicInfoExt3()));
        }
        if(paramBean.getExt3().getE().getShowModel() == null) {
            paramBean.getExt3().getE().setShowModel(DdicConst.DdicInfoShowModelEnum.SimpleName.getKey());
        }
        if(paramBean.getState() == null) {
            paramBean.setState(CommonConst.StateEnum.Enable.getKey());
        }
        if(paramBean.getSysFlag() == null) {
            paramBean.setSysFlag(CommonConst.BooleanEnum.False.getKey());
        }
        paramBean.setGmtCreate(now);
        paramBean.setGmtModified(now);
        paramBean.setSysId(SystemUtils.getCurrentSystemId());
    }

    /**
     * 完善修改数据
     * @param paramBean 接口传递的待修改数据
     * @param dbBean 数据库的原始数据
     * @return
     */
    public static void fillBeforeUpdate(DdicInfo paramBean,DdicInfo dbBean) {
        Date now = new Date();
        paramBean.setGmtModified(now);
    }

    /**
     * 完善新增数据
     * @param paramBean 接口传递的待保存数据
     * @return
     */
    public static void fillBeforeCreate(DdicItem paramBean, DdicItem parentBean) {
        Date now = new Date();
        paramBean.setId(IDUtils.newUUID());
        if(parentBean != null) {
            paramBean.setParentKey(parentBean.getKey());
            paramBean.setLevel(parentBean.getLevel()+CommonConst.INT_1);// 层级+1
            paramBean.setFullName(parentBean.getFullName()+CommonConst.SPACE+paramBean.getName());
            // sort单独处理
        } else {
            paramBean.setParentKey(null);
            paramBean.setLevel(CommonConst.INT_1);// 顶级的设层级1
            paramBean.setFullName(paramBean.getName());
            // sort单独处理
        }
        if(paramBean.getSimpleName() == null) {
            paramBean.setSimpleName(paramBean.getName());
        }
        paramBean.setGmtCreate(now);
        paramBean.setGmtModified(now);
        paramBean.setSysId(SystemUtils.getCurrentSystemId());
    }

    public static void fillBeforeCreateOfSort(DdicItem paramBean, DdicItem parentBean,DdicItem preBean) {
        // 需要默认时
        if(paramBean.getSort() == null) {
            // 优先基于前一个生成
            if(preBean != null) {
                paramBean.setSort(nextStepSort(preBean.getSort(),paramBean.getLevel()));
            } else {
                // 再基于父级生成
                paramBean.setSort(inStepSort(parentBean == null?null:parentBean.getSort(),paramBean.getLevel()));
            }
        }
    }

    /**
     * 完善修改数据
     * @param paramBean 接口传递的待修改数据
     * @param dbBean 数据库的原始数据
     * @return
     */
    public static void fillBeforeUpdate(DdicItem paramBean,DdicItem parentBean,DdicItem dbBean) {
        Date now = new Date();
        if(parentBean != null) {
            paramBean.setParentKey(parentBean.getKey());
            paramBean.setLevel(parentBean.getLevel()+CommonConst.INT_1);// 层级+1
            paramBean.setFullName(parentBean.getFullName()+CommonConst.SPACE+paramBean.getName());
        } else {
            paramBean.setParentKey(null);
            paramBean.setLevel(CommonConst.INT_1);// 顶级的设层级1
            paramBean.setFullName(paramBean.getName());
        }
        if(paramBean.getSimpleName() == null) {
            paramBean.setSimpleName(paramBean.getName());
        }
        paramBean.setGmtModified(now);
    }

    public static void fillBeforeUpdateOfSort(DdicItem paramBean,DdicItem parentBean,DdicItem preBean,DdicItem dbBean) {
        // 需要默认时
        if(paramBean.getSort() == null) {
            // 优先基于数据库复制
            if(dbBean.getSort() != null) {
                paramBean.setSort(dbBean.getSort());
            } else if(preBean != null) {
                // 再基于前一个生成
                paramBean.setSort(nextStepSort(preBean.getSort(),paramBean.getLevel()));
            } else {
                // 再基于父级生成
                paramBean.setSort(inStepSort(parentBean == null?null:parentBean.getSort(),paramBean.getLevel()));
            }
        }
    }

    /**
     * 检查字典项是否是顶级
     * @param info
     * @return
     */
    public static boolean isDdicItemTop(DdicItem info) {
        return StrUtil.isBlank(info.getParentKey());
    }

    /**
     * 获取字典信息
     * @param code
     * @return
     */
    public static DdicInfo findDdic(String code) {
        return getDdicService().findDdicInfoByCode(code);
    }

    /**
     * 获取字典项信息
     * @param code
     * @param key
     * @return
     */
    public static DdicItem findDdicItem(String code,String key) {
        return getDdicService().findDdicItemByKey(code,key);
    }

    public static boolean isNotSame(DdicInfo info1, DdicInfo info2){
        if(ObjectUtil.isAllEmpty(info1, info2)){
            return false;
        }
        return info2 != null && !info1.getId().equals(info2.getId());
    }

    public static boolean isSame(DdicInfo info1,DdicInfo info2) {
        return !isNotSame(info1,info2);
    }

    public static boolean isNotSame(DdicItem info1, DdicItem info2) {
        if(ObjectUtil.isAllEmpty(info1,info2)) {
            return false;
        } else {
            return ObjectUtil.notEqual(info1.getId(),info2.getId());
        }
    }

    public static boolean isSame(DdicItem info1,DdicItem info2) {
        return !isNotSame(info1,info2);
    }

    protected static BigDecimal nextStepSort(BigDecimal pred,Integer level) {
        BigDecimal nows = pred.add(getStepD(D_100,level));
        return nows;
    }

    protected static BigDecimal inStepSort(BigDecimal parentd, Integer level) {
        if(parentd == null) {
            return getStepD(D_100,level);
        } else {
            return parentd.add(getStepD(D_100,level));
        }
    }

    /**
     * 根据起始量级（百位，千位，万位等）与当前层级，计算该量级下步进数
     * @param startAread
     * @param level
     * @return
     */
    protected static BigDecimal getStepD(BigDecimal startAread, Integer level) {
        // 先计算parent代表的值域
        startAread = startAread == null?D_100:startAread;
        BigDecimal dd = startAread;
        for(int i=CommonConst.INT_0;i<level;i++){
            dd = dd.divide(D_100);
        }
        return dd;
    }

    public static void main(String[] args) {
        System.out.println(nextStepSort(new BigDecimal("12.23"),2));
        System.out.println(inStepSort(new BigDecimal("12.23"),3));
    }
}
