package com.kingtom.kirin.app.form.engine.generator;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import com.kingtom.kirin.app.form.FormConst;
import com.kingtom.kirin.app.form.engine.core.def.FormDef;
import com.kingtom.kirin.core.common.exception.AppException;
import lombok.extern.slf4j.Slf4j;

import java.util.List;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/10/13 16:56
 */
@Slf4j
public class FormGenerator{

    private List<IFormGenerator> generators = ListUtil.toList();

    private FormGeneratorConfig config;

    public FormGenerator() {
        this(null);
    }

    public FormGenerator(FormGeneratorConfig config) {
        this.config = config == null?defaultConfig() : config;
        init();
    }

    protected FormGeneratorConfig defaultConfig() {
        FormGeneratorConfig c = new FormGeneratorConfig();
        c.setModel(FormConst.GEN_Model_File);
        return c;
    }

    protected void init(){
        List<String> gs = this.config.findGenerators();
        for (String g : gs) {
            try{
                Class c = Class.forName(g);
                Object o = c.getConstructor().newInstance();
                if(o instanceof IFormGenerator) {
                    generators.add((IFormGenerator) o);
                } else {
                    throw new AppException("类"+g+"不是IFormGenerator的实现类");
                }
            } catch (Exception e){
                log.warn("找不到对应的生成器:g="+g+";e="+e.getMessage());
            }
        }
        CollectionUtil.sort(generators,(g1,g2)->g1.order()-g2.order());
    }

    public void onCreate(FormDef def) {
        FormGeneratorContext context = null;
        for (IFormGenerator g : generators) {
            g.onCreate(context,def);
        }
    }

    public void onUpdate(FormDef def, FormDef oldDef) {
        FormGeneratorContext context = null;
        for (IFormGenerator g : generators) {
            g.onUpdate(context,def, oldDef);
        }
    }

    public void onDelete(FormDef def) {
        FormGeneratorContext context = null;
        for (IFormGenerator g : generators) {
            g.onDelete(context,def);
        }
    }
}
