package com.kingtom.kirin.app.form.engine.generator.impl;

import cn.hutool.core.collection.ListUtil;
import com.kingtom.kirin.app.form.engine.core.IFormBean;
import com.kingtom.kirin.app.form.engine.core.def.FormDef;
import com.kingtom.kirin.app.form.engine.generator.FormGeneratorContext;
import com.kingtom.kirin.app.form.engine.generator.IFormGenerator;
import com.kingtom.kirin.app.form.engine.generator.tools.MemoryFileManager;
import com.kingtom.kirin.app.form.engine.generator.tools.MemoryJavaFileObject;
import com.kingtom.kirin.core.common.CommonConst;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

import javax.tools.*;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 * 生成器：java文件的
 * @author King
 * @since 2023/10/14 10:40
 */
@Slf4j
public class FormJavaGenerator implements IFormGenerator {

    public static final String TEMPLATE_FILE_NAME = "form_java_model.ftl";

    @Override
    public void onCreate(FormGeneratorContext context, FormDef def) {
        Model m = parseModel(def);
        try{
            // 生成
            String sourceCode = context.doGenerateJavaCodeByFM(m,TEMPLATE_FILE_NAME);
            // 编译
            String qualifiedClassName = m.getPackageName()+ CommonConst.PERIOD +m.getClassName();
            JavaCompiler compiler = ToolProvider.getSystemJavaCompiler();
            DiagnosticCollector<JavaFileObject> diagnostics = new DiagnosticCollector<>();
            MemoryFileManager manager = new MemoryFileManager(compiler.getStandardFileManager(null, null, null));
            List<JavaFileObject> sourceFiles = Collections.singletonList(new MemoryJavaFileObject(qualifiedClassName, sourceCode));

            JavaCompiler.CompilationTask task = compiler.getTask(null, manager, diagnostics, null, null, sourceFiles);

            boolean result = task.call();

            if (!result) {
                for(Diagnostic o :diagnostics.getDiagnostics()) {
                    log.error("发生错误："+o.toString());
                }
                log.info("表单编译失败："+qualifiedClassName);
            } else {
                ClassLoader classLoader = manager.getClassLoader(null);
                Class<?> clazz = classLoader.loadClass(qualifiedClassName);
                IFormBean bean = (IFormBean) clazz.getConstructor().newInstance();
                bean.setName("aaaaa");
                log.info("表单输出name："+bean.getName());
            }
        } catch (Exception e) {
            log.error("",e);
        }
    }

    @Override
    public void onUpdate(FormGeneratorContext context,FormDef def, FormDef oldDef) {

    }

    @Override
    public void onDelete(FormGeneratorContext context,FormDef def) {

    }

    protected Model parseModel(FormDef def) {
        Model r = new Model();
        r.setPackageName("com.kingtom.kirin.app.form.instances");
        r.getImports().addAll(ListUtil.toList("com.kingtom.kirin.core.common.json.JObject","java.util.Date"));
        r.setClassName("SystemInfo");
        FieldModel fm = new FieldModel();
        r.getFields().add(fm);
        fm.setName("名称");
        fm.setUniqueCode("name");
        fm.setUpUniqueCode("Name");
        fm.setJavaType("String");
        return r;
    }

    @Data
    public static class Model {
        private String packageName;
        private String className;
        private List<String> imports = new ArrayList<>();
        private List<FieldModel> fields = new ArrayList<>();

    }
    @Data
    public static class FieldModel{
        private String uniqueCode;
        private String upUniqueCode;
        private String name;
        private String javaType;
    }

    public static void main(String[] args) {
        FormJavaGenerator g = new FormJavaGenerator();
        g.onCreate(null,null);
    }
}
