package com.kingtom.kirin.app.form.engine.generator.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import com.kingtom.kirin.app.form.FormConst;
import com.kingtom.kirin.app.form.engine.core.def.FormDef;
import com.kingtom.kirin.app.form.engine.core.def.FormFieldDef;
import com.kingtom.kirin.app.form.engine.core.def.FormIndexDef;
import com.kingtom.kirin.app.form.engine.generator.FormGeneratorContext;
import com.kingtom.kirin.app.form.engine.generator.IFormGenerator;
import com.kingtom.kirin.app.form.utils.FormUtils;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.database.tools.DbExcuter;

import java.text.MessageFormat;
import java.util.List;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 * 生成基于postgresql的表
 * @author King
 * @since 2023/10/14 10:47
 */
public class FormPostgresqlGenerator implements IFormGenerator {

    @Override
    public void onCreate(FormGeneratorContext context, FormDef def) {
        List<String> sqls = fillCreateSql(def);
        DbExcuter excuter = DbExcuter.newInstance(context.getDataSource());
        excuter.excuteStatement(s -> {
            try {
                for (String sql : sqls) {
                    s.execute(sql);
                }
            } catch (Exception e) {
                throw new AppException("创建表单数据库表异常", e);
            }
            return true;
        });
    }

    @Override
    public void onUpdate(FormGeneratorContext context,FormDef def, FormDef oldDef) {

    }

    @Override
    public void onDelete(FormGeneratorContext context,FormDef def) {

    }

    protected List<String> fillCreateSql(FormDef def) {
        List<String> sqls = ListUtil.toList();
        // 根据字段定义建表
        StringBuilder b = new StringBuilder();
        String pattern = " CREATE TABLE {0} ( {1} )WITH (OIDS=FALSE);";
        StringBuilder fieldb = new StringBuilder();
        String fpattern = "\"{0}\" {1} COLLATE \"pg_catalog\".\"default\",";
        for(FormFieldDef e:def.getFields()) {
            FormConst.FormFieldTypeEnum t = FormConst.FormFieldTypeEnum.getEnumByKey(e.getType());
            String fsql = MessageFormat.format(fpattern,e.getUniqueCode(),t.getPostgreField());
            fieldb.append(fsql);
        }
        fieldb.deleteCharAt(fieldb.length() - 1);
        String formTableName = FormUtils.findFormTableName(def);
        b.append(MessageFormat.format(pattern, FormUtils.findFormTableName(def), fieldb.toString()));
        sqls.add(b.toString());
        // 根据唯一约束定义建数据库约束
        String indexPattern = "CREATE INDEX {0} ON {1} ( {2} );";
        List<FormIndexDef> indexDefs = def.getIndexs();
        for (FormIndexDef e : indexDefs) {
            String indexName = FormUtils.findFormIndexName(def,e);
            String fields = CollectionUtil.join(e.getFileds(), CommonConst.COMMA_STRING);
            sqls.add(MessageFormat.format(indexPattern, indexName, formTableName, fields));
        }
        return sqls;
    }
}
