package com.kingtom.kirin.app.form.service;

import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.app.form.engine.core.IFormDao;
import com.kingtom.kirin.app.form.engine.core.IFormData;
import com.kingtom.kirin.app.form.engine.core.IFormInstance;
import com.kingtom.kirin.app.form.engine.core.QueryForm;
import com.kingtom.kirin.app.form.engine.core.def.FormDef;
import com.kingtom.kirin.app.form.model.FormInfo;
import com.kingtom.kirin.core.common.base.Result;
import com.kingtom.kirin.core.common.utils.JsonUtils;

import java.util.List;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/10/11 17:36
 */
public interface IFormService {

    public default Result<FormInfo> createForm(String formJson) {
        return createForm(JsonUtils.toObject(formJson, FormDef.class));
    }

    Result<FormInfo> createForm(FormDef formDef);

    Result<IFormInstance> doLoadFormInstance(String formUniqueCode);

    /**
     * 创建表单数据
     * @param formData
     * @return
     */
    Result<IFormData> createFormData(IFormData formData);

    /**
     * 创建表单数据
     * @param formDatas
     * @return
     */
    Result<List<IFormData>> createFormDatas(List<IFormData> formDatas);

    /**
     * 查询一个表单数据（如果符合条件的多于一个就返回第一个，并报告警告）
     * @param funcId 功能ID
     * @param queryForm
     * @return
     */
    IFormData findFormDataByOne(String funcId, QueryForm queryForm);

    /**
     * 查询表单数据集合
     * @param funcId
     * @param queryForm
     * @return
     */
    List<IFormData> findFormDataByList(String funcId, QueryForm queryForm);

    /**
     * 分页查询表单数据
     * @param funcId
     * @param queryForm
     * @return
     */
    PageInfo<IFormData> findFormDataByPage(String funcId, QueryForm queryForm);

}
