package com.kingtom.kirin.app.formwork.service;

import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ObjectUtil;
import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.app.ddic.model.DdicInfo;
import com.kingtom.kirin.app.ddic.utils.DdicUtils;
import com.kingtom.kirin.app.formwork.FormworkConst;
import com.kingtom.kirin.app.formwork.core.IFormworkHandler;
import com.kingtom.kirin.app.formwork.dao.FormworkInfoMapper;
import com.kingtom.kirin.app.formwork.model.FormworkInfo;
import com.kingtom.kirin.app.formwork.utils.FormworkUtils;
import com.kingtom.kirin.app.space.base.IFileSource;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.utils.CaseUtils;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.Map;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/10/25 17:37
 */
@Component
@Slf4j
public class FormworkServiceImpl implements IFormworkService{

    private FormworkInfoMapper infoMapper;

    private Map<String, IFormworkHandler> handlerMap = MapUtil.newHashMap();

    @Override
    public void registeHandler(IFormworkHandler handler) {
        IFormworkHandler old = handlerMap.put(handler.handlerType(),handler);
        if(old != null) {
            log.warn("存在模板处理器冲突,type="+handler.handlerType()+";class="+ JsonUtils.toJson(new Class[]{old.getClass(),handler.getClass()}));
        }
    }

    @Override
    public FormworkInfo createFormwork(FormworkInfo info) {
        CaseUtils.build(info,
                        CaseUtils.caseBeanAndThrow(ObjectUtil::isNull,"模板不能为空"),
                        CaseUtils.caseAttrBlankAndThrow(FormworkInfo::getName,"模板名不能为空"),
                        CaseUtils.caseRelCheckAndThrow(b-> FormworkConst.FormworkInfoTypeEnum.findEnumByKey(b.getType()), ObjectUtil::isNull,"模板名不能为空"))
                .doCase();
        FormworkUtils.fillBeforeCreate(info);
        infoMapper.insert(info);
        log.info("模板["+info.getName()+"]创建完成");
        return info;
    }

    @Override
    public FormworkInfo updateFormwork(FormworkInfo info) {
        return null;
    }

    @Override
    public FormworkInfo deleteFormwork(String id) {
        return null;
    }

    @Override
    public IFileSource doProcessFormwork(String id, Map<String,Object> data) {
        return null;
    }

    @Override
    public PageInfo<FormworkInfo> findFormworks(String search, Integer pageNum, Integer pageSize) {
        return null;
    }

    @Override
    public FormworkInfo findFormworkInfoById(String id) {
        return null;
    }

    protected IFormworkHandler findHandler(String type) {
        IFormworkHandler r = handlerMap.get(type);
        if(r == null) {
            throw new AppException("模板生成暂不支持该类型").setM(type);
        }
        return r;
    }
}
