package com.kingtom.kirin.app.org.service;

import java.util.List;

import com.kingtom.kirin.app.org.model.OrgStruct;
import com.kingtom.kirin.app.org.model.OrgUnit;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023 /10/20
 */
public interface IOrgAuthRoleService{

    /**
     * 创建组织相关角色.
     *
     * @param org the org
     */
    public void createOrgRoles(OrgUnit org);

    /**
     * 删除组织相关角色.
     *
     * @param org the org
     */
    public void deleteOrgRoles(OrgUnit org);

    /**
     * Find org member code string.
     *
     * @param org the org
     * @return the string
     */
    public String findOrgMemberCode(OrgUnit org);

    /**
     * 增加组织成员.
     *
     * @param org       the org
     * @param personIds the person ids
     */
    public void addOrgMember(OrgUnit org, List<String> personIds);

    /**
     * 移除组织成员.
     *
     * @param org       the org
     * @param personIds the person ids
     */
    public void removeOrgMember(OrgUnit org, List<String> personIds);

    /**
     * 判断是否是组织成员.
     *
     * @param org      the org
     * @param personId the person id
     * @return the boolean
     */
    public boolean isOrgMember(OrgUnit org, String personId);

    /**
     * 查找组织成员的人员Id集合.
     *
     * @param org the org
     * @return the list
     */
    public List<String> findOrgMemberPersonIds(OrgUnit org);

    /**
     * 查找人员是哪些组织的成员.
     *
     * @param personId the person id
     * @return the list
     */
    public List<String> findOrgByMember(String personId);

    /**
     * Find org admin code string.
     *
     * @param org the org
     * @return the string
     */
    public String findOrgAdminCode(OrgUnit org);

    /**
     * 增加组织管理员.
     *
     * @param org       the org
     * @param personIds the person ids
     */
    public void addOrgAdmin(OrgUnit org, List<String> personIds);

    /**
     * 移除组织管理员.
     *
     * @param org       the org
     * @param personIds the person ids
     */
    public void removeOrgAdmin(OrgUnit org, List<String> personIds);

    /**
     * 判断是否是组织管理员.
     *
     * @param org      the org
     * @param personId the person id
     * @return the boolean
     */
    public boolean isOrgAdmin(OrgUnit org, String personId);

    /**
     * 查找组织管理员的人员Id集合.
     *
     * @param org the org
     * @return the list
     */
    public List<String> findOrgAdminPersonIds(OrgUnit org);

    /**
     * 查找人员是哪些组织的管理员.
     *
     * @param personId the person id
     * @return the list
     */
    public List<String> findOrgByAdmin(String personId);

    /**
     * Find org legal code string.
     *
     * @param org the org
     * @return the string
     */
    public String findOrgLegalCode(OrgUnit org);

    /**
     * 更新组织法人。
     *
     * @param org      the org
     * @param personId the person id
     */
    public void updateOrgLegal(OrgUnit org, String personId);

    /**
     * 判断是否是组织法人.
     *
     * @param org      the org
     * @param personId the person id
     * @return the boolean
     */
    public boolean isOrgLegal(OrgUnit org, String personId);

    /**
     * 查找组织法人的人员Id.
     *
     * @param org the org
     * @return the list
     */
    public String findOrgLegalPersonId(OrgUnit org);

    /**
     * 查找人员是哪些组织的法人.
     *
     * @param personId the person id
     * @return the list
     */
    public List<String> findOrgByLegal(String personId);

    /**
     * 创建岗位相关角色.
     *
     * @param post the org
     */
    public void createPostRoles(OrgStruct post);

    /**
     * 删除岗位相关角色.
     *
     * @param post the org
     */
    public void deletePostRoles(OrgStruct post);

    /**
     * Find post member code string.
     *
     * @param post the post
     * @return the string
     */
    public String findPostMemberCode(OrgStruct post);

    /**
     * 增加岗位成员.
     *
     * @param post     the org
     * @param personId the person id
     */
    public void addPostMember(OrgStruct post, List<String> personId);

    /**
     * 移除岗位成员.
     *
     * @param post      the org
     * @param personIds the person ids
     */
    public void removePostMember(OrgStruct post, List<String> personIds);

    /**
     * 判断是否是岗位成员.
     *
     * @param post     the post
     * @param personId the person id
     * @return the boolean
     */
    public boolean isPostMember(OrgStruct post, String personId);

    /**
     * 查找岗位成员的人员Id集合..
     *
     * @param post the post
     * @return the list
     */
    public List<String> findPostMemberPersonIds(OrgStruct post);

    /**
     * 把某个人员加入到多个岗位
     *
     * @param personId the person id
     * @param postIds  the post ids
     */
    public void updatePersonAddPost(String personId, List<String> postIds);

    /**
     * 把某个人员从多个岗位移除
     *
     * @param posts    the post ids
     * @param personId the person id
     */
    public void updatePersonRemovePost(List<OrgStruct> posts, String personId);

    /**
     * 查找人员是哪些岗位的成员.
     *
     * @param personId the person id
     * @return the list
     */
    public List<String> findPostByMember(String personId);

    /**
     * 创建部门相关角色.
     *
     * @param department the org
     */
    public void createDepartmentRoles(OrgStruct department);

    /**
     * 删除部门相关角色.
     *
     * @param department the org
     */
    public void deleteDepartmentRoles(OrgStruct department);


    /**
     * Find department leader code string.
     *
     * @param department the department
     * @return the string
     */
    public String findDepartmentLeaderCode(OrgStruct department);

    /**
     * 更新部门领导.
     *
     * @param department the org
     * @param personId   the person id
     */
    public void updateDepartmentLeader(OrgStruct department, String personId);

    /**
     * 判断是否是部门领导.
     *
     * @param department the department
     * @param personId   the person id
     * @return the boolean
     */
    public boolean isDepartmentLeader(OrgStruct department, String personId);

    /**
     * 查找部门领导的人员Id.
     *
     * @param department the department
     * @return the list
     */
    public String findDepartmentLeaderPersonId(OrgStruct department);

    /**
     * 查找人员是哪些部门的领导.
     *
     * @param personId the person id
     * @return the list
     */
    public List<String> findDepartmentByLeader(String personId);

    /**
     * Find department member code string.
     *
     * @param department the department
     * @return the string
     */
    public String findDepartmentMemberCode(OrgStruct department);

    /**
     * 更新部门成员.
     *
     * @param department the org
     * @param personIds  the person ids
     */
    public void addDepartmentMember(OrgStruct department, List<String> personIds);

    /**
     * 更新部门成员.
     *
     * @param department the org
     * @param personIds  the person ids
     */
    public void removeDepartmentMember(OrgStruct department, List<String> personIds);

    /**
     * 判断是否是部门成员.
     *
     * @param department the department
     * @param personId   the person id
     * @return the boolean
     */
    public boolean isDepartmentMember(OrgStruct department, String personId);

    /**
     * 查找部门成员的人员Id集合.
     *
     * @param department the department
     * @return the list
     */
    public List<String> findDepartmentMemberPersonIds(OrgStruct department);

    /**
     * 把某个人员加入到多个部门
     *
     * @param personId      the person id
     * @param departmentIds the department ids
     */
    public void updatePersonAddDepartment(String personId, List<String> departmentIds);

    /**
     * 把某个人员从多个部门移除
     *
     * @param departments the department ids
     * @param personId    the person id
     */
    public void updatePersonRemoveDepartment(List<OrgStruct> departments, String personId);

    /**
     * 查找人员是哪些部门的领导.
     *
     * @param personId the person id
     * @return the list
     */
    public List<String> findDepartmentByMember(String personId);

}
