package com.kingtom.kirin.app.org.service;

import java.util.List;

import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.app.org.bo.OrgPersonBO;
import com.kingtom.kirin.app.org.bo.OrgUnitBO;
import com.kingtom.kirin.app.org.model.OrgPerson;
import com.kingtom.kirin.app.org.model.OrgStruct;
import com.kingtom.kirin.app.org.model.OrgUnit;
import com.kingtom.kirin.app.org.model.OrgUnitExample;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.utils.CollectionUtils;

/**
 * 组织模块</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023 /10/14
 */
public interface IOrgService extends IOrgPersonService, IOrgUnitService, IOrgStructService{

    /**
     * 获取默认组织
     *
     * @return org unit
     */
    default OrgUnit findDefaultUnit(){
        OrgUnitExample example = new OrgUnitExample();
        example.setOrderByClause("gmt_create asc");
        List<OrgUnit> orgUnits = findUnitList(example);
        return CollectionUtils.isEmpty(orgUnits) ? null : orgUnits.get(CommonConst.INT_0);
    }

    /**
     * 获取组织完整信息
     *
     * @param orgId
     * @return
     */
    OrgUnitBO findOrgUnit(String orgId);

    /**
     * 创建组织，并制定管理员
     *
     * @param unit
     * @param adminIds
     * @return
     */
    OrgUnitBO createUnit(OrgUnit unit, List<String> adminIds);

    /**
     * 创建组织，并制定管理员
     *
     * @param unit
     * @param adminIds
     * @return
     */
    OrgUnitBO updateUnit(OrgUnit unit, List<String> adminIds);

    /**
     * 创建人员并加入组织、部门、岗位.
     *
     * @param orgId         the org id
     * @param departmentIds the department ids
     * @param postIds       the post ids
     * @param person        the person
     * @return the org person bo
     */
    OrgPersonBO createPerson(String orgId, List<String> departmentIds, List<String> postIds, OrgPerson person);

    /**
     * 创建人员 并 更新人员所属组织、部门、岗位.
     *
     * @param orgId         the org id
     * @param departmentIds the department ids
     * @param postIds       the post ids
     * @param person        the person
     * @return the org person bo
     */
    OrgPersonBO updatePerson(String orgId, List<String> departmentIds, List<String> postIds, OrgPerson person);

    /**
     * 删除人员.
     *
     * @param person the person
     */
    void deletePerson(OrgPerson person);

    /**
     * 查找完整信息人员
     *
     * @param personId the person id
     * @param orgId
     */
    OrgPersonBO findPerson(String personId, String orgId);

    /**
     * 翻页查询人员
     *
     * @param structId
     * @param search      the license type
     * @param name        the name
     * @param phone       the phone
     * @param licenseCode the license code
     * @param pageNum     当前页
     * @param pageSize    页面数据条数
     * @return page info
     */
    PageInfo<OrgPerson> findPersonPage(String structId, String search, String name, String phone, String licenseCode,
            int pageNum, int pageSize);

    /**
     * Find department tree list.
     *
     * @param orgId the org id
     * @return the list
     */
    List<OrgStruct> findDepartmentTree(String orgId);

    /**
     * Find post list.
     *
     * @param orgId the org id
     * @return the list
     */
    List<OrgStruct> findPostTree(String orgId);

}
