package com.kingtom.kirin.app.org.service;

import java.util.List;

import com.kingtom.kirin.app.org.config.OrgInitProperties;
import com.kingtom.kirin.app.org.model.OrgPerson;
import com.kingtom.kirin.app.org.model.OrgUnit;
import com.kingtom.kirin.core.common.json.JObject;
import jakarta.annotation.PostConstruct;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 组织模块</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/10/18
 */
@Service
@Slf4j
public class OrgInitService{

    @Autowired
    private OrgInitProperties orgInitProperties;

    @Autowired
    private IOrgService orgService;

    @PostConstruct
    public void init(){
        if(orgService.findDefaultUnit() != null){
            return;
        }
        OrgPerson legal = orgService.findPersonByPhone(orgInitProperties.getLegalPhone());
        if(legal == null){
            OrgPerson person = new OrgPerson();
            person.setName(orgInitProperties.getLegalName());
            person.setPhone(orgInitProperties.getLegalPhone());
            legal = orgService.createPerson(person);
        }
        OrgUnit dbUnit = orgService.findUnitByUsci(orgInitProperties.getUsci());
        if(dbUnit == null){
            log.info("-----开始初始化组织[" + orgInitProperties.getName() + "]-----");
            //初始化组织
            OrgUnit orgUnit = new OrgUnit();
            orgUnit.setName(orgInitProperties.getName());
            orgUnit.setOtherName(orgInitProperties.getOtherName());
            orgUnit.setEnName(orgInitProperties.getEnName());
            orgUnit.setType(orgInitProperties.getType());
            orgUnit.setUsci(orgInitProperties.getUsci());
            orgUnit.setTaxId(orgInitProperties.getTaxId());
            orgUnit.setOrgCode(orgInitProperties.getOrgCode());
            orgUnit.setPhones(new JObject<>(orgInitProperties.getPhones()));
            orgUnit.setEmails(new JObject<>(orgInitProperties.getEmails()));
            orgUnit.setLegalPerson(legal.getId());
            orgService.createUnit(orgUnit, List.of(legal.getId()));
            log.info("------组织[" + orgInitProperties.getName() + "]初始化完成------");
        }
    }

}
