package com.kingtom.kirin.app.org.service.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.kingtom.kirin.app.auth.model.AuthRole;
import com.kingtom.kirin.app.auth.model.AuthRoleSource;
import com.kingtom.kirin.app.auth.service.IAuthRoleService;
import com.kingtom.kirin.app.auth.service.IAuthService;
import com.kingtom.kirin.app.auth.utils.AuthUtils;
import com.kingtom.kirin.app.org.OrgConst;
import com.kingtom.kirin.app.org.model.OrgStruct;
import com.kingtom.kirin.app.org.model.OrgUnit;
import com.kingtom.kirin.app.org.service.IOrgAuthRoleService;
import com.kingtom.kirin.app.system.model.SystemUser;
import com.kingtom.kirin.app.system.service.ISystemService;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.json.JObject;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import com.kingtom.kirin.core.resource.ResourceConst;
import com.kingtom.kirin.core.resource.ResourceObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/10/20
 */
@Service
@Slf4j
public class IOrgAuthRoleServiceImpl implements IOrgAuthRoleService{

    @Autowired
    private IAuthService authService;

    @Autowired
    private IAuthRoleService authRoleService;

    @Autowired
    private ISystemService systemService;

    @Override
    public void createOrgRoles(OrgUnit org){
        //组织成员
        AuthRole member = new AuthRole();
        member.setName(OrgConst.OrgAuthRoleEnum.OrgMember.roleName(org.getName()));
        member.setCode(OrgConst.OrgAuthRoleEnum.OrgMember.roleCode(org.getId()));
        member.setSysId(org.getSysId());
        member.setState(CommonConst.StateEnum.Enable.getKey());
        ResourceObject<String> orgMembers = new ResourceObject<>();
        member.setSource(new JObject<>(new AuthRoleSource(new ArrayList<>())));
        member.setDisplay(CommonConst.BooleanEnum.False.getKey());
        member = authService.addRole(member, null);

        //组织管理员
        AuthRole admin = new AuthRole();
        admin.setName(OrgConst.OrgAuthRoleEnum.OrgAdmin.roleName(org.getName()));
        admin.setCode(OrgConst.OrgAuthRoleEnum.OrgAdmin.roleCode(org.getId()));
        admin.setSysId(org.getSysId());
        admin.setState(CommonConst.StateEnum.Enable.getKey());
        ResourceObject<String> orgAdmins = new ResourceObject<>();
        admin.setSource(new JObject<>(new AuthRoleSource(new ArrayList<>())));
        admin.setDisplay(CommonConst.BooleanEnum.False.getKey());
        admin = authService.addRole(admin, null);

        //组织法人
        AuthRole legal = new AuthRole();
        legal.setName(OrgConst.OrgAuthRoleEnum.OrgLegal.roleName(org.getName()));
        legal.setCode(OrgConst.OrgAuthRoleEnum.OrgLegal.roleCode(org.getId()));
        legal.setSysId(org.getSysId());
        legal.setState(CommonConst.StateEnum.Enable.getKey());
        ResourceObject<String> orgLegals = new ResourceObject<>();
        legal.setSource(new JObject<>(new AuthRoleSource(new ArrayList<>())));
        legal.setDisplay(CommonConst.BooleanEnum.False.getKey());
        legal = authService.addRole(legal, null);
    }

    @Override
    public void deleteOrgRoles(OrgUnit org){
        AuthRole orgMember = authRoleService.findAuthRoleByCode(findOrgMemberCode(org), org.getSysId());
        AuthRole orgAdmin = authRoleService.findAuthRoleByCode(findOrgAdminCode(org), org.getSysId());
        AuthRole orgLegal = authRoleService.findAuthRoleByCode(findOrgLegalCode(org), org.getSysId());
        List<String> roleIds = Stream.of(orgMember, orgAdmin, orgLegal).filter(Objects::nonNull).map(AuthRole::getId)
                .collect(Collectors.toList());
        authRoleService.deleteRoles(roleIds);
    }

    @Override
    public String findOrgMemberCode(OrgUnit org){
        return OrgConst.OrgAuthRoleEnum.OrgMember.roleCode(org.getId());
    }

    @Override
    public void addOrgMember(OrgUnit org, List<String> personIds){
        AuthRole member = authRoleService.findAuthRoleByCode(findOrgMemberCode(org), org.getSysId());
        if(member == null){
            log.error(JsonUtils.toJson(org));
            throw new AppException("组织成员身份不存在，增加组织成员失败！");
        }
        List<SystemUser> systemUsers =
                systemService.findSystemUserByResIds(ResourceConst.ResourceTypeEnum.OrgPerson, personIds);
        if(CollectionUtils.isEmpty(systemUsers)){
            throw new AppException("人员信息缺失用户，增加组织成员失败！");
        }
        //角色追加用户
        List<ResourceObject<String>> resources = createResourceObject(systemUsers);
        authRoleService.updateAuthRoleAddBindResources(member.getId(), resources);
    }

    @Override
    public void removeOrgMember(OrgUnit org, List<String> personIds){
        AuthRole member = authRoleService.findAuthRoleByCode(findOrgMemberCode(org), org.getSysId());
        if(member == null){
            log.error(JsonUtils.toJson(org));
            throw new AppException("组织成员身份不存在，移除组织成员失败！");
        }
        List<SystemUser> systemUsers =
                systemService.findSystemUserByResIds(ResourceConst.ResourceTypeEnum.OrgPerson, personIds);
        if(CollectionUtils.isEmpty(systemUsers)){
            throw new AppException("人员信息缺失用户，移除组织成员失败！");
        }
        //角色移除用户
        List<ResourceObject<String>> resources = createResourceObject(systemUsers);
        authRoleService.updateAuthRoleRemoveBindResources(member.getId(), resources);
    }

    @Override
    public boolean isOrgMember(OrgUnit org, String personId){
        return userIsRole(org.getSysId(), findOrgMemberCode(org), personId);
    }

    @Override
    public List<String> findOrgMemberPersonIds(OrgUnit org){
        String code = findOrgMemberCode(org);
        String sysId = org.getSysId();
        return findPersonIdsByRoleCode(code, sysId);
    }

    @Override
    public List<String> findOrgByMember(String personId){
        return findSourceIdByPerson(OrgConst.OrgAuthRoleEnum.OrgMember, personId);
    }

    @Override
    public String findOrgAdminCode(OrgUnit org){
        return OrgConst.OrgAuthRoleEnum.OrgAdmin.roleCode(org.getId());
    }

    @Override
    public void addOrgAdmin(OrgUnit org, List<String> personIds){
        AuthRole member = authRoleService.findAuthRoleByCode(findOrgAdminCode(org), org.getSysId());
        if(member == null){
            log.error(JsonUtils.toJson(org));
            throw new AppException("组织管理员身份不存在，增加组织管理员失败！");
        }
        List<SystemUser> systemUsers =
                systemService.findSystemUserByResIds(ResourceConst.ResourceTypeEnum.OrgPerson, personIds);
        if(CollectionUtils.isEmpty(systemUsers)){
            throw new AppException("人员信息缺失用户，增加组织管理员失败！");
        }
        //角色追加用户
        List<ResourceObject<String>> resources = createResourceObject(systemUsers);
        authRoleService.updateAuthRoleAddBindResources(member.getId(), resources);
    }

    @Override
    public void removeOrgAdmin(OrgUnit org, List<String> personIds){
        AuthRole admin = authRoleService.findAuthRoleByCode(findOrgAdminCode(org), org.getSysId());
        if(admin == null){
            log.error(JsonUtils.toJson(org));
            throw new AppException("组织管理员身份不存在，移除组织管理员失败！");
        }
        List<SystemUser> systemUsers =
                systemService.findSystemUserByResIds(ResourceConst.ResourceTypeEnum.OrgPerson, personIds);
        if(CollectionUtils.isEmpty(systemUsers)){
            throw new AppException("人员信息缺失用户，移除组织管理员失败！");
        }
        //角色移除用户
        List<ResourceObject<String>> resources = createResourceObject(systemUsers);
        authRoleService.updateAuthRoleRemoveBindResources(admin.getId(), resources);
    }

    @Override
    public boolean isOrgAdmin(OrgUnit org, String personId){
        return userIsRole(org.getSysId(), findOrgAdminCode(org), personId);
    }

    @Override
    public List<String> findOrgAdminPersonIds(OrgUnit org){
        String code = findOrgAdminCode(org);
        String sysId = org.getSysId();
        return findPersonIdsByRoleCode(code, sysId);
    }

    @Override
    public List<String> findOrgByAdmin(String personId){
        return findSourceIdByPerson(OrgConst.OrgAuthRoleEnum.OrgAdmin, personId);
    }

    @Override
    public String findOrgLegalCode(OrgUnit org){
        return OrgConst.OrgAuthRoleEnum.OrgLegal.roleCode(org.getId());
    }

    @Override
    public void updateOrgLegal(OrgUnit org, String personId){
        AuthRole legal = authRoleService.findAuthRoleByCode(findOrgLegalCode(org), org.getSysId());
        if(legal == null){
            log.error(JsonUtils.toJson(org));
            throw new AppException("组织法人身份不存在，更新组织法人失败！");
        }
        List<SystemUser> systemUsers = systemService.findSystemUserByResIds(ResourceConst.ResourceTypeEnum.OrgPerson,
                Collections.singletonList(personId));
        if(CollectionUtils.isEmpty(systemUsers)){
            throw new AppException("人员信息缺失用户，更新组织法人失败！");
        }
        //角色更新用户
        List<ResourceObject<String>> resources = createResourceObject(systemUsers);
        authRoleService.updateAuthRoleUpdateBindResources(legal.getId(), resources);
    }

    @Override
    public boolean isOrgLegal(OrgUnit org, String personId){
        return userIsRole(org.getSysId(), findOrgLegalCode(org), personId);
    }

    @Override
    public String findOrgLegalPersonId(OrgUnit org){
        String code = findOrgLegalCode(org);
        String sysId = org.getSysId();
        List<String> personIds = findPersonIdsByRoleCode(code, sysId);
        return CollectionUtils.isEmpty(personIds) ? null : personIds.get(CommonConst.INT_0);
    }

    @Override
    public List<String> findOrgByLegal(String personId){
        return findSourceIdByPerson(OrgConst.OrgAuthRoleEnum.OrgLegal, personId);
    }

    @Override
    public void createPostRoles(OrgStruct post){
        //岗位成员
        AuthRole member = new AuthRole();
        member.setName(OrgConst.OrgAuthRoleEnum.PostMember.roleName(post.getName()));
        member.setCode(OrgConst.OrgAuthRoleEnum.PostMember.roleCode(post.getId()));
        member.setSysId(post.getSysId());
        member.setState(CommonConst.StateEnum.Enable.getKey());
        ResourceObject<String> orgMembers = new ResourceObject<>();
        member.setSource(new JObject<>(new AuthRoleSource(new ArrayList<>())));
        member.setDisplay(CommonConst.BooleanEnum.False.getKey());
        member = authService.addRole(member, null);
    }

    @Override
    public void deletePostRoles(OrgStruct post){
        AuthRole postMember = authRoleService.findAuthRoleByCode(findPostMemberCode(post), post.getSysId());
        if(postMember == null){
            return;
        }
        authRoleService.deleteAuthRoleById(postMember.getId());
    }

    @Override
    public String findPostMemberCode(OrgStruct post){
        return OrgConst.OrgAuthRoleEnum.PostMember.roleCode(post.getId());
    }

    @Override
    public void addPostMember(OrgStruct post, List<String> personIds){
        AuthRole member = authRoleService.findAuthRoleByCode(findPostMemberCode(post), post.getSysId());
        if(member == null){
            log.error(JsonUtils.toJson(post));
            throw new AppException("岗位成员身份不存在，增加岗位成员失败！");
        }
        List<SystemUser> systemUsers =
                systemService.findSystemUserByResIds(ResourceConst.ResourceTypeEnum.OrgPerson, personIds);
        if(CollectionUtils.isEmpty(systemUsers)){
            throw new AppException("人员信息缺失用户，增加岗位成员失败！");
        }
        //角色追加用户
        List<ResourceObject<String>> resources = createResourceObject(systemUsers);
        authRoleService.updateAuthRoleAddBindResources(member.getId(), resources);
    }

    @Override
    public void removePostMember(OrgStruct post, List<String> personIds){
        AuthRole member = authRoleService.findAuthRoleByCode(findPostMemberCode(post), post.getSysId());
        if(member == null){
            log.error(JsonUtils.toJson(post));
            throw new AppException("岗位成员身份不存在，移除岗位成员失败！");
        }
        List<SystemUser> systemUsers =
                systemService.findSystemUserByResIds(ResourceConst.ResourceTypeEnum.OrgPerson, personIds);
        if(CollectionUtils.isEmpty(systemUsers)){
            throw new AppException("人员信息缺失用户，移除岗位成员失败！");
        }
        //角色移除用户
        List<ResourceObject<String>> resources = createResourceObject(systemUsers);
        authRoleService.updateAuthRoleRemoveBindResources(member.getId(), resources);
    }

    @Override
    public boolean isPostMember(OrgStruct post, String personId){
        return userIsRole(post.getSysId(), findPostMemberCode(post), personId);
    }

    @Override
    public List<String> findPostMemberPersonIds(OrgStruct post){
        String code = findPostMemberCode(post);
        String sysId = post.getSysId();
        return findPersonIdsByRoleCode(code, sysId);
    }

    @Override
    public void updatePersonAddPost(String personId, List<String> postIds){
        if(StringUtils.isEmpty(personId)){
            return;
        }
        if(CollectionUtils.isEmpty(postIds)){
            return;
        }
        SystemUser systemUser = findSystemUserByPersonId(personId);
        if(systemUser == null){
            return;
        }
        List<ResourceObject<String>> resourceObject = createResourceObject(Collections.singletonList(systemUser));
        List<String> roleCodes =
                postIds.stream().map(OrgConst.OrgAuthRoleEnum.PostMember::roleCode).collect(Collectors.toList());
        authRoleService.updateAuthRoleAddBindResourcesByCodes(roleCodes, resourceObject);
    }

    @Override
    public void updatePersonRemovePost(List<OrgStruct> posts, String personId){
        if(StringUtils.isEmpty(personId)){
            return;
        }
        if(CollectionUtils.isEmpty(posts)){
            return;
        }
        SystemUser systemUser = findSystemUserByPersonId(personId);
        if(systemUser == null){
            return;
        }
        List<ResourceObject<String>> resourceObject = createResourceObject(Collections.singletonList(systemUser));
        posts.forEach(e -> {
            AuthRole role = authRoleService.findAuthRoleByCode(findPostMemberCode(e), e.getSysId());
            authRoleService.updateAuthRoleRemoveBindResources(role.getId(), resourceObject);
        });
    }

    @Override
    public List<String> findPostByMember(String personId){
        return findSourceIdByPerson(OrgConst.OrgAuthRoleEnum.PostMember, personId);
    }

    @Override
    public void createDepartmentRoles(OrgStruct department){
        if(!OrgConst.OrgStructSubTypeEnum.SolidDepartment.getKey().equals(department.getSubType())){
            return;
        }
        //部门成员
        AuthRole member = new AuthRole();
        member.setName(OrgConst.OrgAuthRoleEnum.DepartmentMember.roleName(department.getName()));
        member.setCode(OrgConst.OrgAuthRoleEnum.DepartmentMember.roleCode(department.getId()));
        member.setSysId(department.getSysId());
        member.setState(CommonConst.StateEnum.Enable.getKey());
        ResourceObject<String> orgMembers = new ResourceObject<>();
        member.setSource(new JObject<>(new AuthRoleSource(new ArrayList<>())));
        member.setDisplay(CommonConst.BooleanEnum.False.getKey());
        member = authService.addRole(member, null);

        //部门领导
        AuthRole leader = new AuthRole();
        leader.setName(OrgConst.OrgAuthRoleEnum.DepartmentLeader.roleName(department.getName()));
        leader.setCode(OrgConst.OrgAuthRoleEnum.DepartmentLeader.roleCode(department.getId()));
        leader.setSysId(department.getSysId());
        leader.setState(CommonConst.StateEnum.Enable.getKey());
        ResourceObject<String> orgLeaders = new ResourceObject<>();
        leader.setSource(new JObject<>(new AuthRoleSource(new ArrayList<>())));
        leader.setDisplay(CommonConst.BooleanEnum.False.getKey());
        leader = authService.addRole(leader, null);
    }

    @Override
    public void deleteDepartmentRoles(OrgStruct department){

        AuthRole member =
                authRoleService.findAuthRoleByCode(findDepartmentMemberCode(department), department.getSysId());
        AuthRole leader =
                authRoleService.findAuthRoleByCode(findDepartmentLeaderCode(department), department.getSysId());
        List<String> roleIds =
                Stream.of(member, leader).filter(Objects::nonNull).map(AuthRole::getId).collect(Collectors.toList());
        authRoleService.deleteRoles(roleIds);
    }

    @Override
    public String findDepartmentLeaderCode(OrgStruct department){
        return OrgConst.OrgAuthRoleEnum.DepartmentLeader.roleCode(department.getId());
    }

    @Override
    public void updateDepartmentLeader(OrgStruct department, String personId){
        AuthRole leader =
                authRoleService.findAuthRoleByCode(findDepartmentLeaderCode(department), department.getSysId());
        if(leader == null){
            log.error(JsonUtils.toJson(department));
            throw new AppException("部门领导身份不存在，更新部门领导失败！");
        }
        List<SystemUser> systemUsers = systemService.findSystemUserByResIds(ResourceConst.ResourceTypeEnum.OrgPerson,
                Collections.singletonList(personId));
        if(CollectionUtils.isEmpty(systemUsers)){
            throw new AppException("人员信息缺失用户，更新部门领导失败！");
        }
        //角色更新用户
        List<ResourceObject<String>> resources = createResourceObject(systemUsers);
        authRoleService.updateAuthRoleUpdateBindResources(leader.getId(), resources);
    }

    @Override
    public boolean isDepartmentLeader(OrgStruct department, String personId){
        return userIsRole(department.getSysId(), findDepartmentLeaderCode(department), personId);
    }

    @Override
    public String findDepartmentLeaderPersonId(OrgStruct department){
        String code = findDepartmentLeaderCode(department);
        String sysId = department.getSysId();
        List<String> personIds = findPersonIdsByRoleCode(code, sysId);
        return CollectionUtils.isEmpty(personIds) ? null : personIds.get(CommonConst.INT_0);
    }

    @Override
    public List<String> findDepartmentByLeader(String personId){
        return findSourceIdByPerson(OrgConst.OrgAuthRoleEnum.DepartmentLeader, personId);
    }

    @Override
    public String findDepartmentMemberCode(OrgStruct department){
        return OrgConst.OrgAuthRoleEnum.DepartmentMember.roleCode(department.getId());
    }

    @Override
    public void addDepartmentMember(OrgStruct department, List<String> personIds){
        AuthRole member =
                authRoleService.findAuthRoleByCode(findDepartmentMemberCode(department), department.getSysId());
        if(member == null){
            log.error(JsonUtils.toJson(department));
            throw new AppException("组织成员身份不存在，增加部门成员失败！");
        }
        List<SystemUser> systemUsers =
                systemService.findSystemUserByResIds(ResourceConst.ResourceTypeEnum.OrgPerson, personIds);
        if(CollectionUtils.isEmpty(systemUsers)){
            throw new AppException("人员信息缺失用户，增加部门成员失败！");
        }
        //角色追加用户
        List<ResourceObject<String>> resources = createResourceObject(systemUsers);
        authRoleService.updateAuthRoleAddBindResources(member.getId(), resources);
    }

    @Override
    public void removeDepartmentMember(OrgStruct department, List<String> personIds){
        AuthRole member =
                authRoleService.findAuthRoleByCode(findDepartmentMemberCode(department), department.getSysId());
        if(member == null){
            log.error(JsonUtils.toJson(department));
            throw new AppException("组织成员身份不存在，移除部门成员失败！");
        }
        List<SystemUser> systemUsers =
                systemService.findSystemUserByResIds(ResourceConst.ResourceTypeEnum.OrgPerson, personIds);
        if(CollectionUtils.isEmpty(systemUsers)){
            throw new AppException("人员信息缺失用户，移除部门成员失败！");
        }
        //角色移除用户
        List<ResourceObject<String>> resources = createResourceObject(systemUsers);
        authRoleService.updateAuthRoleRemoveBindResources(member.getId(), resources);
    }

    @Override
    public boolean isDepartmentMember(OrgStruct department, String personId){
        return userIsRole(department.getSysId(), findDepartmentMemberCode(department), personId);
    }

    @Override
    public List<String> findDepartmentMemberPersonIds(OrgStruct department){
        String code = findDepartmentMemberCode(department);
        String sysId = department.getSysId();
        return findPersonIdsByRoleCode(code, sysId);
    }

    @Override
    public void updatePersonAddDepartment(String personId, List<String> departmentIds){
        if(StringUtils.isEmpty(personId)){
            return;
        }
        if(CollectionUtils.isEmpty(departmentIds)){
            return;
        }
        SystemUser systemUser = findSystemUserByPersonId(personId);
        if(systemUser == null){
            return;
        }
        List<ResourceObject<String>> resourceObject = createResourceObject(Collections.singletonList(systemUser));
        List<String> roleCodes = departmentIds.stream().map(OrgConst.OrgAuthRoleEnum.DepartmentMember::roleCode)
                .collect(Collectors.toList());
        authRoleService.updateAuthRoleAddBindResourcesByCodes(roleCodes, resourceObject);
    }

    @Override
    public void updatePersonRemoveDepartment(List<OrgStruct> departments, String personId){
        if(StringUtils.isEmpty(personId)){
            return;
        }
        if(CollectionUtils.isEmpty(departments)){
            return;
        }
        SystemUser systemUser = findSystemUserByPersonId(personId);
        if(systemUser == null){
            return;
        }
        List<ResourceObject<String>> resourceObject = createResourceObject(Collections.singletonList(systemUser));
        departments.forEach(e -> {
            AuthRole role = authRoleService.findAuthRoleByCode(findDepartmentMemberCode(e), e.getSysId());
            authRoleService.updateAuthRoleRemoveBindResources(role.getId(), resourceObject);
        });
    }

    @Override
    public List<String> findDepartmentByMember(String personId){
        return findSourceIdByPerson(OrgConst.OrgAuthRoleEnum.DepartmentMember, personId);
    }

    /**
     * 根据角色类型，查找人员对应类型的源(组织、部门、岗位)数据Id
     *
     * @param orgAuthRoleEnum
     * @param personId
     * @return
     */
    private List<String> findSourceIdByPerson(OrgConst.OrgAuthRoleEnum orgAuthRoleEnum, String personId){
        if(orgAuthRoleEnum == null || StringUtils.isEmpty(personId)){
            return new ArrayList<>();
        }
        SystemUser systemUser = findSystemUserByPersonId(personId);
        if(systemUser == null){
            return new ArrayList<>();
        }
        List<AuthRole> roles =
                authRoleService.findBySource(ResourceConst.ResourceTypeEnum.SystemUser, systemUser.getId(), null);
        return roles.stream().filter(e -> orgAuthRoleEnum.isOrgRoleByCode(e.getCode()))
                .map(e -> orgAuthRoleEnum.splitIdByCode(e.getCode())).collect(Collectors.toList());
    }

    /**
     * 判断人员是否拥有某个角色
     *
     * @param sysId
     * @param roleCode
     * @param personId
     * @return
     */
    private boolean userIsRole(String sysId, String roleCode, String personId){
        SystemUser systemUser = findSystemUserByPersonId(personId);
        if(systemUser == null){
            return false;
        }
        AuthRole role = authRoleService.findAuthRoleByCode(roleCode, sysId);
        return AuthUtils.isRoleMember(role, ResourceConst.ResourceTypeEnum.SystemUser, systemUser.getId());
    }

    /**
     * 根据人员id找到对应用户
     *
     * @param personId
     * @return
     */
    private SystemUser findSystemUserByPersonId(String personId){
        List<SystemUser> systemUserByResIds =
                systemService.findSystemUserByResIds(ResourceConst.ResourceTypeEnum.OrgPerson,
                        Collections.singletonList(personId));
        if(CollectionUtils.isEmpty(systemUserByResIds)){
            return null;
        }
        return systemUserByResIds.get(CommonConst.INT_0);
    }

    private List<ResourceObject<String>> createResourceObject(List<SystemUser> systemUsers){
        List<ResourceObject<String>> resources = new ArrayList<>();
        for(SystemUser e : systemUsers){
            ResourceObject<String> resourceObject = new ResourceObject<>();
            resourceObject.setResId(e.getId());
            resourceObject.setResType(ResourceConst.ResourceTypeEnum.SystemUser.getKey());
            resources.add(resourceObject);
        }
        return resources;
    }

    private List<String> findPersonIdsByRoleCode(String code, String sysId){
        AuthRole role = authRoleService.findAuthRoleByCode(code, sysId);
        if(role == null || role.getSource() == null || role.getSource().getE() == null ||
                CollectionUtils.isEmpty(role.getSource().getE().getResources())){
            return new ArrayList<>();
        }
        List<String> userIds = role.getSource().getE().getResources().stream()
                .filter(e -> ResourceConst.ResourceTypeEnum.SystemUser.getKey().equals(e.getResType()))
                .map(ResourceObject::getResId).collect(Collectors.toList());
        List<SystemUser> systemUsers = systemService.findSystemUserByIds(userIds);

        return systemUsers.stream()
                .filter(e -> ResourceConst.ResourceTypeEnum.OrgPerson.getKey().equals(e.getResType()))
                .map(SystemUser::getResId).collect(Collectors.toList());
    }

}
