package com.kingtom.kirin.app.org.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.app.org.OrgConst;
import com.kingtom.kirin.app.org.dao.OrgPersonMapper;
import com.kingtom.kirin.app.org.event.OrgPersonCreateEvent;
import com.kingtom.kirin.app.org.event.OrgPersonDeleteEvent;
import com.kingtom.kirin.app.org.event.OrgPersonUpdateEvent;
import com.kingtom.kirin.app.org.model.OrgPerson;
import com.kingtom.kirin.app.org.model.OrgPersonExample;
import com.kingtom.kirin.app.org.service.IOrgPersonService;
import com.kingtom.kirin.app.system.model.SystemUser;
import com.kingtom.kirin.app.system.model.SystemUserExt3;
import com.kingtom.kirin.app.system.model.SystemUserResAttr;
import com.kingtom.kirin.app.system.service.ISystemService;
import com.kingtom.kirin.app.system.utils.SystemUtils;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.json.JObject;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.IDUtils;
import com.kingtom.kirin.core.event.service.IEventService;
import com.kingtom.kirin.core.resource.ResourceConst;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 组织模块</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/10/14
 */
@Service
@Slf4j
public class OrgPersonServiceImpl implements IOrgPersonService{

    @Autowired
    private OrgPersonMapper mapper;

    @Autowired
    private IEventService eventService;

    @Autowired
    private ISystemService systemService;

    private static boolean retBool(Integer result){
        return null != result && result >= 1;
    }

    @Override
    public OrgPerson createPerson(OrgPerson entity){
        if(entity == null){
            throw new AppException("人员为空，新建失败！");
        }
        if(entity.getId() == null){
            entity.setId(IDUtils.newUUID());
        }
        if(StringUtils.isBlank(entity.getName())){
            throw new AppException("人员名称为空，新建失败！");
        }
        if(entity.getSex() == null){
            entity.setSex(OrgConst.OrgPersonSexEnum.Man.getKey());
        }
        if(StringUtils.isNotBlank(entity.getLicenseType())){
            OrgConst.OrgPersonLicenseTypeEnum licenseTypeEnum =
                    OrgConst.OrgPersonLicenseTypeEnum.findEnumByKey(entity.getLicenseType());
            if(licenseTypeEnum == null){
                throw new AppException("人员证照类型错误，新建失败！");
            }
        }
        if(StringUtils.isBlank(entity.getPhone())){
            throw new AppException("人员手机号为空，新建失败！");
        }
        OrgPerson dbPersonByPhone = findPersonByPhone(entity.getPhone());
        if(dbPersonByPhone != null){
            throw new AppException("人员手机号已存在，新建失败！");
        }
        if(CommonConst.StateEnum.findEnumByKey(entity.getState()) == null){
            entity.setState(CommonConst.StateEnum.Enable.getKey());
        }
        if(CommonConst.BooleanEnum.findEnumByKey(entity.getAuthState()) == null){
            entity.setState(CommonConst.BooleanEnum.True.getKey());
        }
        entity.setGmtCreate(new Date());
        entity.setGmtModified(entity.getGmtCreate());
        entity.setSysId(SystemUtils.getCurrentSystemId());
        boolean f = mapper.insertSelective(entity) == CommonConst.INT_1;
        if(!f){
            throw new AppException("人员信息落库出错，新建失败！");
        }
        OrgPerson person = findPersonById(entity.getId());
        //新建人员时，新建用户
        linkSystemUser(person);
        log.info("人员[" + person.getName() + "]新建完成");
        eventService.publishEvent(new OrgPersonCreateEvent(person));
        return person;
    }

    private SystemUser linkSystemUser(OrgPerson person){
        SystemUser systemUser = null;
        String rest = ResourceConst.ResourceTypeEnum.OrgPerson.getKey();
        // 先根据userId（借用ext1）查询
        if(StringUtils.isNotBlank(person.getExt1())) {
            systemUser = systemService.findSystemUserById(person.getExt1());
        }
        // 再根据自己的id查询
        if(systemUser == null) {
            systemUser = systemService.findSystemUserByRes(rest, person.getId());
        }
        // 再根据自己的phone查询
        if(systemUser == null) {
            systemUser = systemService.findSystemUserByPhone(person.getPhone());
        }
        if(systemUser == null) {
            SystemUser user = new SystemUser();
            user.setName(person.getName());
            user.setOtherName(person.getOtherName());
            user.setImageLogo(null);
            user.setResType(rest);
            user.setResId(person.getId());
            JObject<SystemUserResAttr> resAttr = new JObject<>(new SystemUserResAttr());
            resAttr.getE().setPhone(person.getPhone());
            resAttr.getE().setBirthday(person.getBirthday());
            resAttr.getE().setSex(OrgConst.OrgPersonSexEnum.findEnumByKey(person.getSex()).getName());
            user.setResAttr(resAttr);
            JObject<SystemUserExt3> j = new JObject<>(new SystemUserExt3());
            j.getE().setUsername(person.getPhone());
            user.setExt3(j);
            systemUser = systemService.createSystemUser(user);
            person.setExt1(systemUser.getId());
            mapper.updateByPrimaryKey(person);
        } else {
            systemUser.setResType(rest);
            systemUser.setResId(person.getId());
            systemUser = systemService.updateSystemUser(systemUser);
        }
        return systemUser;
    }

    @Override
    public boolean deletePersonById(String id){
        OrgPerson dbPerson = findPersonById(id);
        //删除用户
        deleteSystemUserByPeople(Collections.singletonList(dbPerson));
        OrgPerson person = new OrgPerson();
        person.setId(id);
        person.setState(CommonConst.StateEnum.Deleted.getKey());
        person.setGmtModified(new Date());
        boolean f = mapper.updateByPrimaryKeySelective(person) == 1;
        if(f){
            log.info("人员[" + person.getName() + "]删除完成");
            eventService.publishEvent(new OrgPersonDeleteEvent(dbPerson));
        }
        return f;
    }

    @Override
    public boolean deletePerson(OrgPersonExample example){
        List<OrgPerson> people = findPersonList(example);
        //删除用户
        deleteSystemUserByPeople(people);
        OrgPerson person = new OrgPerson();
        person.setState(CommonConst.StateEnum.Deleted.getKey());
        person.setGmtModified(new Date());
        boolean f = mapper.updateByExampleSelective(person, example) == people.size();
        if(f){
            for(OrgPerson orgPerson : people){
                log.info("人员[" + orgPerson.getName() + "]删除完成");
                eventService.publishEvent(new OrgPersonDeleteEvent(orgPerson));
            }
        }
        return f;
    }

    @Override
    public boolean deletePersonByIds(List<String> idList){
        List<OrgPerson> people = findPersonListByIds(idList);
        //删除用户
        deleteSystemUserByPeople(people);
        OrgPerson person = new OrgPerson();
        person.setState(CommonConst.StateEnum.Deleted.getKey());
        person.setGmtModified(new Date());

        OrgPersonExample example = new OrgPersonExample();
        example.createCriteria().andIdIn(idList);
        boolean f = mapper.updateByExampleSelective(person, example) == people.size();
        if(f){
            for(OrgPerson orgPerson : people){
                log.info("人员[" + orgPerson.getName() + "]删除完成");
                eventService.publishEvent(new OrgPersonDeleteEvent(orgPerson));
            }
        }
        return f;
    }

    @Override
    public OrgPerson updatePersonById(OrgPerson entity){

        if(entity == null){
            throw new AppException("人员为空，更新失败！");
        }
        OrgPerson db = findPersonById(entity.getId());
        if(db == null){
            throw new AppException("人员不存在，更新失败！");
        }
        if(StringUtils.isBlank(entity.getId())){
            throw new AppException("人员id为空，更新失败！");
        }
        if(StringUtils.isBlank(entity.getName())){
            entity.setName(null);
        }
        if(OrgConst.OrgPersonSexEnum.findEnumByKey(entity.getSex()) == null){
            entity.setSex(null);
        }
        if(StringUtils.isNotBlank(entity.getLicenseType())){
            OrgConst.OrgPersonLicenseTypeEnum licenseTypeEnum =
                    OrgConst.OrgPersonLicenseTypeEnum.findEnumByKey(entity.getLicenseType());
            if(licenseTypeEnum == null){
                throw new AppException("人员证照类型错误，更新失败！");
            }
        }
        if(StringUtils.isBlank(entity.getPhone())){
            throw new AppException("人员手机号为空，更新失败！");
        }
        OrgPerson dbPersonByPhone = findPersonByPhone(entity.getPhone());
        if(dbPersonByPhone != null && !dbPersonByPhone.getId().equals(entity.getId())){
            throw new AppException("人员手机号已存在，更新失败！");
        }

        if(entity.getState() != null && CommonConst.StateEnum.findEnumByKey(entity.getState()) == null){
            entity.setState(CommonConst.StateEnum.Enable.getKey());
        }
        if(entity.getAuthState() != null && CommonConst.BooleanEnum.findEnumByKey(entity.getAuthState()) == null){
            entity.setState(CommonConst.BooleanEnum.True.getKey());
        }
        entity.setGmtModified(new Date());
        boolean f = mapper.updateByPrimaryKeySelective(entity) == 1;
        if(!f){
            throw new AppException("人员信息落库出错，更新失败！");
        }
        OrgPerson person = findPersonById(entity.getId());
        log.info("人员[" + person.getName() + "]更新完成");
        eventService.publishEvent(new OrgPersonUpdateEvent(db, person));
        return person;
    }

    @Override
    public List<OrgPerson> updatePerson(OrgPerson entity, OrgPersonExample example){
        List<OrgPerson> dbPeople = findPersonList(example);
        if(StringUtils.isNotEmpty(entity.getName()) && StringUtils.isBlank(entity.getName())){
            throw new AppException("人员名称错误，批量更新人员失败！");
        }
        if(StringUtils.isNotEmpty(entity.getPhone())){
            throw new AppException("不支持批量更新人员手机号！");
        }
        if(entity.getSex() != null && OrgConst.OrgPersonSexEnum.findEnumByKey(entity.getSex()) == null){
            throw new AppException("人员性别错误，批量更新人员失败！");
        }
        if(entity.getState() != null && CommonConst.StateEnum.findEnumByKey(entity.getState()) == null){
            throw new AppException("人员状态值错误，批量更新人员失败！");
        }
        if(entity.getAuthState() != null && CommonConst.StateEnum.findEnumByKey(entity.getAuthState()) == null){
            throw new AppException("人员状态值错误，批量更新人员失败！");
        }
        entity.setGmtModified(new Date());
        boolean f = mapper.updateByExampleSelective(entity, example) == dbPeople.size();
        if(f){
            for(OrgPerson dbPerson : dbPeople){
                log.info("人员[" + dbPerson.getName() + "]更新完成");
                OrgPerson person = findPersonById(dbPerson.getId());
                eventService.publishEvent(new OrgPersonUpdateEvent(dbPerson, person));
            }
        }
        return findPersonList(example);
    }

    @Override
    public OrgPerson findPersonById(String id){
        if(StringUtils.isEmpty(id)){
            return null;
        }
        OrgPerson person = mapper.selectByPrimaryKey(id);
        //屏蔽掉 查询到被删除的人员
        if(person != null && CommonConst.StateEnum.Deleted.getKey().equals(person.getState())){
            return null;
        }
        return person;
    }

    @Override
    public OrgPerson findPersonByPhone(String phone){
        OrgPersonExample example = new OrgPersonExample();
        example.createCriteria().andPhoneEqualTo(phone);
        excludeDeleteData(example);
        return findPersonOne(example, true);
    }

    @Override
    public List<OrgPerson> findPersonListByIds(List<String> idList){
        OrgPersonExample example = new OrgPersonExample();
        example.createCriteria().andIdIn(idList);
        excludeDeleteData(example);
        return mapper.selectByExample(example);
    }

    @Override
    public OrgPerson findPersonOne(OrgPersonExample example, boolean throwEx){
        excludeDeleteData(example);
        List<OrgPerson> list = mapper.selectByExample(example);
        if(CollectionUtils.isEmpty(list)){
            return null;
        }
        if(throwEx && list.size() > CommonConst.INT_1){
            throw new AppException("查询到多条数据!");
        }
        return list.get(CommonConst.INT_0);
    }

    @Override
    public long findPersonCount(OrgPersonExample example){
        excludeDeleteData(example);
        return mapper.countByExample(example);
    }

    @Override
    public List<OrgPerson> findPersonList(OrgPersonExample example){
        excludeDeleteData(example);
        return mapper.selectByExample(example);
    }

    @Override
    public PageInfo<OrgPerson> findPersonPage(OrgPersonExample example, int pageNum, int pageSize){
        excludeDeleteData(example);
        PageHelper.startPage(pageNum, pageSize);
        List<OrgPerson> list = mapper.selectByExample(example);
        return new PageInfo<>(list);
    }

    @Override
    public PageInfo<OrgPerson> findPersonPageByStruct(String code, String search, String name, String phone,
            String licenseCode, int pageNum, int pageSize){
        if(StringUtils.isEmpty(code)){
            PageInfo<OrgPerson> pageInfo = new PageInfo<>();
            pageInfo.setPageSize(pageSize);
            pageInfo.setPageNum(pageNum);
            pageInfo.setPages(0);
            pageInfo.setTotal(0);
            return pageInfo;
        }
        PageHelper.startPage(pageNum, pageSize);
        List<OrgPerson> list = mapper.selectPageRecordByRoleCode(code, search, name, phone, licenseCode);
        return new PageInfo<>(list);
    }

    /**
     * 排除掉被删除的数据
     *
     * @param example
     */
    private static void excludeDeleteData(OrgPersonExample example){
        example.getOredCriteria().forEach(e -> e.andStateNotEqualTo(CommonConst.StateEnum.Deleted.getKey()));
    }

    private void deleteSystemUserByPeople(List<OrgPerson> people){
        List<String> personIds = people.stream().map(OrgPerson::getId).collect(Collectors.toList());
        List<SystemUser> systemUsers =
                systemService.findSystemUserByResIds(ResourceConst.ResourceTypeEnum.OrgPerson, personIds);
        List<String> sysUserIds = systemUsers.stream().map(SystemUser::getId).collect(Collectors.toList());
        systemService.deleteSystemUsers(sysUserIds);
    }

}
