package com.kingtom.kirin.app.org.service.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import cn.hutool.core.util.StrUtil;
import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.app.org.OrgConst;
import com.kingtom.kirin.app.org.bo.OrgPersonBO;
import com.kingtom.kirin.app.org.bo.OrgUnitBO;
import com.kingtom.kirin.app.org.model.*;
import com.kingtom.kirin.app.org.service.IOrgAuthRoleService;
import com.kingtom.kirin.app.org.service.IOrgService;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.json.JObject;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.IDUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 组织模块</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/10/14
 */
@Service
public class OrgServiceImpl implements IOrgService{

    @Autowired
    private OrgPersonServiceImpl orgPersonService;

    @Autowired
    private OrgUnitServiceImpl orgUnitService;

    @Autowired
    private OrgStructServiceImpl orgStructService;

    @Autowired
    private IOrgAuthRoleService orgAuthRoleService;

    @Override
    public OrgUnitBO findOrgUnit(String orgId){
        if(StringUtils.isEmpty(orgId)){
            return null;
        }
        OrgUnit org = findUnitById(orgId);
        if(org == null){
            return null;
        }
        OrgUnitBO bo = OrgUnitBO.builder().org(org).build();
        List<String> personIds = orgAuthRoleService.findOrgAdminPersonIds(org);
        if(CollectionUtils.isNotEmpty(personIds)){
            bo.setAdmins(findPersonListByIds(personIds));
        }
        if(StringUtils.isNotEmpty(org.getLegalPerson())){
            bo.setLegal(findPersonById(org.getLegalPerson()));
        }
        return bo;
    }

    @Override
    public OrgUnitBO createUnit(OrgUnit unit, List<String> adminIds){
        unit = createUnit(unit);
        //设置组织管理员
        if(CollectionUtils.isNotEmpty(adminIds)){
            List<String> orgMemberPersonIds = orgAuthRoleService.findOrgMemberPersonIds(unit);
            List<List<String>> list = CollectionUtils.analyzeCollection(orgMemberPersonIds, adminIds, null);
            if(CollectionUtils.isNotEmpty(list.get(2))){
                orgAuthRoleService.addOrgMember(unit, list.get(2));
            }
            orgAuthRoleService.addOrgAdmin(unit, adminIds);
        }
        //设置组织法人
        if(StringUtils.isNotBlank(unit.getLegalPerson())){
            boolean isOrgMember = orgAuthRoleService.isOrgMember(unit, unit.getLegalPerson());
            if(!isOrgMember){
                orgAuthRoleService.addOrgMember(unit, Collections.singletonList(unit.getLegalPerson()));
            }
            orgAuthRoleService.updateOrgLegal(unit, unit.getLegalPerson());
        }
        return findOrgUnit(unit.getId());
    }

    @Override
    public OrgUnitBO updateUnit(OrgUnit unit, List<String> adminIds){
        unit = updateUnitById(unit);
        //设置组织管理员
        if(CollectionUtils.isNotEmpty(adminIds)){
            List<String> orgMemberPersonIds = orgAuthRoleService.findOrgMemberPersonIds(unit);
            List<List<String>> list = CollectionUtils.analyzeCollection(orgMemberPersonIds, adminIds, null);
            if(CollectionUtils.isNotEmpty(list.get(2))){
                orgAuthRoleService.addOrgMember(unit, list.get(2));
            }
            List<String> orgAdminPersonIds = orgAuthRoleService.findOrgAdminPersonIds(unit);
            List<List<String>> list1 = CollectionUtils.analyzeCollection(orgAdminPersonIds, adminIds, null);
            if(CollectionUtils.isNotEmpty(list1.get(1))){
                orgAuthRoleService.removeOrgAdmin(unit, list1.get(1));
            }
            if(CollectionUtils.isNotEmpty(list1.get(2))){
                orgAuthRoleService.addOrgAdmin(unit, list1.get(2));
            }
        }
        //设置组织法人
        if(StringUtils.isNotBlank(unit.getLegalPerson())){
            boolean isOrgMember = orgAuthRoleService.isOrgMember(unit, unit.getLegalPerson());
            if(!isOrgMember){
                orgAuthRoleService.addOrgMember(unit, Collections.singletonList(unit.getLegalPerson()));
            }
            orgAuthRoleService.updateOrgLegal(unit, unit.getLegalPerson());
        }
        return findOrgUnit(unit.getId());
    }

    @Override
    public OrgPersonBO createPerson(String orgId, List<String> departmentIds, List<String> postIds, OrgPerson person){
        person = createPerson(person);
        String personId = person.getId();
        // 人员加入组织
        if(StringUtils.isNotEmpty(orgId)){
            OrgUnit orgUnit = orgUnitService.findUnitById(orgId);
            boolean isOrgMember = orgAuthRoleService.isOrgMember(orgUnit, personId);
            if(!isOrgMember){
                orgAuthRoleService.addOrgMember(orgUnit, Collections.singletonList(personId));
            }
        }
        //人员加入部门
        List<OrgStruct> departments = orgStructService.findStructListByIds(departmentIds);
        if(CollectionUtils.isNotEmpty(departments)){
            departmentIds = departments.stream().map(OrgStruct::getId).collect(Collectors.toList());
            orgAuthRoleService.updatePersonAddDepartment(personId, departmentIds);
        }
        // 人员加入岗位
        List<OrgStruct> posts = orgStructService.findStructListByIds(postIds);
        if(CollectionUtils.isNotEmpty(posts)){
            postIds = posts.stream().map(OrgStruct::getId).collect(Collectors.toList());
            orgAuthRoleService.updatePersonAddPost(personId, postIds);
        }
        return OrgPersonBO.builder().person(person).departments(departments).posts(posts).build();
    }

    @Override
    public OrgPersonBO updatePerson(String orgId, List<String> departmentIds, List<String> postIds, OrgPerson person){
        person = updatePersonById(person);
        String personId = person.getId();
        // 人员加入组织
        if(StringUtils.isNotEmpty(orgId)){
            OrgUnit orgUnit = orgUnitService.findUnitById(orgId);
            boolean isOrgMember = orgAuthRoleService.isOrgMember(orgUnit, personId);
            if(!isOrgMember){
                orgAuthRoleService.addOrgMember(orgUnit, Collections.singletonList(personId));
            }
        }
        //人员 部门 覆盖更新
        List<OrgStruct> departments = orgStructService.findStructListByIds(departmentIds);
        List<String> dbDepartmentIds = orgAuthRoleService.findDepartmentByMember(personId);
        //过滤保留当前组织内的部门
        List<OrgStruct> dbDepartment = orgStructService.findStructListByIds(dbDepartmentIds);
        dbDepartmentIds = dbDepartment.stream().filter(e -> orgId.equals(e.getOrgId())).map(OrgStruct::getId)
                .collect(Collectors.toList());
        List<List<String>> list = CollectionUtils.analyzeCollection(departmentIds, dbDepartmentIds, String::equals);
        orgAuthRoleService.updatePersonAddDepartment(personId, list.get(CommonConst.INT_1));
        if(CollectionUtils.isNotEmpty(list.get(CommonConst.INT_2))){
            List<OrgStruct> deleteDepartment =
                    dbDepartment.stream().filter(e -> list.get(CommonConst.INT_2).contains(e.getId()))
                            .collect(Collectors.toList());
            orgAuthRoleService.updatePersonRemoveDepartment(deleteDepartment, personId);
        }


        // 人员 岗位 覆盖更新
        List<OrgStruct> posts = orgStructService.findStructListByIds(postIds);
        List<String> dbPostIds = orgAuthRoleService.findPostByMember(personId);
        //过滤保留当前组织内的岗位
        List<OrgStruct> dbPost = orgStructService.findStructListByIds(dbPostIds);
        dbPostIds = dbPost.stream().filter(e -> orgId.equals(e.getOrgId())).map(OrgStruct::getId)
                .collect(Collectors.toList());
        List<List<String>> list1 = CollectionUtils.analyzeCollection(postIds, dbPostIds, String::equals);
        orgAuthRoleService.updatePersonAddPost(personId, list1.get(CommonConst.INT_1));
        if(CollectionUtils.isNotEmpty(list1.get(CommonConst.INT_2))){
            List<OrgStruct> deletePosts = dbPost.stream().filter(e -> list1.get(CommonConst.INT_2).contains(e.getId()))
                    .collect(Collectors.toList());
            orgAuthRoleService.updatePersonRemovePost(deletePosts, personId);
        }

        return OrgPersonBO.builder().person(person).departments(departments).posts(posts).build();
    }

    @Override
    public void deletePerson(OrgPerson person){
        //todo 暂不实现删除人员
    }

    @Override
    public OrgPersonBO findPerson(String personId, String orgId){
        OrgPerson person = findPersonById(personId);
        if(person == null){
            return new OrgPersonBO();
        }
        List<String> departmentIds = orgAuthRoleService.findDepartmentByMember(personId);
        List<OrgStruct> departments = orgStructService.findStructListByIds(departmentIds);
        List<String> postIds = orgAuthRoleService.findPostByMember(personId);
        List<OrgStruct> posts = orgStructService.findStructListByIds(postIds);
        if(StringUtils.isNotEmpty(orgId)){
            departments = departments.stream().filter(e -> orgId.equals(e.getOrgId())).collect(Collectors.toList());
            posts = posts.stream().filter(e -> orgId.equals(e.getOrgId())).collect(Collectors.toList());
        }
        return OrgPersonBO.builder().person(person).departments(departments).posts(posts).build();
    }

    @Override
    public PageInfo<OrgPerson> findPersonPage(String structId, String search, String name, String phone,
            String licenseCode, int pageNum, int pageSize){

        String code = null;
        OrgStruct struct = orgStructService.findStructById(structId);
        if(struct != null){
            OrgConst.OrgStructSubTypeEnum subTypeEnum =
                    OrgConst.OrgStructSubTypeEnum.findEnumByKey(struct.getSubType());
            code = switch(subTypeEnum){
                case SolidDepartment -> orgAuthRoleService.findDepartmentMemberCode(struct);
                case SolidPost -> orgAuthRoleService.findPostMemberCode(struct);
                case RootDepartment, RootInnerOrg, RootPost -> {
                    OrgUnit org = orgUnitService.findUnitById(struct.getOrgId());
                    yield orgAuthRoleService.findOrgMemberCode(org);
                }
            };
        }
        return orgPersonService.findPersonPageByStruct(code, search, name, phone, licenseCode, pageNum, pageSize);
    }

    @Override
    public List<OrgStruct> findDepartmentTree(String orgId){
        List<OrgStruct> posts = findStructByOrgId(OrgConst.OrgStructSubTypeEnum.RootDepartment, orgId);
        if(CollectionUtils.isEmpty(posts)){
            return new ArrayList<>();
        }
        List<OrgStruct> orgStructs = findStructByOrgId(OrgConst.OrgStructSubTypeEnum.SolidDepartment, orgId);
        posts.addAll(orgStructs);
        return posts;
    }

    @Override
    public List<OrgStruct> findPostTree(String orgId){
        List<OrgStruct> posts = findStructByOrgId(OrgConst.OrgStructSubTypeEnum.RootPost, orgId);
        if(CollectionUtils.isEmpty(posts)){
            return new ArrayList<>();
        }
        List<OrgStruct> orgStructs = findStructByOrgId(OrgConst.OrgStructSubTypeEnum.SolidPost, orgId);
        posts.addAll(orgStructs);
        return posts;
    }

    @Override
    public OrgPerson createPerson(OrgPerson entity){
        return orgPersonService.createPerson(entity);
    }

    @Override
    public boolean deletePersonById(String id){
        return orgPersonService.deletePersonById(id);
    }

    @Override
    public boolean deletePerson(OrgPersonExample example){
        return orgPersonService.deletePerson(example);
    }

    @Override
    public boolean deletePersonByIds(List<String> idList){
        return orgPersonService.deletePersonByIds(idList);
    }

    @Override
    public OrgPerson updatePersonById(OrgPerson entity){
        //todo 更新用户，暂不支持
        return orgPersonService.updatePersonById(entity);
    }

    @Override
    public List<OrgPerson> updatePerson(OrgPerson entity, OrgPersonExample example){
        //todo 更新用户，暂不支持
        return orgPersonService.updatePerson(entity, example);
    }

    @Override
    public OrgPerson findPersonById(String id){
        return orgPersonService.findPersonById(id);
    }

    @Override
    public OrgPerson findPersonByPhone(String phone){
        return orgPersonService.findPersonByPhone(phone);
    }

    @Override
    public List<OrgPerson> findPersonListByIds(List<String> idList){
        return orgPersonService.findPersonListByIds(idList);
    }

    @Override
    public OrgPerson findPersonOne(OrgPersonExample example, boolean throwEx){
        return orgPersonService.findPersonOne(example, throwEx);
    }

    @Override
    public long findPersonCount(OrgPersonExample example){
        return orgPersonService.findPersonCount(example);
    }

    @Override
    public List<OrgPerson> findPersonList(OrgPersonExample example){
        return orgPersonService.findPersonList(example);
    }

    @Override
    public PageInfo<OrgPerson> findPersonPage(OrgPersonExample example, int pageNum, int pageSize){
        return orgPersonService.findPersonPage(example, pageNum, pageSize);
    }

    @Override
    public PageInfo<OrgPerson> findPersonPageByStruct(String code, String search, String name, String phone,
            String licenseCode, int pageNum, int pageSize){
        return findPersonPageByStruct(code, search, name, phone, licenseCode, pageNum, pageSize);
    }

    @Override
    public OrgUnit createUnit(OrgUnit entity){
        //创建企业
        OrgUnit org = orgUnitService.createUnit(entity);
        //创建机构数据：根节点，虚拟部门、虚拟岗位
        unitCreateAfter(org);
        //创建组织相关角色
        orgAuthRoleService.createOrgRoles(org);
        return org;
    }

    @Override
    public boolean deleteUnitById(String id){
        OrgUnit unit = orgUnitService.findUnitById(id);
        unitDeleteBefore(Collections.singletonList(unit));
        return orgUnitService.deleteUnitById(id);
    }

    @Override
    public boolean deleteUnit(OrgUnitExample example){
        List<OrgUnit> units = orgUnitService.findUnitList(example);
        unitDeleteBefore(units);
        return orgUnitService.deleteUnit(example);
    }

    @Override
    public boolean deleteUnitByIds(List<String> idList){
        List<OrgUnit> units = orgUnitService.findUnitListByIds(idList);
        unitDeleteBefore(units);
        return orgUnitService.deleteUnitByIds(idList);
    }


    @Override
    public OrgUnit updateUnitById(OrgUnit entity){
        OrgUnit orgUnit = orgUnitService.updateUnitById(entity);
        updateUnitBefore(orgUnit);
        return orgUnit;
    }

    @Override
    public List<OrgUnit> updateUnit(OrgUnit entity, OrgUnitExample example){
        List<OrgUnit> r = orgUnitService.updateUnit(entity, example);
        r.forEach(this::updateUnitBefore);
        return r;
    }

    @Override
    public OrgUnit findUnitById(String id){
        return orgUnitService.findUnitById(id);
    }

    @Override
    public OrgUnit findUnitByUsci(String phone){
        return orgUnitService.findUnitByUsci(phone);
    }

    @Override
    public List<OrgUnit> findUnitListByIds(List<String> idList){
        return orgUnitService.findUnitListByIds(idList);
    }

    @Override
    public OrgUnit findUnitOne(OrgUnitExample example, boolean throwEx){
        return orgUnitService.findUnitOne(example, throwEx);
    }

    @Override
    public long findUnitCount(OrgUnitExample example){
        return orgUnitService.findUnitCount(example);
    }

    @Override
    public List<OrgUnit> findUnitList(OrgUnitExample example){
        return orgUnitService.findUnitList(example);
    }

    @Override
    public PageInfo<OrgUnit> findUnitPage(OrgUnitExample example, int pageNum, int pageSize){
        return orgUnitService.findUnitPage(example, pageNum, pageSize);
    }

    @Override
    public PageInfo<OrgUnit> findOrgUnitPage(String query, String name, String usci, Integer pageNum, Integer pageSize){
        return orgUnitService.findOrgUnitPage(query, name, usci, pageNum, pageSize);
    }

    @Override
    public OrgStruct createStruct(OrgStruct entity){
        OrgConst.OrgStructSubTypeEnum subTypeEnum = OrgConst.OrgStructSubTypeEnum.findEnumByKey(entity.getSubType());
        entity = switch(subTypeEnum){
            case RootInnerOrg -> {
                entity.setId(IDUtils.newUUID());
                entity.setCode(OrgConst.OrgStructSubTypeEnum.RootInnerOrg.getCodePrefix());
                entity.setParentId(String.valueOf(0));
                entity.setTopId(entity.getId());
                List<String> path = new ArrayList<>();
                path.add(entity.getId());
                entity.setPath(new JObject<>(path));
                entity.setType(OrgConst.OrgStructSubTypeEnum.RootInnerOrg.getStructTypeEnum().getKey());
                entity.setState(CommonConst.StateEnum.Enable.getKey());
                entity.setOrderNum(0);
                yield entity;
            }
            case RootPost -> {
                OrgStruct root = orgStructService.findStructByOrgId(OrgConst.OrgStructSubTypeEnum.RootInnerOrg,
                        entity.getOrgId()).get(0);
                entity.setCode(OrgConst.OrgStructSubTypeEnum.RootPost.getCodePrefix());
                entity.setType(OrgConst.OrgStructSubTypeEnum.RootPost.getStructTypeEnum().getKey());
                entity.setState(CommonConst.StateEnum.Enable.getKey());
                entity.setTopId(root.getId());
                entity.setParentId(root.getId());
                List<String> postPath = new ArrayList<>();
                postPath.add(root.getId());
                entity.setPath(new JObject<>(postPath));
                yield entity;
            }
            case RootDepartment -> {
                OrgStruct root = orgStructService.findStructByOrgId(OrgConst.OrgStructSubTypeEnum.RootInnerOrg,
                        entity.getOrgId()).get(0);
                entity.setCode(OrgConst.OrgStructSubTypeEnum.RootDepartment.getCodePrefix());
                entity.setType(OrgConst.OrgStructSubTypeEnum.RootDepartment.getStructTypeEnum().getKey());
                entity.setState(CommonConst.StateEnum.Enable.getKey());
                entity.setTopId(root.getId());
                entity.setParentId(root.getId());
                List<String> departmentPath = new ArrayList<>();
                departmentPath.add(root.getId());
                entity.setPath(new JObject<>(departmentPath));
                entity.setOrderNum(0);
                yield entity;
            }
            case SolidPost -> {
                entity.setCode(OrgConst.OrgStructSubTypeEnum.SolidPost.getCodePrefix() + entity.getCode());
                entity.setType(OrgConst.OrgStructSubTypeEnum.SolidPost.getStructTypeEnum().getKey());
                entity.setSubType(OrgConst.OrgStructSubTypeEnum.SolidPost.getKey());
                entity.setState(CommonConst.StateEnum.Enable.getKey());
                if(StrUtil.isBlank(entity.getParentId())){
                    List<OrgStruct> orgStructs =
                            findStructByOrgId(OrgConst.OrgStructSubTypeEnum.RootPost, entity.getOrgId());
                    entity.setParentId(CollectionUtils.findFirst(orgStructs).getId());
                }
                OrgStruct parent = orgStructService.findStructById(entity.getParentId());
                OrgConst.OrgStructSubTypeEnum parentSubTypeEnum =
                        OrgConst.OrgStructSubTypeEnum.findEnumByKey(parent.getSubType());
                switch(parentSubTypeEnum){
                    case RootInnerOrg -> {
                        List<OrgStruct> rootDepartment =
                                orgStructService.findStructByOrgId(OrgConst.OrgStructSubTypeEnum.RootPost,
                                        parent.getOrgId());
                        parent = rootDepartment.get(0);
                    }
                    case RootPost -> {
                    }
                    case RootDepartment, SolidDepartment, SolidPost -> {
                        throw new AppException("岗位父节点错误");
                    }
                }
                entity.setParentId(parent.getId());
                entity.setTopId(parent.getTopId());
                entity.setPath(new JObject<>(new ArrayList<>(parent.getPath().getE())));
                yield entity;
            }
            case SolidDepartment -> {
                entity.setCode(OrgConst.OrgStructSubTypeEnum.SolidDepartment.getCodePrefix() + entity.getCode());
                entity.setType(OrgConst.OrgStructSubTypeEnum.SolidDepartment.getStructTypeEnum().getKey());
                entity.setSubType(OrgConst.OrgStructSubTypeEnum.SolidDepartment.getKey());
                entity.setState(CommonConst.StateEnum.Enable.getKey());
                String parentId = entity.getParentId();
                if(StrUtil.isBlank(parentId)){
                    List<OrgStruct> orgStructs =
                            findStructByOrgId(OrgConst.OrgStructSubTypeEnum.RootDepartment, entity.getOrgId());
                    entity.setParentId(CollectionUtils.findFirst(orgStructs).getId());
                }
                OrgStruct parent = orgStructService.findStructById(parentId);
                OrgConst.OrgStructSubTypeEnum parentSubTypeEnum =
                        OrgConst.OrgStructSubTypeEnum.findEnumByKey(parent.getSubType());
                switch(parentSubTypeEnum){
                    case RootInnerOrg -> {
                        List<OrgStruct> rootDepartment =
                                orgStructService.findStructByOrgId(OrgConst.OrgStructSubTypeEnum.RootDepartment,
                                        parent.getOrgId());
                        parent = rootDepartment.get(0);
                    }
                    case RootDepartment, SolidDepartment -> {

                    }
                    case SolidPost, RootPost -> {
                        throw new AppException("部门父节点错误");
                    }
                }
                entity.setParentId(parent.getId());
                entity.setTopId(parent.getTopId());
                entity.setPath(new JObject<>(new ArrayList<>(parent.getPath().getE())));
                yield entity;
            }
        };

        OrgStruct struct = orgStructService.createStruct(entity);
        structCreateAfter(struct);
        return struct;
    }

    @Override
    public boolean deleteStructById(String id){
        OrgStruct struct = orgStructService.findStructById(id);
        structDeleteBefore(Collections.singletonList(struct));
        return orgStructService.deleteStructById(id);
    }

    @Override
    public boolean deleteStructAndChild(String id){
        List<OrgStruct> structs = findStructAndChild(id);
        structDeleteBefore(structs);
        return orgStructService.deleteStructAndChild(id);
    }

    @Override
    public boolean deleteStruct(OrgStructExample example){
        List<OrgStruct> structs = orgStructService.findStructList(example);
        structDeleteBefore(structs);
        return orgStructService.deleteStruct(example);
    }

    @Override
    public boolean deleteStructByIds(List<String> idList){
        List<OrgStruct> structs = orgStructService.findStructListByIds(idList);
        structDeleteBefore(structs);
        return orgStructService.deleteStructByIds(idList);
    }

    @Override
    public OrgStruct updateStructById(OrgStruct entity){
        return orgStructService.updateStructById(entity);
    }

    @Override
    public List<OrgStruct> updateStruct(OrgStruct entity, OrgStructExample example){
        return orgStructService.updateStruct(entity, example);
    }

    @Override
    public OrgStruct findStructById(String id){
        return orgStructService.findStructById(id);
    }

    @Override
    public OrgStruct findStructByCodeAndOrgId(String code, String orgId){
        return orgStructService.findStructByCodeAndOrgId(code, orgId);
    }

    @Override
    public OrgStruct findStructByFullName(String fullName, String orgId){
        return orgStructService.findStructByFullName(fullName, orgId);
    }

    @Override
    public List<OrgStruct> findStructByOrgId(OrgConst.OrgStructSubTypeEnum subTypeEnum, String orgId){
        return orgStructService.findStructByOrgId(subTypeEnum, orgId);
    }

    @Override
    public List<OrgStruct> findStructListByIds(List<String> idList){
        return orgStructService.findStructListByIds(idList);
    }

    @Override
    public OrgStruct findStructOne(OrgStructExample example, boolean throwEx){
        return orgStructService.findStructOne(example, throwEx);
    }

    @Override
    public long findStructCount(OrgStructExample example){
        return orgStructService.findStructCount(example);
    }

    @Override
    public List<OrgStruct> findStructList(OrgStructExample example){
        return orgStructService.findStructList(example);
    }

    @Override
    public PageInfo<OrgStruct> findStructPage(OrgStructExample example, int pageNum, int pageSize){
        return orgStructService.findStructPage(example, pageNum, pageSize);
    }

    @Override
    public PageInfo<OrgStruct> findOrgStructPage(String query, String name, String usci, Integer pageNum,
            Integer pageSize){
        return orgStructService.findOrgStructPage(query, name, usci, pageNum, pageSize);
    }

    @Override
    public List<OrgStruct> findStructAndChild(String id){
        return orgStructService.findStructAndChild(id);
    }


    private void unitCreateAfter(OrgUnit org){
        //组织根节点
        OrgStruct root = new OrgStruct();
        root.setName(org.getName());
        root.setOtherName(org.getOtherName());
        root.setEnName(org.getEnName());
        root.setSubType(OrgConst.OrgStructSubTypeEnum.RootInnerOrg.getKey());
        root.setOrgId(org.getId());
        root = createStruct(root);
        //虚拟部门一级节点
        OrgStruct department = new OrgStruct();
        department.setName(org.getName());
        department.setOtherName(org.getOtherName());
        department.setEnName(org.getEnName());
        department.setSubType(OrgConst.OrgStructSubTypeEnum.RootDepartment.getKey());
        department.setOrgId(org.getId());
        createStruct(department);
        //虚拟岗位一级节点
        OrgStruct post = new OrgStruct();
        post.setName(org.getName());
        post.setOtherName(org.getOtherName());
        post.setEnName(org.getEnName());
        post.setSubType(OrgConst.OrgStructSubTypeEnum.RootPost.getKey());
        post.setOrgId(org.getId());
        createStruct(post);
    }

    private void updateUnitBefore(OrgUnit org){
        //更新树的节点
        OrgStructExample example = new OrgStructExample();
        example.createCriteria().andOrgIdEqualTo(org.getId()).andSubTypeIn(
                Arrays.asList(OrgConst.OrgStructSubTypeEnum.RootInnerOrg.getKey(),
                        OrgConst.OrgStructSubTypeEnum.RootDepartment.getKey(),
                        OrgConst.OrgStructSubTypeEnum.RootPost.getKey()));
        OrgStruct update = new OrgStruct();
        update.setName(org.getName());
        update.setEnName(org.getEnName());
        update.setOtherName(org.getOtherName());
        orgStructService.updateStruct(update, example);
    }

    private void unitDeleteBefore(List<OrgUnit> units){
        if(CollectionUtils.isEmpty(units)){
            return;
        }
        for(OrgUnit unit : units){
            //删除角色
            orgAuthRoleService.deleteOrgRoles(unit);
            //删除机构
            List<OrgStruct> structByOrgId = orgStructService.findStructByOrgId(null, unit.getId());
            deleteStructByIds(structByOrgId.stream().map(OrgStruct::getId).collect(Collectors.toList()));
        }
    }

    private void structCreateAfter(OrgStruct struct){
        //机构相关角色
        if(OrgConst.OrgStructSubTypeEnum.SolidPost.getKey().equals(struct.getSubType())){
            orgAuthRoleService.createPostRoles(struct);
        } else if(OrgConst.OrgStructSubTypeEnum.SolidDepartment.getKey().equals(struct.getSubType())){
            orgAuthRoleService.createDepartmentRoles(struct);
        }
    }

    private void structDeleteBefore(List<OrgStruct> structs){
        if(CollectionUtils.isEmpty(structs)){
            return;
        }
        for(OrgStruct struct : structs){
            //机构相关角色
            if(OrgConst.OrgStructSubTypeEnum.SolidPost.getKey().equals(struct.getSubType())){
                orgAuthRoleService.deletePostRoles(struct);
            } else if(OrgConst.OrgStructSubTypeEnum.SolidDepartment.getKey().equals(struct.getSubType())){
                orgAuthRoleService.deleteDepartmentRoles(struct);
            }
        }
    }

}
