package com.kingtom.kirin.app.org.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.app.org.OrgConst;
import com.kingtom.kirin.app.org.dao.OrgStructMapper;
import com.kingtom.kirin.app.org.event.OrgStructCreateEvent;
import com.kingtom.kirin.app.org.event.OrgStructDeleteEvent;
import com.kingtom.kirin.app.org.event.OrgStructUpdateEvent;
import com.kingtom.kirin.app.org.model.OrgStruct;
import com.kingtom.kirin.app.org.model.OrgStructExample;
import com.kingtom.kirin.app.org.service.IOrgStructService;
import com.kingtom.kirin.app.org.utils.OrgUtils;
import com.kingtom.kirin.app.system.utils.SystemUtils;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.IDUtils;
import com.kingtom.kirin.core.event.service.IEventService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/10/18
 */
@Service
@Slf4j
public class OrgStructServiceImpl implements IOrgStructService{

    @Autowired
    private OrgStructMapper mapper;

    @Autowired
    private IEventService eventService;

    private static boolean retBool(Integer result){
        return null != result && result >= 1;
    }

    @Override
    public OrgStruct createStruct(OrgStruct entity){
        if(entity == null){
            throw new AppException("机构为空，新建失败！");
        }
        if(entity.getId() == null){
            entity.setId(IDUtils.newUUID());
        }
        // TODO 应该按照全名称做排重逻辑
        if(StringUtils.isBlank(entity.getName())){
            throw new AppException("机构名称为空，新建失败！");
        }
        if(OrgConst.OrgStructTypeEnum.findEnumByKey(entity.getType()) == null){
            throw new AppException("机构类型错误，新建失败！");
        }
        if(OrgConst.OrgStructSubTypeEnum.findEnumByKey(entity.getSubType()) == null){
            throw new AppException("机构类型错误，新建失败！");
        }
        if(StringUtils.isBlank(entity.getOrgId())){
            throw new AppException("机构所属组织为空，新建失败！");
        }
        //code查重;
        if(StringUtils.isBlank(entity.getCode())){
            entity.setCode(OrgUtils.findCode());
        }
        OrgStruct orgStruct = findStructByCodeAndOrgId(entity.getCode(), entity.getOrgId());
        if(orgStruct != null){
            throw new AppException("机构编码重复错误，更新失败！");
        }
        //todo 排序号，支持拖拽排序
        if(CommonConst.StateEnum.findEnumByKey(entity.getState()) == null){
            entity.setState(CommonConst.StateEnum.Enable.getKey());
        }
        entity.getPath().getE().add(entity.getId());
        entity.setGmtCreate(new Date());
        entity.setGmtModified(entity.getGmtCreate());
        entity.setSysId(SystemUtils.getCurrentSystemId());
        boolean f = mapper.insertSelective(entity) == CommonConst.INT_1;
        if(!f){
            throw new AppException("机构信息落库出错，新建失败！");
        }
        OrgStruct struct = findStructById(entity.getId());
        log.info("机构[" + struct.getName() + "]新建完成");
        eventService.publishEvent(new OrgStructCreateEvent(struct));
        return struct;
    }

    @Override
    public boolean deleteStructById(String id){
        OrgStruct dbStruct = findStructById(id);
        OrgStruct update = new OrgStruct();
        update.setId(id);
        update.setState(CommonConst.StateEnum.Deleted.getKey());
        update.setGmtModified(new Date());
        boolean f = mapper.updateByPrimaryKeySelective(update) == 1;
        if(f){
            log.info("机构[" + dbStruct.getName() + "]删除完成");
            eventService.publishEvent(new OrgStructDeleteEvent(dbStruct));
        }
        return f;
    }

    @Override
    public boolean deleteStructAndChild(String id){
        OrgStructExample example = new OrgStructExample();
        OrgStructExample.Criteria criteria = example.createCriteria();
        criteria.addCriterion("path::jsonb -> 'e' ?? '" + id + "'");
        return deleteStruct(example);
    }

    @Override
    public boolean deleteStruct(OrgStructExample example){
        List<OrgStruct> people = findStructList(example);
        //todo 待检查
        OrgStruct update = new OrgStruct();
        update.setState(CommonConst.StateEnum.Deleted.getKey());
        update.setGmtModified(new Date());
        boolean f = mapper.updateByExampleSelective(update, example) == people.size();
        if(f){
            for(OrgStruct orgStruct : people){
                log.info("机构[" + orgStruct.getName() + "]删除完成");
                eventService.publishEvent(new OrgStructDeleteEvent(orgStruct));
            }
        }
        return f;
    }

    @Override
    public boolean deleteStructByIds(List<String> idList){
        List<OrgStruct> people = findStructListByIds(idList);

        OrgStruct update = new OrgStruct();
        update.setState(CommonConst.StateEnum.Deleted.getKey());
        update.setGmtModified(new Date());
        OrgStructExample example = new OrgStructExample();
        example.createCriteria().andIdIn(idList);
        boolean f = mapper.updateByExampleSelective(update, example) == people.size();
        if(f){
            for(OrgStruct orgStruct : people){
                log.info("机构[" + orgStruct.getName() + "]删除完成");
                eventService.publishEvent(new OrgStructDeleteEvent(orgStruct));
            }
        }
        return f;
    }

    @Override
    public OrgStruct updateStructById(OrgStruct entity){

        if(entity == null){
            throw new AppException("机构为空，更新失败！");
        }
        OrgStruct db = findStructById(entity.getId());
        if(db == null){
            throw new AppException("机构不存在，更新失败！");
        }
        if(StringUtils.isBlank(entity.getId())){
            throw new AppException("机构id为空，更新失败！");
        }
        if(StringUtils.isBlank(entity.getName())){
            entity.setName(null);
        }
        if(StringUtils.isBlank(entity.getType())){
            entity.setType(null);
        }
        if(StringUtils.isNotBlank(entity.getType()) &&
                OrgConst.OrgStructTypeEnum.findEnumByKey(entity.getType()) == null){
            throw new AppException("机构类型错误，更新失败！");
        }
        if(StringUtils.isNotBlank(entity.getSubType()) &&
                OrgConst.OrgStructSubTypeEnum.findEnumByKey(entity.getSubType()) == null){
            throw new AppException("机构类型错误，更新失败！");
        }
        //code查重;
        if(StringUtils.isBlank(entity.getCode())){
            entity.setCode(null);
        } else{
            OrgStruct orgStruct = findStructByCodeAndOrgId(entity.getCode(), db.getOrgId());
            if(orgStruct != null && !orgStruct.getId().equals(entity.getId())){
                throw new AppException("机构编码重复错误，更新失败！");
            }
        }
        //todo 排序号，支持拖拽排序
        OrgStruct update = new OrgStruct();
        update.setId(entity.getId());
        update.setName(entity.getName());
        update.setEnName(entity.getEnName());
        update.setOtherName(entity.getOtherName());
        update.setCode(entity.getCode());
        update.setOrderNum(entity.getOrderNum());
        update.setGmtModified(new Date());
        update.setExt1(entity.getExt1());
        update.setExt2(entity.getExt2());
        update.setExt3(entity.getExt3());
        boolean f = mapper.updateByPrimaryKeySelective(update) == 1;
        if(!f){
            throw new AppException("机构信息落库出错，更新失败！");
        }
        OrgStruct struct = findStructById(entity.getId());
        log.info("机构[" + struct.getName() + "]更新完成");
        eventService.publishEvent(new OrgStructUpdateEvent(db, struct));
        return struct;
    }

    @Override
    public List<OrgStruct> updateStruct(OrgStruct entity, OrgStructExample example){
        List<OrgStruct> dbPeople = findStructList(example);
        entity.setGmtModified(new Date());
        boolean f = mapper.updateByExampleSelective(entity, example) == dbPeople.size();
        if(f){
            for(OrgStruct dbStruct : dbPeople){
                log.info("机构[" + dbStruct.getName() + "]更新完成");
                OrgStruct struct = findStructById(dbStruct.getId());
                eventService.publishEvent(new OrgStructUpdateEvent(dbStruct, struct));
            }
        }
        return findStructList(example);
    }

    @Override
    public OrgStruct findStructById(String id){
        if(StringUtils.isEmpty(id)){
            return null;
        }
        OrgStruct struct = mapper.selectByPrimaryKey(id);
        //屏蔽掉 查询到被删除的机构
        if(struct != null && CommonConst.StateEnum.Deleted.getKey().equals(struct.getState())){
            return null;
        }
        return struct;
    }

    @Override
    public OrgStruct findStructByCodeAndOrgId(String code, String orgId){
        if(StringUtils.isAnyEmpty(code, orgId)){
            return null;
        }
        OrgStructExample example = new OrgStructExample();
        example.createCriteria().andCodeEqualTo(code).andOrgIdEqualTo(orgId);
        excludeDeleteData(example);
        return findStructOne(example);
    }

    @Override
    public OrgStruct findStructByFullName(String fullName, String orgId) {
        // TODO 应该按照全名称做排重逻辑
        OrgStructExample example = new OrgStructExample();
        example.createCriteria().andNameEqualTo(fullName).andOrgIdEqualTo(orgId);
        excludeDeleteData(example);
        return findStructOne(example);
    }

    @Override
    public List<OrgStruct> findStructByOrgId(OrgConst.OrgStructSubTypeEnum subTypeEnum, String orgId){
        if(StringUtils.isEmpty(orgId)){
            return new ArrayList<>();
        }
        OrgStructExample example = new OrgStructExample();
        OrgStructExample.Criteria criteria = example.createCriteria();
        criteria.andOrgIdEqualTo(orgId);
        if(subTypeEnum != null){
            criteria.andSubTypeEqualTo(subTypeEnum.getKey());
        }
        excludeDeleteData(example);
        return mapper.selectByExample(example);
    }

    @Override
    public List<OrgStruct> findStructListByIds(List<String> idList){
        if(CollectionUtils.isEmpty(idList)){
            return new ArrayList<>();
        }
        OrgStructExample example = new OrgStructExample();
        example.createCriteria().andIdIn(idList);
        excludeDeleteData(example);
        return mapper.selectByExample(example);
    }

    @Override
    public OrgStruct findStructOne(OrgStructExample example, boolean throwEx){
        excludeDeleteData(example);
        List<OrgStruct> list = mapper.selectByExample(example);
        if(CollectionUtils.isEmpty(list)){
            return null;
        }
        if(throwEx && list.size() > CommonConst.INT_1){
            throw new AppException("查询到多条数据!");
        }
        return list.get(CommonConst.INT_0);
    }

    @Override
    public long findStructCount(OrgStructExample example){
        excludeDeleteData(example);
        return mapper.countByExample(example);
    }

    @Override
    public List<OrgStruct> findStructList(OrgStructExample example){
        excludeDeleteData(example);
        return mapper.selectByExample(example);
    }

    @Override
    public PageInfo<OrgStruct> findStructPage(OrgStructExample example, int pageNum, int pageSize){
        excludeDeleteData(example);
        PageHelper.startPage(pageNum, pageSize);
        List<OrgStruct> list = mapper.selectByExample(example);
        return new PageInfo<>(list);
    }

    @Override
    public PageInfo<OrgStruct> findOrgStructPage(String query, String name, String usci, Integer pageNum,
            Integer pageSize){
        OrgStructExample example = new OrgStructExample();
        if(StringUtils.isNotEmpty(query)){
            example.or().andNameLike("%" + query + "%");
            example.or().andOtherNameLike("%" + query + "%");
            example.or().andEnNameLike("%" + query + "%");
            example.or().andCodeLike("%" + query + "%");
        }
        if(StringUtils.isNotEmpty(name)){
            if(example.getOredCriteria().isEmpty()){
                example.or().andNameLike("%" + name + "%");
            } else{
                example.getOredCriteria().forEach(e -> e.andNameLike("%" + name + "%"));
            }
        }
        return findStructPage(example, pageNum, pageSize);
    }

    @Override
    public List<OrgStruct> findStructAndChild(String id){
        if(StringUtils.isEmpty(id)){
            return new ArrayList<>();
        }
        OrgStructExample example = new OrgStructExample();
        OrgStructExample.Criteria criteria = example.createCriteria();
        criteria.addCriterion("path::jsonb -> 'e' ?? '" + id + "'");
        excludeDeleteData(example);
        return mapper.selectByExample(example);
    }

    /**
     * 排除掉被删除的数据
     *
     * @param example
     */
    private static void excludeDeleteData(OrgStructExample example){
        example.getOredCriteria().forEach(e -> e.andStateNotEqualTo(CommonConst.StateEnum.Deleted.getKey()));
    }

}
