package com.kingtom.kirin.app.org.service.impl;

import java.util.Date;
import java.util.List;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.app.org.OrgConst;
import com.kingtom.kirin.app.org.dao.OrgUnitMapper;
import com.kingtom.kirin.app.org.event.OrgUnitCreateEvent;
import com.kingtom.kirin.app.org.event.OrgUnitDeleteEvent;
import com.kingtom.kirin.app.org.event.OrgUnitUpdateEvent;
import com.kingtom.kirin.app.org.model.OrgUnit;
import com.kingtom.kirin.app.org.model.OrgUnitExample;
import com.kingtom.kirin.app.org.service.IOrgUnitService;
import com.kingtom.kirin.app.system.utils.SystemUtils;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.IDUtils;
import com.kingtom.kirin.core.event.service.IEventService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 组织模块</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/10/16
 */
@Service
@Slf4j
public class OrgUnitServiceImpl implements IOrgUnitService{

    @Autowired
    private OrgUnitMapper mapper;

    @Autowired
    private IEventService eventService;

    private static boolean retBool(Integer result){
        return null != result && result >= 1;
    }

    @Override
    public OrgUnit createUnit(OrgUnit entity){
        if(entity == null){
            throw new AppException("企业为空，新建失败！");
        }
        if(entity.getId() == null){
            entity.setId(IDUtils.newUUID());
        }
        if(StringUtils.isBlank(entity.getName())){
            throw new AppException("企业名称为空，新建失败！");
        }
        if(OrgConst.OrgUnitTypeEnum.findEnumByKey(entity.getType()) == null){
            throw new AppException("企业类型错误，新建失败！");
        }
        if(StringUtils.isNotBlank(entity.getUsci())){
            OrgUnit dbUnitByUsci = findUnitByUsci(entity.getUsci());
            if(dbUnitByUsci != null){
                throw new AppException("统一社会信用代码已存在，新建失败！");
            }
        }
        if(CommonConst.StateEnum.findEnumByKey(entity.getState()) == null){
            entity.setState(CommonConst.StateEnum.Enable.getKey());
        }
        if(StringUtils.isNotEmpty(entity.getIndustry()) &&
                OrgConst.OrgUnitIndustryEnum.findEnumByKey(entity.getIndustry()) == null){
            throw new AppException("企业所属行业错误，新建失败！");
        }
        if(StringUtils.isNotEmpty(entity.getLicenseState()) &&
                OrgConst.OrgUnitLicenseStateEnum.findEnumByKey(entity.getLicenseState()) == null){
            throw new AppException("企业证书状态错误，新建失败！");
        }
        entity.setGmtCreate(new Date());
        entity.setGmtModified(entity.getGmtCreate());
        entity.setSysId(SystemUtils.getCurrentSystemId());
        boolean f = mapper.insertSelective(entity) == CommonConst.INT_1;
        if(!f){
            throw new AppException("企业信息落库出错，新建失败！");
        }
        OrgUnit unit = findUnitById(entity.getId());
        log.info("企业[" + unit.getName() + "]新建完成");
        eventService.publishEvent(new OrgUnitCreateEvent(unit));
        return unit;
    }

    @Override
    public boolean deleteUnitById(String id){
        OrgUnit dbUnit = findUnitById(id);

        OrgUnit unit = new OrgUnit();
        unit.setId(id);
        unit.setState(CommonConst.StateEnum.Deleted.getKey());
        unit.setGmtModified(new Date());
        boolean f = mapper.updateByPrimaryKeySelective(unit) == 1;
        if(f){
            log.info("企业[" + unit.getName() + "]删除完成");
            eventService.publishEvent(new OrgUnitDeleteEvent(dbUnit));
        }
        return f;
    }

    @Override
    public boolean deleteUnit(OrgUnitExample example){
        List<OrgUnit> people = findUnitList(example);

        OrgUnit unit = new OrgUnit();
        unit.setState(CommonConst.StateEnum.Deleted.getKey());
        unit.setGmtModified(new Date());
        boolean f = mapper.updateByExampleSelective(unit, example) == people.size();
        if(f){
            for(OrgUnit orgUnit : people){
                log.info("企业[" + orgUnit.getName() + "]删除完成");
                eventService.publishEvent(new OrgUnitDeleteEvent(orgUnit));
            }
        }
        return f;
    }

    @Override
    public boolean deleteUnitByIds(List<String> idList){
        List<OrgUnit> people = findUnitListByIds(idList);
        OrgUnit unit = new OrgUnit();
        unit.setState(CommonConst.StateEnum.Deleted.getKey());
        unit.setGmtModified(new Date());

        OrgUnitExample example = new OrgUnitExample();
        example.createCriteria().andIdIn(idList);
        boolean f = mapper.updateByExampleSelective(unit, example) == people.size();
        if(f){
            for(OrgUnit orgUnit : people){
                log.info("企业[" + orgUnit.getName() + "]删除完成");
                eventService.publishEvent(new OrgUnitDeleteEvent(orgUnit));
            }
        }
        return f;
    }

    @Override
    public OrgUnit updateUnitById(OrgUnit entity){

        if(entity == null){
            throw new AppException("企业为空，更新失败！");
        }
        OrgUnit db = findUnitById(entity.getId());
        if(db == null){
            throw new AppException("企业不存在，更新失败！");
        }
        if(StringUtils.isBlank(entity.getId())){
            throw new AppException("企业id为空，更新失败！");
        }
        if(StringUtils.isBlank(entity.getName())){
            entity.setName(null);
        }
        if(StringUtils.isBlank(entity.getType())){
            entity.setType(null);
        }
        if(StringUtils.isNotBlank(entity.getType()) &&
                OrgConst.OrgUnitTypeEnum.findEnumByKey(entity.getType()) == null){
            throw new AppException("企业类型错误，更新失败！");
        }

        if(StringUtils.isBlank(entity.getUsci())){
            entity.setUsci(null);
        }
        if(StringUtils.isNotBlank(entity.getUsci())){
            OrgUnit dbUnitByUsci = findUnitByUsci(entity.getUsci());
            if(dbUnitByUsci != null && !dbUnitByUsci.getId().equals(entity.getId())){
                throw new AppException("统一社会信用代码已存在，更新失败！");
            }
        }
        if(entity.getState() != null && CommonConst.StateEnum.findEnumByKey(entity.getState()) == null){
            throw new AppException("企业状态错误，更新失败！");
        }
        if(StringUtils.isEmpty(entity.getIndustry())){
            entity.setIndustry(null);
        }
        if(StringUtils.isNotEmpty(entity.getIndustry()) &&
                OrgConst.OrgUnitIndustryEnum.findEnumByKey(entity.getIndustry()) == null){
            throw new AppException("企业所属行业错误，新建更新失败失败！");
        }
        if(StringUtils.isEmpty(entity.getLicenseState())){
            entity.setLicenseState(null);
        }
        if(StringUtils.isNotEmpty(entity.getLicenseState()) &&
                OrgConst.OrgUnitLicenseStateEnum.findEnumByKey(entity.getLicenseState()) == null){
            throw new AppException("企业证书状态错误，更新失败！");
        }
        entity.setGmtModified(new Date());
        boolean f = mapper.updateByPrimaryKeySelective(entity) == 1;
        if(!f){
            throw new AppException("企业信息落库出错，更新失败！");
        }
        OrgUnit unit = findUnitById(entity.getId());
        log.info("企业[" + unit.getName() + "]更新完成");
        eventService.publishEvent(new OrgUnitUpdateEvent(db, unit));
        return unit;
    }

    @Override
    public List<OrgUnit> updateUnit(OrgUnit entity, OrgUnitExample example){
        List<OrgUnit> dbPeople = findUnitList(example);
        entity.setGmtModified(new Date());
        boolean f = mapper.updateByExampleSelective(entity, example) == dbPeople.size();
        if(f){
            for(OrgUnit dbUnit : dbPeople){
                log.info("企业[" + dbUnit.getName() + "]更新完成");
                OrgUnit unit = findUnitById(dbUnit.getId());
                eventService.publishEvent(new OrgUnitUpdateEvent(dbUnit, unit));
            }
        }
        return findUnitList(example);
    }

    @Override
    public OrgUnit findUnitById(String id){
        OrgUnit unit = mapper.selectByPrimaryKey(id);
        //屏蔽掉 查询到被删除的企业
        if(unit != null && CommonConst.StateEnum.Deleted.getKey().equals(unit.getState())){
            return null;
        }
        return unit;
    }

    @Override
    public OrgUnit findUnitByUsci(String usci){
        OrgUnitExample example = new OrgUnitExample();
        example.createCriteria().andUsciEqualTo(usci);
        excludeDeleteData(example);
        return findUnitOne(example, true);
    }

    @Override
    public List<OrgUnit> findUnitListByIds(List<String> idList){
        OrgUnitExample example = new OrgUnitExample();
        example.createCriteria().andIdIn(idList);
        excludeDeleteData(example);
        return mapper.selectByExample(example);
    }

    @Override
    public OrgUnit findUnitOne(OrgUnitExample example, boolean throwEx){
        excludeDeleteData(example);
        List<OrgUnit> list = mapper.selectByExample(example);
        if(CollectionUtils.isEmpty(list)){
            return null;
        }
        if(throwEx && list.size() > CommonConst.INT_1){
            throw new AppException("查询到多条数据!");
        }
        return list.get(CommonConst.INT_0);
    }

    @Override
    public long findUnitCount(OrgUnitExample example){
        excludeDeleteData(example);
        return mapper.countByExample(example);
    }

    @Override
    public List<OrgUnit> findUnitList(OrgUnitExample example){
        excludeDeleteData(example);
        return mapper.selectByExample(example);
    }

    @Override
    public PageInfo<OrgUnit> findUnitPage(OrgUnitExample example, int pageNum, int pageSize){
        excludeDeleteData(example);
        PageHelper.startPage(pageNum, pageSize);
        List<OrgUnit> list = mapper.selectByExample(example);
        return new PageInfo<>(list);
    }

    @Override
    public PageInfo<OrgUnit> findOrgUnitPage(String query, String name, String usci, Integer pageNum, Integer pageSize){
        OrgUnitExample example = new OrgUnitExample();
        if(StringUtils.isNotEmpty(query)){
            example.or().andNameLike("%" + query + "%");
            example.or().andOtherNameLike("%" + query + "%");
            example.or().andEnNameLike("%" + query + "%");
            example.or().andUsciLike("%" + query + "%");
            example.or().andTaxIdLike("%" + query + "%");
            example.or().andOrgCodeLike("%" + query + "%");
        }
        if(StringUtils.isNotEmpty(name)){
            if(example.getOredCriteria().isEmpty()){
                example.or().andNameLike("%" + name + "%");
            } else{
                example.getOredCriteria().forEach(e -> e.andNameLike("%" + name + "%"));
            }
        }
        if(StringUtils.isNotEmpty(usci)){
            if(example.getOredCriteria().isEmpty()){
                example.or().andUsciLike("%" + usci + "%");
            } else{
                example.getOredCriteria().forEach(e -> e.andUsciLike("%" + usci + "%"));
            }
        }
        return findUnitPage(example, pageNum, pageSize);
    }

    /**
     * 排除掉被删除的数据
     *
     * @param example
     */
    private static void excludeDeleteData(OrgUnitExample example){
        example.getOredCriteria().forEach(e -> e.andStateNotEqualTo(CommonConst.StateEnum.Deleted.getKey()));
    }

}
