package com.kingtom.kirin.app.security.base;

import java.util.List;
import java.util.stream.Collectors;

import com.kingtom.kirin.app.auth.model.AuthPermission;
import com.kingtom.kirin.app.security.service.ISecurityContainerService;
import com.kingtom.kirin.core.resource.ResourceConst;
import jakarta.annotation.PostConstruct;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 会话安全模块</p>
 * 菜单权限码的加载器</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/8/31
 */
@Component
public class AuthMenuPermissionLoader implements IPermissionLoader{

    @Autowired
    private ISecurityContainerService container;


    @PostConstruct
    public void init(){
        container.registerPermissionLoader(this);
    }

    @Override
    public ResourceConst.ResourceTypeEnum define(){
        return ResourceConst.ResourceTypeEnum.AuthMenu;
    }

    @Override
    public List<String> parsePermission(List<AuthPermission> list){
        //这里可以考虑换成动态加载（即用菜单code去拼接）
        return list.stream().map(AuthPermission::getCode).collect(Collectors.toList());
    }

}
