package com.kingtom.kirin.app.security.base;

import java.util.List;

import com.kingtom.kirin.app.security.service.ISecurityContainerService;
import com.kingtom.kirin.app.security.service.ISecurityService;
import com.kingtom.kirin.app.system.SystemConst;
import com.kingtom.kirin.app.system.model.UserSecurity;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import jakarta.annotation.PostConstruct;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.context.annotation.ScopedProxyMode;
import org.springframework.stereotype.Component;

/**
 * 会话安全模块</p>
 * 账户认证方式：密码认证令牌</p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/6 19:12
 */
@Component
@Scope(proxyMode = ScopedProxyMode.NO)
public class UserPasswordRealm implements IAuthcRealm{

    @Autowired
    private ISecurityContainerService container;

    @Autowired
    private ISecurityService securityService;

    @PostConstruct
    protected void init(){
        container.registerRealm(this);
    }

    @Override
    public SystemConst.AuthcRealmTypeEnum support(){
        return SystemConst.AuthcRealmTypeEnum.Password;
    }

    @Override
    public Authentication doAuth(Authentication custom, IAuthEncode encode){
        List<UserSecurity> userSecurities =
                securityService.findByPrincipal(support().getTypeEnums(), custom.getPrincipal());
        custom.setUsed(false);
        if(CollectionUtils.isEmpty(userSecurities)){
            return custom;
        }
        UserSecurity userSecurity = userSecurities.get(CommonConst.INT_0);
        custom.setAuthTypeEnum(SystemConst.UserSecurityTypeEnum.findEnumByKey(userSecurity.getType()));
        custom.setUserId(userSecurity.getUserId());
        SystemConst.AuthcEncodeEnum enumByKey = SystemConst.AuthcEncodeEnum.findEnumByKey(userSecurity.getEncode());
        custom.setUsed(userSecurity.getCredentials().equals(enumByKey.encode(custom.getCredentials())));
        return custom;
    }

}
