package com.kingtom.kirin.app.security.base.crypto;

import java.security.KeyPair;

import cn.hutool.core.codec.Base64;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.crypto.asymmetric.KeyType;
import cn.hutool.crypto.asymmetric.RSA;
import com.kingtom.kirin.app.security.SecurityConst;
import com.kingtom.kirin.core.common.exception.AppException;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/11/6
 */
public class RsaCrypto implements ISecurityCrypto{

    private static final KeyPair pair = SecureUtil.generateKeyPair("RSA");

    private static final RSA rsa =
            SecureUtil.rsa(Base64.encode(pair.getPrivate().getEncoded()), Base64.encode(pair.getPublic().getEncoded()));

    private volatile static RsaCrypto instance;


    private RsaCrypto(){

    }


    /**
     * 单例模式
     *
     * @return
     */
    public static RsaCrypto getInstance(){
        if(instance == null){
            synchronized(RsaCrypto.class){
                if(instance == null){
                    instance = new RsaCrypto();
                }
            }
        }
        return instance;
    }

    @Override
    public String getKey(SecurityConst.KeyTypeEnum keyTypeEnum){
        return switch(keyTypeEnum){
            case PrivateKey -> Base64.encode(pair.getPrivate().getEncoded());
            case PublicKey -> Base64.encode(pair.getPublic().getEncoded());
            case SecretKey -> throw new AppException("RSA算法不存在密钥!");
        };
    }

    @Override
    public String encode(String data, SecurityConst.KeyTypeEnum keyTypeEnum){
        return switch(keyTypeEnum){
            case PrivateKey -> rsa.encryptBase64(data, KeyType.PrivateKey);
            case PublicKey -> rsa.encryptBase64(data, KeyType.PublicKey);
            case SecretKey -> throw new AppException("RSA算法不支持密钥加密!");
        };
    }

    @Override
    public String decode(String data, SecurityConst.KeyTypeEnum keyTypeEnum){
        return switch(keyTypeEnum){
            case PrivateKey -> rsa.decryptStr(data, KeyType.PrivateKey);
            case PublicKey -> rsa.decryptStr(data, KeyType.PublicKey);
            case SecretKey -> throw new AppException("RSA算法不支持密钥解密!");
        };
    }

}
