package com.kingtom.kirin.app.security.event.listener;

import com.kingtom.kirin.app.auth.model.AuthRole;
import com.kingtom.kirin.app.auth.service.IAuthService;
import com.kingtom.kirin.app.system.SystemConst;
import com.kingtom.kirin.app.system.event.SystemUserCreateEvent;
import com.kingtom.kirin.app.system.model.SystemUser;
import com.kingtom.kirin.core.event.model.IAppListener;
import com.kingtom.kirin.core.resource.ResourceConst;
import com.kingtom.kirin.core.resource.ResourceObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 会话安全模块</p>
 * 用户创建事件监听器</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023 /8/25
 */
@Component
public class SecuritySystemUserCreateEventListener extends SecuritySystemUserListener
        implements IAppListener<SystemUserCreateEvent>{

    @Autowired
    private IAuthService authService;

    @Override
    public void onApplicationEvent(SystemUserCreateEvent event){
        SystemUser systemUser = event.getSystemUser();
        if(systemUser == null){
            return;
        }
        //创建用户
        createSystemUser(systemUser);
        //超级管理员绑定角色
        updateAuthRole(systemUser);

    }

    /**
     * 超级管理员绑定角色、普通用户绑定角色
     *
     * @param systemUser 用户
     */
    private void updateAuthRole(SystemUser systemUser){
        String sysId = systemUser.getSysId();
        ResourceObject<String> resourceObject = new ResourceObject<>();
        resourceObject.setName(systemUser.getName());
        resourceObject.setResId(systemUser.getId());
        resourceObject.setResType(ResourceConst.ResourceTypeEnum.SystemUser.getKey());
        AuthRole role = null;
        if(SystemConst.SYSTEM_USER_ADMIN_CODE.equals(systemUser.getCode())){
            role = authService.findSuperAdminRole(sysId);
        } else{
            role = authService.findUserRole(sysId);
        }
        if(role != null){
            authService.updateAuthRoleAddBindResource(role.getId(), resourceObject);
        }
    }

}
