package com.kingtom.kirin.app.security.event.listener;

import com.kingtom.kirin.app.auth.model.AuthRole;
import com.kingtom.kirin.app.auth.service.IAuthService;
import com.kingtom.kirin.app.system.SystemConst;
import com.kingtom.kirin.app.system.event.SystemUserCreateEvent;
import com.kingtom.kirin.app.system.event.SystemUserDeleteEvent;
import com.kingtom.kirin.app.system.model.SystemUser;
import com.kingtom.kirin.app.system.model.UserSecurity;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import com.kingtom.kirin.core.event.model.IAppListener;
import com.kingtom.kirin.core.resource.ResourceConst;
import com.kingtom.kirin.core.resource.ResourceObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 会话安全模块</p>
 * 用户创建事件监听器</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023 /8/25
 */
@Component
@Slf4j
public class SecuritySystemUserDeleteEventListener extends SecuritySystemUserListener
        implements IAppListener<SystemUserDeleteEvent>{

    @Autowired
    private IAuthService authService;

    @Override
    public void onApplicationEvent(SystemUserDeleteEvent event){
        SystemUser systemUser = event.getSystemUser();
        if(systemUser == null){
            return;
        }
        //删除用户凭证
        deleteSecurity(systemUser);
        //解除角色绑定
        deleteAuthRole(systemUser);

    }

    private void deleteSecurity(SystemUser systemUser){
        List<UserSecurity> securities = userSecurityService.findByUserId(null, systemUser.getId());
        for(UserSecurity s:securities) {
            userSecurityService.deleteUserSecurity(s.getId());
        }
    }

    /**
     * 删除绑定
     * @param systemUser 用户
     */
    private void deleteAuthRole(SystemUser systemUser){
        log.warn("用户删除了，本来应该解除权限绑定，但是暂时不做了，user="+ JsonUtils.toJson(systemUser));
    }

}
