package com.kingtom.kirin.app.security.event.listener;

import com.kingtom.kirin.app.system.SystemConst;
import com.kingtom.kirin.app.system.model.AppSystemProperties;
import com.kingtom.kirin.app.system.model.SystemUser;
import com.kingtom.kirin.app.system.model.SystemUserExt3;
import com.kingtom.kirin.app.system.model.UserSecurity;
import com.kingtom.kirin.app.system.service.IUserSecurityService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * 会话安全模块</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023 /9/6
 */
public class SecuritySystemUserListener{

    @Autowired
    protected AppSystemProperties appSystemProperties;

    @Autowired
    protected IUserSecurityService userSecurityService;

    protected void createSystemUser(SystemUser systemUser){

        String[] array = getPrincipalCredentials(systemUser);
        String principal = array[0];
        String credentials = array[1];
        //默认密码
        if(StringUtils.isBlank(credentials)){
            credentials = appSystemProperties.getDefaultPassword();
        }
        //用户账户
        if(StringUtils.isNoneBlank(principal, credentials)){
            UserSecurity userName = new UserSecurity();
            userName.setUserId(systemUser.getId());
            userName.setType(SystemConst.UserSecurityTypeEnum.UserName.getKey());
            userName.setPrincipal(principal);
            userName.setCredentials(SystemConst.AuthcEncodeEnum.Md5.encode(credentials));
            userName.setEncode(SystemConst.AuthcEncodeEnum.Md5.getKey());
            userSecurityService.createUserSecurity(userName);
        }
    }

    /**
     * 系统用户解析的到账号密码
     *
     * @param systemUser the system user
     * @return string [ ]
     */
    protected String[] getPrincipalCredentials(SystemUser systemUser){
        SystemUserExt3 ext3 = systemUser.getExt3() == null ? null : systemUser.getExt3().getE();
        String principal = null;
        if(ext3 != null && StringUtils.isNotBlank(ext3.getUsername())){
            principal = ext3.getUsername();
        } else{
            if(StringUtils.isBlank(systemUser.getCode())){
                principal = systemUser.getCode();
            } else if(systemUser.getResAttr() != null && systemUser.getResAttr().getE() != null){
                principal = systemUser.getResAttr().getE().getPhone();
            }
        }
        String credentials = null;
        if(ext3 != null && StringUtils.isNotBlank(ext3.getPassword())){
            credentials = ext3.getPassword();
        }
        return new String[]{principal, credentials};
    }

}
