package com.kingtom.kirin.app.security.event.listener;

import java.util.Arrays;
import java.util.List;

import com.kingtom.kirin.app.system.SystemConst;
import com.kingtom.kirin.app.system.event.SystemUserUpdateEvent;
import com.kingtom.kirin.app.system.model.SystemUser;
import com.kingtom.kirin.app.system.model.UserSecurity;
import com.kingtom.kirin.app.system.service.IUserSecurityService;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.event.model.IAppListener;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 会话安全模块</p>
 * 用户更新事件监听器</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/8/25
 */
@Component
public class SecuritySystemUserUpdateEventListener extends SecuritySystemUserListener
        implements IAppListener<SystemUserUpdateEvent>{

    @Autowired
    private IUserSecurityService userSecurityService;

    @Override
    public void onApplicationEvent(SystemUserUpdateEvent event){
        if(event.getSystemUser() == null || event.getOldSystemUser() == null){
            return;
        }
        updateSystemUser(event);

    }

    private void updateSystemUser(SystemUserUpdateEvent event){
        SystemUser oldSystemUser = event.getOldSystemUser();
        SystemUser systemUser = event.getSystemUser();
        List<UserSecurity> oldAuths =
                userSecurityService.findByUserId(Arrays.asList(SystemConst.UserSecurityTypeEnum.UserName),
                        systemUser.getId());
        UserSecurity oldAuth = CollectionUtils.isEmpty(oldAuths) ? null : oldAuths.get(CommonConst.INT_0);
        if(oldAuth == null){
            //创建账户
            createSystemUser(systemUser);
            return;
        }
        //更新账户
        String[] array = getPrincipalCredentials(systemUser);
        String principal = array[0];
        String credentials = array[1];
        if(StringUtils.isBlank(principal) || oldAuth.getPrincipal().equals(principal)){
            //不更新principal
            principal = null;
        }
        SystemConst.AuthcEncodeEnum enumByKey = SystemConst.AuthcEncodeEnum.findEnumByKey(oldAuth.getEncode());
        if(StringUtils.isBlank(credentials) || enumByKey.encode(credentials).equals(oldAuth.getCredentials())){
            credentials = null;
        }
        if(principal != null || credentials != null){
            //执行更新
            UserSecurity userSecurity = new UserSecurity();
            userSecurity.setId(oldAuth.getId());
            userSecurity.setPrincipal(principal);
            userSecurity.setCredentials(SystemConst.AuthcEncodeEnum.Md5.encode(credentials));
            userSecurity.setEncode(SystemConst.AuthcEncodeEnum.Md5.getKey());
            userSecurityService.updateUserSecurity(userSecurity);
        }

    }

}
