package com.kingtom.kirin.app.security.service;

import java.util.List;

import com.kingtom.kirin.app.auth.model.AuthPermission;
import com.kingtom.kirin.app.security.base.SessionData;
import com.kingtom.kirin.app.system.SystemConst;
import com.kingtom.kirin.app.system.model.UserSecurity;

/**
 * 会话安全模块</p>
 * 身份登录认证 接口</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023 /8/24
 */
public interface ISecurityService{

    /**
     * 登录时候就创建扩展数据
     *
     * @param token token
     * @return abstract session data
     */
    SessionData logIn(String token, String userId);

    /**
     * 指定登出时移除扩展数据
     *
     * @param token token
     * @return abstract session data
     */
    SessionData logOut(String token);

    /**
     * 根据token获取user数据
     * @param token
     * @return
     */
    SessionData findSessionData(String token);

    /**
     * 根据userId获取user数据
     * @param userId
     * @return
     */
    SessionData findSessionDataByUserId(String userId);

    /**
     * 将权限解析成权限码.
     *
     * @param permissions the permissions
     * @return the list
     */
    List<String> parsePermissions(List<AuthPermission> permissions);

    /**
     * 用户个人修改密码.
     *
     * @param oldPassword the old password
     * @param password    the password
     */
    public void updateCredentials(String userId, String oldPassword, String password);

    /**
     * Find by principal list.
     *
     * @param authTypeEnums the auth type enums
     * @param principal     the principal
     * @return the list
     */
    List<UserSecurity> findByPrincipal(List<SystemConst.UserSecurityTypeEnum> authTypeEnums, String principal);

}
