package com.kingtom.kirin.app.space.model;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.kingtom.kirin.app.space.utils.SpaceUtils;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.exception.AppException;
import lombok.Data;

import java.io.File;
import java.io.IOException;
import java.util.Date;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/13 15:54
 */
@Data
public class StorageArea {

    private String id;
    private String areaName;
    /** nginx的文件目录跟路径:/images/ */
    private String nginx;
    private File path;

    /** 方法区 **/

    public boolean isCanNginx() {
        return StrUtil.isNotBlank(nginx);
    }

    public File findFormatFile(SpaceFile file, Boolean canCreate) {
        File df = new File(getPath().getPath()+ CommonConst.FILE_SEPARATOR + findFormatPath(file.getGmtCreate()));
        if(!df.exists()) {
            df.mkdirs();
        }
        File f = new File(df.getPath()+CommonConst.FILE_SEPARATOR+findFormatName(file));
        if(!f.exists() && canCreate) {
            try{
                f.createNewFile();
            } catch (IOException e) {
                throw new AppException("文件创建失败",e).setM("file="+f.getAbsolutePath());
            }
        }
        return f.exists()?f:null;
    }

    public File findFormatFileOfDerivative(SpaceFile file, String derivativeName, Boolean canCreate) {
        File df = new File(getPath().getPath()+ CommonConst.FILE_SEPARATOR + findFormatPath(file.getGmtCreate()));
        if(!df.exists()) {
            df.mkdirs();
        }
        File f = new File(df.getPath()+CommonConst.FILE_SEPARATOR+findFormatNameOfDerivative(file, derivativeName));
        if(!f.exists() && canCreate) {
            try{
                f.createNewFile();
            } catch (IOException e) {
                throw new AppException("文件创建失败",e).setM("file="+f.getAbsolutePath());
            }
        }
        return f.exists()?f:null;
    }

    public String findFormatPath(Date date) {
        return DateUtil.format(date,"yyyy_MM");
    }

    public String findFormatName(SpaceFile file) {
        String ftype = SpaceUtils.findFileType(file);
        if(StrUtil.isNotBlank(ftype)) {
            return file.getId()+CommonConst.PERIOD+ftype;
        } else {
            return file.getId();
        }
    }

    public String findFormatNameOfDerivative(SpaceFile file, String derivativeName) {
        String ftype = SpaceUtils.findFileType(file);
        if(StrUtil.isNotBlank(ftype)) {
            return file.getId()+CommonConst.UNDER_LINE+derivativeName+CommonConst.PERIOD+ftype;
        } else {
            return file.getId()+CommonConst.UNDER_LINE+derivativeName+CommonConst.PERIOD;
        }
    }

    public String findFormatHttpUrl(SpaceFile file) {
        if(isCanNginx()) {
            String r = nginx.endsWith(CommonConst.FILE_SEPARATOR)?nginx:nginx + CommonConst.FILE_SEPARATOR;
            r = r + findFormatPath(file.getGmtCreate()) + CommonConst.FILE_SEPARATOR + findFormatName(file);
            return r;
        } else {
            return null;
        }
    }

    public String findFormatHttpUrlOfDerivative(SpaceFile file, String derivativeName) {
        if(isCanNginx()) {
            String r = nginx.endsWith(CommonConst.FILE_SEPARATOR)?nginx:nginx + CommonConst.FILE_SEPARATOR;
            r = r + findFormatPath(file.getGmtCreate()) + CommonConst.FILE_SEPARATOR + findFormatNameOfDerivative(file,derivativeName);
            return r;
        } else {
            return null;
        }
    }

}
