package com.kingtom.kirin.app.space.service;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.StrUtil;
import com.kingtom.kirin.app.space.base.IFileSource;
import com.kingtom.kirin.app.space.model.SpaceFile;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.utils.CollectionUtils;

import java.util.List;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/9 11:26
 */
public interface ISpaceService {

    /**
     * 二段提交之临时文件创建
     * @param source 资源
     * @return
     */
    default SpaceFile createSpaceFileOfTmp(IFileSource source) {
        if(source == null) {throw new AppException("源资源不能为空");}
        List<SpaceFile> fs = createSpaceFileOfTmp(ListUtil.toList(source));
        return CollectionUtils.findFirst(fs);
    }

    /**
     * 二段提交之临时文件创建
     * @param sources
     * @return
     */
    List<SpaceFile> createSpaceFileOfTmp(List<? extends IFileSource> sources);

    /**
     * 二段提交之正式提交
     * @param fileId
     * @param resType
     * @param resId
     * @param useKey
     * @return
     */
    SpaceFile doFormal(String fileId, String resType, String resId, String useKey);

    /**
     * 二段提交之正式提交
     * @param fileIds
     * @param resType
     * @param resId
     * @param useKey
     * @return
     */
    default List<SpaceFile> doFormal(List<String> fileIds, String resType, String resId, String useKey) {
        List<SpaceFile> r = ListUtil.toList();
        if(CollectionUtil.isNotEmpty(fileIds)) {
            for(String fileId:fileIds) {
                SpaceFile fr = doFormal(fileId,resType,resId,useKey);
                r.add(fr);
            }
        }
        return r;
    }

    default SpaceFile doFormalOfReplace(String fileId, String resType, String resId, String useKey) {
        List<String> fileIds = StrUtil.isNotBlank(fileId)?ListUtil.toList(fileId):null;
        List<SpaceFile> fs =  doFormalOfReplace(fileIds,resType,resId,useKey);
        return CollectionUtils.findFirst(fs);
    }

    /**
     * 二段提交之正式提交，并替换useKey对应信息
     * @param fileIds
     * @param resType
     * @param resId
     * @param useKey
     * @return
     */
    List<SpaceFile> doFormalOfReplace(List<String> fileIds, String resType, String resId, String useKey);

    /**
     * 删除文件
     * @param fileId
     * @return
     */
    Boolean deleteSpaceFile(String fileId);

    /**
     * 当资源删除时连带删除文件数据
     * @param resType
     * @param resId
     * @return
     */
    Boolean deleteSpaceFileOfRes(String resType, String resId);

    /**
     * 查找文件信息
     * @param fileId
     * @return
     */
    SpaceFile findSpaceFile(String fileId);

    /**
     * 查找文件信息
     * @param fileIds
     * @return
     */
    List<SpaceFile> findSpaceFiles(List<String> fileIds);

    /**
     * 查找临时文件
     * @return
     */
    List<SpaceFile> findSpaceFilesOfTmp();

    /**
     * 读取文件
     * @param fileId
     * @return
     */
    IFileSource findSpaceFileAsStream(String fileId);


}
