package com.kingtom.kirin.app.space.service;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ObjectUtil;
import com.kingtom.kirin.app.space.base.IFileSource;
import com.kingtom.kirin.app.space.dao.SpaceFileMapper;
import com.kingtom.kirin.app.space.model.SpaceFile;
import com.kingtom.kirin.app.space.model.SpaceFileExample;
import com.kingtom.kirin.app.space.utils.SpaceUtils;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.ObjectUtils;
import com.kingtom.kirin.core.resource.ResourceConst;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.context.annotation.ScopedProxyMode;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/9 11:26
 */
@Service
@Slf4j
@Scope(proxyMode = ScopedProxyMode.NO)
public class SpaceServiceImpl implements ISpaceService{

    @Autowired
    private SpaceFileMapper spaceFileMapper;

    @Autowired
    private StorageService storageService;

    @Override
    public List<SpaceFile> createSpaceFileOfTmp(List<? extends IFileSource> sources) {
        if(CollectionUtils.isEmpty(sources)) { return null; }
        List<SpaceFile> r = ListUtil.toList();
        for(IFileSource source : sources) {
            SpaceFile f = createSpaceFileOfState(source, CommonConst.StateEnum.Unkown);
            r.add(f);
        }
        return r;
    }

//    @Override
//    public List<SpaceFile> createSpaceFile(List<? extends IFileSource> sources) {
//        if(CollectionUtils.isEmpty(sources)) { return null; }
//        List<SpaceFile> r = ListUtil.toList();
//        for(IFileSource source : sources) {
//            SpaceFile f = createSpaceFileOfState(source,CommonConst.StateEnum.Enable);
//            r.add(f);
//        }
//        return r;
//    }

    protected SpaceFile createSpaceFileOfState(IFileSource source, CommonConst.StateEnum state) {
        if(ObjectUtils.isAnyNull(source)) {
            throw new AppException("保存的文件不能为空");
        }
        SpaceFile f = new SpaceFile();
        SpaceUtils.fillBeanBeforeInsert(f);
        f.setState(state.getKey());
        // TODO
//        f.setOwnerId((String)StpUtil.getLoginId());// 登陆者
        f.setOwnerType(CommonConst.UNKOWN);
        f.setOwnerId(CommonConst.UNKOWN);
        f.setResId(CommonConst.UNKOWN);
        f.setResType(CommonConst.UNKOWN);
        f.setUseKey(CommonConst.UNKOWN);
        f.setName(source.getFileName());
        f.setFileType(source.getContentType());
        storageService.doStorage(f, source);
        spaceFileMapper.insert(f);
        return f;
    }

    @Override
    public SpaceFile doFormal(String fileId, String resType, String resId, String useKey) {
        if(StringUtils.isBlank(fileId)) {throw new AppException("文件不能为空");}
        if(StringUtils.isBlank(fileId)) {
            throw new AppException("文件不能为空");
        }
        SpaceFile f = spaceFileMapper.selectByPrimaryKey(fileId);
        if(f == null) {
            throw new AppException("文件找不到");
        }
        if(!CommonConst.StateEnum.Enable.getKey().equals(f.getState())) {
            if(SpaceUtils.isImage(f)) {
                storageService.doImageScaleStorage(f);
            }
            // 正式保存
            f.setState(CommonConst.StateEnum.Enable.getKey());
            f.setUseKey(useKey);
            f.setResType(resType);
            f.setResId(resId);
            SpaceUtils.fillBeanBeforeUpdate(null,f);
            spaceFileMapper.updateByPrimaryKey(f);
        }
        return f;
    }

    @Override
    public List<SpaceFile> doFormal(List<String> fileIds, String resType, String resId, String useKey) {
        if(ObjectUtils.isAnyNull(resType,resId,useKey)){
            throw new AppException("指定的资源、用途类型不能为空");
        }
        if(CollectionUtils.isNotEmpty(fileIds)) {
            List<SpaceFile> r = ListUtil.toList();
            for(String fileId:fileIds) {
                if(StringUtils.isBlank(fileId)) {return null;}
                SpaceFile f = spaceFileMapper.selectByPrimaryKey(fileId);
                if(ObjectUtils.isNotNull(f)) {
                    r.add(f);
                    if(!CommonConst.StateEnum.Enable.getKey().equals(f.getState())) {
                        if(SpaceUtils.isImage(f)) {
                            storageService.doImageScaleStorage(f);
                        }
                        f.setState(CommonConst.StateEnum.Enable.getKey());
                        f.setUseKey(useKey);
                        f.setResType(resType);
                        f.setResId(resId);
                        SpaceUtils.fillBeanBeforeUpdate(null,f);
                        spaceFileMapper.updateByPrimaryKey(f);
                    }
                } else {
                    throw new AppException("文件不存在").setM(fileId);
                }
            }
            return r;
        }
        return null;
    }

    @Override
    public List<SpaceFile> doFormalOfReplace(List<String> fileIds, String resType, String resId, String useKey) {
        if(ObjectUtils.isAnyNull(resType,resId,useKey)){
            throw new AppException("指定的资源、用途类型不能为空");
        } else {
            List<SpaceFile> oldr = findSpaceFilesByRes(resType, resId, useKey);
            Map<String,SpaceFile> oldrmap = CollectionUtil.toMap(oldr, MapUtil.newHashMap(),SpaceFile::getId);
            List<SpaceFile> nowr = findSpaceFiles(fileIds);
            Map<String,SpaceFile> nowrmap = CollectionUtil.toMap(nowr, MapUtil.newHashMap(),SpaceFile::getId);
            List<List<String>> resultList = CollectionUtils.analyzeCollection(oldrmap.keySet(),nowrmap.keySet(),ObjectUtil::equal);
            // 修改的即不变的，不管
            if(CollectionUtil.isNotEmpty(resultList.get(CommonConst.INT_0))) {
            }
            // 删除的
            if(CollectionUtil.isNotEmpty(resultList.get(CommonConst.INT_1))) {
                for(String id:resultList.get(CommonConst.INT_1)) {
                    deleteSpaceFile(id);
                }
            }
            // 新增的
            if(CollectionUtil.isNotEmpty(resultList.get(CommonConst.INT_2))) {
                for(String id:resultList.get(CommonConst.INT_2)) {
                    doFormal(id,resType,resId,useKey);
                }
            }
            List<SpaceFile> r = findSpaceFilesByRes(resType, resId, useKey);
            return r;
        }
    }

    @Override
    public Boolean deleteSpaceFile(String fileId) {
        SpaceFile f = spaceFileMapper.selectByPrimaryKey(fileId);
        if(ObjectUtils.isNotNull(f)) {
            storageService.deleteStorage(f);
            spaceFileMapper.deleteByPrimaryKey(fileId);
        } else {
            log.warn("文件已经不存在，file="+fileId);
        }
        return Boolean.TRUE;
    }

    @Override
    public Boolean deleteSpaceFileOfRes(String resType, String resId) {
        if(ObjectUtils.isAnyNull(resType,resId)){
            throw new AppException("指定的资源不能为空");
        } else {
            List<SpaceFile> oldr = findSpaceFilesByRes(resType, resId);
            if(CollectionUtil.isNotEmpty(oldr)) {
                for(SpaceFile e:oldr) {
                    deleteSpaceFile(e.getId());
                }
            }
        }
        return Boolean.TRUE;
    }

    @Override
    public SpaceFile findSpaceFile(String fileId) {
        return spaceFileMapper.selectByPrimaryKey(fileId);
    }

    @Override
    public List<SpaceFile> findSpaceFiles(List<String> fileIds) {
        List<String> fids = CollectionUtils.filter(fileIds, ObjectUtil::isNotNull);
        if(CollectionUtils.isNotEmpty(fids)) {
            SpaceFileExample e = new SpaceFileExample();
            e.createCriteria().andIdIn(fids);
            return spaceFileMapper.selectByExample(e);
        } else {
            return CollectionUtil.newArrayList();
        }
    }

    @Override
    public List<SpaceFile> findSpaceFilesOfTmp() {
        SpaceFileExample e = new SpaceFileExample();
        e.createCriteria().andStateEqualTo(CommonConst.StateEnum.Unkown.getKey());
        return spaceFileMapper.selectByExample(e);
    }

    @Override
    public IFileSource findSpaceFileAsStream(String fileId) {
        SpaceFile f = spaceFileMapper.selectByPrimaryKey(fileId);
        if(ObjectUtils.isAnyNull(f)) {
            throw new AppException("文件不存在").setM(fileId);
        } else {
            return storageService.readStorage(f);
        }
    }

    protected List<SpaceFile> findSpaceFilesByRes(String type,String resId) {
        SpaceFileExample e = new SpaceFileExample();
        e.createCriteria().andResTypeEqualTo(type).andResIdEqualTo(resId);
        List<SpaceFile> rs = spaceFileMapper.selectByExample(e);
        return rs;
    }

    protected List<SpaceFile> findSpaceFilesByRes(String type,String resId, String useKey) {
        SpaceFileExample e = new SpaceFileExample();
        e.createCriteria().andResTypeEqualTo(type).andResIdEqualTo(resId).andUseKeyEqualTo(useKey);
        List<SpaceFile> rs = spaceFileMapper.selectByExample(e);
        return rs;
    }
}
