package com.kingtom.kirin.app.space.utils;

import cn.hutool.core.util.StrUtil;
import com.kingtom.kirin.app.space.SpaceConst;
import com.kingtom.kirin.app.space.model.SpaceFile;
import com.kingtom.kirin.app.system.utils.SystemUtils;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.utils.IDUtils;

import java.util.Date;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/9 14:48
 */
public class SpaceUtils {

    private SpaceUtils(){}

    public static <T> void fillBeanBeforeInsert(T b) {
        if(b != null) {
            Date now = new Date();
            if(b instanceof SpaceFile bean) {
                bean.setId(IDUtils.newUUID());
                bean.setStorageArea(SpaceConst.STORAGE_AREA_DEFAULT_ID);
                bean.setStorageEncode(SpaceConst.SpaceFileStorageEncodeEnum.No.getKey());
                bean.setGmtCreate(now);
                bean.setGmtModified(now);
                bean.setSysId(SystemUtils.getCurrentSystemId());
            }
        }
    }

    public static <T> void fillBeanBeforeUpdate(T dbB,T customB) {
        if(customB != null && dbB != null) {
            Date now = new Date();
            if(customB instanceof SpaceFile bean && dbB instanceof SpaceFile dbBean) {
                bean.setId(dbBean.getId());
                bean.setGmtCreate(dbBean.getGmtCreate());
                bean.setGmtModified(now);
                bean.setSysId(dbBean.getSysId());
            }
        } else if (customB != null) {
            Date now = new Date();
            if(customB instanceof SpaceFile bean) {
                bean.setGmtModified(now);
            }
        }
    }

    public static boolean isImage(SpaceFile file) {
        String fileType = findFileType(file);
        return "jpg".equals(fileType) ||
                "jpeg".equals(fileType) ||
                "png".equals(fileType) ||
                "gif".equals(fileType);
    }

    public static String findFileType(SpaceFile file) {
        String fname = file.getName();
        int index = fname.lastIndexOf(CommonConst.PERIOD);
        if(index >= CommonConst.INT_0) {
            return fname.substring(index+CommonConst.INT_1);
        } else {
            String fileType = file.getFileType();
            index = fileType.lastIndexOf(CommonConst.SLASH);
            if(index >= CommonConst.INT_0) {
                return fileType.substring(index+CommonConst.INT_1);
            } else {
                return fileType;
            }
        }
    }

    public static boolean isNotScaled(SpaceFile file) {
        return StrUtil.isBlank(file.getExt1());
    }

}
