package com.kingtom.kirin.app.system;

import java.util.Arrays;
import java.util.List;

import com.kingtom.kirin.app.security.base.crypto.Md5Crypto;
import com.kingtom.kirin.app.security.utils.SecurityUtils;
import com.kingtom.kirin.core.common.base.IBaseEnum;
import org.apache.commons.lang3.StringUtils;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/7 16:48
 */
public class SystemConst{

    private SystemConst(){
    }

    public static final String SYSTEM_DEFAULT_CODE = "platform";

    public static final String SYSTEM_USER_ADMIN_CODE = "admin";

    public static final String SPACE_SYSTEM_Logo = "系统logo";

    public static final String SPACE_SYSTEM_Background = "系统背景图";

    public static final String SPACE_USER_Logo = "用户logo";

    public static final String SESSION_ATTR_SYSTEM_USER = "systemUser";

    /**
     * 系统类型
     */
    public static enum SystemInfoTypeEnum implements IBaseEnum<String>{
        Plat("Plat", "平台"),
        Tenant("Tenant", "租户");

        private String key;

        private String name;

        private SystemInfoTypeEnum(String k, String n){
            this.key = k;
            this.name = n;
        }

        @Override
        public String getKey(){
            return key;
        }

        @Override
        public String getName(){
            return name;
        }

        public static SystemInfoTypeEnum findEnumByKey(String k){
            return IBaseEnum.findEnumByKey(SystemInfoTypeEnum.class, k);
        }
    }

    public enum AuthcResultEnum implements IBaseEnum<String>{

        Ok("Ok", "认证成功"),
        None("None", "账户不存在"),
        Error("Error", "凭证不匹配");

        private String key;

        private String name;

        AuthcResultEnum(String k, String n){
            this.key = k;
            this.name = n;
        }

        @Override
        public String getKey(){
            return this.key;
        }

        @Override
        public String getName(){
            return this.name;
        }

        public static UserSecurityTypeEnum findEnumByKey(String k){
            return IBaseEnum.findEnumByKey(UserSecurityTypeEnum.class, k);
        }
    }

    /**
     * 菜单动作类型枚举
     */
    public enum UserSecurityTypeEnum implements IBaseEnum<String>{

        UserName("UserName", "用户名"),
        Phone("Phone", "手机号"),
        Email("Email", "邮箱"),
        WeChat("WeChat", "微信小程序"),
        WeChatPublicNumber("WeChatPublicNumber", "微信公众号"),
        QQ("QQ", "qq"),
        Sina("Sina", "新浪");

        private String key;

        private String name;

        UserSecurityTypeEnum(String k, String n){
            this.key = k;
            this.name = n;
        }

        @Override
        public String getKey(){
            return this.key;
        }

        @Override
        public String getName(){
            return this.name;
        }

        public static UserSecurityTypeEnum findEnumByKey(String k){
            return IBaseEnum.findEnumByKey(UserSecurityTypeEnum.class, k);
        }
    }

    /**
     * 认证方式枚举
     */
    public enum AuthcRealmTypeEnum implements IBaseEnum<String>{

        Password("Password", "密码登录",
                Arrays.asList(UserSecurityTypeEnum.UserName, UserSecurityTypeEnum.Phone, UserSecurityTypeEnum.Email)),
        Code("Code", "验证码登录", Arrays.asList(UserSecurityTypeEnum.Phone, UserSecurityTypeEnum.Email)),
        WechatMini("WechatMini", "微信小程序用户登录", Arrays.asList(UserSecurityTypeEnum.WeChat));

        private String key;

        private String name;

        private List<UserSecurityTypeEnum> typeEnums;

        AuthcRealmTypeEnum(String k, String n, List<UserSecurityTypeEnum> typeEnums){
            this.key = k;
            this.name = n;
            this.typeEnums = typeEnums;
        }

        @Override
        public String getKey(){
            return this.key;
        }

        @Override
        public String getName(){
            return this.name;
        }

        public List<UserSecurityTypeEnum> getTypeEnums(){
            return typeEnums;
        }

        public static AuthcRealmTypeEnum findEnumByKey(String k){
            return IBaseEnum.findEnumByKey(AuthcRealmTypeEnum.class, k);
        }
    }

    /**
     * 认证编码类型枚举
     */
    public enum AuthcEncodeEnum implements IBaseEnum<Integer>{
        No(0, "无编码"),
        Md5(10, "MD5");

        private Integer key;

        private String name;

        AuthcEncodeEnum(Integer k, String n){
            this.key = k;
            this.name = n;
        }

        @Override
        public Integer getKey(){
            return this.key;
        }

        @Override
        public String getName(){
            return this.name;
        }

        public String encode(String data){
            if(StringUtils.isEmpty(data)){
                return null;
            }
            if(Md5.getKey().equals(this.getKey())){
                return SecurityUtils.md5Encode(data);
            }
            return data;
        }

        public static AuthcEncodeEnum findEnumByKey(Integer k){
            return IBaseEnum.findEnumByKey(AuthcEncodeEnum.class, k);
        }
    }

}
