package com.kingtom.kirin.app.system.service;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.app.system.SystemConst;
import com.kingtom.kirin.app.system.model.SystemInfo;
import com.kingtom.kirin.app.system.model.SystemUser;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.resource.ResourceConst;

import java.util.List;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/7/12 17:24
 */
public interface ISystemService {

    /**
     * 创建系统
     *
     * @param sys
     * @return
     */
    SystemInfo createSystemInfo(SystemInfo sys);

    /**
     * 更新系统
     *
     * @param sys
     * @return
     */
    SystemInfo updateSystemInfo(SystemInfo sys);

    /**
     * 删除系统
     *
     * @param sysId
     * @return 被删除的系统
     */
    SystemInfo deleteSystemInfo(String sysId);

    /**
     * 查找系统
     *
     * @param sysId
     * @return
     */
    SystemInfo findSystemInfo(String sysId);

    /**
     * 根据code查找系统
     *
     * @param sysCode
     * @return
     */
    SystemInfo findSystemInfoByCode(String sysCode);

    /**
     * 根据name查找
     * @param sysName
     * @return
     */
    SystemInfo findSystemInfoByName(String sysName);

    /**** 用户表的操作 ****/
    /**
     * 创建用户
     * @param systemUser
     * @return
     */
    SystemUser createSystemUser(SystemUser systemUser);

    /**
     * 修改用户
     * @param systemUser
     * @return
     */
    SystemUser updateSystemUser(SystemUser systemUser);

    /**
     * 删除用户
     * @param id
     * @return
     */
    default SystemUser deleteSystemUser(String id) {
        return deleteSystemUser(id, Boolean.FALSE);
    }

    /**
     * 删除用户（是否真实删除）
     * @param id
     * @param deleteTrue
     * @return
     */
    SystemUser deleteSystemUser(String id,Boolean deleteTrue);

    default List<SystemUser> deleteSystemUsers(List<String> ids) {
        List<SystemUser> r = CollectionUtil.newArrayList();
        if(CollectionUtils.isNotEmpty(ids)) {
            ids = CollectionUtils.filter(ids, ObjectUtil::isNotNull);
            for(String id:ids) {
                SystemUser user = deleteSystemUser(id);
                if(user != null) {
                    r.add(user);
                }
            }
        }
        return r;
    }

    SystemUser doChangeState(String userId, CommonConst.StateEnum stateEnum);

    /**
     * 切换状态
     * @param userIds
     * @param stateEnum
     * @return
     */
    default List<SystemUser> doChangeState(List<String> userIds, CommonConst.StateEnum stateEnum) {
        List<SystemUser> r = CollectionUtil.newArrayList();
        if(CollectionUtils.isNotEmpty(userIds)) {
            userIds = CollectionUtils.filter(userIds, ObjectUtil::isNotNull);
            for(String id:userIds) {
                SystemUser user = doChangeState(id, stateEnum);
                if(user != null) {
                    r.add(user);
                }
            }
        }
        return r;
    }

    /**
     * 查询用户
     * @param id
     * @return
     */
    default SystemUser findSystemUserById(String id){
        List<SystemUser> rs = findSystemUserByIds(CollectionUtil.newArrayList(id));
        return CollectionUtils.findFirst(rs);
    }

    /**
     * 查询用户
     * @param ids
     * @return
     */
    List<SystemUser> findSystemUserByIds(List<String> ids);

    /**
     * 根据用户对应的实体类型ID查询用户
     * @param resType
     * @param resId
     * @return
     */
    SystemUser findSystemUserByRes(String resType, String resId);

    /**
     * 根据编码查询用户
     * @param code
     * @return
     */
    SystemUser findSystemUserByCode(String code);

    /**
     * 根据手机号查询用户，并不保证唯一
     *
     * @param phone
     * @return
     */
    SystemUser findSystemUserByPhone(String phone);

    /**
     * 根据资源类型查找用户
     *
     * @param typeEnum
     * @param resIds
     * @return
     */
    List<SystemUser> findSystemUserByResIds(ResourceConst.ResourceTypeEnum typeEnum, List<String> resIds);

    /**
     * 分页查询所有用户
     * @param name
     * @param phone
     * @param pageNum
     * @param pageSize
     * @return
     */
    PageInfo<SystemUser> findSystemUsers(String search, String name, String phone, Integer pageNum, Integer pageSize);

}
