package com.kingtom.kirin.app.system.service;

import java.util.List;

import com.kingtom.kirin.app.system.SystemConst;
import com.kingtom.kirin.app.system.model.UserSecurity;

/**
 * 权限模块</p>
 * 安全凭证 pojo service</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023 /8/24
 */
public interface IUserSecurityService{

    /**
     * 创建安全凭证
     *
     * @param security the security
     * @return user security
     */
    UserSecurity createUserSecurity(UserSecurity security);

    /**
     * 更新安全凭证
     *
     * @param security the security
     * @return user security
     */
    UserSecurity updateUserSecurity(UserSecurity security);

    /**
     * 删除安全凭证
     *
     * @param securityId the security id
     * @return 被删除的安全保障 boolean
     */
    boolean deleteUserSecurity(String securityId);

    /**
     * 查找安全凭证
     *
     * @param securityId the security id
     * @return user security
     */
    UserSecurity findUserSecurity(String securityId);

    UserSecurity findUserSecurityByType(String userId,SystemConst.UserSecurityTypeEnum support);

    /**
     * 查找安全凭证
     *
     * @param principal the principal
     * @return user security
     */
    UserSecurity findByPrincipal(String principal);

    /**
     * 查找安全凭证
     *
     * @param support   the support
     * @param principal the principal
     * @return user security
     */
    UserSecurity findByPrincipal(SystemConst.UserSecurityTypeEnum support, String principal);

    /**
     * 查找安全凭证
     *
     * @param supports  the supports
     * @param principal the principal
     * @return list
     */
    List<UserSecurity> findByPrincipal(List<SystemConst.UserSecurityTypeEnum> supports, String principal);

    /**
     * 查找安全凭证
     *
     * @param supports the supports
     * @param userId   the user id
     * @return list
     */
    List<UserSecurity> findByUserId(List<SystemConst.UserSecurityTypeEnum> supports, String userId);

}
