package com.kingtom.kirin.app.system.service;

import com.kingtom.kirin.app.system.SystemConst;
import com.kingtom.kirin.app.system.model.AppSystemProperties;
import com.kingtom.kirin.app.system.model.SystemInfo;
import com.kingtom.kirin.app.system.model.SystemUser;
import com.kingtom.kirin.app.system.model.SystemUserExt3;
import com.kingtom.kirin.core.common.json.JObject;
import jakarta.annotation.PostConstruct;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/7/13 20:10
 */
@Component
@Slf4j
public class SystemDefaultService {

    @Autowired
    private AppSystemProperties appSystemProperties;

    @Autowired
    private ISystemService systemService;

    public static String SYSTEM_ID = null;

    public static String SYSTEM_ADMIN_ID = null;

    public static String SYSTEM_APP_URL = null;

    @PostConstruct
    public void init(){
        doDefaultSystemInfo();
        doDefaultSystemUser();
    }

    /**
     * 根据配置默认系统信息
     * @return
     */
    public SystemInfo doDefaultSystemInfo() {
        SystemInfo dbPlat = findDefaultSystemInfo();
        if(dbPlat == null) {
            SystemInfo plat = new SystemInfo();
            plat.setName(appSystemProperties.getName());
            plat.setOtherName(appSystemProperties.getCompany());
            plat.setSystemProtocol(appSystemProperties.getProtocol());
            plat.setSystemAddress(appSystemProperties.getAddress());
            plat.setSystemPort(appSystemProperties.getPort());
            plat.setSystemApp(appSystemProperties.getApp());
            plat.setCode(SystemConst.SYSTEM_DEFAULT_CODE);
            plat.setSystemType(SystemConst.SystemInfoTypeEnum.Plat.getKey());
            dbPlat = systemService.createSystemInfo(plat);
        } else {
            log.info("系统["+dbPlat.getName()+"]已装备就绪");
            // 系统已经存在时，不做任何动作
//            dbPlat.setName(appSystemProperties.getName());
//            dbPlat.setOtherName(appSystemProperties.getCompany());
//            dbPlat = systemService.updateSystemInfo(dbPlat);
        }
        SYSTEM_ID = dbPlat.getId();
        SYSTEM_APP_URL =
                dbPlat.getSystemProtocol() + "://" + dbPlat.getSystemAddress() + ":" + dbPlat.getSystemPort() + "/" +
                        dbPlat.getSystemApp();
        return dbPlat;
    }

    public SystemUser doDefaultSystemUser() {
        SystemUser systemUser = findDefaultSystemUser();
        SystemInfo dbPlat = findDefaultSystemInfo();
        if(systemUser == null) {
            SystemUser user = new SystemUser();
            user.setName("超级管理员");
            user.setOtherName(user.getName());
            user.setCode(SystemConst.SYSTEM_USER_ADMIN_CODE);
            JObject<SystemUserExt3> j = new JObject<SystemUserExt3>(new SystemUserExt3());
            j.getE().setPassword(appSystemProperties.getSuperPassword());
            j.getE().setUsername(appSystemProperties.getSuperAdmin());
            user.setExt3(j);
            user.setSysId(dbPlat.getId());
            systemUser = systemService.createSystemUser(user);
        } else {
            log.info("系统管理员["+systemUser.getName()+"]已装备就绪");
            // 系统管理员已经存在时，不做任何动作
//            systemUser.getExt3().getE().setUsername(appSystemProperties.getSuperAdmin());
//            systemUser.getExt3().getE().setPassword(appSystemProperties.getSuperPassword());
//            systemUser = systemService.updateSystemUser(systemUser);
        }
        SYSTEM_ADMIN_ID = systemUser.getId();
        return systemUser;
    }

    /**
     * 查找默认系统
     * @return
     */
    public SystemInfo findDefaultSystemInfo() {
        return systemService.findSystemInfoByCode(SystemConst.SYSTEM_DEFAULT_CODE);
    }

    public SystemUser findDefaultSystemUser() {
        return systemService.findSystemUserByCode(SystemConst.SYSTEM_USER_ADMIN_CODE);
    }

}
