package com.kingtom.kirin.app.system.service.impl;

import java.util.ArrayList;
import java.util.List;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.app.space.service.ISpaceService;
import com.kingtom.kirin.app.system.SystemConst;
import com.kingtom.kirin.app.system.dao.SystemInfoMapper;
import com.kingtom.kirin.app.system.dao.SystemUserMapper;
import com.kingtom.kirin.app.system.event.*;
import com.kingtom.kirin.app.system.mapper.SystemMapper;
import com.kingtom.kirin.app.system.model.*;
import com.kingtom.kirin.app.system.service.ISystemService;
import com.kingtom.kirin.app.system.utils.SystemUtils;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.ObjectUtils;
import com.kingtom.kirin.core.event.service.IEventService;
import com.kingtom.kirin.core.resource.ResourceConst;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/19 10:10
 */
@Component
@Slf4j
public class SystemServiceImpl implements ISystemService {

    @Autowired
    private AppSystemProperties appSystemProperties;
    @Autowired
    private SystemInfoMapper systemInfoMapper;
    @Autowired
    private SystemUserMapper systemUserMapper;
    @Autowired
    private IEventService eventService;
    @Autowired
    private ISpaceService spaceService;

    @Override
    public SystemInfo createSystemInfo(SystemInfo sys) {
        if(sys == null){
            throw new AppException("系统不能为空");
        }
        if(sys.getName() == null){
            throw new AppException("系统名不能为空");
        }
        if(SystemConst.SystemInfoTypeEnum.findEnumByKey(sys.getSystemType()) == null) {
            throw new AppException("系统类型不能为空");
        }
        SystemInfo tmp = findSystemInfoByName(sys.getName());
        if(tmp != null) {
            throw new AppException("系统不允许重名").setM(sys.getName());
        }
        // 检查编码重复
        if(sys.getCode() != null) {
            tmp = findSystemInfoByName(sys.getName());
            if(tmp != null) {
                throw new AppException("系统不允许重复编码").setM(sys.getCode());
            }
        }
        SystemUtils.fillBeforeCreate(sys);
        systemInfoMapper.insert(sys);
        spaceService.doFormalOfReplace(sys.getImageLogo(), ResourceConst.ResourceTypeEnum.SystemInfo.getKey(),sys.getId(),SystemConst.SPACE_SYSTEM_Logo);
        spaceService.doFormalOfReplace(sys.getImageBackground(), ResourceConst.ResourceTypeEnum.SystemInfo.getKey(),sys.getId(),SystemConst.SPACE_SYSTEM_Background);
        log.info("系统["+sys.getName()+"]创建完成");
        eventService.publishEvent(new SystemInfoCreateEvent(sys));
        return sys;
    }

    @Override
    public SystemInfo updateSystemInfo(SystemInfo sys) {
        if(sys == null){
            throw new AppException("系统不能为空");
        }
        SystemInfo dbBean = findSystemInfo(sys.getId());
        if(dbBean == null){
            throw new AppException("系统不存在").setM(sys.getId());
        }
        if(sys.getName() == null){
            throw new AppException("系统名不能为空");
        }
        SystemInfo dbNameBean = findSystemInfoByName(sys.getName());
        if(dbNameBean != null && !dbNameBean.getId().equals(dbBean.getId())) {
            throw new AppException("系统不允许重名").setM(sys.getName());
        }
        SystemInfo copyDbBean = SystemMapper.INSTANCT.copyDepth(dbBean);
        ObjectUtils.copyPropertiesByCover(sys,copyDbBean,"name","otherName","code","imageLogo","imageBackground","description");
        SystemUtils.fillBeforeUpdate(copyDbBean,dbBean);
        systemInfoMapper.updateByPrimaryKey(copyDbBean);
        spaceService.doFormalOfReplace(sys.getImageLogo(), ResourceConst.ResourceTypeEnum.SystemInfo.getKey(),sys.getId(),SystemConst.SPACE_SYSTEM_Logo);
        spaceService.doFormalOfReplace(sys.getImageBackground(), ResourceConst.ResourceTypeEnum.SystemInfo.getKey(),sys.getId(),SystemConst.SPACE_SYSTEM_Background);
        log.info("系统["+sys.getName()+"]更新完成");
        eventService.publishEvent(new SystemInfoUpdateEvent(copyDbBean,dbBean));
        return copyDbBean;
    }

    @Override
    public SystemInfo deleteSystemInfo(String sysId) {
        SystemInfo dbInfo = findSystemInfo(sysId);
        if(dbInfo != null) {
            dbInfo.setState(CommonConst.StateEnum.Deleted.getKey());
            systemInfoMapper.updateByPrimaryKey(dbInfo);
            log.info("系统["+dbInfo.getName()+"]逻辑删除完成");
            eventService.publishEvent(new SystemInfoDeleteEvent(dbInfo));
        } else {
            log.warn("系统["+sysId+"]已经不存在，无需删除");
        }
        return dbInfo;
    }

    @Override
    public SystemInfo findSystemInfo(String sysId) {
        if(sysId == null){
            throw new AppException("系统ID不能为空");
        }
        SystemInfo dbInfo = systemInfoMapper.selectByPrimaryKey(sysId);
        return dbInfo;
    }

    @Override
    public SystemInfo findSystemInfoByCode(String sysCode) {
        if(sysCode == null){
            throw new AppException("系统编码不能为空");
        }
        SystemInfoExample e = new SystemInfoExample();
        e.createCriteria().andCodeEqualTo(sysCode);
        List<SystemInfo> dbNameInfos = systemInfoMapper.selectByExample(e);
        SystemInfo dbNameInfo = CollectionUtils.findFirst(dbNameInfos);
        return dbNameInfo;
    }

    @Override
    public SystemInfo findSystemInfoByName(String sysName) {
        if(sysName == null){
            throw new AppException("系统名称不能为空");
        }
        SystemInfoExample e = new SystemInfoExample();
        e.createCriteria().andNameEqualTo(sysName);
        List<SystemInfo> dbNameInfos = systemInfoMapper.selectByExample(e);
        SystemInfo dbNameInfo = CollectionUtils.findFirst(dbNameInfos);
        return dbNameInfo;
    }

    @Override
    public SystemUser createSystemUser(SystemUser systemUser) {
        if( systemUser == null ){
            throw new AppException("用户不能为空");
        }
        if( systemUser.getName() == null ) {
            throw new AppException("用户名不能为空");
        }
        SystemUtils.fillBeforeCreate(systemUser, appSystemProperties.getDefaultPassword());
        systemUserMapper.insert(systemUser);
        spaceService.doFormalOfReplace(systemUser.getImageLogo(), ResourceConst.ResourceTypeEnum.SystemUser.getKey(),systemUser.getId(),SystemConst.SPACE_USER_Logo);
        log.info("用户["+systemUser.getName()+"]创建完成");
        eventService.publishEvent(new SystemUserCreateEvent(systemUser));
        return systemUser;
    }

    @Override
    public SystemUser updateSystemUser(SystemUser systemUser) {
        if(systemUser == null){
            throw new AppException("用户不能为空");
        }
        SystemUser dbBean = findSystemUserById(systemUser.getId());
        if(dbBean == null){
            throw new AppException("用户不存在").setM(systemUser.getId());
        }
        if(systemUser.getName() == null){
            throw new AppException("用户名不能为空");
        }
        SystemUser copyDbBean = SystemMapper.INSTANCT.copyDepth(dbBean);
        ObjectUtils.copyPropertiesByCover(systemUser,copyDbBean,"name","otherName","imageLogo","resAttr", "ext3");
        if(StrUtil.isNotBlank(systemUser.getResId())) {
            copyDbBean.setResId(systemUser.getResId());
        }
        if(StrUtil.isNotBlank(systemUser.getResType())) {
            copyDbBean.setResType(systemUser.getResType());
        }
        SystemUtils.fillBeforeUpdate(copyDbBean,dbBean);
        systemUserMapper.updateByPrimaryKey(copyDbBean);
        spaceService.doFormalOfReplace(copyDbBean.getImageLogo(), ResourceConst.ResourceTypeEnum.SystemUser.getKey(),copyDbBean.getId(),SystemConst.SPACE_USER_Logo);
        log.info("用户["+copyDbBean.getName()+"]更新完成");
        eventService.publishEvent(new SystemUserUpdateEvent(copyDbBean,dbBean));
        return copyDbBean;
    }

    @Override
    public SystemUser deleteSystemUser(String id,Boolean deleteTrue){
        SystemUser dbInfo = findSystemUserById(id);
        if(dbInfo != null) {
            dbInfo.setState(CommonConst.StateEnum.Deleted.getKey());
            if(deleteTrue) {
                systemUserMapper.deleteByPrimaryKey(id);
                log.info("用户["+dbInfo.getName()+"]真实删除完成");
            } else {
                systemUserMapper.updateByPrimaryKey(dbInfo);
                log.info("用户["+dbInfo.getName()+"]逻辑删除完成");
            }
            eventService.publishEvent(new SystemUserDeleteEvent(dbInfo));
        } else {
            log.warn("用户["+id+"]已经不存在，无需删除");
        }
        return dbInfo;
    }

    @Override
    public SystemUser doChangeState(String userId, CommonConst.StateEnum stateEnum) {
        if(!CommonConst.StateEnum.Disable.equals(stateEnum)&&!CommonConst.StateEnum.Enable.equals(stateEnum)) {
            throw new AppException("不支持其他状态修改").setM(userId);
        }
        SystemUser dbBean = findSystemUserById(userId);
        if(dbBean != null) {
            SystemUser copyDbBean = SystemMapper.INSTANCT.copyDepth(dbBean);
            copyDbBean.setState(stateEnum.getKey());
            SystemUtils.fillBeforeUpdate(copyDbBean, dbBean);
            systemUserMapper.updateByPrimaryKey(copyDbBean);
            log.info("用户["+copyDbBean.getName()+"]状态变更完成");
            eventService.publishEvent(new SystemUserUpdateEvent(copyDbBean,dbBean,SystemUserUpdateEvent.E_TYPE_2));
            return copyDbBean;
        } else {
            log.info("用户["+userId+"]不存在");
            return dbBean;
        }
    }

    @Override
    public List<SystemUser> findSystemUserByIds(List<String> ids) {
        ids = CollectionUtils.filter(ids, ObjectUtil::isNotNull);
        if(CollectionUtils.isNotEmpty(ids)) {
            SystemUserExample e = new SystemUserExample();
            e.createCriteria().andIdIn(ids);
            return systemUserMapper.selectByExample(e);
        } else {
            return CollectionUtil.newArrayList();
        }
    }

    @Override
    public SystemUser findSystemUserByRes(String resType, String resId) {
        if(StrUtil.isAllNotBlank(resType, resId)) {
            SystemUserExample e = new SystemUserExample();
            e.createCriteria().andResTypeEqualTo(resType).andResIdEqualTo(resId);
            List<SystemUser> user = systemUserMapper.selectByExample(e);
            return CollectionUtils.findFirst(user);
        } else {
            return null;
        }
    }

    @Override
    public SystemUser findSystemUserByCode(String code) {
        SystemUserExample e = new SystemUserExample();
        if(StrUtil.isNotBlank(code)) {
            e.createCriteria().andCodeEqualTo(code);
            List<SystemUser> user = systemUserMapper.selectByExample(e);
            return CollectionUtils.findFirst(user);
        } else {
            return null;
        }
    }

    @Override
    public SystemUser findSystemUserByPhone(String phone){
        SystemUser r = null;
        if(StrUtil.isNotBlank(phone)){
            SystemUserExample e = new SystemUserExample();
            e.setOrderByClause("gmt_create desc");
            SystemUserExample.Criteria c = e.createCriteria();
            c.addCriterion("res_attr::jsonb -> 'e' ->> 'phone' = '" + phone + "'");
            doIgnoreSome(c, true, true);
            List<SystemUser> users = systemUserMapper.selectByExample(e);
            r = CollectionUtils.findFirst(users);
        }
        return r;
    }

    @Override
    public List<SystemUser> findSystemUserByResIds(ResourceConst.ResourceTypeEnum typeEnum, List<String> resIds){
        if(typeEnum == null || CollectionUtils.isEmpty(resIds)){
            return new ArrayList<>();
        }
        SystemUserExample e = new SystemUserExample();
        SystemUserExample.Criteria c = e.createCriteria();
        c.andResTypeEqualTo(typeEnum.getKey());
        c.andResIdIn(resIds);
        doIgnoreSome(c, true, true);
        List<SystemUser> users = systemUserMapper.selectByExample(e);
        return users;
    }

    @Override
    public PageInfo<SystemUser> findSystemUsers(String search, String name, String phone, Integer pageNum, Integer pageSize) {
        SystemUserExample e = new SystemUserExample();
        SystemUserExample.Criteria c = e.createCriteria();
        if(StrUtil.isNotBlank(search)) {
            c.andNameLike("%"+search+"%");
        }
        if(StrUtil.isNotBlank(name)) {
            c.andNameLike("%"+name+"%");
        }
        if(StrUtil.isNotBlank(phone)){
            c.addCriterion("res_attr::jsonb -> 'e' ->> 'phone' = '" + phone + "'");
        }
        doIgnoreSome(c, true, true);
        e.setOrderByClause("gmt_create desc");
        PageHelper.startPage(pageNum, pageSize);
        List<SystemUser> infos = systemUserMapper.selectByExample(e);
        return new PageInfo<>(infos);
    }

    protected void doIgnoreSome(SystemUserExample.Criteria c, Boolean ignoreDeleted, Boolean ignoreAdmin) {
        if(ignoreDeleted) {
            c.andStateNotEqualTo(CommonConst.StateEnum.Deleted.getKey());
        }
        if(ignoreAdmin) {
            c.andCodeNotEqualTo(SystemConst.SYSTEM_USER_ADMIN_CODE);
        }
    }
}
