package com.kingtom.kirin.app.system.utils;

import cn.hutool.core.util.ObjectUtil;
import com.kingtom.kirin.app.system.SystemConst;
import com.kingtom.kirin.app.system.model.*;
import com.kingtom.kirin.app.system.service.SystemDefaultService;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.json.JObject;
import com.kingtom.kirin.core.common.utils.IDUtils;
import com.kingtom.kirin.core.resource.ResourceConst;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.Date;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/7 16:48
 */
@Slf4j
public class SystemUtils {

    private SystemUtils(){}

    public static String getCurrentSystemId() {
        return SystemDefaultService.SYSTEM_ID;
    }

    public static String getCurrentSystemAdmin() {
        return SystemDefaultService.SYSTEM_ADMIN_ID;
    }

    public static String getCurrentSystemAppUrl(){
        return SystemDefaultService.SYSTEM_APP_URL;
    }

    /**
     * 完善新增数据
     * @param paramBean 接口传递的待保存数据
     * @return
     */
    public static void fillBeforeCreate(SystemInfo paramBean) {
        Date now = new Date();
        paramBean.setId(IDUtils.newUUID());
        if(paramBean.getCode() == null) {
            // 默认一个编码
            String c = paramBean.getSystemType()+ CommonConst.UNDER_LINE + paramBean.getId();
            paramBean.setCode(c);
        }
        if(paramBean.getState() == null) {
            paramBean.setState(CommonConst.StateEnum.Enable.getKey());
        }
        if(paramBean.getOtherName() == null) {
            paramBean.setOtherName(paramBean.getName());
        }
        if(paramBean.getExt3() == null) {
            paramBean.setExt3(new JObject<>(new SystemInfoExt3()));
        }
        paramBean.setGmtCreate(now);
        paramBean.setGmtModified(now);
    }

    /**
     * 完善修改数据
     * @param paramBean 接口传递的待修改数据
     * @param dbBean 数据库的原始数据
     * @return
     */
    public static void fillBeforeUpdate(SystemInfo paramBean,SystemInfo dbBean) {
        Date now = new Date();
        paramBean.setGmtModified(now);
    }

    /**
     * 完善新增数据
     * @param paramBean 接口传递的待保存数据
     */
    public static void fillBeforeCreate(SystemUser paramBean, String defaultPassword) {
        Date now = new Date();
        paramBean.setId(IDUtils.newUUID());
        if(paramBean.getResType() == null) {
            paramBean.setResType(ResourceConst.ResourceTypeEnum.OrgPerson.getKey());
        }
        if(StringUtils.isEmpty(paramBean.getResId())){
            paramBean.setResId(CommonConst.UNKOWN);
        }
        if(paramBean.getCode() == null) {
            // 默认一个编码
            String c = paramBean.getResType()+ CommonConst.UNDER_LINE + paramBean.getId();
            paramBean.setCode(c);
        }
        if(paramBean.getState() == null) {
            paramBean.setState(CommonConst.StateEnum.Enable.getKey());
        }
        if(paramBean.getOtherName() == null) {
            paramBean.setOtherName(paramBean.getName());
        }
        if(paramBean.getResAttr() == null) {
            paramBean.setResAttr(new JObject<>(new SystemUserResAttr()));
        }
        if(paramBean.getExt3() == null) {
            paramBean.setExt3(new JObject<>(new SystemUserExt3()));
        }
        paramBean.setGmtCreate(now);
        paramBean.setGmtModified(now);
        paramBean.setSysId(getCurrentSystemId());
    }

    /**
     * 完善修改数据
     * @param paramBean 接口传递的待修改数据
     * @param dbBean 数据库的原始数据
     * @return
     */
    public static void fillBeforeUpdate(SystemUser paramBean,SystemUser dbBean) {
        Date now = new Date();
        paramBean.setGmtModified(now);
    }

    public static boolean isNotSame(SystemInfo info1, SystemInfo info2) {
        if(ObjectUtil.isAllEmpty(info1,info2)) {
            return false;
        } else {
            return ObjectUtil.notEqual(info1.getId(),info2.getId());
        }
    }

    public static boolean isSame(SystemInfo info1,SystemInfo info2) {
        return !isNotSame(info1,info2);
    }

    public static boolean isNotSame(SystemUser info1, SystemUser info2) {
        if(ObjectUtil.isAllEmpty(info1,info2)) {
            return false;
        } else {
            return ObjectUtil.notEqual(info1.getId(),info2.getId());
        }
    }

    public static boolean isSame(SystemUser info1,SystemUser info2) {
        return !isNotSame(info1,info2);
    }
}
