package com.kingtom.kirin.app.wechat.base;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.kingtom.kirin.app.wechat.WeChatConst;
import com.kingtom.kirin.app.wechat.base.mini.api.token.GetStableToken;
import com.kingtom.kirin.app.wechat.base.mini.api.token.GetToken;
import com.kingtom.kirin.core.common.utils.HttpUtils;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/10/12
 */
@Slf4j
public abstract class AbstractWeChatApi<Q, S> implements IWeChatApi<Q, S>{

    protected String name;

    protected String url;

    protected String method;

    protected Q request;

    protected S response;

    protected int retry = 0;

    protected AbstractWeChatApi(String n, String u, String m, Q r){
        this.name = n;
        this.url = u;
        this.method = m;
        this.request = r;
    }

    @Getter
    private static List<Class<? extends IWeChatApi>> getTokenClassMap = new ArrayList<>();

    static{
        getTokenClassMap.add(GetToken.class);
        getTokenClassMap.add(GetStableToken.class);
        getTokenClassMap.add(com.kingtom.kirin.app.wechat.base.pubnum.api.token.GetToken.class);
        getTokenClassMap.add(com.kingtom.kirin.app.wechat.base.pubnum.api.token.GetStableToken.class);
    }

    @JsonIgnore
    public abstract Class<S> getResponseClass();

    public String getUrl(){
        return url;
    }

    public String getMethod(){
        return method;
    }

    public Q getRequest(){
        return request;
    }

    @Override
    public String getName(){
        return name;
    }

    @Override
    public S getResponse(){
        return response;
    }

    @Override
    public int getRetry(){
        return retry;
    }

    @Override
    public int findAndAddRetry(){
        int r = retry;
        retry++;
        return r;
    }

    @Override
    public Map<String, ?> getRequestParamOfUrl(){
        return JsonUtils.toMap(request);
    }

    @Override
    public Map<String, ?> getRequestParamOfBody(){
        Map<String, ?> r = JsonUtils.toMap(request);
        if(this instanceof IWeChatTokenAware){
            r.remove(IWeChatTokenAware.TOKEN_KEY);
        }
        return r;
    }

    @Override
    public boolean isResponseOk(){
        Integer code = response == null ? null : (Integer) JsonUtils.toMap(response).get("errcode");
        return code == null || WeChatConst.ResultCodeEnum.SUCCESS.getKey().equals(code);
    }

    @Override
    public boolean isTokenError(){
        Integer code = response == null ? null : (Integer) JsonUtils.toMap(response).get("errcode");
        return code != null && WeChatConst.ResultCodeEnum.isTokenError(code) &&
                !getTokenClassMap.contains(this.getClass());
    }

    @Override
    public void call(){
        if(IWeChatApi.METHOD_GET.equals(method)){
            response = HttpUtils.getSend(url, getResponseClass(), getRequestParamOfUrl());
        } else{
            response = HttpUtils.postSend(url, getResponseClass(), getRequestParamOfUrl(), getRequestParamOfBody());
        }
    }

}
