package com.kingtom.kirin.app.wechat.base;

import java.util.Map;

import com.fasterxml.jackson.annotation.JsonIgnore;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @param <Q> the type parameter
 * @param <S> the type parameter
 * @author wany
 * @since 2023 /11/1
 */
public interface IWeChatApi<Q, S>{

    /**
     * The constant METHOD_GET.
     */
    public static final String METHOD_GET = "get";

    /**
     * The constant METHOD_POST.
     */
    public static final String METHOD_POST = "post";

    /**
     * Gets name.
     *
     * @return the name
     */
    public String getName();

    public S getResponse();

    /**
     * Gets retry.
     *
     * @return the retry
     */
    public int getRetry();

    /**
     * Find and add retry int.
     *
     * @return the int
     */
    public int findAndAddRetry();

    /**
     * Gets request param of url.
     *
     * @return the request param of url
     */
    @JsonIgnore
    public Map<String, ?> getRequestParamOfUrl();

    /**
     * Gets request param of body.
     *
     * @return the request param of body
     */
    @JsonIgnore
    public Map<String, ?> getRequestParamOfBody();

    /**
     * 判断请求是否OK
     *
     * @return boolean
     */
    @JsonIgnore
    public boolean isRequestOk();

    /**
     * 判断结果是否OK
     *
     * @return boolean
     */
    @JsonIgnore
    public boolean isResponseOk();

    /**
     * Is get token error boolean.
     *
     * @return the boolean
     */
    @JsonIgnore
    public boolean isTokenError();

    public void call();

}
