package com.kingtom.kirin.app.wechat.base.mini.api;

import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.kingtom.kirin.app.wechat.base.AbstractWeChatApi;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/10/12
 */
public class GetUserPhoneNumber extends AbstractWeChatApi<GetUserPhoneNumber.ReqParam, GetUserPhoneNumber.ResParam>
        implements IWxMiniTokenAware{

    public GetUserPhoneNumber(String code){
        super("getPhoneNumber", "https://api.weixin.qq.com/wxa/business/getuserphonenumber?access_token={access_token}",
                METHOD_POST, new ReqParam());
        request.code = code;
    }

    @Override
    public void setToken(String token){
        request.setAccess_token(token);
    }

    @Override
    public Class<ResParam> getResponseClass(){
        return ResParam.class;
    }

    @Override
    public boolean isRequestOk(){
        return StringUtils.isNotBlank(request.getCode());
    }

    @Data
    @JsonPropertyOrder({"access_token", "code"})
    public static class ReqParam{

        /**
         * 接口调用凭证，该参数为 URL 参数，非 Body 参数。使用access_token或者authorizer_access_token
         */
        private String access_token;

        /**
         * 手机号获取凭证
         */
        private String code;

    }

    @Data
    public static class ResParam{

        /**
         * 错误码
         */
        private Integer errcode;

        /**
         * 错误信息
         */
        private String errmsg;

        /**
         * 用户手机号信息
         */
        private PhoneInfo phone_info;

    }

    @Data
    public static class PhoneInfo{

        /**
         * 用户绑定的手机号（国外手机号会有区号）
         */
        private String phoneNumber;

        /**
         * 没有区号的手机号
         */
        private String purePhoneNumber;

        /**
         * 区号
         */
        private String countryCode;

        /**
         * 数据水印
         */
        private Watermark watermark;

    }

    @Data
    public static class Watermark{

        /**
         * 用户获取手机号操作的时间戳
         */
        private Long timestamp;

        /**
         * 小程序appid
         */
        private String appid;

    }

}
