package com.kingtom.kirin.app.wechat.base.mini.api;

import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.kingtom.kirin.app.wechat.base.AbstractWeChatApi;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/10/12
 */
public class JsCode2Session extends AbstractWeChatApi<JsCode2Session.ReqParam, JsCode2Session.ResParam>
        implements IWxMiniAppIdAware{

    public JsCode2Session(String jsCode){
        super("JsCode2Session",
                "https://api.weixin.qq.com/sns/jscode2session?appid={appid}&secret={secret}&js_code={js_code" +
                        "}&grant_type={grant_type}", "GET", new ReqParam());
        request.setJs_code(jsCode);
    }

    @Override
    public Class<ResParam> getResponseClass(){
        return ResParam.class;
    }

    @Override
    public boolean isRequestOk(){
        return !StringUtils.isAnyBlank(request.getJs_code(), request.getGrant_type());
    }

    @Override
    public void setAppId(String appId){
        request.setAppid(appId);
    }

    @Override
    public void setAppSecret(String appSecret){
        request.setSecret(appSecret);
    }

    @Data
    @JsonPropertyOrder({"appid", "secret", "js_code", "grant_type"})
    public static class ReqParam{

        /**
         * 小程序唯一凭证，即 AppID，可在「微信公众平台 - 设置 - 开发设置」页中获得。（需要已经成为开发者，且帐号没有异常状态）
         */
        private String appid;

        /**
         * 小程序唯一凭证密钥，即 AppSecret，获取方式同 appid
         */
        private String secret;

        /**
         * 填写 client_credential
         */
        private String js_code;

        /**
         * 授权类型，此处只需填写 authorization_code
         */
        private String grant_type = "authorization_code";

    }

    @Data
    public static class ResParam{

        /**
         * 错误时：错误码
         */
        private Integer errcode;

        /**
         * 错误时：错误信息
         */
        private String errmsg;

        /**
         * 正确时：会话密钥
         */
        private String session_key;

        /**
         * 正确时：用户在开放平台的唯一标识符，若当前小程序已绑定到微信开放平台帐号下会返回
         */
        private String unionid;

        /**
         * 用户唯一标识
         */
        private String openid;

    }

}
