package com.kingtom.kirin.app.wechat.base.mini.api.token;

import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.kingtom.kirin.app.wechat.base.AbstractWeChatApi;
import com.kingtom.kirin.app.wechat.base.mini.api.IWxMiniAppIdAware;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/10/12
 */
public class GetToken extends AbstractWeChatApi<GetToken.ReqParam, GetToken.ResParam> implements IWxMiniAppIdAware{

    public GetToken(){
        super("getToken",
                "https://api.weixin.qq.com/cgi-bin/token?grant_type={grant_type}&appid={appid}&secret={secret}", "GET",
                new ReqParam());
    }

    @Override
    public Class<ResParam> getResponseClass(){
        return ResParam.class;
    }

    @Override
    public boolean isRequestOk(){
        return StringUtils.isNotBlank(request.getGrant_type());
    }

    @Override
    public void setAppId(String appId){
        request.setAppid(appId);
    }

    @Override
    public void setAppSecret(String appSecret){
        request.setSecret(appSecret);
    }

    @Data
    @JsonPropertyOrder({"grant_type", "appid", "secret"})
    public static class ReqParam{

        /**
         * 填写 client_credential
         */
        private String grant_type = "client_credential";

        /**
         * 小程序唯一凭证，即 AppID，可在「微信公众平台 - 设置 - 开发设置」页中获得。（需要已经成为开发者，且帐号没有异常状态）
         */
        private String appid;

        /**
         * 小程序唯一凭证密钥，即 AppSecret，获取方式同 appid
         */
        private String secret;

    }

    @Data
    public static class ResParam{

        /**
         * 错误时：错误码
         */
        private Integer errcode;

        /**
         * 错误时：错误信息
         */
        private String errmsg;

        /**
         * 正确时：获取到的凭证
         */
        private String access_token;

        /**
         * 正确时：凭证有效时间，单位：秒。目前是7200秒之内的值
         */
        private int expires_in;

    }

}
