package com.kingtom.kirin.app.wechat.base.mini.authc;

import java.util.List;

import com.kingtom.kirin.app.security.base.Authentication;
import com.kingtom.kirin.app.security.base.IAuthEncode;
import com.kingtom.kirin.app.security.base.IAuthcRealm;
import com.kingtom.kirin.app.security.service.ISecurityContainerService;
import com.kingtom.kirin.app.security.service.ISecurityService;
import com.kingtom.kirin.app.system.SystemConst;
import com.kingtom.kirin.app.system.model.UserSecurity;
import com.kingtom.kirin.app.wechat.base.mini.service.IWeChatMiniChannel;
import com.kingtom.kirin.app.wechat.service.IWeChatSystemUserService;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import jakarta.annotation.PostConstruct;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.context.annotation.ScopedProxyMode;
import org.springframework.stereotype.Component;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/10/12
 */
@Component
@Scope(proxyMode = ScopedProxyMode.NO)
public class WechatMiniAuthcRealm implements IAuthcRealm{

    @Autowired
    private ISecurityContainerService container;

    @Autowired
    private ISecurityService securityService;

    @Autowired
    private IWeChatMiniChannel weChatService;

    @Autowired
    private IWeChatSystemUserService weChatUserService;

    @PostConstruct
    protected void init(){
        container.registerRealm(this);
    }

    @Override
    public SystemConst.AuthcRealmTypeEnum support(){
        return SystemConst.AuthcRealmTypeEnum.WechatMini;
    }

    @Override
    public Authentication doAuth(Authentication custom, IAuthEncode encode){
        custom.setUsed(false);
        // 请求微信登录
        String r = weChatService.code2Session(custom.getCredentials());
        if(StringUtils.isNotBlank(r)){
            List<UserSecurity> userSecurities = securityService.findByPrincipal(support().getTypeEnums(), r);
            if(CollectionUtils.isEmpty(userSecurities)){
                UserSecurity wechatMini = weChatUserService.createWechatMiniUser(r);
                userSecurities.add(wechatMini);
            }
            if(CollectionUtils.isNotEmpty(userSecurities)){
                UserSecurity userSecurity = userSecurities.get(CommonConst.INT_0);
                custom.setAuthTypeEnum(SystemConst.UserSecurityTypeEnum.findEnumByKey(userSecurity.getType()));
                custom.setUserId(userSecurity.getUserId());
                custom.setUsed(true);
            }
        }
        return custom;
    }

}
