package com.kingtom.kirin.app.wechat.base.mini.service.impl;

import com.kingtom.kirin.app.wechat.base.IWeChatApi;
import com.kingtom.kirin.app.wechat.base.mini.api.GetUserPhoneNumber;
import com.kingtom.kirin.app.wechat.base.mini.api.IWxMiniAppIdAware;
import com.kingtom.kirin.app.wechat.base.mini.api.IWxMiniTokenAware;
import com.kingtom.kirin.app.wechat.base.mini.api.JsCode2Session;
import com.kingtom.kirin.app.wechat.base.mini.api.token.GetStableToken;
import com.kingtom.kirin.app.wechat.base.mini.config.WeChatMiniProperties;
import com.kingtom.kirin.app.wechat.base.mini.service.IWeChatMiniChannel;
import com.kingtom.kirin.app.wechat.service.IWeChatUserService;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.lang.FixedLinkedList;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/10/12
 */
@Service
@Slf4j
public class WeChatMiniImpl implements IWeChatMiniChannel{

    @Autowired(required = false)
    private WeChatMiniProperties properties;

    @Autowired
    private IWeChatUserService weChatUserService;

    private FixedLinkedList<String> accessTokens = new FixedLinkedList<String>(CommonConst.INT_5);

    @Override
    public WeChatMiniProperties findConfig(){
        if(properties == null){
            throw new AppException("微信小程序appid未配置！");
        }
        return properties;
    }

    @Override
    public String findAccessToken() throws AppException{
        if(accessTokens.isEmpty()){
            refreshAccessToken();
        }
        if(accessTokens.isEmpty()){
            throw new AppException("微信accessToken获取失败");
        } else{
            return accessTokens.getLast();
        }
    }

    @Override
    public String code2Session(String jsCode) throws AppException{
        JsCode2Session c = new JsCode2Session(jsCode);
        callApi(c);
        //更新WechatUser数据
        if(c.isResponseOk()){
            weChatUserService.saveUser(null, findConfig().getAppId(), c.getResponse().getOpenid(),
                    c.getResponse().getUnionid());
        }
        return c.getResponse().getOpenid();
    }

    @Override
    public String getPhoneNumber(String code) throws AppException{
        GetUserPhoneNumber p = new GetUserPhoneNumber(code);
        callApi(p);
        return p.getResponse().getPhone_info().getPhoneNumber();
    }

    @Override
    public <Q, S> void callApi(IWeChatApi<Q, S> api) throws AppException{
        if(api == null){
            throw new AppException("微信请求不能为空");
        } else if(!api.isRequestOk()){
            throw new AppException("微信请求构建错误");
        } else{
            if(api instanceof IWxMiniAppIdAware){
                ((IWxMiniAppIdAware) api).setAppId(findConfig().getAppId());
                ((IWxMiniAppIdAware) api).setAppSecret(findConfig().getAppSecret());
            }
            if(api instanceof IWxMiniTokenAware){
                ((IWxMiniTokenAware) api).setToken(findAccessToken());
            }
            log.info("微信接口[" + api.getName() + "]开始发送，json=" + JsonUtils.toJson(api));
            api.call();
            if(api.isResponseOk()){
                log.info("微信接口[" + api.getName() + "]执行成功，json=" + JsonUtils.toJson(api));
            } else{
                log.warn("微信接口[" + api.getName() + "]发生错误，json=" + JsonUtils.toJson(api));
                if(api.isTokenError() && api.findAndAddRetry() < CommonConst.INT_2){
                    log.warn("微信接口[" + api.getName() + "]重新尝试[" + api.getRetry() + "]，json=" +
                            JsonUtils.toJson(api));
                    refreshAccessToken();
                    callApi(api);
                }
            }
        }
    }

    //    @Scheduled(cron = "0 0/59 * * * ?")
    //    public void refreshAccessToken() {
    //        log.info("请求微信accessToken开始");
    //        GetAccessToken token = new GetAccessToken();
    //        callWeChat(token);
    //        if(token.isResOk()) {
    //            String t = token.getResponse().getAccess_token();
    //            accessTokens.add(t);
    //            log.info("请求微信accessToken成功，res=："+JsonUtils.toJson(token.getResponse())+"；当前列表：" + JsonUtils
    //            .toJson(accessTokens));
    //        } else {
    //            log.warn("请求微信accessToken失败，res="+JsonUtils.toJson(token.getResponse()));
    //        }
    //    }

    @Scheduled(cron = "0 0/5 * * * ?")
    public void refreshAccessToken(){
        log.info("请求微信accessToken开始");
        GetStableToken token = new GetStableToken();
        callApi(token);
        if(token.isResponseOk()){
            String t = token.getResponse().getAccess_token();
            accessTokens.add(t);
            log.info("请求微信accessToken成功，res=：" + JsonUtils.toJson(token.getResponse()) + "；当前列表：" +
                    JsonUtils.toJson(accessTokens));
        } else{
            log.warn("请求微信accessToken失败，res=" + JsonUtils.toJson(token.getResponse()));
        }
    }

}
