package com.kingtom.kirin.app.wechat.base.pubnum.api.message;

import java.util.HashMap;
import java.util.Map;

import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.kingtom.kirin.app.wechat.base.AbstractWeChatApi;
import com.kingtom.kirin.app.wechat.base.IWeChatTokenAware;
import com.kingtom.kirin.app.wechat.base.pubnum.api.IWxPubNumTokenAware;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/11/1
 */
public class SendTemplateMessage extends AbstractWeChatApi<SendTemplateMessage.ReqParam, SendTemplateMessage.ResParam>
        implements IWxPubNumTokenAware{

    public SendTemplateMessage(String touser, String templateId, Map<String, String> data, String appId,
            String pagePath, String url, String msgId){
        super("微信公众号-发送模板消息",
                "https://api.weixin.qq.com/cgi-bin/message/template/send?access_token={access_token}", METHOD_POST,
                new SendTemplateMessage.ReqParam());
        request.setTouser(touser);
        request.setTemplate_id(templateId);
        Map<String, Object> dataMap = new HashMap<>();
        data.forEach((k, v) -> {
            Map<String, String> value = new HashMap<>();
            value.put("value", v);
            dataMap.put(k, value);
        });
        request.setData(dataMap);

        if(StringUtils.isNotBlank(appId)){
            SendTemplateMessage.MiniProgram miniProgram = new SendTemplateMessage.MiniProgram();
            miniProgram.setAppid(appId);
            if(StringUtils.isNotBlank(pagePath)){
                miniProgram.setPagepath(pagePath);
            }
            request.setMiniprogram(miniProgram);
        }

        if(StringUtils.isNotBlank(url)){
            request.setUrl(url);
        }
        if(StringUtils.isNotBlank(msgId)){
            request.setClient_msg_id(msgId);
        }
    }

    @Override
    public void setToken(String token){
        request.setAccess_token(token);
    }

    @Override
    public Class<ResParam> getResponseClass(){
        return ResParam.class;
    }

    @Override
    public boolean isRequestOk(){
        return StringUtils.isNoneBlank(request.touser, request.template_id) &&
                (request.miniprogram == null || StringUtils.isNotEmpty(request.miniprogram.appid));
    }

    @Override
    public Map<String, ?> getRequestParamOfUrl(){
        Map<String, Object> map = new HashMap<>();
        map.put("access_token", request.getAccess_token());
        return map;
    }

    @Override
    public Map<String, ?> getRequestParamOfBody(){
        Map<String, ?> r = JsonUtils.toMap(request);
        r.remove(IWeChatTokenAware.TOKEN_KEY);
        return r;
    }

    @Data
    @JsonPropertyOrder({"touser", "template_id", "url", "miniprogram", "data", "client_msg_id"})
    public static class ReqParam{

        /**
         * 接口调用凭证
         */
        private String access_token;

        /**
         * 必填	接收者openid
         */
        private String touser;

        /**
         * 必填	模板ID
         */
        private String template_id;

        /**
         * 非必填 模板跳转链接（海外账号没有跳转能力）
         */
        private String url;

        /**
         * 非必填 跳小程序所需数据，不需跳小程序可不用传该数据
         */
        private MiniProgram miniprogram;

        /**
         * 必填 模板数据
         */
        private Map<String, Object> data;

        /**
         * 非必填 防重入id。对于同一个openid +client_msg_id,只发送一条消息,10分钟有效,超过10分钟不保证效果。若无防重入需求，可不填
         */
        private String client_msg_id;

    }

    @Data
    @JsonPropertyOrder({"appid", "pagepath",})
    public static class MiniProgram{

        /**
         * 必填 所需跳转到的小程序appid（该小程序appid必须与发模板消息的公众号是绑定关联关系，暂不支持小游戏）
         */
        private String appid;

        /**
         * 非必填 所需跳转到小程序的具体页面路径，支持带参数,（示例index?foo=bar），要求该小程序已发布，暂不支持小游戏
         */
        private String pagepath;

    }

    @Data
    public static class ResParam{

        /**
         * 错误时：错误码
         */
        private Integer errcode;

        /**
         * 错误时：错误信息
         */
        private String errmsg;

        /**
         * 消息id
         */
        private String msgid;

    }

}
