package com.kingtom.kirin.app.wechat.base.pubnum.api.user;

import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.kingtom.kirin.app.wechat.base.AbstractWeChatApi;
import com.kingtom.kirin.app.wechat.base.pubnum.api.IWxPubNumTokenAware;
import com.kingtom.kirin.app.wechat.base.pubnum.model.UserInfo;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.apache.commons.lang3.StringUtils;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/11/1
 */
public class GetUserInfo extends AbstractWeChatApi<GetUserInfo.ReqParam, GetUserInfo.ResParam>
        implements IWxPubNumTokenAware{


    public GetUserInfo(String openId){
        super("获取用户信息",
                "https://api.weixin.qq.com/cgi-bin/user/info?access_token={access_token}&openid={openid}&lang" +
                        "={lang}", METHOD_GET, new ReqParam());
        request.openid = openId;
    }

    @Override
    public void setToken(String token){
        request.setAccess_token(token);
    }

    @Override
    public Class<ResParam> getResponseClass(){
        return ResParam.class;
    }

    @Override
    public boolean isRequestOk(){
        return request != null && StringUtils.isNotBlank(request.openid);
    }

    @Data
    @JsonPropertyOrder({"grant_type", "appid", "lang"})
    public static class ReqParam{

        /**
         * 必填 调用接口凭证
         */
        private String access_token;

        /**
         * 必填 普通用户的标识，对当前公众号唯一
         */
        private String openid;

        /**
         * 非必填 返回国家地区语言版本，zh_CN 简体，zh_TW 繁体，en 英语
         */
        private String lang = "zh_CN";

    }


    @Data
    @EqualsAndHashCode(callSuper = true)
    public static class ResParam extends UserInfo{

        /**
         * 错误时：错误码
         */
        private Integer errcode;

        /**
         * 错误时：错误信息
         */
        private String errmsg;

    }

}
