package com.kingtom.kirin.app.wechat.base.pubnum.api.user;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.kingtom.kirin.app.wechat.base.AbstractWeChatApi;
import com.kingtom.kirin.app.wechat.base.IWeChatTokenAware;
import com.kingtom.kirin.app.wechat.base.pubnum.api.IWxPubNumTokenAware;
import com.kingtom.kirin.app.wechat.base.pubnum.model.UserInfo;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import lombok.Data;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/11/1
 */
public class GetUserInfoBatch extends AbstractWeChatApi<GetUserInfoBatch.ReqParam, GetUserInfoBatch.ResParam>
        implements IWxPubNumTokenAware{

    public GetUserInfoBatch(List<String> openIds){
        super("批量获取用户信息", "https://api.weixin.qq.com/cgi-bin/user/info/batchget?access_token={access_token}",
                METHOD_POST, new ReqParam());
        List<GetUserInfoBatch.UserListItem> items = openIds.stream().map(e -> {
            GetUserInfoBatch.UserListItem item = new GetUserInfoBatch.UserListItem();
            item.setOpenid(e);
            return item;
        }).collect(Collectors.toList());
        request.setUser_list(items);
    }

    @Override
    public void setToken(String token){
        request.setAccess_token(token);
    }

    @Override
    public Class<ResParam> getResponseClass(){
        return ResParam.class;
    }

    @Override
    public boolean isRequestOk(){
        return request != null && CollectionUtils.isNotEmpty(request.user_list);
    }

    @Override
    public Map<String, ?> getRequestParamOfUrl(){
        Map<String, Object> map = new HashMap<>();
        map.put(TOKEN_KEY, request.getAccess_token());
        return map;
    }

    @Override
    public Map<String, ?> getRequestParamOfBody(){
        Map<String, ?> r = JsonUtils.toMap(request);
        r.remove(IWeChatTokenAware.TOKEN_KEY);
        return r;
    }

    @Data
    @JsonPropertyOrder({"access_token", "user_list"})
    public static class ReqParam{

        /**
         * 必填 调用接口凭证
         */
        private String access_token;

        /**
         * 必填
         */
        private List<UserListItem> user_list;

    }

    @Data
    public static class UserListItem{

        /**
         * 必填 用户的标识，对当前公众号唯一
         */
        private String openid;

        /**
         * 非必填 国家地区语言版本，zh_CN 简体，zh_TW 繁体，en 英语，默认为zh-CN
         */
        private String lang = "zh_CN";

    }

    @Data
    public static class ResParam{

        /**
         * 错误时：错误码
         */
        private Integer errcode;

        /**
         * 错误时：错误信息
         */
        private String errmsg;

        /**
         * 正确时：用户基本信息集合
         */
        private List<UserInfo> user_info_list;

    }

}
