package com.kingtom.kirin.app.wechat.base.pubnum.api.user;

import java.util.List;
import java.util.Map;

import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.kingtom.kirin.app.wechat.base.AbstractWeChatApi;
import com.kingtom.kirin.app.wechat.base.IWeChatTokenAware;
import com.kingtom.kirin.app.wechat.base.pubnum.api.IWxPubNumTokenAware;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/11/1
 */
public class GetUserList extends AbstractWeChatApi<GetUserList.ReqParam, GetUserList.ResParam>
        implements IWxPubNumTokenAware{

    public GetUserList(String nextOpenId){

        super("获取关注用户列表", null, METHOD_GET, new GetUserList.ReqParam());
        if(StringUtils.isBlank(nextOpenId)){
            this.url = "https://api.weixin.qq.com/cgi-bin/user/get?access_token={access_token}";
        } else{
            request.next_openid = nextOpenId;
            this.url = "https://api.weixin.qq.com/cgi-bin/user/get?access_token={access_token}&next_openid" +
                    "={next_openid}";
        }
    }

    @Override
    public void setToken(String token){
        request.setAccess_token(token);
    }

    @Override
    public Class<ResParam> getResponseClass(){
        return ResParam.class;
    }

    @Override
    public boolean isRequestOk(){
        return request != null;
    }

    @Override
    public Map<String, ?> getRequestParamOfUrl(){
        Map<String, ?> map = super.getRequestParamOfUrl();
        if(StringUtils.isBlank(request.getNext_openid())){
            map.remove("next_openid");
        }
        return map;
    }

    @Override
    public Map<String, ?> getRequestParamOfBody(){
        Map<String, ?> r = JsonUtils.toMap(request);
        r.remove(IWeChatTokenAware.TOKEN_KEY);
        return r;
    }

    @Data
    @JsonPropertyOrder({"access_token", "next_openid"})
    public static class ReqParam{

        /**
         * 必填 调用接口凭证
         */
        private String access_token;

        /**
         * 非必填 第一个拉取的OPENID，不填默认从头开始拉取
         */
        private String next_openid;

    }

    @Data
    public static class ResParam{

        /**
         * 错误时：错误码
         */
        private Integer errcode;

        /**
         * 错误时：错误信息
         */
        private String errmsg;

        /**
         * 正确时：关注该公众账号的总用户数
         */
        private long total;

        /**
         * 正确时：拉取的OPENID个数，最大值为10000
         */
        private long count;

        /**
         * 列表数据，OPENID的列表
         */
        private String next_openid;

        /**
         * 拉取列表的最后一个用户的OPENID
         */
        private ResParamData data;

    }

    @Data
    public static class ResParamData{

        private List<String> openid;

    }

}
