package com.kingtom.kirin.app.wechat.base.pubnum.service;

import java.util.List;
import java.util.Map;

import com.kingtom.kirin.app.wechat.base.IWeChatChannel;
import com.kingtom.kirin.app.wechat.base.pubnum.config.WeChatPubNumProperties;
import com.kingtom.kirin.app.wechat.base.pubnum.model.UserInfo;
import com.kingtom.kirin.core.common.exception.AppException;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023 /10/31
 */
public interface IWeChatPubNumChannel extends IWeChatChannel{

    /**
     * 获取配置信息
     * 有可能为空
     *
     * @return 配置 we chat pub num properties
     */
    public WeChatPubNumProperties findConfig();

    /**
     * 获取凭证
     *
     * @return token
     * @throws AppException the app exception
     */
    public String findAccessToken() throws AppException;

    /**
     * 获取用户信息
     *
     * @param openId the open id
     * @return 用户信息
     * @throws AppException the app exception
     */
    public UserInfo findUserInfo(String openId) throws AppException;

    /**
     * 批量获取用户信息
     *
     * @param openIds the open ids
     * @return 用户信息 集合
     * @throws AppException the app exception
     */
    public List<UserInfo> findUserInfos(List<String> openIds) throws AppException;

    /**
     * 批量获取关注用户openId集合 最多1000个
     *
     * @param nextOpenId 开始openId,非必填，如果不填，默认从第一个开始查询
     * @return 用户openId 集合
     * @throws AppException the app exception
     */
    public List<String> findUserList(String nextOpenId) throws AppException;

    /**
     * 发送模板消息
     *
     * @param touser     接受者openId
     * @param templateId 模板id
     * @param data       模板数据
     * @param appId      所需跳转到的小程序appid 非必填
     * @param pagePath   所需跳转到的小程序 页面路径 非必填
     * @param url        模板跳转链接（海外账号没有跳转能力） 非必填
     * @param msgId      消息id 用于消息排重 非必填
     * @throws AppException the app exception
     */
    public void sendTemplateMessage(String touser, String templateId, Map<String, String> data, String appId,
            String pagePath, String url, String msgId) throws AppException;

}
