package com.kingtom.kirin.app.wechat.base.pubnum.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import cn.hutool.core.map.MapUtil;
import com.kingtom.kirin.app.wechat.base.IWeChatApi;
import com.kingtom.kirin.app.wechat.base.pubnum.api.IWxPubNumAppIdAware;
import com.kingtom.kirin.app.wechat.base.pubnum.api.IWxPubNumTokenAware;
import com.kingtom.kirin.app.wechat.base.pubnum.api.message.SendTemplateMessage;
import com.kingtom.kirin.app.wechat.base.pubnum.api.token.GetStableToken;
import com.kingtom.kirin.app.wechat.base.pubnum.api.user.GetUserInfo;
import com.kingtom.kirin.app.wechat.base.pubnum.api.user.GetUserInfoBatch;
import com.kingtom.kirin.app.wechat.base.pubnum.api.user.GetUserList;
import com.kingtom.kirin.app.wechat.base.pubnum.config.WeChatPubNumProperties;
import com.kingtom.kirin.app.wechat.base.pubnum.model.UserInfo;
import com.kingtom.kirin.app.wechat.base.pubnum.service.IWeChatPubNumChannel;
import com.kingtom.kirin.app.wechat.service.IWeChatUserService;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.lang.FixedLinkedList;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/10/31
 */
@Service
@Slf4j
public class WechatPubNumImpl implements IWeChatPubNumChannel{

    @Autowired(required = false)
    private WeChatPubNumProperties properties;

    @Autowired
    private IWeChatUserService weChatUserService;

    private FixedLinkedList<String> accessTokens = new FixedLinkedList<String>(CommonConst.INT_5);

    @Override
    public <Q, S> void callApi(IWeChatApi<Q, S> api) throws AppException{
        if(api == null){
            throw new AppException("微信请求不能为空");
        }
        if(!api.isRequestOk()){
            throw new AppException("微信请求构建错误");
        }

        if(api instanceof IWxPubNumAppIdAware){
            ((IWxPubNumAppIdAware) api).setAppId(findConfig().getAppId());
            ((IWxPubNumAppIdAware) api).setAppSecret(findConfig().getAppSecret());
        }
        if(api instanceof IWxPubNumTokenAware){
            ((IWxPubNumTokenAware) api).setToken(findAccessToken());
        }
        log.info("微信公众号接口[" + api.getName() + "]开始发送，json=" + JsonUtils.toJson(api));
        api.call();
        if(api.isResponseOk()){
            log.info("微信公众号接口[" + api.getName() + "]执行成功，json=" + JsonUtils.toJson(api));
        } else{
            log.warn("微信公众号接口[" + api.getName() + "]发生错误，json=" + JsonUtils.toJson(api));
            if(api.isTokenError() && api.findAndAddRetry() < CommonConst.INT_2){
                log.warn("微信公众号接口[" + api.getName() + "]重新尝试[" + api.getRetry() + "]，json=" +
                        JsonUtils.toJson(api));
                refreshAccessToken();
                callApi(api);
            }
        }

    }

    @Override
    public WeChatPubNumProperties findConfig(){
        if(properties == null){
            throw new AppException("微信公众号appId未配置！");
        }
        return properties;
    }

    @Override
    public String findAccessToken() throws AppException{
        if(accessTokens.isEmpty()){
            refreshAccessToken();
        }
        if(accessTokens.isEmpty()){
            throw new AppException("微信accessToken获取失败");
        } else{
            return accessTokens.getLast();
        }
    }

    //@Scheduled(cron = "0 0/5 * * * ?")
    private void refreshAccessToken(){
        log.info("请求微信公众号access_token开始");
        GetStableToken token = new GetStableToken();
        callApi(token);
        if(token.isResponseOk()){
            String t = token.getResponse().getAccess_token();
            accessTokens.add(t);
            log.info("请求微信公众号access_token成功，res=：" + JsonUtils.toJson(token.getResponse()) + "；当前列表：" +
                    JsonUtils.toJson(accessTokens));
        } else{
            log.warn("请求微信公众号access_token失败，res=" + JsonUtils.toJson(token.getResponse()));
        }
    }

    @Override
    public UserInfo findUserInfo(String openId) throws AppException{
        if(StringUtils.isBlank(openId)){
            throw new AppException("参数错误，获取用户信息失败");
        }
        GetUserInfo g = new GetUserInfo(openId);
        callApi(g);
        //更新WechatUser数据
        if(g.isResponseOk()){
            weChatUserService.saveUser(null, findConfig().getAppId(), g.getResponse().getOpenid(),
                    g.getResponse().getUnionid());
        }
        return g.getResponse();
    }

    @Override
    public List<UserInfo> findUserInfos(List<String> openIds) throws AppException{
        if(CollectionUtils.isEmpty(openIds)){
            throw new AppException("参数错误，批量获取用户信息失败");
        }
        GetUserInfoBatch g = new GetUserInfoBatch(openIds);
        callApi(g);
        List<UserInfo> r = g.getResponse().getUser_info_list();
        //更新WechatUser数据
        if(g.isResponseOk()){
            for(UserInfo userInfo : r){
                weChatUserService.saveUser(null, findConfig().getAppId(), userInfo.getOpenid(), userInfo.getUnionid());
            }
        }
        return r;
    }

    @Override
    public List<String> findUserList(String nextOpenId) throws AppException{
        GetUserList g = new GetUserList(nextOpenId);
        callApi(g);
        return g.getResponse().getData() == null ? new ArrayList<>() : g.getResponse().getData().getOpenid();
    }

    @Override
    public void sendTemplateMessage(String touser, String templateId, Map<String, String> data, String appId,
            String pagePath, String url, String msgId) throws AppException{
        if(StringUtils.isAnyBlank(touser, templateId) || MapUtil.isEmpty(data)){
            throw new AppException("参数错误，发送模板消息失败");
        }
        SendTemplateMessage s = new SendTemplateMessage(touser, templateId, data, appId, pagePath, url, msgId);
        callApi(s);
    }

}
