package com.kingtom.kirin.app.wechat.service;

import java.util.List;

import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.app.wechat.model.WeChatUser;
import com.kingtom.kirin.app.wechat.model.WeChatUserExample;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023 /11/1
 */
public interface IWeChatUserService{

    /**
     * Save user we chat user.
     *
     * @param name    the name
     * @param appId   the app id
     * @param openId  the open id
     * @param unionId the union id
     * @return the we chat user
     */
    WeChatUser saveUser(String name, String appId, String openId, String unionId);

    /**
     * Create user we chat user.
     *
     * @param name    the name
     * @param appId   the app id
     * @param openId  the open id
     * @param unionId the union id
     * @return the we chat user
     */
    WeChatUser createUser(String name, String appId, String openId, String unionId);

    /**
     * Update user we chat user.
     *
     * @param name    the name
     * @param appId   the app id
     * @param openId  the open id
     * @param unionId the union id
     * @return the we chat user
     */
    WeChatUser updateUser(String name, String appId, String openId, String unionId);

    /**
     * Find by union id list.
     *
     * @param unionId the union id
     * @return the list
     */
    List<WeChatUser> findByUnionId(String unionId);

    /**
     * Find by app open id we chat user.
     *
     * @param appId  the app id
     * @param openId the open id
     * @return the we chat user
     */
    WeChatUser findByAppOpenId(String appId, String openId);

    /**
     * Find by app union id we chat user.
     *
     * @param appId   the app id
     * @param unionId the union id
     * @return the we chat user
     */
    WeChatUser findByAppUnionId(String appId, String unionId);

    /**
     * Page page info.
     *
     * @param example  the example
     * @param pageNum  the page num
     * @param pageSize the page size
     * @return the page info
     */
    PageInfo<WeChatUser> page(WeChatUserExample example,int pageNum, int pageSize);

    /**
     * 根据微信小程序openId，获取微信公众号用户信息
     *
     * @param miniOpenId the mini open id
     * @return the we chat pub num user
     */
    public WeChatUser findPubNumUserByMiniOpenId(String miniOpenId);

}
