package com.kingtom.kirin.app.wechat.service.impl;

import com.kingtom.kirin.app.system.SystemConst;
import com.kingtom.kirin.app.system.model.AppSystemProperties;
import com.kingtom.kirin.app.system.model.SystemUser;
import com.kingtom.kirin.app.system.model.UserSecurity;
import com.kingtom.kirin.app.system.service.ISystemService;
import com.kingtom.kirin.app.system.service.IUserSecurityService;
import com.kingtom.kirin.app.wechat.WeChatConst;
import com.kingtom.kirin.app.wechat.service.IWeChatSystemUserService;
import com.kingtom.kirin.app.wechat.utils.WechatUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/11/1
 */
@Service
public class WeChatSystemUserServiceImpl implements IWeChatSystemUserService{

    @Autowired
    private ISystemService systemService;

    @Autowired
    private IUserSecurityService userSecurityService;

    @Autowired
    private AppSystemProperties appSystemProperties;

    @Override
    public UserSecurity createWechatMiniUser(String openId){
        //        SystemInfo dbPlat = systemService.findSystemInfoByCode(SystemConst.SYSTEM_DEFAULT_CODE);
        SystemUser user = new SystemUser();
        user.setName(WeChatConst.WECHAT_MINI_USER_NAME_PREFIX + WechatUtils.findCode());
        user.setOtherName(openId);
        //        JObject<SystemUserExt3> j = new JObject<SystemUserExt3>(new SystemUserExt3());
        //        j.getE().setPassword(appSystemProperties.getSuperPassword());
        //        j.getE().setUsername(openId);
        //        user.setExt3(j);
        //        user.setSysId(dbPlat.getId());
        SystemUser systemUser = systemService.createSystemUser(user);
        UserSecurity wechatMini = new UserSecurity();
        wechatMini.setUserId(systemUser.getId());
        wechatMini.setType(SystemConst.UserSecurityTypeEnum.WeChat.getKey());
        wechatMini.setPrincipal(openId);
        wechatMini.setCredentials(SystemConst.AuthcEncodeEnum.No.encode(appSystemProperties.getDefaultPassword()));
        wechatMini.setEncode(SystemConst.AuthcEncodeEnum.No.getKey());
        wechatMini = userSecurityService.createUserSecurity(wechatMini);
        return wechatMini;
    }

    @Override
    public SystemUser updateWechatMiniUser(String userId, String phone){
        SystemUser currentUser = systemService.findSystemUserById(userId);
        //检查当前手机号是否已经存在用户
        SystemUser phoneUser = systemService.findSystemUserByPhone(phone);
        if(phoneUser != null){
            if(phoneUser.getId().equals(userId)){
                //绑定关系正确，就检查security信息加入UserName登录
                UserSecurity wechatMini = new UserSecurity();
                wechatMini.setUserId(currentUser.getId());
                wechatMini.setType(SystemConst.UserSecurityTypeEnum.UserName.getKey());
                wechatMini.setPrincipal(phone);
                wechatMini.setCredentials(
                        SystemConst.AuthcEncodeEnum.Md5.encode(appSystemProperties.getDefaultPassword()));
                wechatMini.setEncode(SystemConst.AuthcEncodeEnum.Md5.getKey());
                userSecurityService.createUserSecurity(wechatMini);
                return currentUser;
            } else{
                //绑定关系错误，就删除currentUser对应的用户，并依据phoneUser完善信息
                // 完善信息：查找currentUser的微信信息
                UserSecurity currentWechatMini = userSecurityService.findUserSecurityByType(currentUser.getId(),
                        SystemConst.UserSecurityTypeEnum.WeChat);
                // 完善信息：把微信信息绑定到phoneUser上去
                UserSecurity wechatMini = new UserSecurity();
                wechatMini.setUserId(phoneUser.getId());
                wechatMini.setType(SystemConst.UserSecurityTypeEnum.WeChat.getKey());
                wechatMini.setPrincipal(currentWechatMini.getPrincipal());
                wechatMini.setCredentials(appSystemProperties.getDefaultPassword());
                wechatMini.setEncode(SystemConst.AuthcEncodeEnum.No.getKey());
                wechatMini = userSecurityService.createUserSecurity(wechatMini);
                // 删除currentUser
                systemService.deleteSystemUser(currentUser.getId(), Boolean.TRUE);
                return phoneUser;
            }
        } else{
            //更新用户的phone信息
            currentUser.getExt3().getE().setUsername(phone);
            currentUser.getResAttr().getE().setPhone(phone);
            systemService.updateSystemUser(currentUser);
            return currentUser;
        }
    }

}
