package com.kingtom.kirin.app.wechat.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.app.system.utils.SystemUtils;
import com.kingtom.kirin.app.wechat.dao.WeChatUserMapper;
import com.kingtom.kirin.app.wechat.model.WeChatUser;
import com.kingtom.kirin.app.wechat.model.WeChatUserExample;
import com.kingtom.kirin.app.wechat.service.IWeChatUserService;
import com.kingtom.kirin.app.wechat.utils.WechatUtils;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.IDUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/11/1
 */
@Service
public class WechatUserServiceImpl implements IWeChatUserService{

    @Autowired
    private WeChatUserMapper mapper;

    @Override
    public WeChatUser saveUser(String name, String appId, String openId, String unionId){
        WeChatUser weChatUser = findByAppOpenId(appId, openId);
        if(weChatUser == null){
            return createUser(name, appId, openId, unionId);
        } else{
            return updateUser(name, appId, openId, unionId);
        }
    }

    @Override
    public WeChatUser createUser(String name, String appId, String openId, String unionId){
        WeChatUser weChatUser = findByAppOpenId(appId, openId);
        if(weChatUser != null){
            throw new AppException("微信用户已存在！");
        }
        weChatUser = new WeChatUser();
        weChatUser.setId(IDUtils.newUUID());
        weChatUser.setName(name);
        weChatUser.setAppId(appId);
        weChatUser.setOpenId(openId);
        weChatUser.setUnionId(unionId);
        weChatUser.setGmtCreate(new Date());
        weChatUser.setGmtModified(weChatUser.getGmtCreate());
        weChatUser.setSysId(SystemUtils.getCurrentSystemId());
        int n = mapper.insertSelective(weChatUser);
        if(n != 1){
            throw new AppException("落库失败，新增微信用户失败！");
        }
        return findByAppOpenId(appId, openId);
    }

    @Override
    public WeChatUser updateUser(String name, String appId, String openId, String unionId){
        WeChatUser weChatUser = findByAppOpenId(appId, openId);
        if(weChatUser == null){
            throw new AppException("微信用户不存在！");
        }
        if((name == null || name.equals(weChatUser.getName())) &&
                (unionId == null || unionId.equals(weChatUser.getUnionId()))){
            return weChatUser;
        }
        WeChatUser update = new WeChatUser();
        update.setId(weChatUser.getId());
        update.setName(name);
        update.setAppId(appId);
        update.setOpenId(openId);
        update.setUnionId(unionId);
        update.setGmtModified(new Date());
        int n = mapper.updateByPrimaryKeySelective(update);
        if(n != 1){
            throw new AppException("落库失败，新增微信用户失败！");
        }
        return findByAppOpenId(appId, openId);
    }

    @Override
    public List<WeChatUser> findByUnionId(String unionId){
        if(StringUtils.isBlank(unionId)){
            return new ArrayList<>();
        }
        WeChatUserExample example = new WeChatUserExample();
        example.createCriteria().andUnionIdEqualTo(unionId);
        return mapper.selectByExample(example);
    }

    @Override
    public WeChatUser findByAppOpenId(String appId, String openId){
        if(StringUtils.isAnyBlank(appId, openId)){
            return null;
        }
        WeChatUserExample example = new WeChatUserExample();
        example.createCriteria().andAppIdEqualTo(appId).andOpenIdEqualTo(openId);
        List<WeChatUser> list = mapper.selectByExample(example);
        return CollectionUtils.isEmpty(list) ? null : list.get(0);
    }

    @Override
    public WeChatUser findByAppUnionId(String appId, String unionId){
        if(StringUtils.isAnyBlank(appId, unionId)){
            return null;
        }
        WeChatUserExample example = new WeChatUserExample();
        example.createCriteria().andAppIdEqualTo(appId).andUnionIdEqualTo(unionId);
        List<WeChatUser> list = mapper.selectByExample(example);
        return CollectionUtils.isEmpty(list) ? null : list.get(0);
    }

    @Override
    public PageInfo<WeChatUser> page(WeChatUserExample example, int pageNum, int pageSize){
        if(example == null){
            example = new WeChatUserExample();
        }
        PageHelper.startPage(pageNum, pageSize);
        List<WeChatUser> list = mapper.selectByExample(example);
        return new PageInfo<>(list);
    }

    @Override
    public WeChatUser findPubNumUserByMiniOpenId(String miniOpenId){
        String miniAppId = WechatUtils.findMiniAppId();
        if(StringUtils.isAnyBlank(miniAppId, miniOpenId)){
            return null;
        }
        WeChatUser miniUser = findByAppOpenId(miniAppId, miniOpenId);
        if(miniUser == null || StringUtils.isBlank(miniUser.getUnionId())){
            return null;
        }
        String pubNumAppId = WechatUtils.findPubNumAppId();
        if(StringUtils.isBlank(pubNumAppId)){
            return null;
        }
        WeChatUser weChatUser1 = findByAppUnionId(pubNumAppId, miniUser.getUnionId());
        if(weChatUser1 == null || StringUtils.isBlank(weChatUser1.getOpenId())){
            return null;
        }
        return weChatUser1;
    }

}
