-- ----------------------------
-- Table structure for auth_role
-- ----------------------------
DROP TABLE IF EXISTS "public"."auth_role";
CREATE TABLE "public"."auth_role" (
    "id"           VARCHAR(36) COLLATE "pg_catalog"."default"   NOT NULL,
    "name"         VARCHAR(1024) COLLATE "pg_catalog"."default" NOT NULL,
    "code"         VARCHAR(64) COLLATE "pg_catalog"."default"   NOT NULL,
    "source" jsonb,
    "display" int4 NOT NULL,
    "state"        int4                                         NOT NULL,
    "ext1"         VARCHAR(1024) COLLATE "pg_catalog"."default",
    "ext2"         VARCHAR(1024) COLLATE "pg_catalog"."default",
    "ext3"         jsonb,
    "gmt_create"   TIMESTAMP(3),
    "gmt_modified" TIMESTAMP(3),
    "sys_id"       VARCHAR(36) COLLATE "pg_catalog"."default"   NOT NULL
);
COMMENT ON TABLE "public"."auth_role" IS '授权角色表';
COMMENT ON COLUMN "public"."auth_role"."id" IS 'id';
COMMENT ON COLUMN "public"."auth_role"."name" IS '名称';
COMMENT ON COLUMN "public"."auth_role"."code" IS '角色码';
COMMENT ON COLUMN "public"."auth_role"."source" IS '绑定的资源信息';
COMMENT ON COLUMN "public"."auth_role"."display" IS '是否显示';
COMMENT ON COLUMN "public"."auth_role"."state" IS '角色状态';
COMMENT ON COLUMN "public"."auth_role"."ext1" IS '扩展字段1';
COMMENT ON COLUMN "public"."auth_role"."ext2" IS '扩展字段2';
COMMENT ON COLUMN "public"."auth_role"."ext3" IS '扩展字段3';
COMMENT ON COLUMN "public"."auth_role"."gmt_create" IS '创建时间';
COMMENT ON COLUMN "public"."auth_role"."gmt_modified" IS '更新时间';
COMMENT ON COLUMN "public"."auth_role"."sys_id" IS '系统ID';

-- ----------------------------
-- Primary Key structure for table auth_role
-- ----------------------------
ALTER TABLE "public"."auth_role" ADD CONSTRAINT "auth_role_pkey" PRIMARY KEY ("id");


-- ----------------------------
-- Table structure for auth_menu
-- ----------------------------
DROP TABLE IF EXISTS "public"."auth_menu";
CREATE TABLE "public"."auth_menu" (
    "id"           VARCHAR(36) COLLATE "pg_catalog"."default"   NOT NULL,
    "name"         VARCHAR(1024) COLLATE "pg_catalog"."default" NOT NULL,
    "other_name"  VARCHAR(1024) COLLATE "pg_catalog"."default",
    "code"         VARCHAR(64) COLLATE "pg_catalog"."default"   NOT NULL,
    "parent_id"    VARCHAR(36) COLLATE "pg_catalog"."default"   NOT NULL,
    "route"       jsonb,
    "hide"        int4,
    "preset"      int4,
    "order_num"   int4,
    "type"         VARCHAR(64) COLLATE "pg_catalog"."default"   NOT NULL,
    "action_type" VARCHAR(64) COLLATE "pg_catalog"."default",
    "icon"        VARCHAR(64) COLLATE "pg_catalog"."default",
    "route_uri"   VARCHAR(1024) COLLATE "pg_catalog"."default",
    "ext1"         VARCHAR(1024) COLLATE "pg_catalog"."default",
    "ext2"         VARCHAR(1024) COLLATE "pg_catalog"."default",
    "ext3"         jsonb,
    "gmt_create"   TIMESTAMP(3),
    "gmt_modified" TIMESTAMP(3),
    "sys_id"       VARCHAR(36) COLLATE "pg_catalog"."default"   NOT NULL
);
COMMENT ON TABLE "public"."auth_menu" IS '授权菜单表';
COMMENT ON COLUMN "public"."auth_menu"."id" IS 'id';
COMMENT ON COLUMN "public"."auth_menu"."name" IS '名称';
COMMENT ON COLUMN "public"."auth_menu"."other_name" IS '别名';
COMMENT ON COLUMN "public"."auth_menu"."code" IS '标识符';
COMMENT ON COLUMN "public"."auth_menu"."parent_id" IS '父菜单id';
COMMENT ON COLUMN "public"."auth_menu"."route" IS '路径';
COMMENT ON COLUMN "public"."auth_menu"."hide" IS '是否隐藏菜单';
COMMENT ON COLUMN "public"."auth_menu"."preset" IS '是否预置菜单';
COMMENT ON COLUMN "public"."auth_menu"."order_num" IS '排序号';
COMMENT ON COLUMN "public"."auth_menu"."type" IS '菜单类型：目录、菜单，按钮';
COMMENT ON COLUMN "public"."auth_menu"."action_type" IS '动作类型';
COMMENT ON COLUMN "public"."auth_menu"."icon" IS '菜单图标';
COMMENT ON COLUMN "public"."auth_menu"."route_uri" IS '路由地址';
COMMENT ON COLUMN "public"."auth_menu"."ext1" IS '扩展字段1';
COMMENT ON COLUMN "public"."auth_menu"."ext2" IS '扩展字段2';
COMMENT ON COLUMN "public"."auth_menu"."ext3" IS '扩展字段3';
COMMENT ON COLUMN "public"."auth_menu"."gmt_modified" IS '更新时间';
COMMENT ON COLUMN "public"."auth_menu"."sys_id" IS '系统ID';

-- ----------------------------
-- Primary Key structure for table auth_menu
-- ----------------------------
ALTER TABLE "public"."auth_menu" ADD CONSTRAINT "auth_menu_pkey" PRIMARY KEY ("id");


-- ----------------------------
-- Table structure for auth_permission
-- ----------------------------
DROP TABLE IF EXISTS "public"."auth_permission";
CREATE TABLE "public"."auth_permission" (
    "id"           VARCHAR(36) COLLATE "pg_catalog"."default"  NOT NULL,
    "code"         VARCHAR(256) COLLATE "pg_catalog"."default" NOT NULL,
    "res_type"     VARCHAR(64) COLLATE "pg_catalog"."default"  NOT NULL,
    "res_id"       VARCHAR(36) COLLATE "pg_catalog"."default",
    "opera_type"   VARCHAR(64) COLLATE "pg_catalog"."default"  NOT NULL,
    "role_id"      VARCHAR(36) COLLATE "pg_catalog"."default"  NOT NULL,
    "ext1"         VARCHAR(1024) COLLATE "pg_catalog"."default",
    "ext2"         VARCHAR(1024) COLLATE "pg_catalog"."default",
    "ext3"         jsonb,
    "gmt_create"   TIMESTAMP(3),
    "gmt_modified" TIMESTAMP(3),
    "sys_id"       VARCHAR(36) COLLATE "pg_catalog"."default"  NOT NULL
);
COMMENT ON TABLE "public"."auth_permission" IS '授权权限表';
COMMENT ON COLUMN "public"."auth_permission"."id" IS 'id';
COMMENT ON COLUMN "public"."auth_permission"."code" IS '权限码';
COMMENT ON COLUMN "public"."auth_permission"."res_type" IS '资源类型';
COMMENT ON COLUMN "public"."auth_permission"."res_id" IS '资源id:';
COMMENT ON COLUMN "public"."auth_permission"."opera_type" IS '操作类型：新增、修改、查看、删除、执行等';
COMMENT ON COLUMN "public"."auth_permission"."role_id" IS '角色Id';
COMMENT ON COLUMN "public"."auth_permission"."ext1" IS '扩展字段1';
COMMENT ON COLUMN "public"."auth_permission"."ext2" IS '扩展字段2';
COMMENT ON COLUMN "public"."auth_permission"."ext3" IS '扩展字段3';
COMMENT ON COLUMN "public"."auth_permission"."gmt_create" IS '创建时间';
COMMENT ON COLUMN "public"."auth_permission"."gmt_modified" IS '更新时间';
COMMENT ON COLUMN "public"."auth_permission"."sys_id" IS '系统ID';

-- ----------------------------
-- Primary Key structure for table auth_permission
-- ----------------------------
ALTER TABLE "public"."auth_permission" ADD CONSTRAINT "auth_permission_pkey" PRIMARY KEY ("id");