-- ----------------------------
-- Table structure for affair_info
-- ----------------------------
DROP TABLE IF EXISTS "public"."affair_info";
CREATE TABLE "public"."affair_info" (
    "id" varchar(36) COLLATE "pg_catalog"."default" NOT NULL,
    "name" varchar(1024) COLLATE "pg_catalog"."default" NOT NULL,
    "full_name" varchar(65534) COLLATE "pg_catalog"."default",
    "group_name" varchar(1024) COLLATE "pg_catalog"."default",
    "affair_token" varchar(65534) COLLATE "pg_catalog"."default",
    "affair_url" varchar(1024) COLLATE "pg_catalog"."default",
    "action_model" VARCHAR(1024) COLLATE "pg_catalog"."default" NOT NULL,
    "important" varchar(1024) COLLATE "pg_catalog"."default" NOT NULL,
    "urgent" varchar(1024) COLLATE "pg_catalog"."default" NOT NULL,
    "res_type" varchar(64) COLLATE "pg_catalog"."default" NOT NULL,
    "res_id" varchar(36) COLLATE "pg_catalog"."default" NOT NULL,
    "ext1" varchar(65534) COLLATE "pg_catalog"."default",
    "ext2" varchar(65534) COLLATE "pg_catalog"."default",
    "gmt_create" timestamp(3) NOT NULL,
    "gmt_modified" timestamp(3) NOT NULL,
    "sys_id" varchar(36) COLLATE "pg_catalog"."default" NOT NULL
)
;
COMMENT ON COLUMN "public"."affair_info"."id" IS '主键';
COMMENT ON COLUMN "public"."affair_info"."name" IS '事项名称';
COMMENT ON COLUMN "public"."affair_info"."full_name" IS '事项全称';
COMMENT ON COLUMN "public"."affair_info"."group_name" IS '事项分组名';
COMMENT ON COLUMN "public"."affair_info"."affair_token" IS '事项对接业务时的关键字（由业务识别）';
COMMENT ON COLUMN "public"."affair_info"."affair_url" IS '事项对接的URL';
COMMENT ON COLUMN "public"."affair_info"."action_model" IS '动作模式';
COMMENT ON COLUMN "public"."affair_info"."important" IS '重要程度';
COMMENT ON COLUMN "public"."affair_info"."urgent" IS '紧急程度';
COMMENT ON COLUMN "public"."affair_info"."res_type" IS '事项对接业务类型';
COMMENT ON COLUMN "public"."affair_info"."res_id" IS '事项对接业务ID';
COMMENT ON COLUMN "public"."affair_info"."ext1" IS '扩展字段1';
COMMENT ON COLUMN "public"."affair_info"."ext2" IS '扩展字段2';
COMMENT ON COLUMN "public"."affair_info"."gmt_create" IS '创建时间';
COMMENT ON COLUMN "public"."affair_info"."gmt_modified" IS '更新时间';
COMMENT ON COLUMN "public"."affair_info"."sys_id" IS '系统ID';
COMMENT ON TABLE "public"."affair_info" IS '事项表';

-- ----------------------------
-- Primary Key structure for table affair_info
-- ----------------------------
ALTER TABLE "public"."affair_info" ADD CONSTRAINT "affair_info_pkey" PRIMARY KEY ("id");

-- ----------------------------
-- Table structure for affair_token
-- ----------------------------
DROP TABLE IF EXISTS "public"."affair_token";
CREATE TABLE "public"."affair_token" (
    "id" varchar(36) COLLATE "pg_catalog"."default" NOT NULL,
    "affair_id" varchar(1024) COLLATE "pg_catalog"."default" NOT NULL,
    "token" varchar(65534) COLLATE "pg_catalog"."default",
    "effect_model" VARCHAR(1024) COLLATE "pg_catalog"."default" NOT NULL,
    "effect_condition" VARCHAR(65534) COLLATE "pg_catalog"."default",
    "res_type" VARCHAR(1024) COLLATE "pg_catalog"."default" NOT NULL,
    "res_id" VARCHAR(1024) COLLATE "pg_catalog"."default" NOT NULL,
    "ext1" varchar(65534) COLLATE "pg_catalog"."default",
    "ext2" varchar(65534) COLLATE "pg_catalog"."default",
    "state" int4 NOT NULL,
    "gmt_create" timestamp(3) NOT NULL,
    "gmt_modified" timestamp(3) NOT NULL,
    "sys_id" varchar(36) COLLATE "pg_catalog"."default" NOT NULL
);
COMMENT ON COLUMN "public"."affair_token"."id" IS '主键';
COMMENT ON COLUMN "public"."affair_token"."affair_id" IS '事项ID';
COMMENT ON COLUMN "public"."affair_token"."token" IS '对接业务时身份专用信息（由业务识别）';
COMMENT ON COLUMN "public"."affair_token"."effect_model" IS '生效模式';
COMMENT ON COLUMN "public"."affair_token"."effect_condition" IS '生效条件';
COMMENT ON COLUMN "public"."affair_token"."res_type" IS '身份对应的实体类型';
COMMENT ON COLUMN "public"."affair_token"."res_id" IS '身份对应的实体ID';
COMMENT ON COLUMN "public"."affair_token"."state" IS '状态';
COMMENT ON COLUMN "public"."affair_token"."ext1" IS '扩展字段1';
COMMENT ON COLUMN "public"."affair_token"."ext2" IS '扩展字段2';
COMMENT ON COLUMN "public"."affair_token"."gmt_create" IS '创建时间';
COMMENT ON COLUMN "public"."affair_token"."gmt_modified" IS '更新时间';
COMMENT ON COLUMN "public"."affair_token"."sys_id" IS '系统ID';
COMMENT ON TABLE "public"."affair_token" IS '事项凭证表';

-- ----------------------------
-- Primary Key structure for table affair_token
-- ----------------------------
ALTER TABLE "public"."affair_token" ADD CONSTRAINT "affair_token_pkey" PRIMARY KEY ("id");

-- ----------------------------
-- Table structure for affair_member
-- ----------------------------
DROP TABLE IF EXISTS "public"."affair_member";
CREATE TABLE "public"."affair_member" (
    "id" varchar(36) COLLATE "pg_catalog"."default" NOT NULL,
    "affair_id" varchar(1024) COLLATE "pg_catalog"."default" NOT NULL,
    "token_id" varchar(1024) COLLATE "pg_catalog"."default" NOT NULL,
    "user_id" VARCHAR(1024) COLLATE "pg_catalog"."default" NOT NULL,
    "state" int4 NOT NULL,
    "ext1" varchar(65534) COLLATE "pg_catalog"."default",
    "ext2" varchar(65534) COLLATE "pg_catalog"."default",
    "gmt_create" timestamp(3) NOT NULL,
    "gmt_modified" timestamp(3) NOT NULL,
    "sys_id" varchar(36) COLLATE "pg_catalog"."default" NOT NULL
);
COMMENT ON COLUMN "public"."affair_member"."id" IS '主键';
COMMENT ON COLUMN "public"."affair_member"."affair_id" IS '事项ID';
COMMENT ON COLUMN "public"."affair_member"."token_id" IS '事项凭证ID';
COMMENT ON COLUMN "public"."affair_member"."user_id" IS '具体用户ID';
COMMENT ON COLUMN "public"."affair_member"."state" IS '状态';
COMMENT ON COLUMN "public"."affair_member"."ext1" IS '扩展字段1';
COMMENT ON COLUMN "public"."affair_member"."ext2" IS '扩展字段2';
COMMENT ON COLUMN "public"."affair_member"."gmt_create" IS '创建时间';
COMMENT ON COLUMN "public"."affair_member"."gmt_modified" IS '更新时间';
COMMENT ON COLUMN "public"."affair_member"."sys_id" IS '系统ID';
COMMENT ON TABLE "public"."affair_member" IS '事项成员表';

-- ----------------------------
-- Primary Key structure for table affair_member
-- ----------------------------
ALTER TABLE "public"."affair_member" ADD CONSTRAINT "affair_member_pkey" PRIMARY KEY ("id");