package com.kingtom.kirin.core.common.base;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import com.kingtom.kirin.core.common.utils.BaseEnumUtils;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import com.kingtom.kirin.core.common.utils.RefUtils;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/5 15:01
 */
public interface IBaseEnum<K> {
    /**
     * 获取枚举的唯一码
     * @return
     */
    public K getKey();

    /**
     * 获取枚举名称
     * @return
     */
    public String getName();

    /**
     * 获取本枚举定义的其他值
     * @return
     */
    default Map<String, Object> getOther() {
        return null;
    }

    /**
     * 获取本枚举定义的所有的值
     * @return
     */
    default Map<String, Object> getAllAttribute() {
        Map<String, Method> methoMap = RefUtils.getGetterMap(this.getClass());
        Map<String, Object> map = new HashMap<>(2);
        String[] excludeKey = new String[] { "declaringClass", "class" };
        methoMap.entrySet().stream().forEach(e -> {
            if ((!excludeKey[0].equals(e.getKey())) && (!excludeKey[1].equals(e.getKey()))) {
                map.put(e.getKey(), RefUtils.invokeMethod(e.getValue(), this, null));
            }

        });
        return map;
    }

    /**
     * 根据key获取枚举值
     * @param ec 枚举类
     * @param key 枚举值
     * @return e
     */
    public static <K, E extends IBaseEnum<K>> E findEnumByKey(Class<E> ec, K key) {
        Map<K,E> m = BaseEnumUtils.findEnumKeyMap(ec);
        E r = key == null?null:m.get(key);
        return r;
    }

    /**
     * 根据key列表获取枚举列表
     * @param ec
     * @param keys
     * @param <K>
     * @param <E>
     * @return
     */
    public static <K, E extends IBaseEnum<K>> List<E> findEnumByKeys(Class<E> ec, List<K> keys) {
        if (keys == null || keys.size() == 0) {
            return null;
        }
        List<E> r = CollUtil.newArrayList();
        for(K k:keys) {
            r.add(findEnumByKey(ec, k));
        }
        return r;
    }

    /**
     * 根据枚举列表获取枚举的key列表
     * @param enums
     * @param <K>
     * @param <E>
     * @return
     */
    public static <K, E extends IBaseEnum<K>> List<K> getKeyByEnumList(List<E> enums) {
        if (enums == null || enums.size() == 0) {
            return null;
        }
        List<K> r = CollUtil.newArrayList();
        for(E e:enums) {
            r.add(e.getKey());
        }
        return r;
    }

    /**
     * 根据name获取枚举值
     * @param ec
     * @param name
     * @return
     */
    public static <E extends IBaseEnum<?>> E findEnumByName(Class<E> ec, String name) {
        if (Enum.class.isAssignableFrom(ec)) {
            E[] es = ec.getEnumConstants();
            for (E e : es) {
                if (e.getName().equals(name)) {
                    return e;
                }
            }
        }
        return null;
    }

    /**
     * 根据枚举值获取枚举(全小写来比对)
     * @param ec
     * @param value
     * @return
     */
    public static <E extends IBaseEnum<?>> E findEnumByValue(Class<E> ec, String value) {
        if (ec.isAssignableFrom(Enum.class) && value != null) {
            E[] es = ec.getEnumConstants();
            String lv = value.toLowerCase();
            for (E e : es) {
                if (e.toString().toLowerCase().equals(lv)) {
                    return e;
                }
            }
        }
        return null;
    }

    /**
     * 转换为key,name的jsonString
     * @param ec
     * @param <E>
     * @return
     */
    public static <E extends IBaseEnum<?>> String translateToJson(Class<E> ec) {
        LinkedHashMap<Object, Object> map = new LinkedHashMap<>();
        E[] es = ec.getEnumConstants();
        if (es != null) {
            for (E e : es) {
                map.put(e.getKey(), e.getName());
            }
        }
        return JsonUtils.toJson(map);
    }
}
