package com.kingtom.kirin.core.common.base;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 * 统一返回对象
 * @author King
 * @since 2023/6/1 11:09
 */
public class Result<T> {

    private Integer code;

    private String message;

    private T data;

    private Result(Integer code, String message, T data) {
        this.code = code;
        this.message = message;
        this.data = data;
    }

    public Integer getCode() {
        return code;
    }

    public void setCode(Integer code) {
        this.code = code;
    }

    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    public T getData() {
        return data;
    }

    public void setData(T data) {
        this.data = data;
    }

    public boolean checkOk() {
        return ResultCodeEnum.Success.getKey().equals(this.code);
    }

    public static <T> Result<T> ok(T data){
        return ok(ResultCodeEnum.Success.getName(), data);
    }

    public static <T> Result<T> ok(String msg, T data){
        return new Result<T>(ResultCodeEnum.Success.getKey(), msg, data);
    }

    public static <T> Result<T> fail(String msg) {
        return new Result<T>(ResultCodeEnum.ServerError.getKey(), msg, null);
    }

    public static <T> Result<T> clientfail(String msg) {
        return new Result<T>(ResultCodeEnum.ClientError.getKey(), msg, null);
    }

    public static <T> Result<T> build(ResultCodeEnum errorCodeEnum, String message, T data){
        if(errorCodeEnum == null){
            errorCodeEnum = ResultCodeEnum.ServerError;
        }
        return new Result<T>(errorCodeEnum.getKey(), message, data);
    }


}
