package com.kingtom.kirin.core.common.lang;

import java.util.Collection;
import java.util.LinkedList;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/4 14:59
 */
public class FixedLinkedList<E> extends LinkedList<E> {

    private int fixedSize;

    /**
     * 0-保持固定长度的模式：列首先出（适用于add|offer+getLast）
     * 1-保持固定长度的模式：列尾先出（适用于push+getFirst|peek）
     */
    private int model = 0;

    public static int MODEL_0 = 0;
    public static int MODEL_1 = 1;

    public FixedLinkedList(int fixedSize) {
        this(fixedSize,MODEL_0);
    }

    public FixedLinkedList(int fixedSize, int model) {
        super();
        setFixedSize(fixedSize);
        setModel(model);
    }

    protected void setFixedSize(int fixedSize) {
        if(fixedSize <= 0) {
            throw new IllegalArgumentException("fixedSize必须大于0，fixedSize="+fixedSize);
        }
        this.fixedSize = fixedSize;
    }

    protected void setModel(int model) {
        if(model != MODEL_0 && model != MODEL_1) {
            throw new IllegalArgumentException("model只能是0或1，model="+model);
        }
        this.model = model;
    }

    @Override
    public void push(E e) {
        super.push(e);
        fixed();
    }

    @Override
    public boolean add(E e) {
        boolean r = super.add(e);
        fixed();
        return r;
    }

    @Override
    public void addFirst(E e) {
        super.addFirst(e);
        fixed();
    }

    @Override
    public void addLast(E e) {
        super.addLast(e);
        fixed();
    }

    @Override
    public boolean addAll(Collection<? extends E> c) {
        boolean r = super.addAll(c);
        fixed();
        return r;
    }

    @Override
    public boolean addAll(int index, Collection<? extends E> c) {
        boolean r = super.addAll(index, c);
        return r;
    }

    @Override
    public void add(int index, E element) {
        super.add(index, element);
        fixed();
    }

    private void fixed() {
        // 如果大小超标，持续移除直到符合
        while(size() > fixedSize) {
            if(model == MODEL_0) {
                this.removeFirst();
            } else {
                this.removeLast();
            }
        }
    }
}
