package com.kingtom.kirin.core.common.utils;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import com.kingtom.kirin.core.common.base.IBaseEnum;
import com.kingtom.kirin.core.common.exception.AppException;
import lombok.extern.slf4j.Slf4j;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/5 15:02
 */
@Slf4j
public class BaseEnumUtils {

    private static final Map<Class, Map> map = MapUtil.newHashMap();

    /**
     * 查缓存
     * @param <K>
     * @param <E>
     * @param c
     * @return
     */
    public static <K,E extends IBaseEnum<K>> Map<K,E> findEnumKeyMap(Class<E> c) {
        Map r = map.get(c);
        if(r == null) {
            synchronized (BaseEnumUtils.class) {
                if(Enum.class.isAssignableFrom(c)) {
                    Map<K,E> m = MapUtil.newHashMap();
                    E[] es = c.getEnumConstants();
                    for(E e:es) {
                        m.put(e.getKey(), e);
                    }
                    map.put(c, m);
                    r = map.get(c);
                } else {
                    throw new RuntimeException("不是标准的枚举对象");
                }
            }
        }
        return (Map<K,E>)r;
    }

    public static <K,E extends IBaseEnum<K>> List<E> findEnumItemList(Class<E> c) {
        Map<K,E> m = findEnumKeyMap(c);
        return m == null?null:ListUtil.toList(m.values());
    }

    public static <K,E extends IBaseEnum<K>> List<E> findEnumItemList(String cname) {
        try{
            Class c = Class.forName(cname);
            return findEnumItemList(c);
        } catch (Exception e) {
            throw new AppException("获取枚举异常",e);
        }
    }

}
