package com.kingtom.kirin.core.common.utils;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.ArrayUtil;
import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.core.common.CommonConst;

import java.util.*;
import java.util.function.BiFunction;
import java.util.function.Function;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/10 14:42
 */
public class CollectionUtils {

    private CollectionUtils(){}

    public static <E> boolean isEmpty(Collection<E> c) {
        return org.springframework.util.CollectionUtils.isEmpty(c);
    }

    public static <E> boolean isNotEmpty(Collection<E> c) {
        return !isEmpty(c);
    }

    /**
     * 一种元素集合转另外一种元素集合
     * @param c
     * @param f
     * @return
     * @param <E>
     * @param <R>
     */
    public static <E,R> List<R> convert(Collection<E> c, Function<E,R> f) {
        if(ObjectUtils.isNotNull(c)) {
            List<R> r = CollUtil.newArrayList();
            for(E e:c) {
                r.add(f.apply(e));
            }
            return r;
        }
        return null;
    }

    public static <R> List<R> filter(Collection<R> c, Function<R,Boolean> f) {
        if(ObjectUtils.isNotNull(c)) {
            List<R> r = CollUtil.newArrayList();
            for(R e:c) {
                if(f.apply(e)){
                    r.add(e);
                }
            }
            return r;
        }
        return null;
    }

    /**
     * 查找集合中第一个元素
     * @param c
     * @return
     * @param <R>
     */
    public static <R> R findFirst(Collection<R> c) {
        if(c == null || c.isEmpty()) {
            return null;
        } else {
            List<R> tc = ListUtil.toList(c);
            return tc.get(CommonConst.INT_0);
        }
    }

    public static <R> R findLast(Collection<R> c) {
        if(c == null || c.isEmpty()) {
            return null;
        } else {
            List<R> tc = ListUtil.toList(c);
            return tc.get(tc.size()-CommonConst.INT_1);
        }
    }

    public static <K,E> Map<K,List<E>> toGroupMap(Collection<E> c, Function<E,K> keyfunc) {
        if(ObjectUtils.isNotNull(c)) {
            Map<K,List<E>> r = new LinkedHashMap<>();
            for(E e:c) {
                K k = keyfunc.apply(e);
                r.computeIfAbsent(k, key->new ArrayList<>()).add(e);
            }
            return r;
        }
        return null;
    }

    public static <K,E,V> Map<K,V> convertMap(Collection<E> c, Function<E,K> keyfunc, Function<E,V> valfunc) {
        if(ObjectUtils.isNotNull(c)) {
            Map<K,V> r = new LinkedHashMap<>();
            for(E e:c) {
                K k = keyfunc.apply(e);
                V v = valfunc.apply(e);
                r.put(k,v);
            }
            return r;
        }
        return null;
    }

    /**
     * 集合分析
     * 1.比较collection1,collection2，得出collection2相对于collection1的增加、减少、修改
     * 2.r.get(0)=collection2中修改的；r.get(1)=collection2中删除的；r.get(2)=collection2中新增的；
     * @param collection1 老数据
     * @param collection2 新数据
     * @param c 比较器，判断两个对象是不是同一个对象(最简单的情况下用equals)
     * @return
     */
    public static <T> List<List<T>> analyzeCollection(Collection<T> collection1, Collection<T> collection2, BiFunction<T, T, Boolean> c) {
        List<List<T>> r = new ArrayList<>();
        r.add(new ArrayList<T>());
        r.add(new ArrayList<T>());
        r.add(new ArrayList<T>());
        if(!org.springframework.util.CollectionUtils.isEmpty(collection1)) {
            r.get(1).addAll(collection1);
        }
        if(!org.springframework.util.CollectionUtils.isEmpty(collection2)) {
            r.get(2).addAll(collection2);
        }
        BiFunction<T, T, Boolean> tc = c == null?(e1,e2)->e1.equals(e2):c;
        if(!org.springframework.util.CollectionUtils.isEmpty(collection2) && !org.springframework.util.CollectionUtils.isEmpty(collection1)){
            collection1.forEach(e1->{
                collection2.forEach(e2->{
                    if(tc.apply(e1, e2)) {
                        r.get(CommonConst.INT_0).add(e2);
                        r.get(CommonConst.INT_1).remove(e1);
                        r.get(CommonConst.INT_2).remove(e2);
                    }
                });
            });
        }
        return r;
    }

    /**
     * 把一堆集合里的数据融合成一个不重复的集合
     * @param collections
     * @return
     */
    @SuppressWarnings("unchecked")
    public static <T> List<T> mergeCollection(Collection<T>...collections) {
        Set<T> r = Set.of();
        if(ArrayUtil.isNotEmpty(collections)) {
            for(Collection<T> c:collections) {
                if(CollectionUtil.isNotEmpty(c)) {
                    c.forEach(e->r.add(e));
                }
            }
        }
        return CollectionUtil.newArrayList(r);
    }

    /**
     * 判断是不是某个类型的集合
     * @param <T>
     * @param l
     * @param c
     * @return
     */
    public static <T> boolean isObjectList(List<?> l,Class<?> c) {
        if(!org.springframework.util.CollectionUtils.isEmpty(l)) {
            Object o = l.stream().filter(e->e!=null).findFirst().orElseGet(null);
            return o == null?true:c.isInstance(o);
        } else {
            return true;
        }
    }

    /**
     * 获取一个集合的下标集合
     * @param l
     * @return
     */
    public static List<Integer> analyzeIndex(List l) {
        if(l == null) {
            return null;
        } else {
            int s = l.size();
            List<Integer> r = new ArrayList<Integer>(s);
            for(int i=CommonConst.INT_0;i<s;i++) {
                r.add(i);
            }
            return r;
        }
    }

    public static <P,R> PageInfo<R> convertPage(PageInfo<P> page, Function<P, R> converter) {
        PageInfo<R> r = new PageInfo<R>();
        r.setPageSize(page.getPageSize());
        r.setPageNum(page.getPageNum());
        r.setTotal(page.getTotal());
        List<R> rs = CollectionUtil.newArrayList();
        if(!org.springframework.util.CollectionUtils.isEmpty(page.getList()) && converter != null) {
            page.getList().forEach(e->rs.add(converter.apply(e)));
        }
        r.setList(rs);
        return r;
    }
}
