package com.kingtom.kirin.core.common.utils;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.UUID;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/2 15:39
 */
@Slf4j
public class IDUtils {

    private IDUtils() {
    }

    /**
     * 创建UUID
     *
     * @return uuid
     */
    public static String newUUID() {
        return UUID.randomUUID().toString();
    }

    /**
     * 是否UUID
     *
     * @param id id
     * @return uuid
     */
    public static Boolean isUUID(String id) {
        if (StringUtils.isBlank(id)) {
            return false;
        }
        return true;
    }

    /**
     * 是否不是UUID
     *
     * @param id
     * @return boolean
     */
    public static Boolean isNotUUID(String id) {
        return !isUUID(id);
    }

    /**
     * 如果id不是有效的UUID，则创建一个新的UUID
     *
     * @param id id
     * @return uuid
     */
    public static String newUUID(String id) {
        if (isNotUUID(id)) {
            return newUUID();
        } else {
            return id;
        }
    }

    /**
     * 根据version2构建uuid
     *
     * @param s s
     * @return uuid
     */
    public static String newUUIDByString(String s) {
        try {
            return UUID.nameUUIDFromBytes(s.getBytes("utf-8")).toString();
        } catch (Exception e) {
            log.error("newUUIDByString错误", e);
            return null;
        }
    }
}
