package com.kingtom.kirin.core.common.utils;

import com.esotericsoftware.kryo.Kryo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;

import java.beans.PropertyDescriptor;
import java.util.Arrays;
import java.util.List;
import java.util.Set;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/2 16:42
 */
@Slf4j
public class ObjectUtils {

    private static ThreadLocal<Kryo> kryoThreadLocal = new ThreadLocal<Kryo>();

    private ObjectUtils(){}

    public static boolean isAnyNull(Object... os) {
        if(os == null) {
            return true;
        } else {
            for(Object o:os) {
                if(o == null) {
                    return true;
                }
            }
            return false;
        }
    }

    public static boolean isNotNull(Object... os) {
        return !isAnyNull(os);
    }

    public static <T> T copyObject(T obj) {
        return obj == null?null:getKryo(obj.getClass()).copy(obj);
    }

    protected static Kryo getKryo(Class c) {
        Kryo r = kryoThreadLocal.get();
        if(r == null) {
            r = new Kryo();
            r.setRegistrationRequired(false);
            kryoThreadLocal.set(r);
        }
        r.register(c);
        return r;
    }

    /**
     * 忽略字段复制
     * @param source
     * @param target
     * @param ignoreProperties
     * @param <T>
     */
    public static <T> void copyPropertiesByIgnore(T source, T target, String... ignoreProperties) {
        BeanUtils.copyProperties(source,target,ignoreProperties);
    }

    /**
     * 包含字段复制
     * @param source
     * @param target
     * @param coverProperties
     * @param <T>
     */
    public static <T> void copyPropertiesByCover(T source, T target, String... coverProperties) {
        Set<String> coverPropertiesSet = Set.of(coverProperties);
        String[] excludedProperties = Arrays.stream(BeanUtils.getPropertyDescriptors(source.getClass()))
                        .map(PropertyDescriptor::getName)
                        .filter(name -> !coverPropertiesSet.contains(name))
                        .toArray(String[]::new);
        BeanUtils.copyProperties(source,target,excludedProperties);
    }

}
