package com.kingtom.kirin.core.common.utils;

import cn.hutool.core.collection.CollUtil;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.ref.ReferenceImpl;
import com.kingtom.kirin.core.common.ref.ReflectFilterObject;
import com.kingtom.kirin.core.common.ref.ReflectVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;

import java.lang.reflect.*;
import java.util.*;
import java.util.function.Function;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/5 15:02
 */
@Slf4j
public class RefUtils {

    private RefUtils(){}

    private static final String LENGTH = "length";
    private static final Map<Class<?>, Map<String, Method>> READER_MAP = new WeakHashMap<Class<?>, Map<String, Method>>();
    private static final Map<Class<?>, Map<String, Method>> WRITER_MAP = new WeakHashMap<Class<?>, Map<String, Method>>();
    private static final Map<Class<?>, Map<String, Class<?>>> TYPE_MAP = new WeakHashMap<Class<?>, Map<String, Class<?>>>();
    private static Object initLock = new Object();

    /**
     * 判断c是不是class的名称
     * @param cs cs
     * @return
     */
    public static boolean isClassString(String cs) {
        if(cs == null) {
            return false;
        } else {
            try {
                Class c = Class.forName(cs);
                return true;
            } catch (Exception e) {
                log.debug("不是class名的字符串:%",cs);
                return false;
            }
        }
    }

    /**
     * 获取方法
     *
     * @param startc
     * @param methodName
     * @param cs
     * @return
     */
    public static Method getDeclaredMethod(Class startc, String methodName, Class... cs) {
        Method m = null;
        try {
            int t = 0;
            Class c = startc;
            do {
                try {
                    m = c.getDeclaredMethod(methodName, cs);
                } catch (Exception e) {
                }
                c = c.getSuperclass();
                t++;
            } while (m == null && !c.equals(Object.class) && t < 10);
            return m;
        } catch (Exception ex) {
            throw new IllegalStateException("invokeMethod error", ex);
        }
    }


    /**
     * @param target
     * @param args
     * @param need
     * @param method
     * @param cs
     * @return
     */
    public static Object invokeMethod(Object target, Object[] args, boolean need, String method, Class... cs) {
        Method m = null;
        boolean acc = false;
        try {
            m = getDeclaredMethod(target.getClass(), method, cs);
            if (!m.isAccessible() && need) {
                acc = m.isAccessible();
                m.setAccessible(true);
            }
            return invokeMethod(m, target, args);
        } catch (Exception ex) {
            throw new IllegalStateException("invokeMethod error", ex);
        } finally {
            if (m != null && need && !acc) {
                m.setAccessible(acc);
            }
        }
    }

    public static Object invokeMethod(Method method, Object target, Object[] args) {
        try {
            return method.invoke(target, args);
        } catch (Exception ex) {
            throw new IllegalStateException("invokeMethod error", ex);
        }
    }

    public static Map<String, Method> getGetterMap(final Class<?> clazz) {
        Map<String, Method> propertyMap = READER_MAP.get(clazz);
        if (propertyMap == null) {
            initProperties(clazz);
            propertyMap = READER_MAP.get(clazz);
        }
        return propertyMap;
    }

    /**
     * 获取本对象的所有值为map
     *
     * @param o
     * @param filter 指定过滤
     * @return
     */
    public static Map<String, Object> getValueGetterMap(final Object o, Function<Map.Entry<String, Method>, Boolean> filter) {
        if (o == null) {
            return null;
        } else {
            Map<String, Object> vmap = new HashMap<String, Object>(CommonConst.INT_128);
            Map<String, Method> gettermap = getGetterMap(o.getClass());
            gettermap.entrySet().stream().forEach((e) -> {
                if (filter == null || filter.apply(e)) {
                    Object v = invokeMethod(e.getValue(), o, null);
                    vmap.put(e.getKey(), v);
                }
            });
            return vmap;
        }
    }

    public static Map<String, Method> getSetterMap(final Class<?> clazz) {
        Map<String, Method> propertyMap = WRITER_MAP.get(clazz);
        if (propertyMap == null) {
            initProperties(clazz);
            propertyMap = WRITER_MAP.get(clazz);
        }
        return propertyMap;
    }

    /**
     * 从map设置值到对象中
     *
     * @param o
     * @param vmap
     * @param filter 需要指定过滤的值
     */
    public static void setValueSetterMap(final Object o, Map<String, Object> vmap, Function<ReflectVO, ReflectFilterObject> filter) {
        if (o != null && vmap != null) {
            Map<String, Method> settermap = RefUtils.getSetterMap(o.getClass());
            Function<ReflectVO, ReflectFilterObject> defaultFilter = e -> {
                if (e.getValue() == null) {
                    return ReflectFilterObject.builder().object(null).continueAssignment(true).build();
                }
                if (Date.class.equals(e.getParameterType()) && Long.class.equals(e.getValue().getClass())) {
                    return ReflectFilterObject.builder().continueAssignment(true).object(new Date((Long) e.getValue())).build();
                }
                return ReflectFilterObject.builder().object(e.getValue()).continueAssignment(true).build();
            };
            settermap.entrySet().stream().forEach((e) -> {
                Set<String> keySet = vmap.keySet();
                if (keySet.contains(e.getKey())) {
                    ReflectVO p = new ReflectVO(e.getKey(), vmap.get(e.getKey()), e.getValue());
                    if (filter == null) {
                        ReflectFilterObject defaultFilterResult = defaultFilter.apply(p);
                        if (defaultFilterResult != null && defaultFilterResult.getContinueAssignment()) {
                            try {
                                RefUtils.invokeMethod(e.getValue(), o, new Object[]{defaultFilterResult.getObject()});
                            } catch (Exception ex) {
                                LogManager.getLogger(RefUtils.class).debug("警告：自动写入属性失败!class={};method={},exception={}", e.getValue().getDeclaringClass().getName(), e.getValue().getName(), ex.getMessage());
                            }
                        }
                    } else {
                        ReflectFilterObject filterResult = filter.apply(p);
                        if (filterResult != null && filterResult.getContinueAssignment()) {
                            try {
                                RefUtils.invokeMethod(e.getValue(), o, new Object[]{filterResult.getObject()});
                            } catch (Exception ex) {
                                LogManager.getLogger(RefUtils.class).debug("警告：自动写入属性失败!class={};method={},exception={}", e.getValue().getDeclaringClass().getName(), e.getValue().getName(), ex.getMessage());
                            }

                        }
                    }
                }
            });
        }
    }

    public static void test() {
        Map m = null;
        Object o = null;
        setValueSetterMap(o, m, e -> {
            return null;
        });
    }

    /**
     * 根据匹配关系拷贝数据
     *
     * @param o
     * @param bean
     * @return
     */
    public static <E> E copyObjectByMap(Object o, E bean) {
        if (o != null && bean != null) {
            Map<String, Object> vmap = RefUtils.getValueGetterMap(o, null);
            RefUtils.setValueSetterMap(bean, vmap, null);
            return bean;
        } else {
            return null;
        }
    }

    private static Map<String, Class<?>> getTypeMap(final Class<?> clazz) {
        Map<String, Class<?>> propertyMap = TYPE_MAP.get(clazz);
        if (propertyMap == null) {
            initProperties(clazz);
            propertyMap = TYPE_MAP.get(clazz);
        }
        return propertyMap;
    }

    private static void initProperties(final Class<?> clazz) {
        synchronized (initLock) {
            HashMap<String, Method> getterMap = new HashMap<String, Method>(CommonConst.INT_128);
            HashMap<String, Method> setterMap = new HashMap<String, Method>(CommonConst.INT_128);
            HashMap<String, Class<?>> propertyMap = new HashMap<String, Class<?>>(CommonConst.INT_128);
            try {
                if (!clazz.equals(Object.class)) {
                    getterMap.putAll(getGetterMap(clazz.getSuperclass()));
                    setterMap.putAll(getSetterMap(clazz.getSuperclass()));
                    propertyMap.putAll(getTypeMap(clazz.getSuperclass()));
                }
                Method[] methods = clazz.getDeclaredMethods();

                for (Method m : methods) {
                    if ((m.getModifiers() & Modifier.PUBLIC) > 0) {
                        Class<? extends Object> type = m.getReturnType();
                        Class<? extends Object>[] params = m.getParameterTypes();
                        String name = m.getName();
                        if (type == Void.TYPE) {
                            if (params.length == 1 && name.startsWith("set")) {
                                type = params[0];
                                initMethod(setterMap, propertyMap, m, name.substring(3));
                            }
                        } else {
                            if (params.length == 0) {
                                if (name.startsWith("get")) {
                                    initMethod(getterMap, propertyMap, m, name.substring(3));
                                } else if (type == Boolean.TYPE && name.startsWith("is")) {
                                    initMethod(getterMap, propertyMap, m, name.substring(2));
                                }
                            }
                        }
                    }
                }
            } catch (Exception e) {
                if(log.isWarnEnabled()) {
                    log.warn("初始化属性集合异常", e);
                }
            } finally {
                READER_MAP.put(clazz, Collections.unmodifiableMap(getterMap));
                WRITER_MAP.put(clazz, Collections.unmodifiableMap(setterMap));
                TYPE_MAP.put(clazz, Collections.unmodifiableMap(propertyMap));
            }
        }
    }

    private static void initMethod(Map<String, Method> propertyMap, Map<String, Class<?>> typeMap, Method m,
                                   String name) {
        if (name.length() > 0) {
            char c = name.charAt(0);
            if (Character.isUpperCase(c)) {
                name = Character.toLowerCase(c) + name.substring(1);
                m.setAccessible(true);
                propertyMap.put(name, m);
                Class<?> type = m.getReturnType();
                if (type == Void.TYPE) {
                    type = m.getParameterTypes()[0];
                }
                Class<?> ot = typeMap.get(name);
                if (ot != null) {
                    if (ot != type) {
                        log.warn("属性类型冲突：" + ot + "!=" + type);
                    }
                }
                typeMap.put(name, type);
            }
        }

    }

    @SuppressWarnings("unchecked")
    public static Map<String, ? extends Object> map(final Object context) {
        if (context == null) {
            return Collections.EMPTY_MAP;
        } else if (context instanceof Map) {
            return (Map<String, ? extends Object>) context;
        }
        return proxyMap(context, getTypeMap(context.getClass()).keySet());
    }

    private static HashMap<String, Object> proxyMap(Object base, Set<String> ps) {
        HashMap<String, Object> result = new HashMap<String, Object>(CommonConst.INT_128);
        for (String key : ps) {
            result.put(key, new ReferenceImpl(base, key));
        }
        return result;
    }

    private static int toIndex(Object key) {
        return key instanceof Number ? ((Number) key).intValue() : Integer.parseInt(String.valueOf(key));
    }

    /**
     * 获取所有的变量
     * @param clazz 类
     * @param rootclazz 查找至于哪个类
     * @param containStatic 是否包含静态
     * @return
     */
    public static List<Field> getDeclaredFieldsToRoot(Class clazz, Class rootclazz, boolean containStatic) {
        List<Field> r = CollUtil.newArrayList();
        Class tmp = clazz;
        do {
            try {
                Field[] fs = tmp.getDeclaredFields();
                for(Field f:fs) {
                    if(containStatic || !Modifier.isStatic(f.getModifiers())) {
                        r.add(f);
                    }
                }
            } catch (Exception e) {
            }
            Class sclazz = tmp.getSuperclass();
            if (sclazz == null || (rootclazz != null && rootclazz == sclazz)) {
                break;
            }
            tmp = sclazz;
        } while (true);
        return r;
    }

    /**
     * 查找成员变量，一直到根
     *
     * @param clazz clazz
     * @param fieldName fieldName
     */
    public static Field getDeclaredFieldToRoot(Class clazz, Class rootclazz, String fieldName) {
        if(clazz == null || StringUtils.isBlank(fieldName)) {
            return null;
        }
        Field f = null;
        Class tmp = clazz;
        do {
            try {
                f = tmp.getDeclaredField(fieldName);
            } catch (Exception e) {
                f = null;
            }
            Class sclazz = tmp.getSuperclass();
            if (f != null || sclazz == null || (rootclazz != null && rootclazz == sclazz)) {
                break;
            }
            tmp = sclazz;
        } while (true);
        return f;
    }


    /**
     * 查找成员变量，一直到根
     *
     * @param obj obj
     * @param rootClazz rootClazz
     * @param fieldName fieldName
     * @return
     */
    public static Object getDeclaredFieldValue(Object obj, Class rootClazz, String fieldName) {
        if (obj == null || StringUtils.isBlank(fieldName)) {
            return null;
        }
        Object result = null;
        try {
            Field f = getDeclaredFieldToRoot(obj.getClass(), rootClazz, fieldName);
            result = getDeclaredFieldValue(obj,f);
        } catch (Exception e) {
            if(log.isErrorEnabled()) {
                log.error("获取" + rootClazz.toString() + "的" + fieldName + "错误", e);
            }
        }
        return result;
    }

    public static Object getDeclaredFieldValue(Object obj, Field f) {
        if (obj == null || f == null) {
            return null;
        }
        Object result = null;
        try {
            boolean b = f.isAccessible();
            f.setAccessible(true);
            result = f.get(obj);
            f.setAccessible(b);
        } catch (Exception e) {
            if(log.isErrorEnabled()) {
                log.error("获取" + obj.getClass().toString() + "的" + f.toString() + "错误", e);
            }
        }
        return result;
    }

    public static void setDeclaredFieldValue(Object obj, Object v, String fieldName) {
        Field f = getDeclaredFieldToRoot(obj.getClass(), null, fieldName);
        setDeclaredFieldValue(obj, f, v);
    }

    public static void setDeclaredFieldValue(Object obj, Field f,Object v) {
        if (obj != null && f != null) {
            try {
                boolean b = f.isAccessible();
                f.setAccessible(true);
                f.set(obj, v);;
                f.setAccessible(b);
            } catch (Exception e) {
                if(log.isErrorEnabled()) {
                    log.error("设置" + obj.getClass().toString() + "的" + f.toString() + "错误", e);
                }
            }
        }
    }

    public static Class<? extends Object> getValueType(Type type) {
        Type result = null;
        Class<? extends Object> clazz = null;
        if (type instanceof ParameterizedType) {
            clazz = (Class<?>) ((ParameterizedType) type).getRawType();
        } else {
            clazz = (Class<?>) type;
        }
        if (Collection.class.isAssignableFrom(clazz)) {
            result = getParameterizedType(type, Collection.class, 0);
        } else if (Map.class.isAssignableFrom(clazz)) {
            result = getParameterizedType(type, Map.class, 1);
        }
        if (result != null) {
            return findClass(result);
        }
        return Object.class;
    }

    public static Class<?> getKeyType(Type type) {
        Class<?> clazz = null;
        if (type instanceof ParameterizedType) {
            clazz = (Class<?>) ((ParameterizedType) type).getRawType();
        } else {
            clazz = (Class<?>) type;
        }
        if (Map.class.isAssignableFrom(clazz)) {
            Type result = getParameterizedType(type, Map.class, 0);
            if (result != null) {
                return findClass(result);
            }
        }
        return Integer.TYPE;
    }

    private static Type getParameterizedType(Type type, Class<?> destClass, int paramIndex) {
        Class<?> clazz = null;
        ParameterizedType pt = null;
        Type[] ats = null;
        TypeVariable<?>[] tps = null;
        if (type instanceof ParameterizedType) {
            pt = (ParameterizedType) type;
            clazz = (Class<?>) pt.getRawType();
            ats = pt.getActualTypeArguments();
            tps = clazz.getTypeParameters();
        } else {
            clazz = (Class<?>) type;
        }
        if (destClass.equals(clazz)) {
            if (pt != null) {
                if( paramIndex< pt.getActualTypeArguments().length){
                    return pt.getActualTypeArguments()[paramIndex];
                }
                throw new IllegalArgumentException("数组越界");
            }
            return Object.class;
        }
        Class<?>[] ifs = clazz.getInterfaces();
        Type[] pis = clazz.getGenericInterfaces();
        for (int i = 0; i < ifs.length; i++) {
            Class<?> ifc = ifs[i];
            if (destClass.isAssignableFrom(ifc)) {
                return getTureType(getParameterizedType(pis[i], destClass, paramIndex), tps, ats);
            }
        }
        Class<?> type2 = clazz.getSuperclass();
        if (type2 != null) {
            if (destClass.isAssignableFrom(type2)) {
                return getTureType(getParameterizedType(clazz.getGenericSuperclass(), destClass, paramIndex), tps, ats);
            }
        }
        throw new IllegalArgumentException("必须是Collection 子类");
    }

    private static Type getTureType(Type type, TypeVariable<?>[] typeVariables, Type[] actualTypes) {
        if (type instanceof Class<?>) {
            return type;
        } else if (type instanceof TypeVariable<?>) {
            TypeVariable<?> tv = (TypeVariable<?>) type;
            String name = tv.getName();
            if (actualTypes != null) {
                for (int i = 0; i < typeVariables.length; i++) {
                    if (name.equals(typeVariables[i].getName())) {
                        return actualTypes[i];
                    }
                }
            }
            return tv;
        }
        return type;
    }

    private static Class<? extends Object> findClass(Type result) {
        if (result instanceof Class<?>) {
            return (Class<?>) result;
        } else if (result instanceof ParameterizedType) {
            return findClass(((ParameterizedType) result).getRawType());
        } else if (result instanceof WildcardType) {
            return findClass(((WildcardType) result).getUpperBounds()[0]);
        }
        return null;
    }

    public static Class<?> getPropertyType(Type type, Object key) {
        Class<?> clazz = findClass(type);
        if (clazz != null) {
            if (clazz.isArray()) {
                if (LENGTH.equals(key)) {
                    return Integer.TYPE;
                } else if (Number.class.isInstance(key)) {
                    return clazz.getComponentType();
                }
            } else if (Collection.class.isAssignableFrom(clazz)) {
                return getValueType(type);
            } else if (Map.class.isAssignableFrom(clazz)) {
                return getValueType(type);
            } else {
                Class<?> pd = getTypeMap(clazz).get(String.valueOf(key));
                if (pd != null) {
                    return pd;
                }
            }
        }
        return null;

    }

    public static Object getValue(Object context, Object key) {
        if (context != null) {
            try {
                if (context.getClass().isArray()) {
                    if (LENGTH.equals(key)) {
                        return Array.getLength(context);
                    } else {
                        return Array.get(context, toIndex(key));
                    }
                } else if (context instanceof Collection<?>) {
                    if (LENGTH.equals(key)) {
                        return ((Collection<?>) context).size();
                    } else if (context instanceof List<?>) {
                        return ((List<?>) context).get(toIndex(key));
                    }
                } else if (context instanceof String) {
                    if (LENGTH.equals(key)) {
                        return ((String) context).length();
                    }
                }
                if (context instanceof Map<?, ?>) {
                    return ((Map<?, ?>) context).get(key);
                }
                Method method = getGetterMap(context.getClass()).get(String.valueOf(key));
                if (method != null) {
                    return method.invoke(context);
                }
            } catch (Exception e) {
                if (log.isDebugEnabled()) {
                    log.debug("", e);
                }
            }
        }
        if (key instanceof String) {
            try {

                if (context instanceof Class<?>) {
                    return ((Class<?>) context).getField((String) key).get(context);
                } else {
                    return context.getClass().getField((String) key).get(context);
                }
            } catch (NoSuchFieldException e) {
            } catch (Exception e) {
                if (log.isDebugEnabled()) {
                    log.debug("", e);
                }
            }
        }
        return null;
    }

    public static <T> T setValues(T base, Map<String, Object> attributeMap) {
        for (String key : attributeMap.keySet()) {
            RefUtils.setValue(base, key, attributeMap.get(key));
        }
        return base;
    }

    @SuppressWarnings("unchecked")
    public static <T> T setValue(T base, Object key, Object value) {
        if (base != null) {
            try {
                if (base.getClass().isArray()) {
                    Array.set(base, toIndex(key), value);
                } else if (base instanceof List<?>) {
                    ((List<Object>) base).set(toIndex(key), value);
                }
                if (base instanceof Map) {
                    ((Map<Object, Object>) base).put(key, value);
                }
                Method method = getSetterMap(base.getClass()).get(String.valueOf(key));
                if (method != null) {
                    if (value != null) {
                        Class<?> type = method.getParameterTypes()[0];
                        if (!type.isInstance(value)) {
                            type = toWrapper(type);
                            if (Number.class.isAssignableFrom(type)) {
                                value = toValue((Number) value, type);
                            }
                        }
                    }
                    method.invoke(base, value);
                }
            } catch (Exception e) {
                if (log.isDebugEnabled()) {
                    log.debug("", e);
                }
            }
        }
        return base;
    }

    public static Number toValue(Number value, Class<? extends Object> type) {
        if (type == Long.class) {
            return value.longValue();
        } else if (type == Integer.class) {
            return value.intValue();
        } else if (type == Short.class) {
            return value.shortValue();
        } else if (type == Byte.class) {
            return value.byteValue();
        } else if (type == Double.class) {
            return value.doubleValue();
        } else if (type == Float.class) {
            return value.floatValue();
        } else {
            Class<? extends Object> clazz = RefUtils.toWrapper(type);
            if (clazz == type) {
                return null;
            } else {
                return toValue(value, clazz);
            }
        }
    }

    public final static Class<?> toWrapper(Class<? extends Object> type) {
        if (type.isPrimitive()) {
            if (Byte.TYPE == type) {
                return Byte.class;
            } else if (Short.TYPE == type) {
                return Short.class;
            } else if (Integer.TYPE == type) {
                return Integer.class;
            } else if (Long.TYPE == type) {
                return Long.class;
            } else if (Float.TYPE == type) {
                return Float.class;
            } else if (Double.TYPE == type) {
                return Double.class;
            } else if (Character.TYPE == type) {
                return Character.class;
            } else if (Boolean.TYPE == type) {
                return Boolean.class;
            }
        }
        return type;
    }
}
