package com.kingtom.kirin.core.database.tools;

import cn.hutool.db.DbUtil;
import com.kingtom.kirin.core.common.base.FunctionExp;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.utils.SpringContextUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.jdbc.datasource.DataSourceUtils;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 * 数据库交互
 * @author King
 * @since 2023/10/17 20:10
 */
@Slf4j
public class DbExcuter {

    private DataSource dataSource;

    private DbExcuter(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    public <R> R excuteStatement(FunctionExp<Statement, R> f) {
        R r = excute(c -> {
            R r1 = null;
            Statement s = null;
            try {
                s = c.createStatement();
                r1 = f.apply(s);
            } catch (Exception e) {
                log.error("", e);
                throw new AppException(e);
            } finally {
                DbUtil.close(s);
            }
            return r1;

        });
        return r;
    }

    public <R> R excutePreStatement(FunctionExp<PreparedStatement, R> f, String presql) {
        R r = excute(c -> {
            R r1 = null;
            PreparedStatement ps = null;
            try {
                ps = c.prepareStatement(presql);
                r1 = f.apply(ps);
            } catch (Exception e) {
                log.error("", e);
                throw new AppException(e);
            } finally {
                DbUtil.close(ps);
            }
            return r1;
        });
        return r;
    }

    public <R> R excute(FunctionExp<Connection, R> f) {
        R r = null;
        Connection c = null;
        try {
            c = DataSourceUtils.getConnection(getDataSource());
            r = f.apply(c);
        } catch (AppException e){
            log.error("", e);
            throw e;
        }catch (Exception e) {
            log.error("", e);
            throw new AppException(e);
        } finally {
            DataSourceUtils.releaseConnection(c, dataSource);
        }
        return r;
    }

    protected DataSource getDataSource() {
        if (dataSource == null) {
            synchronized (this) {
                if (dataSource == null) {
                    dataSource = SpringContextUtils.getBean("dataSource", DataSource.class);
                }
            }
        }
        return dataSource;
    }

    @SuppressWarnings("rawtypes")
    public static void setPreparedStatements(PreparedStatement ps, List params) throws SQLException {
        try {
            for (int i = 0, s = params.size(); i < s; i++) {
                Object o = params.get(i);
                ps.setObject(i + 1, o);
            }
        } catch (SQLException e) {
            throw new AppException("",e);
        }
    }

    public static DbExcuter newInstance(DataSource dataSource) {
        return new DbExcuter(dataSource);
    }

}
