package com.kingtom.kirin.generator.orm.mybatis;

import org.mybatis.generator.api.IntrospectedColumn;
import org.mybatis.generator.api.IntrospectedTable;
import org.mybatis.generator.api.dom.java.*;
import org.mybatis.generator.internal.DefaultCommentGenerator;
import org.mybatis.generator.internal.util.StringUtility;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Properties;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/7/2 20:19
 */
public class KirinCommentGenerator extends DefaultCommentGenerator {

    /**
     * properties配置文件
     */
    private Properties properties = new Properties();
    /**
     * properties配置文件
     */
    private Properties systemPro;
    private boolean suppressDate = false;
    private boolean suppressAllComments = false;
    private boolean addRemarkComments = false;
    private SimpleDateFormat dateFormat;

    public static final String MARK = "(MyBatis-Generator)";

    @Override
    public void addConfigurationProperties(Properties properties) {
        super.addConfigurationProperties(properties);
        this.systemPro = System.getProperties();
        this.properties.putAll(properties);
        this.suppressDate = StringUtility.isTrue(properties.getProperty("suppressDate"));
        this.suppressAllComments = StringUtility.isTrue(properties.getProperty("suppressAllComments"));
        this.addRemarkComments = StringUtility.isTrue(properties.getProperty("addRemarkComments"));
        String dateFormatString = properties.getProperty("dateFormat");
        if (StringUtility.stringHasValue(dateFormatString)) {
            this.dateFormat = new SimpleDateFormat(dateFormatString);
        }
    }

    @Override
    public void addModelClassComment(TopLevelClass topLevelClass, IntrospectedTable introspectedTable) {
        if (suppressAllComments || !addRemarkComments) {
            super.addModelClassComment(topLevelClass, introspectedTable);
        } else {
            // 给生成的bean生成注释
            topLevelClass.addJavaDocLine("/**");
            topLevelClass.addJavaDocLine("* <p>");
            topLevelClass.addJavaDocLine("* Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>");
            topLevelClass.addJavaDocLine("* Company: 成都诚唐科技有限公司<p>");
            topLevelClass.addJavaDocLine("* " + introspectedTable.getRemarks() +"(" +introspectedTable.getFullyQualifiedTable()+")"+ MARK);
            topLevelClass.addJavaDocLine("* @author " + systemPro.getProperty("user.name"));
            topLevelClass.addJavaDocLine("* @since " + dateFormat.format(new Date()));
            topLevelClass.addJavaDocLine("*/");
        }
    }

    @Override
    public void addClassComment(InnerClass innerClass, IntrospectedTable introspectedTable) {
        addClassComment(innerClass, introspectedTable, false);
    }

    @Override
    public void addClassComment(InnerClass innerClass, IntrospectedTable introspectedTable, boolean markAsDoNotDelete) {
        if (suppressAllComments) {
            super.addClassComment(innerClass, introspectedTable, markAsDoNotDelete);
        } else {
            // 给example的内部类生成注释
            innerClass.addJavaDocLine("/**"); //$NON-NLS-1$
            innerClass.addJavaDocLine(" * " + introspectedTable.getRemarks() + " " +innerClass.getType().getShortName() + MARK); //$NON-NLS-1$
            innerClass.addJavaDocLine(" */"); //$NON-NLS-1$
        }
    }

    @Override
    public void addFieldComment(Field field, IntrospectedTable introspectedTable, IntrospectedColumn introspectedColumn) {
        if (this.suppressAllComments) {
            super.addFieldComment(field, introspectedTable, introspectedColumn);
        } else {
            field.addJavaDocLine("/**");
            field.addJavaDocLine(" * " + introspectedColumn.getRemarks() + "("+introspectedTable.getFullyQualifiedTable() + "." + introspectedColumn.getActualColumnName()+")" + MARK);
            field.addJavaDocLine(" */");
        }
    }

    public void addFieldComment(Field field, IntrospectedTable introspectedTable) {
        if (this.suppressAllComments) {
            super.addFieldComment(field, introspectedTable);
        } else {
            field.addJavaDocLine("/**");
            field.addJavaDocLine(" * " + field.getName());
            field.addJavaDocLine(" */");
        }
    }

    @Override
    public void addGetterComment(Method method, IntrospectedTable introspectedTable, IntrospectedColumn introspectedColumn) {
        if (suppressAllComments) {
            super.addGetterComment(method, introspectedTable, introspectedColumn);
        } else {
            method.addJavaDocLine("/**");
            method.addJavaDocLine(" * 获取 " + introspectedColumn.getRemarks() + "("+introspectedTable.getFullyQualifiedTable()+"."+introspectedColumn.getActualColumnName()+")" + MARK);
            method.addJavaDocLine(" * @return " + introspectedColumn.getRemarks());
            method.addJavaDocLine(" */");
        }
    }

    @Override
    public void addSetterComment(Method method, IntrospectedTable introspectedTable, IntrospectedColumn introspectedColumn) {
        if (suppressAllComments) {
            super.addSetterComment(method, introspectedTable, introspectedColumn);
        } else {
            method.addJavaDocLine("/**");
            method.addJavaDocLine(" * 设置 " + introspectedColumn.getRemarks() + "("+introspectedTable.getFullyQualifiedTable()+"."+introspectedColumn.getActualColumnName()+")" + MARK);
            Parameter parm = method.getParameters().get(0);
            method.addJavaDocLine(" * @param " + parm.getName()+ " " + introspectedColumn.getRemarks());
            method.addJavaDocLine(" */");
        }
    }

    @Override
    public void addGeneralMethodComment(Method method, IntrospectedTable introspectedTable) {
        if (suppressAllComments) {
            super.addGeneralMethodComment(method,introspectedTable);
        } else {
            method.addJavaDocLine("/**");
            method.addJavaDocLine(" * " + introspectedTable.getRemarks() + " " + method.getName() + MARK);
            method.addJavaDocLine(" */");
        }
    }
}
